/*
 * @DEC_COPYRIGHT@
 */
/*
 * HISTORY
 * $Log: cam_data.c,v $
 * Revision 1.1.102.3  1997/04/17  20:18:16  Fred_Knight
 * 	 	** Merge Information **
 * 		** Command used:	bmerge -jPTBOS_BL4:PTHW1OS_BL3 **
 * 		** Ancestor revision:	1.1.77.9 **
 * 		** Merge revision:	1.1.100.2 **
 * 	 	** End **
 * 	[1997/04/17  19:42:15  Fred_Knight]
 *
 * Revision 1.1.102.2  1997/04/03  20:16:31  Dick_Bagley
 * 	Added "isp_fw_load" variable, which can be disabled via
 * 	boot command line kernarg, to support QLogic firmware
 * 	loading from the root filesystem.
 * 	[1997/04/03  14:16:27  Dick_Bagley]
 * 
 * Revision 1.1.100.2  1997/01/10  00:15:28  Fred_Knight
 * 	Add ata/ide entry (ata_bus_table).
 * 	[1997/01/09  20:57:53  Fred_Knight]
 * 
 * Revision 1.1.77.9  1995/08/22  20:21:30  David_Gagne
 * 	Dev Nmg: Explicit use of NDPS_X & NLPT_X not just NDPS/NLPT.
 * 	[1995/08/21  17:56:15  David_Gagne]
 * 
 * Revision 1.1.77.8  1995/07/21  17:45:58  Fred_Knight
 * 	Merge from ptliteos BL4->BL6 including the following:
 * 	(note, new device support added elsewhere in V4.0)
 * 	Revision 1.1.76.4  1995/06/12  18:13:21  Janet_Schank
 * 	      Added simport_softc_directory[].
 * 	      [1995/06/07  13:02:17  Janet_Schank]
 * 
 * 	Revision 1.1.76.3  1995/05/22  19:47:01  Fred_Knight
 * 	      Add entries for RZ25F, RZ26F, RZ226J, RZ28D, and RZ28J disks.
 * 	      [1995/05/19  22:36:50  Fred_Knight]
 * 	[1995/07/21  17:40:25  Fred_Knight]
 * 
 * Revision 1.1.77.7  1995/07/05  15:11:30  James_Smart
 * 	Include camdb.h and kds.h.
 * 	[1995/07/04  11:30:47  James_Smart]
 * 
 * Revision 1.1.77.6  1995/06/23  19:15:41  James_Smart
 * 	Add psiop_min_selection_timeout for NPSIOP
 * 	[1995/06/21  20:02:12  James_Smart]
 * 
 * Revision 1.1.77.5  1995/06/16  13:46:20  James_Smart
 * 	We're re-deleting the device information and enable DDR again.
 * 	[1995/06/13  16:31:50  James_Smart]
 * 
 * Revision 1.1.77.4  1995/05/24  18:27:03  Carol_Sheridan
 * 	Merge of Platinum Lite BL4 changes
 * 	[1995/05/24  18:15:06  Carol_Sheridan]
 * 
 * Revision 1.1.77.3  1995/04/24  18:03:31  James_Smart
 * 	Due to console problems, DDR is unable to access it's database
 * 	on bootup. Thus, devices requiring "exceptions" are encountering
 * 	errors. Until problem resolved - back in device information.
 * 	[1995/04/24  17:41:42  James_Smart]
 * 
 * Revision 1.1.77.2  1995/04/14  15:30:01  James_Smart
 * 	DDR: strip out Digital-specific elements as they are now part of
 * 	     the DDR database. Left just enough to support skeleton of
 * 	     the interface so that it can still be used.
 * 	[1995/03/09  19:52:51  James_Smart]
 * 
 * Revision 1.1.76.2  1995/03/16  18:20:19  Frederick_Canter
 * 	Moved SCSI CAM XPT pool parameters to param.c, so
 * 	they can be configured via /etc/sysconfigtab.
 * 	[1995/03/14  18:16:35  Frederick_Canter]
 * 
 * Revision 1.1.71.8  1994/12/19  21:38:21  Fred_Knight
 * 	Add TZK13, update several tag queue depths, update RZ29 sizes
 * 	to what the real device is.  Also reduce HSZ40 match to HSZ4
 * 	to allow HSZ42/45 etc... support.
 * 	[1994/12/19  21:33:53  Fred_Knight]
 * 
 * Revision 1.1.71.7  1994/12/06  14:41:14  William_Grava
 * 	merge of Hardware BL7 changes
 * 	[1994/12/06  14:30:03  William_Grava]
 * 
 * Revision 1.1.71.6  1994/11/21  23:44:52  Fred_Knight
 * 	Add TZK20 tape drive using default densities.
 * 	[1994/11/21  19:51:54  Fred_Knight]
 * 
 * 	Set flag for HSZ40 to allow long select timeout retry for dual
 * 	controller failover.
 * 	[1994/11/21  18:59:15  Fred_Knight]
 * 
 * Revision 1.1.71.5  1994/11/21  21:53:12  John_Meneghini
 * 	Bug fix QAR 25590 - Add PDRV_UNIT_INFO table. This table contains an
 * 	array of cam_pd_open_unit_li lock data structures used to synchronize
 * 	access to the corresponding pdrv_unit_table during Opens.
 * 	[1994/11/17  18:17:51  John_Meneghini]
 * 
 * Revision 1.1.71.4  1994/11/18  18:59:58  James_Smart
 * 	Pre-merge of HW3 BL7
 * 	[1994/11/17  17:38:41  James_Smart]
 * 
 * Revision 1.1.69.3  1994/11/11  19:49:30  Fred_Knight
 * 	Adjust RZ25M entry (correct queue depth).
 * 	[1994/11/11  16:34:23  Fred_Knight]
 * 
 * Revision 1.1.71.3  1994/11/07  21:33:55  Fred_Knight
 * 	Add TZ88, TZ885, and TZ887.  Also fix TZK10 (QIC150 - 1 file mark)
 * 	and TKZ60C (set COMPRESS_VALID) problems.
 * 	[1994/11/04  18:23:14  Fred_Knight]
 * 
 * Revision 1.1.71.2  1994/10/04  15:31:26  James_Smart
 * 	QAR 25393: Moved "default_810_clk_speed" from driver_support.c to here.
 * 	[1994/09/21  19:54:29  James_Smart]
 * 
 * Revision 1.1.69.2  1994/08/26  21:33:09  Fred_Knight
 * 	Add RZ26N entry.
 * 	[1994/08/23  19:09:40  Fred_Knight]
 * 
 * Revision 1.1.48.13  1994/05/23  21:12:56  Fred_Knight
 * 	Change cdisk_to_def to 10 since floppies need 5-8 seconds.
 * 	This is for QAR 22390
 * 	[1994/05/23  20:49:36  Fred_Knight]
 * 
 * 	Add RZ25M, RZ29, RZ29B, RZ29L, RZ29M, and RZ59.
 * 	[1994/05/12  21:55:29  Fred_Knight]
 * 
 * Revision 1.1.48.12  1994/05/10  21:25:34  Donald_Dutile
 * 	Add ISP/isp/qlogic.
 * 	[1994/05/05  19:04:14  Donald_Dutile]
 * 
 * Revision 1.1.48.11  1994/05/10  13:29:06  Robin_Miller
 * 	Fix for QARs 18311 & 22323:
 * 	Disable the sim_allow_io_priority_sorting flag since this has a negative
 * 	affect on synchronous I/O requests (as instructed by Bill Dallas).
 * 	Fix for QAR 22410:
 * 	Change the field where the dynamic geometry device flag
 * 	(SZ_DYNAMIC_GEOM) gets set.  Use dd_flags field, not dd_scsi_optcmds
 * 	(optional cmds) field.
 * 	[1994/05/10  13:24:31  Robin_Miller]
 * 
 * Revision 1.1.48.10  1994/04/19  21:54:37  Stuart_Hollander
 * 	merge agoshw2 bl5 to gold
 * 	Revision 1.1.47.6  1994/04/15  17:21:14  Mike_MacKay
 * 	Moved the static array allocation (pci_script_space and pci_local_space)
 * 	from psiop_pci.c to here so that it can be conditionalized by NPSIOP.
 * 	In addition, made these arrays two dimensional for multi-NCR'810
 * 	support.
 * 
 * 	Note that this static allocation should be replaced by dynamic
 * 	allocation when dynamic contiguous memory allocation is supported.
 * 	[1994/04/15  16:18:49  Mike_MacKay]
 * 
 * Revision 1.1.48.9  1994/04/07  13:09:19  Fred_Knight
 * 	Merge
 * 	[1994/04/04  20:00:49  Fred_Knight]
 * 
 * 	Fix some queue depths (RZ26L and RZ28B) and add new CD's.
 * 	[1994/04/04  19:51:21  Fred_Knight]
 * 
 * 	Add Generic DEC RZ device support.  Turn off RZ25L and RZ28B tagged
 * 	queueing until the HW is fixed.  Turn reordering back on.
 * 	[1994/04/04  19:26:51  Fred_Knight]
 * 
 * Revision 1.1.48.8  1994/03/29  21:46:55  Janet_Schank
 * 	Added PZA support.
 * 	[1994/03/21  13:56:12  Janet_Schank]
 * 
 * Revision 1.1.48.7  1994/03/28  01:43:34  Tim_Hoskins
 * 	convert rcsid to pragma ident
 * 	[1994/03/28  01:34:59  Tim_Hoskins]
 * 
 * Revision 1.1.48.6  1994/03/25  14:02:11  Stuart_Hollander
 * 	Revision 1.1.47.5  1994/03/21  22:29:53  Fred_Knight
 * 	Turn off tags for RZ28B since they have the same problem as RZ25Ls.
 * 	[1994/03/21  22:29:02  Fred_Knight]
 * 
 * 	Revision 1.1.47.4  1994/03/18  21:58:29  Fred_Knight
 * 	Check John's change in (and temporarly set NO_TAG on the RZ25L).
 * 	[1994/03/18  21:54:54  Fred_Knight]
 * 
 * 	Revision 1.1.61.2  1994/03/17  21:28:37  John_Gallant
 * 	Add sim810_fast_enable to allow control over psiop speed.
 * 	[1994/03/24  22:52:59  Stuart_Hollander]
 * 
 * Revision 1.1.48.5  1994/03/03  00:12:30  Mike_MacKay
 * 	Merge to nightly.
 * 	[1994/03/02  19:39:16  Mike_MacKay]
 * 
 * 	Made number of asc controllers also dependent on the
 * 	number of tcds controllers (times 2).
 * 	[1994/02/28  18:36:27  Mike_MacKay]
 * 
 * 	removed NSIOP from NCAM define
 * 	[1994/02/26  01:23:31  Mike_MacKay]
 * 
 * 	Changes for SCSI Controller renaming project.  See proj. plan for
 * 	details.  Removed references to "sii" and "kzq."
 * 	[1994/02/26  01:14:41  Mike_MacKay]
 * 
 * Revision 1.1.48.4  1994/02/05  00:33:29  Joseph_Szczypek
 * 	merge of Sterling BL9 changes
 * 	[1994/02/04  23:32:02  Joseph_Szczypek]
 * 
 * Revision 1.1.48.3  1994/01/23  21:09:27  Stuart_Hollander
 * 	Merge agoshw2 bl2 to gold bl8
 * 	[1994/01/12  20:07:19  Stuart_Hollander]
 * 
 * Revision 1.1.48.2  1994/01/10  16:31:56  Theresa_Chin
 * 	Added KZTSA counters enable flag.  Added SIMport carriers and BSMs pool
 * 	variables.
 * 	[1993/12/22  20:02:30  Theresa_Chin]
 * 
 * Revision 1.1.47.2  1994/01/06  20:06:59  John_Gallant
 * 	Included the NCR 53c810 include file, psiop.h, and added the NPSIOP
 * 	count to NCAM.
 * 	[1994/01/06  15:35:28  John_Gallant]
 * 	Revision 1.1.48.2  1994/01/10  16:31:56  Theresa_Chin
 * 	Added KZTSA counters enable flag.  Added SIMport carriers and BSMs pool
 * 	variables.
 * 	[1993/12/22  20:02:30  Theresa_Chin]
 * 
 * Revision 1.1.37.11  1993/12/08  23:08:48  Fred_Knight
 * 	Increase (to 90) HSZxx maximum spinup delays (this was necessary
 * 	because "full" HSZ configurations may take a while to spin up all
 * 	the attached disks).  Add NO_TAG and REORDER back to the RZ58 since
 * 	it can't do tagged queueing.  Also change EZ spinup delays back to
 * 	120 (merge problem from 1.1.37.6).
 * 	[1993/12/07  19:07:43  Fred_Knight]
 * 
 * Revision 1.1.37.10  1993/11/19  21:26:37  Fred_Knight
 * 	Add tcds interrupt loop limit value.
 * 	Add tlz07 density table entry.
 * 	[1993/11/19  16:25:15  Fred_Knight]
 * 
 * Revision 1.1.37.9  1993/10/14  20:19:33  William_Dallas
 * 	Fixed tape tables for tzk11 and others
 * 	[1993/10/14  19:59:55  William_Dallas]
 * 
 * Revision 1.1.37.8  1993/10/14  12:49:43  Robin_Miller
 * 	Removed EZ58 device entries (EZ58R now used for EZ58).
 * 	Removed the "R" from EZ device names (EZ51R, EZ54R, & EZ58R).
 * 	[1993/10/14  12:47:46  Robin_Miller]
 * 
 * Revision 1.1.37.7  1993/09/09  18:37:48  William_Dallas
 * 	Added Dev_XX device names where there should be some, fixed
 * 	tlz6 & 7 names. Add disperse queue for target/lun combos.
 * 	[1993/09/03  18:51:38  William_Dallas]
 * 
 * Revision 1.1.37.6  1993/08/24  16:50:45  Maria_Vella
 * 	Added device descriptor entries for the RZ28L, RZ28M, and EZ58R.
 * 	Changed  SZ_READY_DEF time to 120 (seconds) for all of the EZ
 * 	devices (EZ51R, EZ54R, EZ58).
 * 	[1993/08/16  14:54:14  Maria_Vella]
 * 
 * Revision 1.1.37.5  1993/08/13  19:42:35  Kuo_Hsieh
 * 	Merged with AGOSMINOR_NIGHTLY.
 * 	[1993/08/13  19:36:59  Kuo_Hsieh]
 * 
 * 	Added TZA adapter support.
 * 	[1993/07/30  19:27:40  Kuo_Hsieh]
 * 
 * Revision 1.1.37.4  1993/07/27  17:27:38  William_Dallas
 * 	Turned on Tag Queuing for devices that support it. Added HSZ20 and
 * 	HSZ40 descriptors (RAID controllers).
 * 	[1993/07/27  13:23:21  William_Dallas]
 * 
 * Revision 1.1.37.3  1993/07/07  19:37:15  Stuart_Hollander
 * 	merge of ag maintenance BL6 changes
 * 	[1993/07/07  19:24:43  Stuart_Hollander]
 * 
 * Revision 1.1.37.2  1993/06/24  22:32:07  Randall_Brown
 * 	Submit of AGOSHW pool into Sterling
 * 	[1993/06/24  21:13:11  Randall_Brown]
 * 
 * Revision 1.1.14.6  1993/06/11  23:10:33  Randall_Brown
 * 	Revision 1.1.22.4  1993/04/30  20:41:26  Robin_Miller
 * 	Fix for QAR 11970:
 * 	Removed SZ_PREV_ALLOW flag so removable media can be ejected
 * 	while the device is still open.  Necessary for SoftPC.
 * 	[1993/04/30  20:39:17  Robin_Miller]
 * 
 * Revision 1.1.22.8  1993/05/28  19:58:09  Ritu_Bahl
 * 	Added #include for cam_tape.h
 * 	[1993/05/28  19:47:36  Ritu_Bahl]
 * 
 * Revision 1.1.22.7  1993/05/26  18:17:29  Ritu_Bahl
 * 	Changed default blocking for TZK10/QIC-320 to variable.
 * 	Added "den_blocking" field to the density_table structure.
 * 	[1993/05/26  18:16:36  Ritu_Bahl]
 * 
 * Revision 1.1.22.6  1993/05/25  19:44:50  Maria_Vella
 * 	Added device descriptor entries for the for the EZ51R, EZ54R, EZ58,
 * 	TLZ07 and TLZ7.
 * 	[1993/05/24  17:07:43  Maria_Vella]
 * 
 * Revision 1.1.22.5  1993/05/24  16:46:59  Maria_Vella
 * 	Changed the hard and soft error reported limits to 1000.
 * 	[1993/05/20  17:02:17  Maria_Vella]
 * 
 * 	Increased the hard and soft error reported limits.
 * 	Added device descriptor entries for RZ26L and RZ26M.
 * 	Added two new densities for the TZK11 tape drive in the
 * 	tape density table.
 * 	[1993/05/18  21:01:04  Maria_Vella]
 * 
 * Revision 1.1.14.5  1993/05/19  19:35:33  Gary_Dupuis
 * 	Merge with Ag maintenance BL3.
 * 	[1993/05/19  19:10:12  Gary_Dupuis]
 * 
 * Revision 1.1.22.4  1993/04/30  20:41:26  Robin_Miller
 * 	Fix for QAR 11970:
 * 	Removed SZ_PREV_ALLOW flag so removable media can be ejected
 * 	while the device is still open.  Necessary for SoftPC.
 * 	[1993/04/30  20:39:17  Robin_Miller]
 * 
 * Revision 1.1.22.3  1993/04/09  21:46:42  Fred_Knight
 * 	bmerge
 * 	[1993/04/08  20:02:06  Fred_Knight]
 * 
 * 	Add tcds structure allocation and sim94_fast_enable flag
 * 	[1993/04/07  15:51:03  Fred_Knight]
 * 
 * Revision 1.1.22.2  1993/04/08  18:09:45  Maria_Vella
 * 	 Modified the RZ28 partition table to reflect a change in capacity.
 * 	 Added device descriptor entries for RZ28B, RZ26B TKZ60C, TKZ60L, and
 * 	 TKZ60CL. Corrected device descriptor entry for TKZ60 to non-loader.
 * 	[1993/04/05  16:18:49  Maria_Vella]
 * 
 * Revision 1.1.14.4  1993/03/12  20:00:07  Paul_Grist
 * 	     Jensen shared sandbox merge to BL12 (V1.2) backing tree
 * 	[1993/03/12  19:58:52  Paul_Grist]
 * 
 * Revision 1.1.3.37  1993/02/10  11:09:34  cam_proj_Maria_Vella
 * 	Fix name of tz30 device descriptor entry.
 * 	[93/02/10  10:32:43  Ritu_Bahl]
 * 
 * Revision 1.1.3.36  93/02/04  13:40:23  Thomas_Tierney
 * 	Modified the RZxx partition table to be in line with other
 * 	root and swap sizes to allow third-party device installs
 * 	to work.
 * 	[93/02/04  12:40:45  Thomas_Tierney]
 * 
 * Revision 1.1.3.35  93/02/02  15:49:16  Thomas_Tierney
 * 	Change id strings for HSX00, HSX00, HSZ10 and HSZ15
 * 	to be 13 characters in length.  Moved rz25l before
 * 	rz25 to avoid incorrect match during id string comparison.
 * 	[93/02/01  13:05:56  Thomas_Tierney]
 * 
 * Revision 1.1.3.34  93/01/22  15:32:20  cam_proj_Maria_Vella
 * 	Added HSZ15 (DEC RAID Toto+).
 * 	[93/01/21  20:08:14  Thomas_Tierney]
 * 
 * 	Added device descriptor entries for rrd43, rrd44, rz25l, rz28.
 * 	Fixed name in rz74 descriptor table entry.
 * 	Changed name of tzk20 to tzk11 and modified density table.
 * 	[93/01/20  17:37:32  Maria_Vella]
 * 
 * Revision 1.1.3.33  93/01/15  16:06:25  Thomas_Tierney
 * 	Added the HSX00 (single-spindle RAID), HSX01 (multi-spindle
 * 	RAID) and HSZ10 (Toto RAID) device types.
 * 	[93/01/15  14:27:03  Thomas_Tierney]
 * 
 * Revision 1.1.3.32  93/01/08  15:15:35  Peter_Schulter
 * 	Disable tagged queuing for Dec4000.  Part of fix for QAR 9912.
 * 	[93/01/08  15:13:10  Peter_Schulter]
 * 
 * 	Add entries for RZ24l, RZ26, and TKZ09.
 * 	[92/06/09  16:54:38  Maria_Vella]
 * 
 * Revision 1.1.14.3  1993/01/06  20:12:42  Paul_Grist
 * 	     Jensen shared sandbox merge from BL9 to BL11
 * 	[93/01/06  20:05:44  Paul_Grist]
 * 
 * Revision 1.1.3.31  92/12/11  14:59:29  cam_proj_Maria_Vella
 * 	Changed the queue depth for the rz26, rz35, and rz73 to 74.
 * 	Added entries for the rz27, rz74, tz87, tz877, tz867, tzk20
 * 	and tlz6.  Changed the max record size for the tz85 and tz857.
 * 	Fix comments for rz26 disk partitions.
 * 	[92/12/07  15:00:00  Roger_Richardson]
 * 
 * Revision 1.1.14.2  92/11/13  16:33:40  Paul_Grist
 * 	Initial EISA SCSI submit for jtw - Add support for aha (adaptech
 * 	1742 scsi)
 * 	[92/11/13  16:05:39  Paul_Grist]
 * 
 * Revision 1.1.3.30  92/11/06  15:15:36  cam_proj_Maria_Vella
 * 	Increased the low water mark for the ccb pool to allow the detection
 * 	of a possible pool depletetion sooner. Also increased the base ccb
 * 	pool for performance.
 * 	[92/11/06  10:03:52  William_Dallas]
 * 
 * Revision 1.1.3.29  92/10/23  15:35:24  Peter_Schulter
 * 	Removed duplicate rz26 entry.
 * 	Set SZ_REORDER on RZ26, 25, and 73.
 * 	[92/10/23  15:34:22  Peter_Schulter]
 * 
 * Revision 1.1.3.28  92/10/14  12:40:51  Peter_Schulter
 * 	merged in Silver BL10_6
 * 	Revision 1.1.3.13  92/09/16  09:53:32  cam_proj_Maria_Vella
 * 	Added int to allow issuing bdr's to scsi tapes on boot.
 * 	Moved ansi density table to cam_data.c to allow the support
 * 	of new densities easier.
 * 	[92/09/10  15:32:50  William_Dallas]
 * 
 * Revision 1.1.3.27  92/09/29  14:24:03  Peter_Schulter
 * 	merged in Silver BL10
 * 	Revision 1.1.3.12  92/08/31  13:41:30  cam_proj_Maria_Vella
 * 	Remove sim_softc and sim94_softc since they are now dynamically
 * 	allocated.
 * 	[92/08/27  12:41:19  Maria_Vella]
 * 
 * Revision 1.1.3.26  92/08/25  14:43:30  Farrell_Woods
 * 	remove unused static data structures from this file
 * 	[92/08/25  14:29:49  Farrell_Woods]
 * 
 * Revision 1.1.3.25  92/08/25  13:24:55  Peter_Schulter
 * 	Enable tagged queueing for Alpha DEC 4000 only.
 * 	[92/08/25  13:23:54  Peter_Schulter]
 * 
 * Revision 1.1.3.24  92/08/21  14:49:54  Paul_Hansen
 * 	Changes and corrections to various devices from Brain Nadeau
 * 	[92/08/21  14:39:25  Paul_Hansen]
 * 
 * Revision 1.1.3.20  92/07/14  13:34:20  Peter_Schulter
 * 	merged in Silver BL8_3
 * 	Revision 1.1.3.11  92/06/30  11:35:55  cam_proj_Janet_Schank
 * 	Fixed merge problem.
 * 	[92/06/25  17:11:59  Maria_Vella]
 * 
 * 	Modified default disk partition tables to be consistent with what is
 * 	in /etc/disktab and changed name of newly supported TKZ09 tape device.
 * 	[92/06/25  16:55:41  Maria_Vella]
 * 
 * 	Revision 1.1.3.10  92/06/16  09:58:36  cam_proj_Janet_Schank
 * 	Turn off tagged queueing for rz26.
 * 	[92/06/12  13:23:57  Maria_Vella]
 * 
 * 	Add entries for RZ24l, RZ26, and TKZ09.
 * 	[92/06/09  16:54:38  Maria_Vella]
 * 
 * 	Revision 1.1.3.9  92/06/02  10:49:26  cam_proj_Janet_Schank
 * 	Added mode select for SONY drive to select a 512 byte block size and
 * 	added SZ_NOSYNC flag for tlz04.
 * 	[92/05/29  14:44:54  Maria_Vella]
 * 
 * 	Moved header files out of sys. Removed all usage of "BOP" macros.
 * 	[92/05/26  09:46:50  Janet_Schank]
 * 
 * 	Revision 1.1.3.8  92/05/19  10:18:10  Maria_Vella
 * 	Added CAM controller string declaration used by SIM's to identify a CAM
 * 	controller structure.
 * 	[92/05/19  08:47:57  Maria_Vella]
 * 
 * 	Added CAM_log_label_info, a flag to allow a site to toggle
 * 	whether disk label informative messages are output.
 * 	[92/05/05  14:09:36  Thomas_Tierney]
 * 
 * 	Revision 1.1.3.6  92/05/04  11:15:44  cam_proj_Janet_Schank
 * 	Added u_long's to specify the base timeouts for the tape driver.
 * 	[92/04/24  07:29:00  Janet_Schank]
 * 
 * 	Changed the XPT pool sizes.
 * 	[92/04/23  16:51:24  Janet_Schank]
 * 
 * Revision 1.1.3.19  92/07/08  10:26:42  Peter_Schulter
 * 	Fixed duplicate entry for rz26_sizes introduced by merge.
 * 	[92/07/08  10:25:18  Peter_Schulter]
 * 
 * Revision 1.1.3.18  92/07/07  17:24:31  Peter_Schulter
 * 	merged in Silver BL8_2
 * 	Revision 1.1.3.10  92/06/16  09:58:36  cam_proj_Janet_Schank
 * 	Turn off tagged queueing for rz26.
 * 	[92/06/12  13:23:57  Maria_Vella]
 * 
 * Revision 1.1.3.17  92/06/25  17:44:44  Peter_Schulter
 * 	merged in Silver BL8
 * 	Revision 1.1.3.9  92/06/02  10:49:26  cam_proj_Janet_Schank
 * 	Added mode select for SONY drive to select a 512 byte block size and
 * 	added SZ_NOSYNC flag for tlz04.
 * 	[92/05/29  14:44:54  Maria_Vella]
 * 
 * 	Moved header files out of sys. Removed all usage of "BOP" macros.
 * 	[92/05/26  09:46:50  Janet_Schank]
 * 
 * Revision 1.1.3.16  92/06/12  15:15:22  Farrell_Woods
 * 	Fix comments for rz57 disk partitions.
 * 	[92/06/12  14:28:30  Farrell_Woods]
 * 
 * 	Implement new partition scheme for rz25, fix partitions for
 * 	rz57 to reflect smaller actual capacity than the geometry
 * 	would have us believe.
 * 	[92/06/12  12:37:08  Farrell_Woods]
 * 
 * Revision 1.1.3.15  92/06/05  14:22:06  Peter_Schulter
 * 	Changed all partition sizes to the new OSF partitioning scheme.  This
 * 	new scheme uses a 64 Mbyte root, a 128 Mbyte swap, and a 400 Mbyte usr
 * 	(g partition, where possible).
 * 	[92/06/05  13:43:02  Peter_Schulter]
 * 
 * Revision 1.1.3.14  92/06/03  12:08:25  Peter_Schulter
 * 	merged in Silver BL7
 * 	Revision 1.1.3.8  92/05/19  10:18:10  Maria_Vella
 * 	Added CAM controller string declaration used by SIM's to identify a CAM
 * 	controller structure.
 * 	[92/05/19  08:47:57  Maria_Vella]
 * 
 * 	Revision 1.1.3.7  92/05/08  12:42:44  Thomas_Tierney
 * 	Added CAM_log_label_info, a flag to allow a site to toggle
 * 	whether disk label informative messages are output.
 * 	[92/05/05  14:09:36  Thomas_Tierney]
 * 
 * 	Revision 1.1.3.6  92/05/04  11:15:44  cam_proj_Janet_Schank
 * 	Added u_long's to specify the base timeouts for the tape driver.
 * 	[92/04/24  07:29:00  Janet_Schank]
 * 
 * 	Changed the XPT pool sizes.
 * 	[92/04/23  16:51:24  Janet_Schank]
 * 
 * Revision 1.1.3.13  92/05/21  12:22:08  Farrell_Woods
 * 	Apply Bill Dallas' fix for the TLZ04 "off-line" problem.
 * 	[92/05/21  12:21:18  Farrell_Woods]
 * 
 * Revision 1.1.3.12  92/05/15  09:19:11  Farrell_Woods
 * 	Add entry for RZ26 disk appropriate for Alpha.  Leave it
 * 	empty for everyone else.
 * 	[92/05/14  14:24:57  Farrell_Woods]
 * 
 * Revision 1.1.3.11  92/05/14  17:21:27  Peter_Schulter
 * 	Changed default partition sizes for RZ57, RZ35, and RZ73.
 * 	[92/05/14  17:19:46  Peter_Schulter]
 * 
 * Revision 1.1.3.10  92/05/12  12:22:27  Peter_Schulter
 * 	Fixed entries for RZ73 Disk drive:
 * 		Fixed partition table based on actual size of the drive.
 * 		Added SZ_NO_TAG since Cobra does not support tagged
 * 	queueing yet.
 * 	[92/05/12  12:21:34  Peter_Schulter]
 * 
 * Revision 1.1.3.9  92/05/05  17:47:58  Peter_Schulter
 * 	merged in Silver BL6_4
 * 	Revision 1.1.6.5  92/04/16  12:35:53  Janet_Schank
 * 
 * Revision 1.1.3.8  92/04/17  21:48:50  Shashi_Mangalat
 * 	Removed rcsid from this file.  This conflicts with rcsid in files
 * 	which include this file.
 * 	[92/04/17  21:43:57  Shashi_Mangalat]
 * 
 * Revision 1.1.6.5  92/04/16  12:35:53  Janet_Schank
 * 	Decreased initial XPT pool sizes.
 * 	[92/04/16  12:23:48  Janet_Schank]
 * 	Revision 1.1.6.4  92/04/13  15:28:38  Janet_Schank
 * 	Added SIM global variables for I/O reordering and added
 * 	SZ_REORDER flag to disk descriptor entries.
 * 	[92/04/13  09:17:22  Janet_Schank]
 * 	Revision 1.1.6.3  92/04/07  07:36:04  Janet_Schank
 * 	Added the global EDT size variable.  This allows
 * 	checking on the edt_dir size.
 * 	[92/04/07  07:26:31  Janet_Schank]
 * 
 * 	Revision 1.1.6.2  92/03/24  13:45:58  Janet_Schank
 * 	Merged in changes from LA
 * 	[92/03/24  13:14:48  Janet_Schank]
 * 
 * 	Revision 1.1.4.2  92/03/06  20:06:04  cam_proj_Janet_Schank
 * 	Added KZQSA support.
 * 	[92/05/05  17:31:18  Peter_Schulter]
 * 
 * Revision 1.1.3.7  92/04/14  15:22:16  Peter_Schulter
 * 	merged in Silver BL5
 * 	Revision 1.1.4.2  92/03/06  20:06:04  cam_proj_Janet_Schank
 * 	     Added KZQSA support.
 * 	[92/04/14  15:06:03  Peter_Schulter]
 * 
 * Revision 1.1.3.6  92/04/03  16:03:34  Ronald_Heglie
 * 	Added RZ73 information.
 * 	[92/04/03  15:59:33  Ronald_Heglie]
 * 
 * Revision 1.1.3.5  92/04/02  00:13:14  Ronald_Heglie
 * 	Added LASER/Ruby XZA/SCSI support.
 * 	[92/04/02  00:04:36  Ronald_Heglie]
 * 
 * Revision 1.1.3.4  92/04/01  19:04:20  Peter_Schulter
 * 	Added interface to Cobra SCSI Driver (SIOP) and RZ35 definitions.
 * 	[92/04/01  19:03:20  Peter_Schulter]
 * 
 * Revision 1.1.3.3  92/03/24  15:07:47  Farrell_Woods
 * 	Fix partition sizes for RZ25's for Alpha
 * 	[92/03/24  15:06:56  Farrell_Woods]
 * 
 * Revision 1.1.3.2  92/03/18  16:14:41  Al_Delorey
 * 	From Silver: AG_BL5
 * 	[92/03/18  12:20:18  Al_Delorey]
 * 
 * 	Added include of <mach/vm_param.h> for Alpha.
 * 	[92/02/21  13:27:41  Janet_Schank]
 * 
 * 	Made 64bit modifications
 * 	[92/01/31  09:36:34  Janet_Schank]
 * 
 * 	Merged in changes from LA.
 * 	[92/01/28  18:02:42  Janet_Schank]
 * 
 * 	Added CAM support
 * 	[92/01/17  15:30:30  Janet_Schank]
 * 
 * Revision 1.1.1.2  92/03/17  15:17:19  devbld_gsf
 * 	First AG_BL5 version.  Moved from Ag baselevel delivery tree.
 * 
 * $EndLog$
 */
#pragma ident "@(#)$RCSfile: cam_data.c,v $ $Revision: 1.1.102.3 $ (DEC) $Date: 1997/04/17 20:18:16 $"

/* ---------------------------------------------------------------------- */

/* cam_data.c			Version 1.25		Dec. 11, 1991 */

/* The CAM data file.  This file contains the system setable parameters and
static data arrays needed for the CAM subsystem.

Modification History

	Version	  Date		Who	Reason

	1.00	03/27/91	jag	Creation date, from xpt_data.c. 
	1.01	04/10/91	dallas	Added device descriptor tables
					and supporting structures
					Incrmented the CCB pool size defines.
	1.02	06/03/91	jag	Fixed the N_cam_conftbl calc.
	1.03	06/07/91	maria	Changed alll NCAM references to
					NASC.
					Added DTYPE_SHIFT in device
					descriptor table entries.
					Added the default_scsiid
					declaration.
	1.04	06/19/91	dallas  Added max record size to the dev desc.
					This gives the user/operator the
					ability to match the device.
					Fixed TZK10 density table to write
					1 file mark on close.
					Fixed comment line for tz07
					Added descriptions on how to add
					a device.
	1.05	07/03/91	rps	Added sii defines.

	1.06	07/09/91	janet	Fixed "sim.h" include to be "sii.h"

	1.07	07/17/91	maria	Added SZ_NO_DISC, SZ_TRY_SYNC bits
					and removed SCSI2 for device descriptor
					flags.

	1.08	07/31/91	dallas	Fixed density tables to correspond
					to old major/minor pairs.. For
					compat until OSF.

	1.09	08/02/91	rln	Added () to NCAM

	1.10	08/21/91	dallas	Added TZ85 and TLZ04 to the
					device tables.
					
	1.11	09/03/91	maria	Added unknown device descriptor
					entries for all types of devices.
					
	1.12	09/11/91	maria	Changed SZ_TRY_SYNC flag to
					SZ_NOSYNC in device descrptor
					table entries.  Synchronous
					will not be attempted on: RRD40,
					RRD42, RZ23, TK50, TZ30, TZ05.
					Added entry RWZ01.

	1.13	09/12/91	jag	Added the CDrv EDT scan INQUIRY
					retry count.

	1.14	09/13/91	dallas	Added compression for tlz06.

	1.15	09/16/91	maria	Added RWZ01 partition table.

	1.16	09/19/91	maria	Removed the prevent/allow media
					removal optional command for RRD40.

	1.17	10/24/91	maria 	Updated the RZ58 queue size to
					its full size of 32 and disabled
					tagged queueing for RZ58 (SZ_NO_TAGS).
	1.18	11/19/91	dallas  Added densities for tapes and new
					updated rz58 partition sizes..
					Fixed unknown tape entry to reflect
					default density. Added error log
					limits for device drivers..
	1.19	11/20/91	janet	Merged in sim_data.c and xpt_data.c
	1.20	11/20/91	dallas  Deleted the use of the tzk08_dens
					table for the exabyte. It seems 
					that the device does not like 
					anything other then 0's for the
					density setting..
	1.21	11/22/91	dallas  Added the sending of the vendor
					unique data on open for the TKZ08.
					This disables odd byte disconnects for
					the TKZ08 which fixes the 3min problem.
	1.22	12/04/91	dallas	Changed the tzk10_dens table for 
					TZK10's. The unit will reject the 
					QIC-24 density setting.. Now we 
					will send a default setting.
	1.23	12/09/91	maria	Added SZ_NO_TAG for UNKNOWN disk
					device descriptor entry to turn
					off tagged queueing.
	1.24	12/11/91	maria	Added common buf struture pool
	1.25	12/11/91	jag	Added the global EDT scanning wait
					variable for the CDrv.
					size variables.

	NOTES:

*/

/* ---------------------------------------------------------------------- */

#include <io/common/iotypes.h>
#include <sys/types.h>			/* system level types */
#include <sys/time.h>			/* system level time defs */
#include <sys/param.h>			/* system level parameter defs */

#include <io/cam/dec_cam.h>		/* DEC CAM include file */
#include <mach/vm_param.h>
#include <io/cam/cam_debug.h>		/* debug structures for the CAM code */

#include <io/cam/cam.h>			/* General CAM include file */
#include <io/common/devio.h>
#include <io/cam/scsi_all.h>		/* SCSI 2 defines (structs) */
#include <io/cam/scsi_direct.h>		/* SCSI 2 defines (structs) */
#include <io/cam/scsi_sequential.h>	/* SCSI 2 defines (structs) */
#include <io/common/kds.h>		/* Kernel database structures */
#include <io/cam/camdb.h>		/* Cam database structures */
#include <io/cam/pdrv.h>		/* General PDrv include file */
#include <io/cam/cam_tape.h>
#include <io/cam/sim_target.h>		/* Target mode defines and macros */
#include <io/cam/sim_cirq.h>		/* Circular Q defines and macros */
#include <io/cam/dme.h>			/* DME include file */
#include <io/cam/sim.h>			/* SIMs include file */
#include <io/cam/sim_94.h>		/* SIM94 include file */
#include <io/cam/ccfg.h>		/* Configuration driver include file */
#include <io/cam/uagt.h>		/* UAgt include file */
#include <io/cam/xpt.h>			/* XPT include file */
#include <io/cam/tcds_adapt.h>		/* TCDS include file */
#include <io/cam/ata/sim_ata.h>		/* ATA/EIDE include file */
#include <io/cam/siop/psiopdefs.h>	/* psiop include file */

#include "asc.h"			/* Config generated defines file */
#include "tcds.h"			/* Config generated defines file */
#include "tza.h"                        /* Config generated defines file */
#include "pza.h"                        /* Config generated defines file */

#ifdef __alpha
#include "xza.h"		/* Defines # of config'ed XZA SCSI busses */
#include "siop.h"
#include "psiop.h"		/* PCI - SIOP's, NCR 53c810 */
#include "aha.h"
#include "ata.h"
#include "isp.h"
#include "simos.h"
#else
#define NXZA 0
#define NSIOP 0
#define NAHA 0
#define NATA 0
#define NISP 0
#endif

#include "scsi.h"			/* Total Number of SCSI controllers
					 * connected to the above "buses"
					 */

/* NSCSI currently covers NASC, NTZA, and NPZA, and NSIOP
 */
#define NCAM	(NSCSI)

/*
 * Define the total number of SIMport adapters.  This is used for
 * calculating SIMport common resource pool variable values.
 */
#define NSIMPORT (NTZA + NPZA)


/* ---------------------------------------------------------------------- */

/*
 * The system's SCSI bus id.
 */
int default_scsiid = DEFAULT_SCSIID;

/* ---------------------------------------------------------------------- */

/* The CAM CCB pool parameters.  These settings define the initial size, 
hi/low water marks and increment values.  These settings are used by the
XPT module. */

/* Moved to param.c; Configured via /etc/sysconfigtab. */
/*U32 cam_ccb_pool_size = 200;*/
/*U32 cam_ccb_high_water = 1000;	NOT USED */
/*U32 cam_ccb_low_water = 100;*/
/*U32 cam_ccb_increment = 50;*/

/*
 * The SIMport resource pool parameters. These settings define the initial
 * size, low watermarks and increment values.  These settings are used by
 * the SIMport module.
 */

U32 spo_car_pool_size = 400;		
U32 spo_car_low_water = 50;
U32 spo_car_increment = 50;

U32 spo_bsm_pool_size = 50;
U32 spo_bsm_low_water = 10;
U32 spo_bsm_increment = 20;


/* ---------------------------------------------------------------------- */
/* The CAM CDrv EDT scanning parameters.  These settings help to define
the scanning behavior when the CDrv is filling in or updating the EDT
data structures. */

U32 ccfg_inquiry_retry_limit = 3;
U32 ccfg_use_sdtr_inquiry = CAM_TRUE;
U32 ccfg_wait_delay_loop = CCFG_WAIT_DELAY_LOOP;

/* ---------------------------------------------------------------------- */

/* JAG: These two arrays need to reference the BUS count. */

EDT *edt_dir[ NCAM ]; 		/* ptrs for EDT grid per HBA */

unsigned long N_edt_dir = (sizeof(edt_dir)/sizeof(EDT *));

/* ---------------------------------------------------------------------- */

CAM_SIM_ENTRY *cam_conftbl[ NCAM ];

U32 N_cam_conftbl = (sizeof(cam_conftbl)/sizeof(CAM_SIM_ENTRY *));

/* ---------------------------------------------------------------------- */
/*
 * CAM SIM data.
 */
#ifdef CAMDEBUG
/*
 * Camdbg_flag and camdbg_id are only used during debug.  The shipped
 * source files will not use them.
 */
U32 camdbg_flag = (U32)0;
U32 camdbg_id = (U32)0;
#endif CAMDEBUG

/*
 * Used to identify a CAM controller structure.
 */
char cam_ctlr_string[] = { "CAM" };

/*
 * Configuration structures.
 */
int nNHBA94 = NASC + (2 * NTCDS);
int nNTZA = NTZA;
int nNPZA = NPZA;
int nNSIMPORT = NSIMPORT;
int nCAMBUS = NCAM;
struct device *camdinfo[NCAM*NDPS_16*NLPT_8];
struct controller *camminfo[NCAM];
SIM_SOFTC *softc_directory[NCAM];

#if NPSIOP > 0
int psiop_bus_table[NPSIOP];
#endif	/* NPSIOP */

#if NTZA > 0
int tza_bus_table[NTZA];
U32 kztsa_cntrs_enable = 0;
#endif /* NTZA */

#if NPZA > 0
int pza_bus_table[NPZA];
U32 pza_cntrs_enable = 0;
#endif /* NPZA */

#if NSIMPORT > 0
void *simport_softc_directory[NSIMPORT];
#endif

#if NASC > 0 || NTCDS > 0
int asc_bus_table[NASC + (2 * NTCDS)];
#endif /* NASC || NTCDS */

#if NAHA > 0
int aha_bus_table[NAHA];
#endif /* NAHA */

#if NATA > 0
ATA_BUS ata_bus_table[NATA];
#endif /* NATA */

#if NISP > 0
    /*
     * NOTE:
     *
     *    The boot flag "isp_fw_load" specified on the 
     *    command line during interactive boot takes
     *    precedence because kernargs are handled after
     *    configure. Here, isp_fw_load is set to a default
     *    value of 1. This means that the Qlogic sim driver
     *    will attempt to load a firmware image file from
     *    the root filesystem if it is present.
     *
     *    Since isp_fw_load is extern'd in kernargs.c, it
     *    must be defined whether or not there is a QLogic
     *    option present in the kernel configuration.
     */
    long isp_fw_load = 1;
    int isp_bus_table[NISP];
#else
    long isp_fw_load = 0;
#endif /* NISP */

#if NSIMOS >0
int simos_bus_table[NSIMOS];
#endif

#if NTCDS > 0
TCDS_BUS tcds_bus_table[NTCDS];

/*
 * sim94_fast_enable is only here to quickly and easily turn OFF all fast
 * buses.  The driver now examines the H/W state flags in the V3.0 console
 * eeprom, or in the PMAZC-AA option rom to determine which buses are fast.
 *
 * The tcds_inter_loop_limit value determines the number of scsi bus
 * interrupts that can be handled during 1 interrupt cycle before forcing
 * an exit.  The lower the value, the better the interrupt latency for
 * other devices (at the same priority).  The higher the value, the lower
 * the scsi interrupt overhead is.
 */
U32 sim94_fast_enable = 1;

#ifdef RT_PREEMPT_OPT
U32 tcds_inter_loop_limit = 1;
#else
U32 tcds_inter_loop_limit = 4;
#endif
#endif

/*
 * sim810_fast_enable is here to allow the psiop (810 chip) to run in
 * slow mode.  This would be needed if a long bus cable (over 3 meters)
 * is attached to the controller.  The default is fast - since the normal
 * system will use the internal cable only.  If a cable is added that
 * causes it to become "long" then set this to 0 to disable fast mode
 * (and therefore allow cable lengths of up to 6 meters).
 *
 * Supply the value of 40MHz for the default 810 SCLK speed. The default
 * speed will be used for any 810 not recognized by the system.
 *
 * The pci_script_space is loaded with script code during initialization;
 * this is where the NCR script processor executes from.
 *
 * The pci_local_space is a general r/w area for the NCR'810 processor.
 *
 * The psiop_min_selection_timeout specifies a minimum value (in 
 * microseconds (us)) for selection timeout. As the values available to
 * the chip are based on the SCLK frequency as well as a multiplier that
 * doubles for each of 15 possible selections, we cannot guarantee that
 * this will be the exact timeout value. Rather, we will select the first
 * value that is greater than or equal to the minimum timeout. If we exhaust
 * all 15 selections and still cannot exceed the minimum timeout, we
 * will use the last (largest) selection. The SCSI-2 specification
 * recommends a value of 250ms.
 */
#if NPSIOP > 0
U32 sim810_fast_enable = 1;
int default_810_clk_speed = PSIOP_810_SCLK_40MHz;  /* NOTE!: Units are MHz * 100 */
char pci_script_space[NPSIOP][PSIOP_SCRIPT_RAM_SIZE];	/* script area */
char pci_local_space [NPSIOP][PSIOP_LOCAL_RAM_SIZE];	/* local area */
int psiop_min_selection_timeout = 250000;               /* 250ms timeout */
#endif

#if NXZA > 0
void* xza_softc_directory[NXZA];
#endif

/*
 * One state machine queue is kept for all controllers.
 */
U32 sm_queue_sz = SM_QUEUE_SZ * NCAM;
SIM_SM_DATA sm_data[SM_QUEUE_SZ * NCAM];
SIM_SM sim_sm;

/*
 * If scsi_bus_reset_at_boot is set to CAM_TRUE, a SCSI bus reset
 * will be performed at boot time.  This should not be set to
 * CAM_TRUE unless you are SURE that it is necessary for your device
 * to operate.
 */
I32 scsi_bus_reset_at_boot = CAM_FALSE;

/*
 * The SIM default value.  This value is initially SIM_DEFAULT_TIMEOUT
 * which is 5 seconds.  This shouldn't be changed unless you are SURE
 * that it is necessary for you device to operate.
 */
I32 sim_default_timeout = SIM_DEFAULT_TIMEOUT;

/*
 * The SIM will perform I/O reordering if the CCB's "cam_sort" value
 * is non-zero.  In a busy system it is possible that certain I/O's
 * will have to wait a while if reordering is allowing a lot of other
 * I/O to go first.  "sim_sort_age_time" is the maximum number of
 * microseconds that an I/O is allowed to sit and wait.
 *
 * The default is 2 sec (2000000 usec).
 */
U32 sim_sort_age_time = 2000000;
U32 sim_allow_io_sorting = 1;
U32 sim_allow_io_priority_sorting = 0;
U32 sim_min_sort_depth = 3;

/* ---------------------------------------------------------------------- */

/*
 * The prihperal driver unit table.
 */
PDRV_UNIT_ELEM pdrv_unit_table[(NCAM) * NDPS_8 * NLPT_8];
PDRV_UNIT_INFO pdrv_unit_info[(NCAM) * NDPS_8 * NLPT_8];

/*
 * The following are the variables used for the buf structure pool.
 */
U32 ccmn_bp_pool_size = 50;
U32 ccmn_bp_high_water = 90;
U32 ccmn_bp_low_water = 10;
U32 ccmn_bp_increment = 50;


/*
 * Error limits for the disk and tape drivers... Once the limits
 * are reached on a per device basis no more errlog entries will
 * be seen. A reboot clears...
 */

U32 cam_harderr_limit = 1000;
U32 cam_softerr_limit = 1000;

/*
 * The following allows changing the base timeout values for tape's
 * PLEASE NOTE ALL TIMES ARE IN SECONDS
 *
 * ctape_io_base_timo is the base timeout value for reads and write's.
 * For every 10000 bytes request a second is added to the base for the
 * timeout value.
 *
 * ctape_move_timo is the timeout value for a tape movement operations
 * that does not involve reads or writes.
 *
 * ctape_wfm_base_timo is the base value for writing filemarks. For
 * each filemark to be written 1 second is added.
 */
u_long ctape_io_base_timo = 300; 	/* 5 minutes			*/
u_long ctape_move_timo = 3600; 		/* 60 minutes			*/
u_long ctape_wfm_base_timo = 300;	/* 5 minutes Buffer flushes	*/

/*
 * For machines that don't issue a bus reset on boot issue a BDR to 
 * every tape drive seen. This is needed so when the machine boots
 * we can do our setups of the device. This will change once policy 
 * is determined for multi-initiator tapes.
 */
u_long ctape_boot_bdr = 1;		/* Issue a Bus device reset (tapes) */

/* 
 * Changable disk driver timeouts. cdisk_to_def for non read/write
 * commands to the disks (test unit ready, mode select etc.)
 * cdisk_io_def Hard fixed disk timeout value for i/o, and
 * cdisk_io_rmb for removable media disks since they are slow.
 */
u_long cdisk_to_def = 10;		/* 10 seconds			*/
u_long cdisk_io_def = 60;		/* 60 seconds (Tagged commands)	*/
u_long cdisk_io_rmb = 120;		/* 120 seconds (slow removables */

/*
 * (CAM_log_label_info):
 * Setting this flag to a non-zero value will cause informational
 * messages about disk labels to be logged to the syslog when an open
 * occurs for a SCSI/CAM disk.  This flag can be used to help debug
 * problems moving from ULTRIX disks to OSF/1 labelled disks.
 *
 * NOTE: When enabled, informational messages are always logged to
 * syslog (see /usr/adm/syslog.dated/<date>/kern.log for any messages).
 */
int CAM_log_label_info = 0;



/*
 * ************************************************************************* *
 * ************************************************************************* *
 * ************************************************************************* *
 *
 *                 FOR ALL INFORMATION THAT FOLLOWS:
 *
 * AS OF V4.0:
 *
 * The device information that has previously been added to this file
 * has been replaced by the Dynamic Device Recognition (DDR) Subsystem
 * and its database. This new subsystem's database contains device
 * information "exceptions" that patch a device type's "defaults" to create
 * an entry with the appropriate information. Please refer to the "ddr_config"
 * utility man page and associated documentation (referenced in the man page).
 *
 * To provide a path of backward compatibility, we will continue to
 * support the addition of device information to this file. However,
 * this support will terminate in a future release. Documentation on the
 * DDR subsystem includes migration information and pointers to tools
 * to aid in the migration.
 *
 * Please note that all future device support will *NOT* be added to this
 * file, but rather will utilize the DDR database.
 *
 * ************************************************************************* *
 * ************************************************************************* *
 * ************************************************************************* *
 */


/* ---------------------------------------------------------------------- */
/*
 * How to add "your own" SCSI (disk/tape/cdrom) device:
 *
 *
 *    For those who would like to connect their own (non-DEC) devices to the
 *    SCSI bus the ULTRIX CAM driver allows the addition and definition of new
 *    devices in the "cam_data.c" file supplied with every binary or
 *    source license.
 *
 *    A good starting place would be to review the table before reading
 *    this description.
 *    A new entry should be added to cam_devdesc_tab describing the new device.  
 *    An example disk and tape entry might look like the following:
 *
 *    DEV_DESC  cam_devdesc_tab[] = {
 *
 *    {"DEC     RZ55", 12, DEV_RZ55, (ALL_DTYPE_DIRECT << DTYPE_SHFT) | SZ_HARD_DISK, 
 *    (struct pt_info *)ccmn_rz55_sizes, 512, DEC_MAX_REC, NO_DENS_TAB, NO_MODE_TAB, 
 *    SZ_BBR, 
 *    NO_OPT_CMDS, SZ_READY_DEF, SZ_NO_QUE, DD_REQSNS_VAL | DD_INQ_VAL ,
 *    36, 64
 *    },
 *
 *    {"DEC     TZK10", 13, DEV_TZK10, (ALL_DTYPE_SEQUENTIAL << DTYPE_SHFT) | SZ_QIC_CLASS,
 *    (struct pt_info *)ccmn_null_sizes, SZ_NO_BLK, DEC_MAX_REC, &tzk10_dens, &tzk10_mod,
 *    SZ_NO_FLAGS,
 *    NO_OPT_CMDS, SZ_READY_DEF, SZ_NO_QUE, DD_REQSNS_VAL | DD_INQ_VAL,
 *    36, 64
 *    },
 *
 *    { 0 },
 *
 *    };
 *
 *    The first field is the vendor returned string identifying the drive.
 *    This string is composed of the vendor id (first 8 chars) followed by
 *    the product id.
 *
 *    The second field is the length of the string, the total string returned 
 *    by the unit is what we match on.
 *
 *    The third field is the ULTRIX name for the device (DEV_RZ55 for this
 *    disk, DEV_TZK10 for this tape. For disk devices that are not from DIGITAL
 *    a generic name of DEV_RZxx should be used. For tapes a couple of generic
 *    categories are supplied. DEV_TZQIC (Quarter inch cartridge tape unit), 
 *    DEV_TZ9TRK (9 Track tape unit), DEV_TZ8MM ( 8 millimeter tape units),
 *    DEV_TZRDAT ( RDAT tape units), DEV_TZ3480 ( 3480 tape units), and
 *    for tape devices that do not fit into any of the above generic 
 *    classifications DEV_TZxx should be used.
 *
 *
 *    The fourth field is the class of SCSI device and sub_class. The upper
 *    4 bits are the class of the device. Please refer to 
 *    /usr/sys/h/scsi_all.h for a complete list of defines. The lower 28
 *    bits are the sub class. 
 *    Currently for disks the sub classes are:
 *    SZ_HARD_DISK	Normal disk unit.
 *    SZ_FLOP_3		Floppy disk 3.5"
 *    SZ_FLOP_5		Floppy disk 5.25"
 *
 *    For tapes the sub classes are:
 *    SZ_TK_CLASS	TK50 and TK30 Class tape.
 *    SZ_RDATE_CLASS	TLZ04, RDAT Class tape.
 *    SZ_9TRK_CLASS	9 Track Class tape.
 *    SZ_QIC_CLASS	Quarter inch Class tape.
 *    SZ_8MM_CLASS	8 Millimeter Class tape.
 *    SZ_3480_CLASS	3480 Cartridge Class tape.
 *    SZ_LOADER		Is there a loader tied to the unit that does not
 *			have a separate id. (Not implemented as of yet ).
 *    These defines can be found in /usr/sys/h/pdrv.h
 *    
 *    The fifth field is the partition table entry.  Tape units should use
 *    sz_null_sizes.  Disk devices may use sz_rzxx_sizes.  The sz_rzxx_sizes
 *    assumes that the disk is at least 48Mb.  DO NOT MODIFY sz_rzxx_sizes!
 *    To create your own partition table, you must make an entry in the
 *    "size" struct below.
 *
 *    The sixth field is the block/sector size of the unit. This field is
 *    used for disks and cdroms. The field is the number of bytes in a
 *    sector/block that the unit uses. Refer to the technical manual of 
 *    your device to get that number.
 *    
 *    The sixth field is the block/sector size of the unit is now being phased
 *    out. The cam_disk driver will get the block size from the device now.
 *    The field is being reused for RAID boxes. Since each SCSI RAID box
 *    is different the lower byte is used to tell the driver which page
 *    contains the RAID level for this LUN, the next byte is the offset
 *    within the page that contains the RAID level. The upper 2 bytes 
 *    the size of the page to request.
 *
 *    The seventh field is the maximun record size the unit can do in a single
 *    transfer. This field is used for raw transfers only and not nbuf/io.
 *    The field allows for single raw transfers to be allowed up the number
 *    of bytes defined by the field. Primarily used for tapes that contain
 *    seimic data which can have hugh record sizes. Please note that if your
 *    machine does not have the physical memory or the unit can not do the size
 *    transfer requested, errors will be a result.
 *    
 *    The eighth field is used for tapes only. The entry is a pointer to the
 *    tape units density structure. Please refer to the density structure 
 *    write up below.
 *
 *    The ninth field is a pointer to a mode select table. If the field
 *    is filled in the disk/tape/cdrom drivers will do mode selects
 *    for each page defined in the table on initial opens of the device.
 *    this field is optional and is Not recommended to be set up unless
 *    it is truely needed. Please refer to the write up for the mode select
 *    table write up below.
 *
 *    The tenth field are flags which direct the disk/tape/cdrom drivers
 *    on how to treat the device. Current flags are:
 *    SZ_NO_FLAGS	No flags defined.
 *    SZ_BBR		Device allows bad block replacement (disks only).
 *    SZ_NOSYNC		Device does not allow synchronous data transfer.
 *    SZ_NO_DISC	Device does not allow disconnects.
 *    SZ_NO_TAG		Do not use tagged queueing even though device
 *			supports it.
 *    SZ_REORDER	Allow the SIM to reorder specific CCB's.
 *    SZ_DISPERSE_QUE   Allow the queue depth to be spread across all
 *			LUNS connected to the device.
 *
 *    The eleventh field are optional commands. Some devices support a
 *    larger command set then just the mandatory ones defined for the
 *    device defined by the SCSI specs. Currently the optional command
 *    that are recognized by the disk/tape/cdrom drivers are:
 *    NO_OPT_CMDS	Device does not support optional commands
 *    SZ_RW10 		Device supports 10 byte CDB's for read and write.
 *    SZ_PREV_ALLOW     Device supports prevent/allow media removal commnad
 *    SZ_EXT_RESRV	Device supports extended reserve/release command.
 *
 *    The twelfth field is maximun ready time is seconds. For disks this
 *    field represents powerup/spinup time combined. For tape it represents
 *    powerup/load/rewind from End of Tape. This is the maximun amount of
 *    time in seconds the disk/tape/cdrom drivers will wait for a unit to
 *    become ready.
 *
 *    The thirteenth field is tag queue depth. If the device supports tag
 *    queuing this field represents the depth of the queue the device 
 *    supports. Refer to your device manual to see if your device supports
 *    tag queuing and the depth of the queue.
 *
 *    The fourteenth field are valid flags for the inquirey data lenght 
 *    and request sense data lenght. Indicates which fields are valid.
 *    Defines are:
 *    DD_REQSNS_VAL	Request Sense lenght is valid.
 *    DD_INQ_VAL	Inquirey lenght is valid.
 *
 *    The fifteenth is the inquirey data lenght for the device.
 *    Must be used in conjunction with the DD_INQ_VAL flag.
 *
 *    The sixteenth field is the request sense data lenght for 
 *    the device. Must be used in conjuction with DD_REQSNS_VAL flag.
 *
 *    Below is the structure definition for the struct dev_desc 
 * 
 *
 *    Device Descriptor Structure
 *    There will be a device descriptor structure entry for each
 *    specific device supported by DEC.  A user may supply a new
 *    entry to the device descriptor table by adding it to TBD and
 *    relinking the kernel or by sending it via an ioctl() call
 *    (TBD).  The following describes an entry in the device
 *    descriptor table:
 *
 *    typedef struct dev_desc {
 *	   u_char	dd_pv_name[IDSTRING_SIZE];  
 *	   u_char  	dd_length;
 *	   u_char	dd_dev_name[DEV_NAME_SIZE];	
 *	   u_long  	dd_device_type;
 *	   struct  	pt_info *dd_def_partition;
 *	   u_long  	dd_block_size;
 *	   u_long	dd_max_record;
 *	   DENSITY_TBL *dd_density_tbl;
 *	   MODESEL_TBL *dd_modesel_tbl;
 *	   u_long  dd_flags;
 *	   u_long  dd_scsi_optcmds;
 *	   u_long  dd_ready_time;
 *	   u_short dd_que_depth;
 *	   u_char  dd_valid;
 *	   u_char  dd_inq_len; 
 *	   u_char  dd_req_sense_len;
 *    }DEV_DESC;
 *
 *
 *    The density table allows the definition of eight densities
 *    for each type of tape unit. The current method of writing
 *    a particular density is by selection of the units minor 
 *    numbers lower 3 bits. Please refer to the units technical
 *    manual for the density/compression codes and blocking factors
 *    for each density. Each density is made up of 
 *    the following:
 *    	The first field are valid flags which signify which 
 *	of the following fileds for this density are valid.
 *	Flags are
 *	DENS_VALID              Is the structure valid 
 *	ONE_FM                  Write 1 filemark on close QIC
 *	DENS_SPEED_VALID        Speed setting valid. Multi speed
 *	DENS_BUF_VALID		Run in buffered mode (cache)
 *	DENS_COMPRESS_VALID 	Compression code if supported
 *
 *	The second field is the scsi density code for this density.
 *
 *	The third field is the scsi compression code for this density
 *	if the unit supports compression.
 *
 *	The fourth field is the setting speeding for this density.
 *	Some units are varible speed for certain densities.
 *
 *	The fifth field buffer control setting. 
 *
 *	The sixth field is the blocking factor for this density
 *	Certain units (QIC) will only write data in fixed lenght
 *	blocks to the tape. This field is the number of bytes in
 *	it's blocking factor. The field is NULL if this density
 *	blocking factor is variable.
 *
 *	Below is the TZK10's density table. Please note that this
 *	unit supports both fix and variable records.
 *
 *   DENSITY_TBL 
 *   tzk10_dens = {
 *       { Minor 00 or rmtXl 
 *
 *       Flags 
 *       DENS_VALID | DENS_BUF_VALID |ONE_FM ,
 *
 *       Density code	Compression code	Speed setting
 *       SEQ_QIC120,		NULL,			NULL,
 *
 *       Buffered setting	Blocking		
 *       1,			512
 *       },
 *       { Minor 01 or rmtXh 
 *
 *       Flags 
 *       DENS_VALID | DENS_BUF_VALID |ONE_FM ,
 *
 *       Density code	Compression code	Speed setting	
 *       SEQ_QIC320,		NULL,			NULL,
 *
 *       Buffered setting	Blocking				
 *       1,			512
 *       },
 *       { Minor 02 or rmtXm 
 *
 *       Flags 
 *       DENS_VALID | DENS_BUF_VALID |ONE_FM ,
 *
 *       Density code	Compression code	Speed setting	
 *       SEQ_QIC150,		NULL,			NULL,
 *
 *       Buffered setting	Blocking				
 *       1,			512
 *       },
 *       { Minor 03 or rmtXa 
 *
 *       Flags 
 *       DENS_VALID | DENS_BUF_VALID  |ONE_FM,
 *
 *       Density code	Compression code	Speed setting	
 *       SEQ_8000R_BPI,		NULL,			NULL,
 *
 *       Buffered setting	Blocking				
 *       1,			512
 *       },
 *       { Minor 04 
 *
 *       Flags 
 *       DENS_VALID | DENS_BUF_VALID |ONE_FM ,
 *
 *       Density code	Compression code	Speed setting	
 *       SEQ_QIC150,		NULL,			NULL,
 *
 *       Buffered setting	Blocking				
 *       1,			512
 *       },
 *       { Minor 05 
 *
 *       Flags 
 *       DENS_VALID | DENS_BUF_VALID |ONE_FM ,
 *
 *       Density code	Compression code	Speed setting	
 *       SEQ_QIC320,		NULL,			NULL,
 *
 *       Buffered setting	Blocking				
 *       1,			1024
 *       },
 *       { Minor 06 
 *
 *       Flags 
 *       DENS_VALID | DENS_BUF_VALID |ONE_FM ,
 *
 *       Density code	Compression code	Speed setting	
 *       SEQ_QIC320,		NULL,			NULL,
 *
 *       Buffered setting	Blocking				
 *       1,			1024
 *       },
 *       { Minor 07 
 *
 *       Flags 
 *       DENS_VALID | DENS_BUF_VALID |ONE_FM ,
 *
 *       Density code	Compression code	Speed setting	
 *       SEQ_QIC320,		NULL,			NULL,
 *
 *       Buffered setting	Blocking				
 *       1,			NULL
 *       }
 *   };	end of tzk10_dens 
 *
 *    Below is the structure definition for the density_tbl
 *
 *    typedef struct density_tbl {
 *    	struct density{
 *    	   u_char    den_flags;		
 *    	   u_char    den_density_code;
 *    	   u_char    den_compress_code;
 *    	   u_char    den_speed_setting;
 *    	   u_char    den_buffered_setting; 
 *    	   u_long    den_blocking;	
 *    	}density[MAX_TAPE_DENSITY]; MAX_TAPE_DENSITY defined as 8 
 *    }DENSITY_TBL;
 *
 *    The mode select table allows up to 8 mode selects to be
 *    done for a device type on initial open of a unit.
 *    The mode select table for a device consists of 8 entries.
 *    Each entry has the following format:
 *    
 *    The first field is the SCSI page number for that device
 *    type. If the device is a tape and you want to set the
 *    device configuration page for tapes, the page number
 *    would be a hex 10.
 *
 *    The second field is the pointer to the mode select data.
 *    You must set up the page data and place the address
 *    of the page structure in this field.
 *
 *    The third field is the lenght of the page. This is the
 *    number of bytes of the page size you want sent to the 
 *    device.
 *
 *    The fourth field are flags for the mode select cdb the
 *    driver formats. Bit 0 a zero don't save page , a one
 *    save the page. Bit 1 a zero SCSI 1 device, a one 
 *    means a SCSI 2 device.
 *
 *    Below you will find the the following:
 *    1. Mode select table struct definition
 *    2. An example filled out mode select table for a TZK10 
 *    3. An filled out page definition for page hex 10 for
 *       the TZK10.
 *    
 *
 *     Mode Select Table Structure Definition:
 *
 *     typedef struct modesel_tbl {
 *     	   struct ms_entry{
 *     	       u_char  ms_page;	
 *     	       u_char  *ms_data;
 *     	       u_char  ms_data_len;
 *     	       u_char  ms_ent_sp_pf;
 *     	   }ms_entry[MAX_OPEN_SELS]; MAX_OPEN_SELS defined as 8
 *     }MODESEL_TBL;
 *
 *    MODESEL_TBL
 *    tzk10_mod = {
 *    	{ MODE PAGE ENTRY 1	
 *
 *    	Page number		The data pointer		
 *    	0x02,			(u_char *)&tzk10_page2,
 *
 *    	Data len		SCSI2??				
 *    	   28,			0x2
 *    	},
 *    	{ MODE PAGE ENTRY 2	
 *
 *    	Page number		The data pointer		
 *    	0x10,			(u_char *)&tzk10_page10,
 *
 *    	Data len		SCSI2??				
 *    	   28,			0x2
 *    	},
 *    	{ MODE PAGE ENTRY 3	
 *
 *    	Page number		The data pointer		
 *    	NULL,			(u_char *)NULL,
 *
 *    	Data len		SCSI2??				
 *    	NULL,			NULL
 *    	},
 *    	{ MODE PAGE ENTRY 4	
 *
 *    	Page number		The data pointer		
 *    	NULL,			(u_char *)NULL,
 *
 *    	Data len		SCSI2??				
 *    	NULL,			NULL
 *    	},
 *    	{ MODE PAGE ENTRY 5	
 *
 *    	Page number		The data pointer		
 *    	NULL,			(u_char *)NULL,
 *
 *    	Data len		SCSI2??				
 *    	NULL,			NULL
 *    	},
 *    	{ MODE PAGE ENTRY 6	
 *
 *    	Page number		The data pointer		
 *    	NULL,			(u_char *)NULL,
 *
 *    	Data len		SCSI2??				
 *    	NULL,			NULL
 *    	},
 *    	{ MODE PAGE ENTRY 7	
 *
 *    	Page number		The data pointer		
 *    	NULL,			(u_char *)NULL,
 *
 *    	Data len		SCSI2??				
 *    	NULL,			NULL
 *    	},
 *    	{ MODE PAGE ENTRY 8	
 *
 *    	Page number		The data pointer		
 *    	NULL,			(u_char *)NULL,
 *
 *    	Data len		SCSI2??				
 *    	NULL,			NULL
 *    	},
 *    };
 *
 *    SEQ_MODE_DATA6
 *    tzk10_page10 = {
 *    
 *    	{ Parameter header 
 *
 *    	mode_len	medium type	speed	
 *    	NULL,		NULL,		NULL,
 *
 *    	Buf_mode	wp		blk_desc_len	
 *    	0x01,		NULL,		sizeof(SEQ_MODE_DESC)
 *    	},
 *    	{ Mode descriptor 
 *
 *    	Density	num_blks2	num_blks1	
 *    	NULL,		NULL,		NULL,
 *
 *    	num_blks0	reserved	blk_len2	
 *    	NULL,				NULL,
 *
 *    	blk_len1	blk_len0			
 *    	NULL,		NULL
 *    	},
 *    	{
 *    	Page data for page 0x2 			
 *
 *    	PAGE header 
 *    	byte0	byte1				
 *    	   0x10,	0x0e,
 *
 *    	byte2    byte3    byte4    byte5    byte6	
 *    	   0x00,    0x00,    40,      40,      NULL,
 *
 *    	byte7    byte8    byte9    byte10   byte11	
 *    	   NULL,    0xe0,    NULL,    0x38,    NULL,
 *
 *    	byte12   byte13   byte14   byte15   		
 *    	   NULL,    NULL,    NULL,    NULL
 *    	}
 *     };
 *
 *********************************************************************/

/*
 * ************************************************************************* *
 * ************************************************************************* *
 * ************************************************************************* *
 *
 * AS OF V4.0:
 *
 * The Dynamic Device Recognition (DDR) subsystem is replacing the
 * following table.
 *
 * Please refer to the "ddr_config" utility man page and associated
 * documentation (referenced in the man page) for further information.
 *
 * ************************************************************************* *
 * ************************************************************************* *
 * ************************************************************************* *
 */

DENS_TBL_ENTRY   density_table[] = {
	0,		NULL,		/* 0x00 - Default density.	*/
};
int density_entrys = sizeof(density_table) / sizeof(struct dens_tbl_entry);


/*
 * The density structures for the known devices. You can
 * added your own at the bottom
 */
DENSITY_TBL 
def_dens = {
    {
    { /* Minor 00  or rmtXl*/
    /* Flags */ 
    DENS_VALID | DENS_BUF_VALID ,
    /* Density code	Compression code	Speed setting	*/
    SEQ_DEF_BPI,		NULL,			NULL,
    /* Buffered setting	Blocking				*/
    1,			NULL 
    },
    { /* Minor 01 or rmtXh */
    /* Flags */ 
    DENS_VALID | DENS_BUF_VALID,
    /* Density code	Compression code	Speed setting	*/
    SEQ_DEF_BPI,		NULL,			NULL,
    /* Buffered setting	Blocking				*/
    1,			NULL
    },
    { /* Minor 02 or rmtXm*/
    /* Flags */ 
    DENS_VALID | DENS_BUF_VALID ,
    /* Density code	Compression code	Speed setting	*/
    SEQ_DEF_BPI,		NULL,			NULL,
    /* Buffered setting	Blocking				*/
    1,			NULL
    },
    { /* Minor 03  or rmtXa*/
    /* Flags */ 
    DENS_VALID | DENS_BUF_VALID ,
    /* Density code	Compression code	Speed setting	*/
    SEQ_DEF_BPI,		NULL,			NULL,
    /* Buffered setting	Blocking				*/
    1,			NULL
    },
    { /* Minor 04 */
    /* Flags */ 
    DENS_VALID | DENS_BUF_VALID ,
    /* Density code	Compression code	Speed setting	*/
    SEQ_DEF_BPI,		NULL,			NULL,
    /* Buffered setting	Blocking				*/
    1,			NULL
    },
    { /* Minor 05 */
    /* Flags */ 
    DENS_VALID | DENS_BUF_VALID ,
    /* Density code	Compression code	Speed setting	*/
    SEQ_DEF_BPI,		NULL,			NULL,
    /* Buffered setting	Blocking				*/
    1,			NULL
    },
    { /* Minor 06 */
    /* Flags */ 
    DENS_VALID | DENS_BUF_VALID ,
    /* Density code	Compression code	Speed setting	*/
    SEQ_DEF_BPI,		NULL,			NULL,
    /* Buffered setting	Blocking				*/
    1,			NULL
    },
    { /* Minor 07 */
    /* Flags */ 
    DENS_VALID | DENS_BUF_VALID ,
    /* Density code	Compression code	Speed setting	*/
    SEQ_DEF_BPI,		NULL,			NULL,
    /* Buffered setting	Blocking				*/
    1,			NULL
    }
    }
}; /* end of def_dens */
/* End density structures.. Please extend the table
 * for your own tape drives.. The table is designed
 * to allow for flexibility and customer selections
 */

/* 
 * Mode Select issued to CD-ROM devices to set block length to 512 bytes.
 */
DIR_MODE_DATA6
cdrom_page0 = {

	{ /* Parameter header */
	/* mode_len	medium type	dpofua */
	0,		0,		0,
	/* wp	blk_desc_len	*/
	0,	sizeof(DIR_MODE_DESC)
	},
	{ /* Mode descriptor */
	/* Density	num_blks2	num_blks1	*/
	0,		0,		0,
	/* num_blks0	reserved	blk_len2	*/
	0,				00,
	/* blk_len1	blk_len0			*/
	0x2,		00
	},
};


SEQ_MODE_DATA6
dummy_seq_mode_data = {
	{ /* Parameter header */
	/* mode_len	medium type	speed	*/
	NULL,		NULL,		NULL,
	/* Buf_mode	wp		blk_desc_len	*/
	0x01,		NULL,		sizeof(SEQ_MODE_DESC)
	},
	{ /* Mode descriptor */
	/* Density	num_blks2	num_blks1	*/
	NULL,		NULL,		NULL,
	/* num_blks0	reserved	blk_len2	*/
	NULL,		0,		NULL,
	/* blk_len1	blk_len0			*/
	NULL,		NULL
	},
	{
	/* Page data since scsi 1 vendor unique	*/
	/* byte0	byte1		byte2		*/
	0x06,		0x00,		0x80,		
	/* byte3	byte4				*/
	0x80,		0x07
	}
	
};


/* 
 * Mode select tables
 */
MODESEL_TBL
cdrom_mode_sel = {
	{
	{ /* MODE PAGE ENTRY 1	*/
	/* Page number		The data pointer		*/
	00,			(u_char *)&cdrom_page0,
	/* Data len		SCSI2??				*/
	12,			NULL
	},
	{ /* MODE PAGE ENTRY 2	*/
	/* Page number		The data pointer		*/
	NULL,			(u_char *)NULL,
	/* Data len		SCSI2??				*/
	NULL,			NULL
	}
	}
};


/*
 * Default partition tables to be used if the
 * disk does not contain a parition table.
 */
struct size {
	daddr_t	nblocks;
	int	blkoffs;
}  ccmn_rzxx_sizes[8] = {
	131072,	0,		/* A=blk 0 thru 131071        (64Mb)     */
	262144, 131072,         /* B=blk 131072 thru 393215   (128Mb)    */
	-1,	0,		/* C=blk 0      thru ??????   (???Mb/Gb) */
	0,	0,
	0,	0,
	0,	0,
	-1, 	393216,         /* G=blk 393216 thru end (?)             */
	0,	0,
}, ccmn_rrd40_sizes[8] = {
	-1,	0,              /* A=blk 0 thru end (CDROM size variable) */
	0,	0,
	-1,	0,		/* C=blk 0 thru end (CDROM size variable) */
	0,	0,
	0,	0,
	0,	0,
	0,	0,
	0,	0,
}, ccmn_null_sizes[8] = {		/* Dummy for tapes */
	0,	0,
	0,	0,
	0,	0,
	0,	0,
	0,	0,
	0,	0,
	0,	0,
	0,	0,
};


/*
 * ************************************************************************* *
 * ************************************************************************* *
 * ************************************************************************* *
 *
 * AS OF V4.0:
 *
 * The following table will no longer be used. The Dynamic Device
 * Recognition (DDR) subsystem will provide the defaults.
 *
 * Please refer to the "ddr_config" utility man page and associated
 * documentation (referenced in the man page) for further information.
 *
 * ************************************************************************* *
 * ************************************************************************* *
 * ************************************************************************* *
 */

/*
 * This is the unknown device table. If a device is not in the
 * the cam_devdesc_tab[] then we default it to this table based
 * on device type found in the inquiry data.
 * Do Not change this table or the order of the entries.
 */
DEV_DESC dev_desc_unknown[] = {

/* UNKNOWN DISK */
{"UNKNOWN", 7, DEV_RZxx, (ALL_DTYPE_DIRECT << DTYPE_SHFT) | SZ_HARD_DISK, 
  (struct pt_info *)ccmn_rzxx_sizes, 512, DEC_MAX_REC, NO_DENS_TAB, NO_MODE_TAB,
  (SZ_BBR | SZ_NO_TAG | SZ_REORDER),
  NO_OPT_CMDS, SZ_READY_DEF, SZ_NO_QUE, DD_REQSNS_VAL | DD_INQ_VAL,
  36, 64
},
/* UNKNOWN TAPE */
{"UNKNOWN", 7, DEV_TZxx, (ALL_DTYPE_SEQUENTIAL << DTYPE_SHFT) | SZ_9TRK_CLASS, 
  (struct pt_info *)ccmn_null_sizes, NULL, (DEC_MAX_REC - 1), &def_dens, 
  NO_MODE_TAB, SZ_NO_FLAGS,
  NO_OPT_CMDS, SZ_READY_DEF, SZ_NO_QUE, DD_REQSNS_VAL | DD_INQ_VAL,
  36, 64
},

/* UNKNOWN Printer */
{"\0", 0, "\0", 0, 0, 0, 0, 0, 	
 0, 0, 0, 0, 0, 0, 0, 0},	

/* UNKNOWN Processor */
{"\0", 0, "\0", 0, 0, 0, 0, 0, 	
 0, 0, 0, 0, 0, 0, 0, 0},	

/* UNKNOWN WORM */
{"\0", 0, "\0", 0, 0, 0, 0, 0, 	
 0, 0, 0, 0, 0, 0, 0, 0},	

/* UNKNOWN cd-rom */
{"UNKNOWN", 7, DEV_RRDxx, (ALL_DTYPE_RODIRECT << DTYPE_SHFT) , 
  (struct pt_info *)ccmn_rrd40_sizes, 512, DEC_MAX_REC, NO_DENS_TAB,
  &cdrom_mode_sel, SZ_NO_FLAGS,
  SZ_PREV_ALLOW, 80, SZ_NO_QUE, DD_REQSNS_VAL | DD_INQ_VAL,
  36, 64
},
/* UNKNOWN Scanner */
{"\0", 0, "\0", 0, 0, 0, 0, 0, 	
 0, 0, 0, 0, 0, 0, 0, 0},	

/* UNKNOWN Optical */
{"\0", 0, "\0", 0, 0, 0, 0, 0, 	
 0, 0, 0, 0, 0, 0, 0, 0},	

/* UNKNOWN Medium Changer */
{"\0", 0, "\0", 0, 0, 0, 0, 0, 	
 0, 0, 0, 0, 0, 0, 0, 0},	

/* UNKNOWN Communication*/
{"\0", 0, "\0", 0, 0, 0, 0, 0, 	
 0, 0, 0, 0, 0, 0, 0, 0},	

};

/*
 * Number of valid unknown descriptors (size of table)
 */
I32 num_unknown_dev_desc = sizeof(dev_desc_unknown)/sizeof(DEV_DESC);

/*
 * ************************************************************************* *
 * ************************************************************************* *
 * ************************************************************************* *
 *
 * AS OF V4.0:
 *
 * The Dynamic Device Recognition (DDR) subsystem database is replacing
 * all information previously provided by the following table. DDR will
 * provide backward compatibility by searching for entries in this table
 * first before attempting to create an entry based on its database.
 *
 * HOWEVER: The information and interface provided by this table will be
 * removed in a future release.
 *
 * Please refer to the "ddr_config" utility man page and associated
 * documentation (referenced in the man page) for further information.
 *
 * ************************************************************************* *
 * ************************************************************************* *
 * ************************************************************************* *
 */

/*
 * CAM SCSI device descriptor information table.
 */
			/* DO NOT CHANGE MAX_DEVICE_DESC */
/* there should not be more than MAX_DEVICE_DESC entries */
DEV_DESC cam_devdesc_tab[] = {
{ 0 }
};


/*
 * Number of valid descriptors (size of table)
 */
I32 num_dev_desc = sizeof(cam_devdesc_tab)/sizeof(DEV_DESC);

/* END OF FILE */
/* ---------------------------------------------------------------------- */
