

/*
 * originally hichjaked from qlogic/isp1020.c
 */

/* ---------------------------------------------------------------------- *
 *                                                                        *
 * cam_scsi.c:      *
---------------------------------------------------------------------- */

/* ---------------------------------------------------------------------- */
/*              INCLUDE FILES        */
/* ---------------------------------------------------------------------- */

#define CAMERRLOG
#include <sys/types.h>
#include <sys/proc.h>
#include <sys/buf.h>
#include <sys/lock.h>
#include <sys/malloc.h>
#include <kern/lock.h>
#include <kern/thread.h>
#include <kern/sched.h>
#include <kern/parallel.h>
#include <kern/sched_prim.h>
#include <vm/vm_kern.h>
#include <dec/binlog/errlog.h>
#include <arch/alpha/machparam.h>
#include <io/common/devdriver.h>
#include <io/common/iotypes.h>
#include <io/common/kds.h>
#include <io/dec/pci/pci.h>
#include <io/cam/dec_cam.h>
#include <io/cam/cam.h>
#include <io/cam/scsi_status.h>
#include <io/cam/scsi_all.h>
#include <io/cam/scsi_phases.h>
#include <io/cam/cam_logger.h>
#include <io/cam/sim_cirq.h>
#include <io/cam/dme.h>
#include <io/cam/cam_debug.h>
#include <io/cam/sim_target.h>
#include <io/cam/sim.h>
#include <io/cam/sim_common.h>
#include <io/cam/cam_errlog.h>
#include <io/cam/camdb.h>
#include <io/cam/pdrv.h>
#include <io/cam/ccfg.h>
#include <io/cam/simos/cam_simos.h>

#include <io/cam/simos/simos_scsi.h>
/*
For now I have included this file in the kernel tree
But it is actually borrowed from the simulation tree
The two need to be kept consistent
#include "/udir/bugnion/simulation/cpus-alpha/simos/simos_scsi.h"
*/


#ifdef TM_SUPPORTED
#include <io/cam/sim_htm.h>
bomb. 
#endif

/* ---------------------------------------------------------------------- */
/*              HEADER STUFF        */
/* ---------------------------------------------------------------------- */

/* external declarations */
extern SIM_SOFTC  *softc_directory[];
extern CAM_SIM_ENTRY  dec_sim_entry;
extern struct driver  simos_driver;
extern int   simos_bus_table[];
extern U32       cam_at_boottime();
extern I32   scsi_bus_reset_at_boot;
extern char   cam_ctlr_string[];
extern long   sable_direct_map_base;
extern SIM_SM   sim_sm;
extern void   ss_reset_detected(); /* Notify bus reset detected */
extern int              sc_alloc_nexus_heads(SIM_SOFTC *sc);
extern long  lbolt;
extern int  hz;
#ifdef TM_SUPPORTED
extern void  htm_process_bdr(SIM_SOFTC *);
extern SIM_WS  *sx_setup_ws();
#endif
#ifdef __alpha
extern long   rpcc();
#endif

/* forward references */
void   simos_send_marker( SIMOS_SOFTC * simos_softc, U8 target, U8 lun, U8 modifier );
void                    simos_process_done( SIMOS_JOB *job );
U32   simos_cam_bus_reset();
U32   simos_cam_bus_reset_tmo( SIM_SOFTC * sim_softc );
U32   simos_termio_abort_bdr();
U32   simos_cam_init();
int   simos_set_retry_count();
U32    simos_unload();
U32   simos_risc_register_setup();
U32   simos_pci_register_setup();
U32   simos_sxp_register_setup();
U32   simos_mbox_register_setup();
U32   simos_softc_init();
U32   simos_go();
int   simos_intr();
U32   simos_null( SIMOS_SOFTC * simos_softc );
U32    simos_bus_reset();
U32   simos_chip_init();
SIMOS_MAIL *   simos_allocate_msg();
void   simos_process_response_queue();
U32   simos_dumpreg(SIMOS_SOFTC *simos_softc);
u_long   simos_segment_load();
void   simos_read_nvram_word();
int   simos_dump_ram( SIMOS_SOFTC *simos_softc, U16 start_word, U16 num_word, U32 dest );
int   simos_write_ram_word( SIM_SOFTC *sim_softc, U16 address, U16 data );
int   simos_read_ram_word( SIM_SOFTC *sim_softc, U16 address, U16 *data );
int   simos_set_device_queue_parameters( SIMOS_SOFTC *simos_softc, U16 target, U16 lun );
int   simos_stop_firmware( SIMOS_SOFTC *simos_softc );
int   simos_verify_checksum( SIMOS_SOFTC *simos_softc, int *checksum );
int   simos_get_target_status( SIMOS_SOFTC *, U16);
int   simos_get_target_parameters( SIMOS_SOFTC *simos_softc, U16 target_id, U16 *target_control_flags, U16 *offset, U16 *period );
int   simos_set_target_parameters( SIMOS_SOFTC *simos_softc, U16 target_id, U16 target_control_flags, U16 offset, U16 period );
int   simos_get_firmware_status( SIMOS_SOFTC *simos_softc );
int   simos_get_device_queue_parameters( SIMOS_SOFTC *simos_softc, U16 target, U16 lun );
int   simos_get_device_queue_status( SIMOS_SOFTC *simos_softc, U16 target, U16 lun );
int   simos_abort( SIMOS_SOFTC *simos_softc, U16 target, U16 lun, U32 handle );
void   simos_init_target_parameters( SIMOS_SOFTC *simos_softc, U16 target );
int   simos_init( SIMOS_SOFTC * simos_softc);
int   simos_reinit( SIMOS_SOFTC * simos_softc);
int   simos_get_console_image( SIMOS_SOFTC * simos_softc );
int   simos_execute_iocb( SIMOS_SOFTC *simos_softc, SIMOS_IOCB_IOSB *simos_iocb_iosb );
int   simos_enable_subsystem_features( SIMOS_SOFTC *simos_softc );
int                     simos_enable_target( SIMOS_SOFTC * simos_softc, U16 flags );
int                     simos_get_target_status( SIMOS_SOFTC * simos_softc, U16 lun );
int   simos_enable_overrun_recovery( SIMOS_SOFTC *simos_softc );
void   simos_free_handles( SIMOS_SOFTC *simos_softc, SIMOS_JOB * job );
void   simos_set_nvram( SIMOS_SOFTC *simos_softc );
void   simos_invalidate_all_jobs( SIMOS_SOFTC *simos_softc );
void   simos_soft_reset( SIMOS_SOFTC *simos_softc );
void   simos_hard_reset( SIMOS_SOFTC * simos_softc );
void   simos_disable_scsi_filters( SIMOS_SOFTC *simos_softc );
void   simos_enable_scsi_filters( SIMOS_SOFTC *simos_softc );
void   simos_logger( u_char *func, u_char *msg, U32 flags, SIM_SOFTC *sim_softc,
              SIMOS_SOFTC *simos_softc );

I32   simos_enable_lun(SIM_SOFTC *, CCB_ENABLE_LUN *);
I32   simos_accept(SIM_SOFTC *, CCB_HEADER *);
I32   simos_continue(SIM_SOFTC *, CCB_CONT_TARG *);



static void   (*local_errlog)() = simos_logger;

/* pci read/write */
#define SIMOS_RCSR8(csr)  READ_BUS_D8(csr)
#define SIMOS_RCSR16(csr)  READ_BUS_D16(csr)
#define SIMOS_RCSR32(csr)  READ_BUS_D32(csr)

#define SIMOS_WCSR8(csr, value) WRITE_BUS_D8(csr, (value)); mb();
#define SIMOS_WCSR16(csr, value) WRITE_BUS_D16(csr, (value)); mb();
#define SIMOS_WCSR32(csr, value) WRITE_BUS_D32(csr, (value)); mb();

/*
 * ev2 gamma bug where write are not ordered between cpus
 */
#define EV5_GAMMA(simos_sc) {     \
    U16 foo;       \
    extern long lockmode;     \
    if (lockmode) foo = SIMOS_RCSR16((simos_sc)->simos_reg.p_icr); \
}

/*
 * Allocate a job (CAM ZONE).  Must be called at splbio to
 * sync against sc_free.
 */
#define SIMOS_GET_JOB(simos_softc, job) {   \
    if (((simos_softc)->spares > 2) ||   \
        (((simos_softc)->spares) && cam_inisr())) { \
 (simos_softc)->spares--;    \
 (job) = (simos_softc)->jspare[(simos_softc)->spares]; \
    } else {      \
        (simos_softc)->memory_allocations++;  \
        SIMOS_UNLOCK_FAST((simos_softc)->chiplock);  \
        (job) = (SIMOS_JOB *)sc_alloc(sizeof(SIMOS_JOB)); \
        SIMOS_LOCK_FAST((simos_softc)->chiplock);  /* already at splbio */ \
    }       \
}

#define SIMOS_RELEASE_JOB(simos_softc, job) {   \
    int ipl;                                                    \
    if ((simos_softc)->spares < SIMOS_MAXSPARE) {   \
        SIMOS_LOCK((simos_softc)->chiplock, ipl)   \
     if ((simos_softc)->spares < SIMOS_MAXSPARE) {  \
  (simos_softc)->jspare[simos_softc->spares] = (job); \
                (simos_softc)->spares++;    \
                (job) = (SIMOS_JOB *)NULL;   \
     }       \
 SIMOS_UNLOCK(simos_softc->chiplock, ipl);   \
    }        \
    if (job) {       \
        SIMOS_FREE_MEMORY *fm = (SIMOS_FREE_MEMORY *)(job);  \
        fm->flink = 0;      \
        fm->blink = 0;      \
        fm->addr = (char *)(job);    \
        fm->size = sizeof(SIMOS_JOB);    \
        SIMOS_LOCK(simos_softc->chiplock, ipl)   \
        insque((void *)fm, (void *)&simos_softc->free_memory); \
        SIMOS_UNLOCK(simos_softc->chiplock, ipl)   \
    }        \
}

#define SIMOS_RELEASE_JOB_FAST(simos_softc, job) {   \
    if ((simos_softc)->spares < SIMOS_MAXSPARE) {   \
        SIMOS_LOCK_FAST((simos_softc)->chiplock)   \
     if ((simos_softc)->spares < SIMOS_MAXSPARE) {  \
  (simos_softc)->jspare[simos_softc->spares] = (job); \
                (simos_softc)->spares++;    \
                (job) = (SIMOS_JOB *)NULL;   \
     }       \
 SIMOS_UNLOCK_FAST(simos_softc->chiplock);   \
    }        \
    if (job) {       \
        SIMOS_FREE_MEMORY *fm = (SIMOS_FREE_MEMORY *)(job);  \
        fm->flink = 0;      \
        fm->blink = 0;      \
        fm->addr = (char *)(job);    \
        fm->size = sizeof(SIMOS_JOB);    \
        SIMOS_LOCK_FAST(simos_softc->chiplock)   \
        insque((void *)fm, (void *)&simos_softc->free_memory); \
        SIMOS_UNLOCK_FAST(simos_softc->chiplock)   \
    }        \
}


/* global defines */
 
#define MAX_ENABLE_TARG_RETRY 5

#define SIMOS_DEBUG 1
#define SIMOS_DEBUG_NVRAM 1

#ifdef SIMOS_DEBUG
int simos_debug = 0;
#define Eprintf(s)   if( simos_debug ) printf(s)
#define Eprintf1(s, p1)  if( simos_debug ) printf(s, p1)
#define Eprintf2(s, p1, p2) if( simos_debug ) printf(s, p1, p2)
#define Eprintf3(s, p1, p2, p3) if( simos_debug ) printf(s, p1, p2, p3)
#define Qprintf(s)   if( simos_debug > 1 ) printf(s)
#define Qprintf1(s, p1)  if( simos_debug > 1 ) printf(s, p1)
#define Qprintf2(s, p1, p2)  if( simos_debug > 1 ) printf(s, p1, p2)
#define Qprintf3(s, p1, p2, p3) if( simos_debug > 1 ) printf(s, p1, p2, p3)
#else
#define Eprintf(s)
#define Eprintf1(s, p1)
#define Eprintf2(s, p1, p2)
#define Eprintf3(s, p1, p2, p3)
#define Qprintf(s)
#define Qprintf1(s, p1)
#define Qprintf2(s, p1, p2)
#define Qprintf3(s, p1, p2, p3)
#endif

/* default 1 (on) parameters */
int simos_tag_ok = 1;
int simos_firmware_load = 1;

/* default 0 (off) parameters */
int simos_enable_filters = 0;
int simos_disable_filters = 0;
int simos_try_firmware_load = 0;
int     simos_no_printf = 0;

/* locking */
struct lockinfo *cam_simos_li;

/* ---------------------------------------------------------------------- */
/*              DRIVER CONFIGURATION AND INITIALIZATION            */
/* ---------------------------------------------------------------------- */

/* ---------------------------------------------------------------------- *
 *                                                                        *
 * ROUTINE NAME: simos_confl1                                                *
 *                                                                        *
 * FUNCTIONAL DESCRIPTION:                                                *
 *      "Bus" configuration function for the isp1020.  Called during boot.*
 *                                                                        *
 * CALLED BY:                                                             *
 *                                                                        *
 * FORMAL PARAMETERS:                                                     *
 *  Call Syntax:         *
 *          simos_confl1(bustype, binfo, bus)      *
 *       int bustype;        *
 *       struct pci_bus_info *binfo;      *
 *       struct bus *bus;       *
 *                                                                        *
 * IMPLICIT INPUTS:                                                       *
 *                                                                        *
 * IMPLICIT OUTPUTS:                                                      *
 *                                                                        *
 * RETURN VALUE:                                                          *
 *  CAM_TRUE if configuration succeeded, CAM_FALSE otherwise   *
 *                                                                        *
 * SIDE EFFECTS:                                                          *
 *                                                                        *
 * ---------------------------------------------------------------------- */
 
int
simosconfl1( bustype, binfo, bus )
    int bustype;
    struct pci_bus_info *binfo;
    struct bus *bus;
{
    struct controller *ctlr;
    struct pci_slot *pci_slot;
    struct driver *savedriver;
    int saveslot;

    SIM_MODULE( simos_confl1 );

    pci_slot = bus->pcislot;
    bus->bus_type = BUS_SCSI;
    bus->alive |= ALV_ALIVE;

    printf( "%s%d at %s%d slot %d\n",bus->bus_name, bus->bus_num,
     bus->connect_bus, bus->connect_num, pci_slot->slot );


    /*
     * Get the controller structure
     */
    if( (ctlr = get_ctlr("scsi",                /* "scsi" */
                         0,                     /* take scsi at slot 0 */
                         bus->bus_name,         /* "isp" */
                         bus->bus_num)) ||      /* which isp */
       (ctlr = get_ctlr("scsi",                 /* "scsi" */
                         -1,                    /* take any scsi */
                         bus->bus_name,         /* "isp" */
                         bus->bus_num)) ||      /* which isp */
       (ctlr = get_ctlr("scsi",
                        0,                      /* take scsi at slot 0 */
                        bus->bus_name,
                        -1)) ||                 /* wild isp number */
       (ctlr = get_ctlr("scsi",
                        -1,
                        bus->bus_name,
                        -1)) ||                 /* wild isp number */
       (ctlr = get_ctlr("scsi",
                        -1,
                        "*",                    /* finally take ANY "isp" */
                        -99))) {

        saveslot = ctlr->slot;
        savedriver = ctlr->driver;

        ctlr->slot     = 0;          /* ? */
        ctlr->pcislot  = pci_slot;
        ctlr->driver   = &simos_driver;            /* goto right probe */
        *ctlr->intr    = simos_intr;
        ctlr->bus_name = bus->bus_name;
        ctlr->bus_num  = bus->bus_num;

        /*
         * Continue the configuration.
         */
	
        if (!(pci_config_cont(pci_slot, ctlr, bus ))) {
           printf( "%s%d: Not probed.\n",
		   pci_slot->driver_name, pci_slot->unit );
           ctlr->slot = saveslot;
           ctlr->driver = savedriver;
        } else {
           conn_ctlr(bus, ctlr);
           return( 1 );
        }
#if 0
	conn_ctlr(bus,ctlr);
	return 1;
#endif
    }
    return(0);
}

/* ---------------------------------------------------------------------- *
 *                                                                        *
 * ROUTINE NAME: simos_confl2                                                *
 *                                                                        *
 * FUNCTIONAL DESCRIPTION:  not used                                      *
 *                                                                        *
 * CALLED BY:                                                             *
 *                                                                        *
 * FORMAL PARAMETERS:                                                     *
 *                                                                        *
 * IMPLICIT INPUTS:                                                       *
 *                                                                        *
 * IMPLICIT OUTPUTS:                                                      *
 *                                                                        *
 * RETURN VALUE:                                                          *
 *                                                                        *
 * SIDE EFFECTS:                                                          *
 *                                                                        *
 * ---------------------------------------------------------------------- */

int
simosconfl2(bustype, binfo, bus)
int bustype;
struct pci_bus_info *binfo;
struct bus *bus;
{
    return(1);
}



/* ---------------------------------------------------------------------- *
 *                                                                        *
 * ROUTINE NAME: simos_probe                                                *
 *                                                                        *
 * FUNCTIONAL DESCRIPTION:                                                *
 * Get the isp1020 chip ready to run and probe for devices.   *
 *                                                                        *
 * CALLED BY:                                                             *
 *                                                                        *
 * FORMAL PARAMETERS:                                                     *
 *                                                                        *
 * IMPLICIT INPUTS:                                                       *
 *                                                                        *
 * IMPLICIT OUTPUTS:                                                      *
 *                                                                        *
 * RETURN VALUE:                                                          *
 * CAM_TRUE on success        *
 * CAM_FALSE on failure        *
 *                                                                        *
 * SIDE EFFECTS:                                                          *
 *                                                                        *
 * ---------------------------------------------------------------------- */

int
simos_probe( pci_cnfg_hdr, ctlr )
struct pci_config_hdr *pci_cnfg_hdr;
struct controller *ctlr;
{
    SIM_SOFTC   *sim_softc = softc_directory[ ctlr->ctlr_num ];
    SIMOS_SOFTC   *simos_softc = 0;
    SIMOS_JOB   *job;
    ihandler_t   simos_ihandle;
    struct handler_intr_info simos_intr_info;
    u_long   bytes_mapped;
    int   i,j, rev;
    U16   data;
    EDT   *eg;
    U8   saved_sync_data_transfers[16];
    U8   saved_wide_data_transfers[16];
    volatile   CAM_EDT_ENTRY *ee;

    SIM_MODULE( simos_probe );

    /*
     * Have we been here before?
     */

    if( sim_softc != NULL ) {
 if( ctlr->addr == sim_softc->csr_probe ) {
     SIMOS_ERROR( sim_softc, 
  "isp controller already probed" );
            return ( 1 );
 } else {
            SIMOS_ERROR( sim_softc, 
  "isp controller already probed at another CSR" );
            return( 0 );
 }
    }

    /*
     * only support on ctrl 0 for now (bugnion)
     */

    if ( ctlr->ctlr_num ) {
       printf("simos_probe: cannot attach to ctlr %d \n",ctlr->ctlr_num );
       return 0;
    }
    /*
     * Check the pci config base alignment
     */

    if( pci_cnfg_hdr->bar1 & 0x3 ) {
 SIMOS_ERROR( sim_softc,
     "isp config base is illegal" );
 return( 0 );
    }

    /*
     * Allocate some memory for softc structures
     */

    sim_softc = (SIM_SOFTC *)sc_alloc(sizeof(SIM_SOFTC));
    simos_softc = (SIMOS_SOFTC *)sc_alloc(sizeof(SIMOS_SOFTC));
    if( !sim_softc || !simos_softc ) {
 if( sim_softc )
     sc_free( sim_softc, sizeof( SIM_SOFTC ) );
 if( simos_softc )
     sc_free( simos_softc, sizeof( SIMOS_SOFTC ) );
 printf( "isp: Unable to allocate memory for probe!\n" );
 return( 0 );
    }

    /* 
     * Setup the isp softc 
     */

    bzero( simos_softc, sizeof(SIMOS_SOFTC) );
    simos_softc->sim_softc      = sim_softc;
    simos_softc->pci_cnfg_hdr     = pci_cnfg_hdr;
    simos_softc->simos_base      = pci_cnfg_hdr->bar1; /* change to bar1 */
    simos_softc->pci_base      = pci_cnfg_hdr->bar1; /* ditto */
    simos_softc->simos_ctlr      = ctlr;
    simos_softc->simos_bus      = ctlr->bus_hd;
    simos_softc->fw_dmahandle     = (dma_handle_t)NULL;
    simos_softc->spares      = 0;
    simos_softc->reinits      = 0;
    simos_softc->mailbox_semaphore    = 0;
    simos_softc->mailbox_status       = SIMOS_BUSY;
    simos_softc->request_info.depth   = MAX_QUEUE_ENTRIES;
    simos_softc->response_info.depth  = MAX_QUEUE_ENTRIES;

    SIMOS_LOCK_INIT( simos_softc->chiplock );

    /*
     * Setup the sim softc
     */

    bzero( sim_softc, sizeof(SIM_SOFTC) );
    sim_softc->hba_sc        = simos_softc;
    sim_softc->csr_probe      = (void *)pci_cnfg_hdr;
    sim_softc->um_probe      = (void *)ctlr;
    sim_softc->reg       = (void *)simos_softc;
    sim_softc->cntlr       = ctlr->ctlr_num;
    sim_softc->dme       = 0;      

    if( simos_tag_ok )
        sim_softc->path_inq_flags   = PI_TAG_ABLE;
    sim_softc->max_targets          = NDPS_16;
    sim_softc->max_luns             = NLPT_8;

    if (sc_alloc_nexus_heads(sim_softc) != CAM_TRUE) {
 CAM_ERROR(module, "SIM_SOFTC initialization failed",
           SIM_LOG_PRISEVERE, NULL, NULL, NULL);
 return ( 0 );
    }

    /*
     * Print the controller name. ADD NEW NAMES HERE AS REQUIRED.
     *
     *    Version 1: ISP1020
     *    Version 2: ISP1020A
     *    Version 3: ISP1020A/V2
     */

    printf( "simos%d: SIMOS 0.0", simos_softc->simos_bus->bus_num );
    printf("\n");

    /*
     * Allocate some memory for the request and response queues
     * Each entry is 64 bytes and must be phys contig, so we'll
     * allocate at most 1 page until we have a phys contig memory
     * allocator in OSF/1.
     *
     * All some basic checks to make sure things are ok...
     *
     */

    if( sizeof( SIMOS_QUEUE ) > NBPG )
 panic("isp queue size may not be larger than 1 page");

    if( sizeof( SIMOS_QUEUE ) != NBPG )
        printf( "simos%d: queue size is %d\n",
  simos_softc->simos_bus->bus_num, sizeof( SIMOS_QUEUE ) );

    simos_softc->request_queue = (SIMOS_QUEUE *)sc_alloc( sizeof(SIMOS_QUEUE) );
    simos_softc->response_queue = (SIMOS_SQUEUE *)sc_alloc( sizeof(SIMOS_QUEUE) );
    if( (simos_softc->request_queue == (SIMOS_QUEUE *)NULL) ||
        (simos_softc->response_queue == (SIMOS_SQUEUE *)NULL) ) {
        printf( "simos%d: Unable to allocate memory for Qlogic driver\n",
  simos_softc->simos_bus->bus_num );
 /*
  * Deallocate dynamic memory before failing probe.
  */
 if( simos_softc->request_queue != (SIMOS_QUEUE *)NULL )
     sc_free( simos_softc->request_queue, sizeof( SIMOS_QUEUE ) );
 if( simos_softc->response_queue != (SIMOS_SQUEUE *)NULL )
     sc_free( simos_softc->response_queue, sizeof( SIMOS_QUEUE ) );
 sc_free( sim_softc, sizeof( SIM_SOFTC ) );
 sc_free( simos_softc, sizeof( SIMOS_SOFTC ) );

        return( 0 );
    }

    
    /*
     * Allocate memory for jobs
     */

    for( i=0; i<SIMOS_MAXSPARE; i++ ) {
        job = (SIMOS_JOB *)sc_alloc( sizeof( SIMOS_JOB ) );
 if( job == (SIMOS_JOB *)NULL ) {

     /*
      * Deallocate dynamic memory before failing probe.
      */
     for( ; i>=0; --i )
  sc_free( simos_softc->jspare[i], sizeof( SIMOS_JOB ) );

     dma_map_unload( DMA_DEALLOC, simos_softc->request_info.dmahandle );
     dma_map_unload( DMA_DEALLOC, simos_softc->response_info.dmahandle );
     sc_free( simos_softc->request_queue, sizeof( SIMOS_QUEUE ) );
     sc_free( simos_softc->response_queue, sizeof( SIMOS_QUEUE ) );
     sc_free( sim_softc, sizeof( SIM_SOFTC ) );
     sc_free( simos_softc, sizeof( SIMOS_SOFTC ) );

     return( 0 );
 }
 if( simos_softc->spares < SIMOS_MAXSPARE ) {
     simos_softc->jspare[simos_softc->spares] = job;
            simos_softc->spares++;
      }
    }

    /*
     * Enable interrupt handler - this must happen before
     * any mailbox operations are attempted.
     */

    Qprintf( "Adding handler\n" );
    simos_intr_info.configuration_st = (caddr_t)ctlr;
    simos_intr_info.intr = simos_intr;
    simos_intr_info.param = (caddr_t)simos_softc;
#ifdef SHARED_INTR_CAPABLE
    simos_intr_info.config_type =
 ( CONTROLLER_CONFIG_TYPE | SHARED_INTR_CAPABLE );
#else
    simos_intr_info.config_type = CONTROLLER_CONFIG_TYPE;
#endif

    simos_ihandle.ih_bus = ctlr->bus_hd->bus_hd; /* scsi->isp->pci */
    simos_ihandle.ih_bus_info = (char *)&simos_intr_info;

    printf("SIMOS probe::add_handler bus=%d conf_ctrl=%d type=0x%x \n",
	   simos_ihandle.ih_bus,
	   ctlr, simos_intr_info.config_type);

#ifdef gone
    simos_softc->hid = handler_add( &simos_ihandle );
    if( simos_softc->hid == (ihandler_id_t *)(NULL) ) {
       printf( "simoos%d: Failed to add interrupt handler!\n",
	       simos_softc->simos_bus->bus_num );
       /* ... */
    }
#endif
    {
       vm_offset_t vecaddr;
       if(allocvec(1, &vecaddr) != KERN_SUCCESS)
	  panic("cam_simos: coudl not alloc vector\n");
       printf("simos: vecaddr=0x%lx offset=0x%x \n",
	      vecaddr,vecoffset(vecaddr));
       
       intrsetvec(vecoffset(vecaddr), simos_intr, simos_softc);
    }

    softc_directory[ctlr->ctlr_num] = sim_softc;
    if(ccfg_simattach(&dec_sim_entry, ctlr) == CAM_FAILURE) {
       panic("could not ccfg_simattach \n");
        return( 0 );
    }
    simos_softc->state |= SIMOS_ALIVE;

    Qprintf( "In simos_probe exit\n" );
    ctlr->private[0] = cam_ctlr_string;

    return( 1 );
}

/* ---------------------------------------------------------------------- */
/*              ISP CAM INTERFACE       */
/* ---------------------------------------------------------------------- */

/* ---------------------------------------------------------------------- *
 *                                                                        *
 * ROUTINE NAME: simos_attach()                                             *
 *                                                                        *
 * FUNCTIONAL DESCRIPTION:                                                *
 *      This routine sets up the HBA function vectors in the SIM_SOFTC    *
 *      structure.         *
 *                                                                        *
 * CALLED BY:  SIM config routine, name_lookup_hba_attach()               *
 *                                                                        *
 * FORMAL PARAMETERS:                                                     *
 *              sim_softc -- Pointer to the SIM_SOFTC structure           *
 *                                                                        *
 * IMPLICIT INPUTS:                                                       *
 *                                                                        *
 * IMPLICIT OUTPUTS:                                                      *
 *              HBA function vectors are set up in SIM_SOFTC              *
 *                                                                        *
 * RETURN VALUE:                                                          *
 *              CAM_REQ_CMP     if successful attach                      *
 *              CAM_REQ_CMP_ERR if attach failed                          *
 *                                                                        *
 * SIDE EFFECTS:                                                          *
 *                                                                        *
 * ---------------------------------------------------------------------- */

U32
simos_attach( SIM_SOFTC *sim_softc )

{
    SIM_MODULE( simos_attach );

    Qprintf2( "simos%d: %s\n",
       ((SIMOS_SOFTC *)sim_softc->hba_sc)->simos_bus->bus_num, module );

    /* 
     * Setup vectors 
     */
    sim_softc->hba_init  = simos_cam_init;
    sim_softc->hba_go   = simos_go;
    sim_softc->hba_bus_reset  = simos_cam_bus_reset;
    sim_softc->hba_sel_msgout   = simos_termio_abort_bdr;



    return( CAM_REQ_CMP );
}

/* ---------------------------------------------------------------------- *
 *                                                                        *
 * ROUTINE NAME: simos_unload        *
 *                                                                        *
 * FUNCTIONAL DESCRIPTION:                                                *
 *                                                                        *
 * CALLED BY:            *
 *                                                                        *
 * FORMAL PARAMETERS:                                                     *
 *                                                                        *
 * IMPLICIT INPUTS:                                                       *
 *                                                                        *
 * IMPLICIT OUTPUTS:                                                      *
 *                                                                        *
 * RETURN VALUE:                                                          *
 *                                                                        *
 * SIDE EFFECTS:                                                          *
 *                                                                        *
 * ---------------------------------------------------------------------- */

U32
simos_unload()
{

   SIM_MODULE( simos_unload );

   SIM_PRINTD(NOBTL,NOBTL,NOBTL,(CAMD_ERRORS),("simos_unload called\n"));
   return (CAM_REQ_CMP);

}
U32
simos_chip_reset( sim_softc )
SIM_SOFTC *sim_softc;
{

   SIM_MODULE( simos_chip_reset );

   SIM_PRINTD(NOBTL,NOBTL,NOBTL,(CAMD_ERRORS),("simos_chip_reset called\n"));
   return (CAM_REQ_CMP);

}

/* ---------------------------------------------------------------------- *
 *                                                                        *
 * ROUTINE NAME: simos_cam_init()                                           *
 *                                                                        *
 * FUNCTIONAL DESCRIPTION:                                                *
 *      This routine is called by the sim_init() via SC_HBA_INIT macro.   *
 *                                                                        *
 * CALLED BY:  sim_init()                                                 *
 *                                                                        *
 * FORMAL PARAMETERS:                                                     *
 *              sim_softc -- Pointer to the SIM_SOFTC structure           *
 *                                                                        *
 * IMPLICIT INPUTS:                                                       *
 *                                                                        *
 * IMPLICIT OUTPUTS:                                                      *
 *                                                                        *
 * RETURN VALUE:                                                          *
 *              CAM_REQ_CMP     if successful initialization              *
 *              CAM_REQ_CMP_ERR if initialization fails                   *
 *                                                                        *
 * SIDE EFFECTS:                                                          *
 *                                                                        *
 * ---------------------------------------------------------------------- */

U32
simos_cam_init( sim_softc )
SIM_SOFTC *sim_softc;
{
    SIMOS_SOFTC *simos_softc = sim_softc->hba_sc;

    SIM_MODULE( simos_cam_init );

    Qprintf2( "simos%d: %s\n", simos_softc->simos_bus->bus_num, module );


    Qprintf( "In simos_cam_init, no action taken at this time...\n" );

}

/* ---------------------------------------------------------------------- */
/*              I/O, DEVICE, BUS HANDLING                            */
/* ---------------------------------------------------------------------- */

/* ---------------------------------------------------------------------- *
 *                                                                        *
 * ROUTINE NAME: simos_go()                                                 *
 *                                                                        *
 * FUNCTIONAL DESCRIPTION:                                                *
 *      This is the HBA_GO routine for this driver.       *
 *                                                                        *
 * CALLED BY:  ss_start() in sim_sched.c                                  *
 *                                                                        *
 * FORMAL PARAMETERS:                                                     *
 *              sws -- Pointer to the SIM working set for the request     *
 *                      to be sent                                        *
 *                                                                        *
 * IMPLICIT INPUTS:                                                       *
 *                                                                        *
 * IMPLICIT OUTPUTS:                                                      *
 *                                                                        *
 * RETURN VALUE:                                                          *
 *              CAM_REQ_CMP -- CCB request sent without error             *
 *              CAM_BUSY -- CAM HBA subsystem is busy                     *
 *              CAM_PROVIDE_FAIL -- Unable to provide requested capability*
 *              CAM_NO_HBA -- The adapter is off-line, i.e. is dead       *
 *                                                                        *
 * SIDE EFFECTS:                                                          *
 *                                                                        *
 * ---------------------------------------------------------------------- */

U32
simos_go( SIM_WS *sws )
{
   SIM_SOFTC *sim_softc = sws->sim_sc;
   SIMOS_SOFTC *simos_softc = (SIMOS_SOFTC *)(sim_softc->hba_sc);
   CCB_SCSIIO *ccb = sws->ccb;
   int entry = 0;
   int s;
   int segment;
   SIMOS_JOB *job;

   SIM_MODULE( simos_go );

   /*
    * Debug 
    */
   Qprintf2( "simos%d: %s\n", simos_softc->simos_bus->bus_num, module );
   Qprintf1( "      lbolt = 0x%x\n", lbolt);
   Qprintf1( "      sws = 0x%x\n", sws);
#ifdef SIMOS_DEBUG
   if( simos_debug && (sws->cam_flags & CAM_DIS_SYNC) )
      Qprintf1( "simos%d: sync disabled\n", simos_softc->simos_bus->bus_num );
#endif

   /*
    * CDB checks
    *    max supported in isp driver is 12 bytes
    */
   if( ccb->cam_cdb_len > 12 ) {
      SIM_PRINTD(NOBTL,NOBTL,NOBTL,(CAMD_ERRORS),
                 ("simos_go - max size of CDB is 12 bytes\n"));
      return( CAM_PROVIDE_FAIL );
   }

   /*
    * Disable linked cdb support
    */
   if( sws->cam_flags & CAM_CDB_LINKED ) {
      SIM_PRINTD(NOBTL,NOBTL,NOBTL,(CAMD_ERRORS),
                 ("simos_go - no support for linked commands\n"));
      return (CAM_PROVIDE_FAIL);
   }
    
   SIMOS_LOCK( simos_softc->chiplock, s );

   /*
    * Sanity Checks 
    */
#if 0
   if( !(simos_softc->state & SIMOS_ALIVE) ) {
      SIMOS_UNLOCK( simos_softc->chiplock, s );
      return( CAM_NO_HBA );
   }

   if( simos_softc->state &
       (CHIP_RESET_IN_PROG|BUS_RESET_IN_PROG) ) {
      SIMOS_UNLOCK( simos_softc->chiplock, s );
      return( CAM_BUSY );
   }
#endif

   simos_softc->state |= CMD_RING_LOCK;

#if 0 
   /*
    * tmp code: unsafe to free memory in interrupt context
    * Check to see if there is any memory that we need to free.
    * This code should be moved to a thread since this can not be
    * done in an isr.
    */
   if(( simos_softc->free_memory.flink != (SIMOS_FREE_MEMORY *)&simos_softc->free_memory ) && 
      ( !cam_inisr() )) {
      SIMOS_FREE_MEMORY *fm = simos_softc->free_memory.flink;
      remque(fm);
      SIMOS_UNLOCK_FAST( simos_softc->chiplock );
      sc_free( fm, fm->size );
      SIMOS_LOCK_FAST( simos_softc->chiplock );        /* already at splbio */
   }

#endif

   /*
    * Already in progress
    */

   if (sws->targid <0 || sws->targid >= 16) {
      panic("simos targid out of range %d \n",sws->targid);
   }
       
   if (simos_softc->activeJobs[sws->targid]) {
      Qprintf1("simos: disk request %d already in progress. CAM_BUSY \n",sws->targid);
      return CAM_BUSY;
   }
 
   /*
    * Allocate a job
    */
   SIMOS_GET_JOB(simos_softc, job);
   if( job == (struct simos_job *)NULL ) {
      simos_softc->state &= ~CMD_RING_LOCK;
      SIMOS_UNLOCK( simos_softc->chiplock, s );
      return( CAM_BUSY );
   }
   SIMOS_SETUP_JOB( job, sws, lbolt );
   simos_softc->activeJobs[sws->targid] = job;

   entry = simos_setup_io( simos_softc, sws, job );

#ifdef gone
   if( entry < 0 ) {
      simos_softc->setup_failures++;
      simos_softc->state &= ~CMD_RING_LOCK;
      SIMOS_UNLOCK( simos_softc->chiplock, s );
      SIMOS_RELEASE_JOB( simos_softc, job );
      return( CAM_BUSY );
   }
#endif
   

   /* 
    * This should return CAM_REQ_INPROG, however the sched layer
    * requires the status to be CAM_REQ_CMP; else the request will
    * get rescheduled. - bgn
    */
   simos_softc->state &= ~CMD_RING_LOCK;
   SIMOS_UNLOCK( simos_softc->chiplock, s );
   return( CAM_REQ_CMP );
}

/* ---------------------------------------------------------------------- *
 *                                                                        *
 * ROUTINE NAME: simos_setup_io()                                           *
 *                                                                        *
 * FUNCTIONAL DESCRIPTION:                                                *
 *      This routine allocations system dma and isp resources             *
 *                                                                        *
 *      isp1020 Queue Protocol:        *
 *           *
 *      The firmware maintains two queues, the request queue and the      *
 *      response queue.  The queues are organized as circular, fixed-     *
 *      length lists of 64-byte entries (FIFOs).  Each queue has an       *
 *      in-pointer and an out-pointer, which are implemented as indexes.  *
 *      Because each entry in the queue is a fixed length, a pointer can  *
 *      be converted to an address by the following formula:     *
 *           *
 *     ENTRY ADDRESS = QUEUE BASE ADDRESS + POINTER_VALUE * 64       *
 *           *
 *      The maximum pointer (index) value is one less than the queue   *
 *      entry length.  For Example, if the queue length is 128 entries,   *
 *      the pointer values are in the range 0-127.  The queue status      *
 *      (empty, not full, or full) is determined by comparing the two   *
 *      pointers.  When the in-pointer and out-pointer are equal, the     *
 *      queue is empty.  When the in-pointer is one less than the   *
 *      out-pointer (taking into account a wraparound condition), the     *
 *      queue is full.  A full queue contains one entry less than the     *
 *      length of the queue.  The number of empty queue entries is        *
 *      calculated by subtracting the in-pointer from the out-pointer     *
 *      (taking into account wraparound condition).      *
 *           *
 *      DEC OSF/1 Notes:        *
 *           *
 *      Until the contiguous memory allocator is available we will only   *
 *      allocate 128 entries, or 1 8192 byte page for each ISP1020.   *
 *           *
 * CALLED BY:  simos_go                                                     *
 *                                                                        *
 * FORMAL PARAMETERS:                                                     *
 *                                                                        *
 * IMPLICIT INPUTS:                                                       *
 *                                                                        *
 * IMPLICIT OUTPUTS:                                                      *
 *                                                                        *
 * RETURN VALUE:                                                          *
 *                                                                        *
 * SIDE EFFECTS:                                                          *
 *                                                                        *
 * SMP:           *
 * This routine must be called with the isp locked     *
 * ---------------------------------------------------------------------- */

int
simos_setup_io( SIMOS_SOFTC *simos_softc, SIM_WS *sws, SIMOS_JOB *job ) 
{
    CCB_SCSIIO * ccb = sws->ccb;
    U8 *   cam_cdb;
    int   free_entries = 0;
    int   i, s;
    COMMAND_ENTRY *  ce;
    int   saved_daip;
    int   retval;

    SIM_MODULE( simos_setup_io );

    Qprintf2( "simos%d: %s\n", simos_softc->simos_bus->bus_num, module );


    ce = &simos_softc->request_queue->entry[ DAIP++ ];
    if( DAIP == simos_softc->request_info.depth )
       DAIP = 0;

    /* 
     * Setup queue header
     */
    ce->header.entry_type = 0x1;
    ce->header.entry_count = 0x1;
    ce->header.system_defined_1 = 0x0;
    ce->header.flags = 0x0;

    /* 
     * get a unique 32 bit handle and save it in
     * the command queue entry and the isp job struct
     */
    ce->handle = job->handle = simos_softc->next_handle++;

    Qprintf2( "simos%d: handle = 0x%x",
       simos_softc->simos_bus->bus_num, ce->handle );
    Qprintf2( " targ=%d lun=%d\n",
       sws->targid, sws->lun );

    ce->lun = sws->lun;
    ce->target =  sws->targid;
    ce->cdb_length = ccb->cam_cdb_len; 
    ce->timeout = 0;  
    ce->data_segment_count = 0;
    ce->control_flags = 0;

    /*
     * If CAM_INITIATE_SYNC or CAM_DIS_SYNC are set for this
     * command, also set the firmware flag to initiate a wide
     * negotiation (BIT11) if the NVRAM parameters allow wide
     * transfers to this target.
     *
     * Note that this fix requires firmware rev 2.0 or greater
     * (otherwise the flag is reserved).
     */

    if( sws->cam_flags & CAM_INITIATE_SYNC )
        ce->control_flags |= BIT10;

    if( sws->cam_flags & CAM_DIS_SYNC )
        ce->control_flags |= BIT9;

    if( ce->control_flags & (BIT9 | BIT10) &&
 (simos_softc->nvram.target[sws->targid].wide_data_transfers) )
     ce->control_flags |= BIT11;

    if( sws->cam_flags & CAM_DIS_AUTOSENSE )
 ce->control_flags |= BIT8;

    if( sws->cam_flags & CAM_DIR_OUT )
        ce->control_flags |= BIT6;

    if( sws->cam_flags & CAM_DIR_IN )
        ce->control_flags |= BIT5;

    /* BIT4 is the target routine bit */

    if( (sws->flags & SZ_TAGGED) &&
 (simos_softc->nvram.target[sws->targid].tagged_queuing ) ) {
        if( ccb->cam_tag_action == CAM_SIMPLE_QTAG ) {
     ce->control_flags |= BIT3;
 } else if( ccb->cam_tag_action == CAM_ORDERED_QTAG ) {
     ce->control_flags |= BIT2;
 } else if( ccb->cam_tag_action == CAM_HEAD_QTAG ) {
     ce->control_flags |= BIT1;
 }
    }

    /*
     * If disconnects are disabled, also disable tagged queueing.
     */

    if( (sws->cam_flags & CAM_DIS_DISCONNECT) ||
       !(simos_softc->nvram.target[sws->targid].disconnect_allowed) ) {
 ce->control_flags |= BIT0;
 ce->control_flags &= ~(BIT1 | BIT2 | BIT3 );
    }

    Qprintf2( "simos%d: Control flags = 0x%x\n",
       simos_softc->simos_bus->bus_num, ce->control_flags );

    if( ccb->cam_ch.cam_flags & CAM_CDB_POINTER )
 cam_cdb = ccb->cam_cdb_io.cam_cdb_ptr;
    else
 cam_cdb = ccb->cam_cdb_io.cam_cdb_bytes;

    for( i = 0; i < 12; i++ )
        ce->cdb[i] = *cam_cdb++;

    

    simos_convert_request(ce,ccb);
    /*
     * This is so upper layers can get the resid
     */
    sws->data_xfer.data_count = ccb->cam_dxfer_len;

#ifdef gone
    /*
     * Add the job to the queue
     */
    insque( (void *)job, (void *)&simos_softc->jobs);
    
#endif
    return( DAIP );

}


int simos_convert_request( COMMAND_ENTRY *ce, CCB_SCSIIO *ccb)
{
   int i;
   int sgLen = ccb->cam_sglist_cnt;
   SimOS_SCSI *reg = SIMOS_SCSI_REGS;
   SG_ELEM *sg_element = (SG_ELEM *)ccb->cam_data_ptr;      
   
   reg->target = ce->target;
   reg->lun = ce->lun;
   reg->length = ce->cdb_length;
   for (i=0;i<12;i++) {
      reg->cmd[i] = ce->cdb[i];
   }
   /*
    * ignore the 32bit reverse TLB crap for now. 
    */
   if (ccb->cam_ch.cam_flags & CAM_SCATTER_VALID) {
      reg->sgLen = sgLen;
      panic("simos_convert_request: want to use S/G ");
      for (i=0;i<sgLen;i++) {
	 reg->sgMap[i].pAddr = 
	    (vm_offset_t) sg_element[i].cam_sg_address;
	 reg->sgMap[i].len = sg_element[i].cam_sg_count;
      }
   } else  if (ccb->cam_data_ptr) {
      struct proc *	dma_procp = 0;
      int todo = ccb->cam_dxfer_len;
      vm_offset_t vAddr = (vm_offset_t) ccb->cam_data_ptr; 
      vm_offset_t 	phys_addr;
   
      if (CAM_IS_KUSEG(ccb->cam_data_ptr)) {
	 dma_procp = ((struct buf *)ccb->cam_req_map)->b_proc;
      }
      sgLen = 0;
      while (todo) {
	 int len = page_size - (vAddr %page_size); 
	 if (len > todo) len = todo;
	 if (dma_procp) {
	    phys_addr = pmap_extract(proc_to_task(dma_procp)->map->vm_pmap,vAddr);
	 } else	{	/* 0 == kernel */
	    svatophys(vAddr,&phys_addr);
	 }
	 reg->sgMap[sgLen].pAddr = phys_addr;
	 reg->sgMap[sgLen].len   = len;
	 if (simos_debug>1) { 
	    printf("simos_convert sg=%d vAddr=0x%lx pAddr=0x%lx len=0x%x\n",
		   sgLen,vAddr,phys_addr,len);
	 }
	 todo -= len;
	 vAddr +=len;
	 sgLen++;
      }
      reg->sgLen = sgLen;
   } else {
      reg->sgLen = 0;
   }
      
   
   reg->startIO = 1; /* start */
}



U32
simos_cam_bus_reset( SIM_SOFTC * sim_softc )
{
    SIMOS_SOFTC *simos_softc = sim_softc->hba_sc;
    int s;
    SIM_MODULE( simos_cam_bus_reset );
    return( CAM_REQ_CMP );

}

U32
simos_cam_bus_reset_tmo( SIM_SOFTC * sim_softc )
{

    SIMOS_SOFTC * simos_softc = sim_softc->hba_sc;
    int i, s;
    int no_reinit = 0;
    int status;

    SIM_MODULE( simos_cam_bus_reset_tmo );

    return( CAM_REQ_CMP );
}


void
simos_bus_reset_detected( SIMOS_SOFTC * simos_softc )
{
    SIM_SOFTC * sim_softc = simos_softc->sim_softc;
    int s;

    SIM_MODULE( simos_bus_reset_detected );

}


void
simos_invalidate_all_jobs( SIMOS_SOFTC * simos_softc )
{
    SIMOS_JOB *job;

    job = simos_softc->jobs.flink;
    while( job != (SIMOS_JOB *)&simos_softc->jobs ) {
 job->flags |= JOB_INVALIDATED;
        job = job->flink;
    }

}

void
simos_invalidate_jobs( SIMOS_SOFTC *simos_softc, int targid, int lun )
{
    SIMOS_JOB *job;

    job = simos_softc->jobs.flink;
    while( job != (SIMOS_JOB *)&simos_softc->jobs ) {
 if( (job->sws->targid == targid) && (job->sws->lun == lun) ) {
     job->flags |= JOB_INVALIDATED;
 }
        job = job->flink;
    }

}

/* ---------------------------------------------------------------------- *
 *                                                                        *
 * ROUTINE NAME: simos_process_done                                    *
 *                                                                        *
 * FUNCTIONAL DESCRIPTION:                                                *
 *           *
 *     This routine is called via timeout to complete IO processing       *
 *     for response queue interrupts received during Abort processing.    *
 *                                                                        *
 * CALLED BY:                                                             *
 *                                                                        *
 * FORMAL PARAMETERS:                                                     *
 *           *
 *     SIMOS_JOB *job                                                       *
 *                                                                        *
 * IMPLICIT INPUTS:                                                       *
 *                                                                        *
 * IMPLICIT OUTPUTS:                                                      *
 *                                                                        *
 * RETURN VALUE:                                                          *
 *                                                                        *
 * SIDE EFFECTS:                                                          *
 *                                                                        *
 * ---------------------------------------------------------------------- */

void
simos_process_done( SIMOS_JOB *job )
{
   SIM_WS *        sws = job->sws;
   SIM_SOFTC *     sim_softc = sws->sim_sc;
   SIMOS_SOFTC *     simos_softc = (SIMOS_SOFTC *)(sim_softc->hba_sc);
   int s;

   SIM_MODULE( simos_process_done );

   SIMOS_LOCK( simos_softc->chiplock, s );

   /*
    * First perform locking check to prevent lock timeout
    * on SIM_SOFTC_LOCK.
    */
   if( simos_softc->state & ABORT_IN_PROG ){
      untimeout( simos_process_done, job );
      timeout( simos_process_done, job, hz/60 );
      SIMOS_UNLOCK( simos_softc->chiplock, s );
      return;
   }

   SIMOS_UNLOCK( simos_softc->chiplock, s );

   /*
    * Call sm_bus_free() to finally make the call to
    * ss_abort_done().
    */

   sws->flags |= SZ_EXP_BUS_FREE;

   sm_bus_free( sws );

   /* 
    * Dealloc DMA resources and free the job
    */
   simos_free_handles( simos_softc, job );

   /*
    * Release the job memory back to the zone
    * or cache it.
    */
   SIMOS_RELEASE_JOB( simos_softc, job );
}


void simosintr(void)
{
   panic("simosintr \n");
}



/* ---------------------------------------------------------------------- */
/*              THREADS, INTERRUPT HANDLING      */
/* ---------------------------------------------------------------------- */

/* ---------------------------------------------------------------------- *
 *                                                                        *
 * ROUTINE NAME: simos_intr()                                           *
 *                                                                        *
 * FUNCTIONAL DESCRIPTION:                                                *
 *      This routine will process an interrupt from the isp1020    *
 *           *
 * The firmware issues an interrupt to request service from the   *
 * host whenever a mailbox command is completed, the request queue   *
 * has been updated, or an asynchronous event has occurred.  The     *
 * following procedure describes how an interrupt is processed by    *
 * the host:         *
 *           *
 *     1. The host responds to the firmware service request by   *
 *        either a hardware interrupt (preferred) or by polling   *
 *        the PCI Interface Status register RISC Interrupt bit   *
 *        (bit 2).         *
 *           *
 *     2. The host reads Incoming Mailbox 5 register, saves the   *
 *        data for processing, then clears the interrupt request     *
 *        by writing a Clear RISC Interrupt command to the HCCR.     *
 *           *
 *     3. The host determintes if this interrupt is the result of   *
 *        a mailbox command completion or an asynchronous event   *
 *        by polling the Semaphore Lock Bit (bit 0) in the PCI   *
 *        Semaphore register.  If the Semaphore Lock Bit is zero,   *
 *        the host processes the interrupt as a response queue   *
 *        update interrupt and skips to step 5.      *
 *           *
 *     4. The host saves and/or processes the infomation in    *
 *        Outgoing Mailbox 0-4 registers before clearing the    *
 *        Semaphore Lock Bit in the PCI Semaphore register.  The   *
 *        Semaphore Lock Bit is cleared by writing zero to bit 0   *
 *             of the PCI Semaphore register.      *
 *            *
 *     5. The host processes the data from Outgoing Mailbox 5   *
 *        register, the response queue in-pointer.     *
 *                                                                        *
 * CALLED BY:                                                             *
 *                                                                        *
 * FORMAL PARAMETERS:                                                     *
 *                                                                        *
 * IMPLICIT INPUTS:                                                       *
 *                                                                        *
 * IMPLICIT OUTPUTS:                                                      *
 *                                                                        *
 * RETURN VALUE:                                                          *
 *                                                                        *
 * SIDE EFFECTS:                                                          *
 *                                                                        *
 * SMP:           *
 *           *
 * ---------------------------------------------------------------------- */

int
simos_intr( SIMOS_SOFTC * simos_softc )
{
   U16         response_queue_ip;
   U16  semaphore, hccr;
   int  reset_needed = 0;
   int  reset_done = 0;
   int         mailbox_semaphore;
   int         simos_bus = 0;
   int         i, j,s;

   SIM_MODULE( simos_intr );

   /*
    * check out the isp softc
    */
   if( simos_softc == (SIMOS_SOFTC *)NULL ) {
      Qprintf1( "%s: Interrupt received prior to simos controller initialization!\n",
                module );
#ifdef INTR_NOT_SERVICED
      return( INTR_NOT_SERVICED );
#else
      return;
#endif
   }

   s = splbio();
   SIMOS_LOCK_FAST( simos_softc->chiplock );
   if( simos_softc->simos_bus )
      simos_bus = simos_softc->simos_bus->bus_num;

   Qprintf2( "simos%d: %s\n", simos_bus, module );

   for( i=0;i<SIMOS_MAX_TARGETS;i++) {
      SimOS_SCSI *reg = SIMOS_SCSI_REGS;
      if (!simos_softc->activeJobs[i]) continue;
      if (reg->done[i]) { 
	 reg->done[i] = 0;
         simos_process_response_queue(simos_softc,i);
      }
   }

   SIMOS_UNLOCK_FAST( simos_softc->chiplock );

#ifdef INTR_SERVICED
   return( INTR_SERVICED );
#else
   return;
#endif

}

void
simos_process_response_queue( SIMOS_SOFTC *simos_softc, int targid )
{
   SIM_SOFTC *         sim_softc = simos_softc->sim_softc;
   SIM_WS *   sws;
   SIMOS_JOB *  job;
   int   done = 0;
   int                 serviced = 0;
   int                 reset_needed = 0;
   int                 simos_bus = simos_softc->simos_bus->bus_num;
   vm_offset_t  sense_target;

   SIM_MODULE( simos_process_response_queue );

   job = simos_softc->activeJobs[targid];
   if (!job) {
      panic("simos_process_response_queue: NULL \n");
   }
   simos_softc->activeJobs[targid] = 0;
#ifdef gone
   remque( job );
#endif
   sws = job->sws;
      
   /*
    * If the job is not on the job list, the IO has already
    * (probably) been returned. If we keep going, the chip
    * tends to hang. Reset it...
    */
   
   Qprintf2("%s: sws = 0x%x\n", module, sws);
   
   if( job->flags & JOB_INVALIDATED ) {
      Qprintf1( "simos%d: Job invalidated - let the job go...\n", simos_bus);
      SIMOS_UNLOCK_FAST( simos_softc->chiplock );
      simos_free_handles( simos_softc, job );
      SIMOS_RELEASE_JOB_FAST( simos_softc, job );
      SIMOS_LOCK_FAST( simos_softc->chiplock );
      /* se->header.entry_type = 0; */
      return;
   }
   
   serviced++;
   
   sws->scsi_status = 0; /* bogus bugnion */
   
   /* 
    * Update # of bytes transferred
    */
   sws->data_xfer.xfer_count = sws->ccb->cam_dxfer_len;
   
   /*
    * Check completion status, handle errors
    */
   sws->flags |= SZ_CMD_CMPLT;
   
   Qprintf1( "simos%d: No transport errors, command complete\n",
             simos_bus );

   sws->cam_status |= CAM_REQ_CMP;
   sws->flags |= SZ_EXP_BUS_FREE;


   /* 
    * update our stats while still locked
    */
   if( !simos_softc->reset_stats ) {
      simos_softc->num_io_completed++;
         simos_softc->lbolt_total += lbolt - job->lbolt;
   } else {
      simos_softc->reset_stats = 0;
      simos_softc->num_io_completed = 0;
      simos_softc->lbolt_total = 0;
         simos_softc->memory_allocations = 0;
         simos_softc->interrupts = 0;
   }
   
   /*
    * Return the sws, must unlock since we can be called back with the
    * next IO.
    * 
    * If this status entry was received during Abort processing, we will
    * return the sws from another thread (simos_process_done() called via
    * timeout). Otherwise, one of two possible problems can occur:
    *
    *    o We can get so many Aborts piled up by the call to ss_sched()
    *      from sm_bus_free() that we'll get a Kernel Stack Not Valid Halt.
    *    o We will get a lock timeout on the SIM_SOFTC_LOCK currently
    *      held by the caller (ss_perform_abort()).
    */
   
   SIMOS_UNLOCK_FAST( simos_softc->chiplock );
   
   sm_bus_free( sws );
   
   /*
    * Release the job memory back to the zone
    * or cache it.
    */
   SIMOS_RELEASE_JOB_FAST(simos_softc, job);
   
   SIMOS_LOCK_FAST( simos_softc->chiplock );

}


U32
simos_thread( )
{
    SIM_MODULE( simos_thread );
}
U32
simos_chip_init( SIMOS_SOFTC *simos_softc )
{
    SIM_MODULE( simos_chip_init );

    Qprintf2( "simos%d: %s\n", simos_softc->simos_bus->bus_num, module );

    Qprintf1( "      simos_softc->nvram.fifo_threshold = 0x%x\n",
       simos_softc->nvram.fifo_threshold );

}

/* ---------------------------------------------------------------------- *
 *                                                                        *
 * ROUTINE NAME: simos_init                                                 *
 *                                                                        *
 * FUNCTIONAL DESCRIPTION: Common probe init/reinit of isp1020    *
 *                                                                        *
 * CALLED BY: simos_probe, simos_reinit                                       *
 *                                                                        *
 * FORMAL PARAMETERS:                                                     *
 *                                                                        *
 * IMPLICIT INPUTS:                                                       *
 *                                                                        *
 * IMPLICIT OUTPUTS:                                                      *
 *                                                                        *
 * RETURN VALUE:                                                          *
 *                                                                        *
 * SIDE EFFECTS:                                                          *
 *                                                                        *
 * ---------------------------------------------------------------------- */

int
simos_init( SIMOS_SOFTC * simos_softc)
{
   SIM_SOFTC * sim_softc = simos_softc->sim_softc;
   int  firmware_major, firmware_minor; /* ident of firmware */
   int  firmware_loaded = 0;
   int  i,j;

   SIM_MODULE( simos_init );

   Qprintf2( "simos%d: %s\n", simos_softc->simos_bus->bus_num, module );

   /*
    * verify we're alive
    */
   if( simos_no_operation( simos_softc ) != SIMOS_SUCCESS ) {
      printf( "simos%d: Failed to start in %s\n",
              simos_softc->simos_bus->bus_num, module );
      return( SIMOS_FAILURE );
   }


   /*
    * free memory list
    */
   simos_softc->free_memory.flink = (SIMOS_FREE_MEMORY *)&simos_softc->free_memory;
   simos_softc->free_memory.blink = (SIMOS_FREE_MEMORY *)&simos_softc->free_memory;

   /*
    * start the queues
    */
   simos_softc->jobs.flink = (SIMOS_JOB *)&simos_softc->jobs;
   simos_softc->jobs.blink = (SIMOS_JOB *)&simos_softc->jobs;
   simos_softc->next_handle = 123; /* any non-zero seed is fine */

   bzero( simos_softc->request_queue, sizeof(SIMOS_QUEUE) );
   bzero( simos_softc->response_queue, sizeof(SIMOS_SQUEUE) );

   simos_softc->request_info.ip = 0;
   simos_softc->request_info.op = 0xffff;

   simos_softc->response_info.ip = 0xffff;
   simos_softc->response_info.op = 0;


   return( SIMOS_SUCCESS );
}

/* ---------------------------------------------------------------------- *
 *                                                                        *
 * ROUTINE NAME: simos_reinit                                               *
 *                                                                        *
 * FUNCTIONAL DESCRIPTION: Restart the chip after a failure    *
 *                                                                        *
 * CALLED BY:                                                             *
 *                                                                        *
 * FORMAL PARAMETERS:                                                     *
 *                                                                        *
 * IMPLICIT INPUTS:                                                       *
 *                                                                        *
 * IMPLICIT OUTPUTS:                                                      *
 *                                                                        *
 * RETURN VALUE:                                                          *
 *                                                                        *
 * SIDE EFFECTS:                                                          *
 *            *
 * ---------------------------------------------------------------------- */

int
simos_reinit( SIMOS_SOFTC * simos_softc)
{
    SIM_SOFTC *sim_softc = simos_softc->sim_softc;
    SIMOS_JOB_Q deadjobs;
    SIMOS_JOB *job, *nextjob;
    SIM_WS  *sws;
    int  s,i;

    SIM_MODULE( simos_reinit );

    Qprintf2( "simos%d: %s\n", simos_softc->simos_bus->bus_num, module );

    /*
     * stop any new I/O operations from starting
     */
    SIM_SOFTC_LOCK( s, sim_softc );
    sim_softc->error_recovery |= ERR_BUS_RESET;
    SIM_SOFTC_UNLOCK(s, sim_softc);

    /*
     * lock the isp and check to see if reset is already in
     * progress.
     */
    SIMOS_LOCK( simos_softc->chiplock, s );
    if( simos_softc->state & CHIP_RESET_IN_PROG ) {
        SIMOS_UNLOCK( simos_softc->chiplock, s );
        return( SIMOS_SUCCESS );
    }

    /*
     * If we have attempted to re-init the chip more than
     * 10 times without success, panic the system.
     */
    if( simos_softc->reinits++ > 10 )
 panic( "simos_reinit: Unable to bring Qlogic chip back online!" );

    /*
     * dump registers on the console
     */
    if( !simos_no_printf ) {
 printf( "simos%d: Restarting\n", simos_softc->simos_bus->bus_num );

#if 0
 simos_dumpreg( simos_softc );
#endif
    }

    /*
     * save the current job list
     */
    deadjobs = simos_softc->jobs;

    /* 
     * soft reset the chip
     */
    simos_softc->state |= CHIP_RESET_IN_PROG;
    simos_softc->mailbox_semaphore = 0;
    simos_softc->mailbox_status = SIMOS_BUSY;
    simos_chip_init( simos_softc );

    SIMOS_UNLOCK( simos_softc->chiplock, s );

    /* 
     * what if an interrupt occurs right here, will the job be
     * returned twice? - bgn
     */

    /*
     * Call common init code
     */
    if( simos_init( simos_softc ) != SIMOS_SUCCESS )
 return( SIMOS_FAILURE );

    /*
     * Fail all current I/O with bus reset status
     * HW4, BL3 change: don't return the I/O, just release the 
     * held resources
     */
    job = deadjobs.flink;
    while( job && (job != (SIMOS_JOB *)&simos_softc->jobs) ) {
 nextjob = job->flink;
        remque( job );
#if 0
 simos_free_handles( simos_softc, job );
#endif
 SIMOS_RELEASE_JOB( simos_softc, job );
 job = nextjob;
    }

    /*
     * Allow I/O to continue
     * 
     * Turn on error recovery bit so no I/O  will come back to
     * simos_go before the 250 ms timeout occurs.  Without this it is
     * possible for I/O to start in the isp and then have the sim xpt
     * layer return it to the caller while it is active in the isp.
     * 
     * Note: the error recovery flag is turned off for us
     */ 
    ss_reset_detected( sim_softc );

    SIMOS_LOCK( simos_softc->chiplock, s );
    simos_softc->state &= ~CHIP_RESET_IN_PROG;
    SIMOS_UNLOCK( simos_softc->chiplock, s );

    return( SIMOS_SUCCESS );
}

U32
simos_mailboxissue( SIMOS_SOFTC *simos_softc, SIMOS_MAIL *mail)
{
  
   panic("simos_mailboxissue\n");
   return SIMOS_SUCCESS;

}


int
simos_mailboxcomplete( SIMOS_SOFTC *simos_softc, SIMOS_MAIL *mail, int s )
{
  
   panic("simos_mailboxcomplete\n");
   return SIMOS_SUCCESS;
}

int
simos_poll_mailboxcomplete( SIMOS_SOFTC *simos_softc )
{
    panic("simos_poll_mailboxcomplete\n");
   return SIMOS_SUCCESS;
}


int
simos_no_operation( SIMOS_SOFTC *simos_softc )
{
   panic("simos_no_operation\n");
}


int
simos_load_ram( SIM_SOFTC *sim_softc )
{
   panic("simos_load_ram\n");

}
int
simos_verify_checksum( SIMOS_SOFTC *simos_softc, int *checksum )
{
    int         status = 0;
    return( status );
}


int
simos_about_firmware( SIM_SOFTC *sim_softc, int *major, int *minor )
{
   panic("simos_about_firmware \n");
   return SIMOS_SUCCESS;
}



int
simos_checksum_firmware( SIMOS_SOFTC *simos_softc, int *checksum )
{
    int         status = 0;
    return( status );
}



int
simos_wake_up( SIM_SOFTC *sim_softc )
{
   panic("simos_wake_up\n");
   return SIMOS_SUCCESS;
}



int
simos_stop_queue( SIM_SOFTC *sim_softc, unsigned int targid, unsigned int lun )
{
    SIMOS_SOFTC   *simos_softc = sim_softc->hba_sc;
    int  status = 0;
    SIMOS_MAIL *mail;
    int         i;

    SIM_MODULE( simos_stop_queue );

    status = SIMOS_SUCCESS;

    return( status );
}



int
simos_start_queue( SIM_SOFTC *sim_softc, unsigned int targid, unsigned int lun )
{
    SIMOS_SOFTC   *simos_softc = sim_softc->hba_sc;
    int  status = 0;
    SIMOS_MAIL *mail;
    int         i = 0;

    SIM_MODULE( simos_start_queue );
    return( status );
}


int
simos_single_step_queue( SIM_SOFTC *sim_softc )
{
    SIMOS_SOFTC   *simos_softc = sim_softc->hba_sc;
    int  status = 0;
    SIMOS_MAIL *mail;


    return( status );
}


int
simos_abort_queue( SIM_SOFTC *sim_softc )
{
    SIMOS_SOFTC   *simos_softc = sim_softc->hba_sc;
    int  status = 0;
    SIMOS_MAIL *mail;

    return( status );
}



int
simos_get_device_queue_status( SIMOS_SOFTC *simos_softc, U16 target, U16 lun)
{
    int  status = 0;

    return( status );
}


int
simos_get_retry_count( SIM_SOFTC *sim_softc )
{
    SIMOS_SOFTC   *simos_softc = sim_softc->hba_sc;
    int  status = 0;

    return( status );
}


int
simos_get_tag_age_limit( SIM_SOFTC *sim_softc )
{
    SIMOS_SOFTC   *simos_softc = sim_softc->hba_sc;
    int  status = 0;

    return( status );
}


int
simos_get_clock_rate( SIM_SOFTC *sim_softc )
{
    SIMOS_SOFTC   *simos_softc = sim_softc->hba_sc;
    int  status = 0;

    return( status );
}


int
simos_get_active_negation_state( SIM_SOFTC *sim_softc )
{
    SIMOS_SOFTC   *simos_softc = sim_softc->hba_sc;
    int  status = 0;
    return( status );
}

int
simos_get_asynchronous_data_setup_time( SIM_SOFTC *sim_softc )
{
   panic("simos_get_asynchronous_data_setup_time");
   return SIMOS_SUCCESS;
}

int
simos_get_pci_control_parameters( SIM_SOFTC *sim_softc )
{
   panic("simos_get_pci_control_parameters");
   return SIMOS_SUCCESS;

}



/* ---------------------------------------------------------------------- */
/*              SUBROUTINES        */
/* ---------------------------------------------------------------------- */

/* ---------------------------------------------------------------------- *
 *                                                                        *
 * ROUTINE NAME: simos_null                                                 *
 *                                                                        *
 * FUNCTIONAL DESCRIPTION:                                                *
 *                                                                        *
 * CALLED BY:                                                             *
 *                                                                        *
 * FORMAL PARAMETERS: SIMOS_SOFTC *simos_softc                                *
 *                                                                        *
 * IMPLICIT INPUTS:                                                       *
 *                                                                        *
 * IMPLICIT OUTPUTS:                                                      *
 *                                                                        *
 * RETURN VALUE:                                                          *
 *                                                                        *
 * SIDE EFFECTS:                                                          *
 *                                                                        *
 * ---------------------------------------------------------------------- */

U32 
simos_null( SIMOS_SOFTC *simos_softc )
{

    SIM_MODULE( simos_null );

    Qprintf2( "simos%d: %s called\n", simos_softc->simos_bus->bus_num, module );
}

/* ---------------------------------------------------------------------- *
 *                                                                        *
 * ROUTINE NAME: simos_get_free_entries                                    *
 *                                                                        *
 * FUNCTIONAL DESCRIPTION:                                                *
 *                                                                        *
 * CALLED BY:                                                             *
 *                                                                        *
 * FORMAL PARAMETERS:                                                     *
 *                                                                        *
 * IMPLICIT INPUTS:                                                       *
 *                                                                        *
 * IMPLICIT OUTPUTS:                                                      *
 *                                                                        *
 * RETURN VALUE:                                                          *
 *                                                                        *
 * SIDE EFFECTS:                                                          *
 *                                                                        *
 * SMP: This must be called with the isp locked!     *
 *                                                                        *
 * ---------------------------------------------------------------------- */

int
simos_get_free_entries_count( SIMOS_SOFTC * simos_softc )
{
    int free_entries;
    int i;

    SIM_MODULE( simos_get_free_entries_count );

    /*
     * Check for the number of free entries, don't
     * perform the [expensive] read operation if we
     * don't really need to
     */
    for( i = 0; i < 2; i++ ) {
        if( i )
            DAOP = SIMOS_RCSR16( simos_softc->simos_reg.mbox_out[4] );

        if( DAIP == DAOP )
            free_entries = simos_softc->request_info.depth;
        else if( DAIP < DAOP)
            free_entries = DAOP - DAIP;
        else if( DAIP > DAOP)
            free_entries = simos_softc->request_info.depth + DAOP - DAIP;

        if( free_entries > 10 )
            break;
    }

    return( free_entries );
}

/* ---------------------------------------------------------------------- *
 *                                                                        *
 * ROUTINE NAME: simos_printinfo                                            *
 *                                                                        *
 * FUNCTIONAL DESCRIPTION:                                                *
 *                                                                        *
 * CALLED BY:                                                             *
 *                                                                        *
 * FORMAL PARAMETERS:                                                     *
 *                                                                        *
 * IMPLICIT INPUTS:                                                       *
 *                                                                        *
 * IMPLICIT OUTPUTS:                                                      *
 *                                                                        *
 * RETURN VALUE:                                                          *
 *                                                                        *
 * SIDE EFFECTS:                                                          *
 *                                                                        *
 * ---------------------------------------------------------------------- */

U32
simos_printinfo( pci_cnfg_hdr )
struct pci_config_hdr *pci_cnfg_hdr; 

{
    SIM_MODULE( simos_printinfo );

    Qprintf1( "vendor_id:         0x%8x\n", pci_cnfg_hdr->vendor_id );
    Qprintf1( "device_id:         0x%8x\n", pci_cnfg_hdr->device_id );
    Qprintf1( "command:           0x%8x\n", pci_cnfg_hdr->command );
    Qprintf1( "status:            0x%8x\n", pci_cnfg_hdr->status );
    Qprintf1( "rev_id:            0x%8x\n", pci_cnfg_hdr->rev_id );
    Qprintf1( "bar0:              0x%8x\n", pci_cnfg_hdr->bar0 );
    Qprintf1( "bar1:              0x%8x\n", pci_cnfg_hdr->bar1 );
    Qprintf1( "bar2:              0x%8x\n", pci_cnfg_hdr->bar2 );
    Qprintf1( "bar3:              0x%8x\n", pci_cnfg_hdr->bar3 );
    Qprintf1( "bar4:              0x%8x\n", pci_cnfg_hdr->bar4 );
    Qprintf1( "bar5:              0x%8x\n", pci_cnfg_hdr->bar5 );
    Qprintf1( "exp_rom_bar:       0x%8x\n", pci_cnfg_hdr->exp_rom_bar );
    Qprintf1( "config_base:       0x%8x\n", pci_cnfg_hdr->config_base );

    return( 0 );
}


/* ---------------------------------------------------------------------- *
 *                                                                        *
 * ROUTINE NAME: simos_logger                                    *
 *                                                                        *
 * FUNCTIONAL DESCRIPTION:                                                *
 *                                                                        *
 * CALLED BY:                                                             *
 *                                                                        *
 * FORMAL PARAMETERS:                                                     *
 *                                                                        *
 * IMPLICIT INPUTS:                                                       *
 *                                                                        *
 * IMPLICIT OUTPUTS:                                                      *
 *                                                                        *
 * RETURN VALUE:                                                          *
 *                                                                        *
 * SIDE EFFECTS:                                                          *
 *                                                                        *
 * ---------------------------------------------------------------------- */

void
simos_logger( u_char *func, u_char *msg, U32 flags, SIM_SOFTC *sim_softc,
           SIMOS_SOFTC *simos_softc )
{
    CAM_ERR_HDR hdr;
    static CAM_ERR_ENTRY entrys[SIM_LOG_SIZE];
    CAM_ERR_ENTRY *entry;
    char bus = -1, pun = -1, lun = -1;

    SIM_MODULE( simos_logger );

    hdr.hdr_type = CAM_ERR_PKT;
    hdr.hdr_class = 0;
    hdr.hdr_subsystem = 0;
    hdr.hdr_size = 0;
    hdr.hdr_entries = 0;
    hdr.hdr_list = entrys;
    if (flags & SIM_LOG_PRISEVERE)
        hdr.hdr_pri = EL_PRISEVERE;

    else if (flags & SIM_LOG_PRIHIGH)
        hdr.hdr_pri = EL_PRIHIGH;
    else
        hdr.hdr_pri = EL_PRILOW;

    /*
     * Setup pointers
     */
    if( sim_softc == (SIM_SOFTC *)NULL ) {
        if (simos_softc != (SIMOS_SOFTC *)NULL)
            sim_softc = simos_softc->sim_softc;
    }
 
    /*
     * Setup the bus #
     */
    bus = sim_softc->cntlr;

    /*
     * Log the module name.
     */
    if (func != (u_char *)NULL) {
        entry = &hdr.hdr_list[hdr.hdr_entries++];
        entry->ent_type = ENT_STR_MODULE;
        entry->ent_size = strlen(func) + 1;
        entry->ent_vers = 0;
        entry->ent_data = func;
        entry->ent_pri = PRI_BRIEF_REPORT;
    }

    /*
     * Log the message.
     */
    if (msg != (u_char *)NULL) {
        entry = &hdr.hdr_list[hdr.hdr_entries++];
        entry->ent_type = ENT_STRING;
        entry->ent_size = strlen(msg) + 1;
        entry->ent_vers = 0;
        entry->ent_data = msg;
        entry->ent_pri = PRI_BRIEF_REPORT;
    }

    /*
     * Call the CAM error logger.
     */
    if (!(flags & SIM_LOG_NOLOG)) {
        cam_logger(&hdr, bus, pun, lun);
    }

}

U32 simos_termio_abort_bdr(void)
{
   panic("simos_termio_abord_bdr\n");
}

void   simos_free_handles( SIMOS_SOFTC *simos_softc, SIMOS_JOB * job )
{
   Qprintf("simos_free_handles called \n");
}

/* end of isp1020.c */
