/*
 * @DEC_COPYRIGHT@
 */
/*
 * HISTORY
 * $Log: if_tu.c,v $
 * 
 * $EndLog$
 */
#pragma ident "@(#)$RCSfile: if_tu.c,v $ $Revision: 1.1.19.11 $ (DEC) $Date: 1996/03/06 19:17:07 $"

/* 
 * SimOS Ethernet Network Interface (simether - se)
 * (Mostly stolen from the tulip interface)
 */ 
#include <data/if_sim_data.c> 

extern struct timeval time;

int	seprobe(), seattach(), seintr(), sereset();
int	seinit(), sestart(), seioctl(), sewatch();
static void ifsim_SetRcvBuffer(int index, struct mbuf *m, struct IfsimState *ifs);

struct driver simetherdriver = {seprobe, 0, seattach, 0, 0, 0 , 0, 0,  "simether", seinfo};

decl_simple_lock_info(, se_softc_lockinfo);

/*
 * This is used to run our "watchdog" routine on a periodic basis. The watchdog
 * routine is where we detect timeouts on transmission.
 */
#define	TU_WATCH_FREQUENCY	5	/* in seconds */

/*
 * Macro to increment x up to a maximum of m:
 */
#define INCR(x,m)	((x != (unsigned) m) ? x++ : m)

/*
 * Macro to get a 2k cluster mbuf
 */
#define TUMCLGET(m, wait_flag) {		\
      MGETHDR((m), wait_flag, MT_DATA);		\
      if ((m)) { 				\
	 MCLGET2((m), wait_flag);		\
	 if (((m)->m_flags & M_EXT) == 0) { 	\
	     m_freem((m));			\
	     (m) = (struct mbuf *)NULL; 	\
         } 					\
      } 					\
}


/*
 * Template to use for a broadcast address
 */
static u_char tu_bcast[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};

void tu_transmit_int(), tu_transmit_errors(), tu_addressRomCRC(),
     tu_select_port();

/*
 * Save some cycles: we really don't need to do unloads
 */
#define dma_map_unload


/*
 * These are "jackets" for tuinit, tureset, and tustart. They simply set ipl 
 * to splimp, and call their respective routines after locking softc. Having
 * these jackets helps because the network layer now doesn't have to worry 
 * about the spl level and locking softc before calling if_init(), if_reset(),
 * or if_start().
 */
se_init(int unit)
{
	register struct se_softc *sc = &se_softc[unit];
	int status, s = splimp();
	simple_lock(&sc->lk_se_softc);
	status = seinit(unit);
	simple_unlock(&sc->lk_se_softc);
	splx(s);
	return (status);
}

se_start(struct ifnet *ifp)
{
	register struct se_softc *sc = &se_softc[ifp->if_unit];
	int s = splimp();

	/*
	 * If we're already busy transmitting or processing completions (in
	 * the interrupt routine), there's no point in blocking the caller.
	 * The interface queue will be examined and serviced anyway.
	 */
	if (!simple_lock_try(&sc->lk_se_softc)) {
		splx(s);
		return;
	}
	sestart(ifp);
	simple_unlock(&sc->lk_se_softc);
	splx(s);
}

se_reset(int unit, int unused)
{
	register struct se_softc *sc = &se_softc[unit];
	int s = splimp();
	simple_lock(&sc->lk_se_softc);
	sereset(unit);
	simple_unlock(&sc->lk_se_softc);
	splx(s);
}

/*
 * This little jacket is used for registering a shutdown routine for Tulip.
 * All we need to  do is reset the chip.
 */
void se_shutdown(struct controller *ctlr)
{
}


/*
 * tuprobe()
 *	Initialize data in the softc structure for the driver
 *	Read the ethernet rom address and also reset the chip
 *	Allocate and initialize transmit and receive rings
 *
 * Arguments:
 *	io_handle	io-handle for this device as determined by caller.
 *	ctlr		pointer to controller struct for the interface.
 *
 * Return Value:
 *	Success:	~0
 *	Failure:	0
 */
seprobe(register io_handle_t io_handle, register struct controller *ctlr)
{	
	/*
	 * Do one-time initialization of our softc structure. In particular,
	 * setup the absolute io-handles for accessing CSRs, etc. Also get 
	 * the Tulip's revision and id.
	 */
	if (se_init_softc(io_handle, ctlr) != ESUCCESS)
		return(0);

	/*
	 * Read and save the Ethernet ROM address. The reading of this Id is 
	 * based on how the Tulip is served to us (de425, de435, pass #, etc.)
	 */
	if (se_read_enet_id(ctlr) != ESUCCESS)
		return(0);

	return(~0);
}


/*
 * se_init_softc()
 *	Initialize the driver's softc structure. In particular, setup the
 *	io-handles for accessing CSRs, and read the chip's revision, etc.
 *
 * Arguments:
 *	io_handle	io-handle for this device as passed on probe.
 *	ctlr		pointer to controller struct for the interface.
 *
 * Return Value:
 *	- ESUCCESS
 *	- ENOMEM
 *	- EIO
 */
se_init_softc(io_handle_t io_handle, struct controller *ctlr)
{
	register int unit = ctlr->ctlr_num;
	register struct se_softc *sc = &se_softc[unit];
	register struct pci_config_hdr *pci_cfg_hdr = (void*)io_handle;
	register struct ifnet *ifp = &sc->is_if;
	register struct IfsimState *ifs = &sc->ifsimState;
	register vm_offset_t csr_base;
	struct handler_intr_info se_intr_info;
	register u_int i, fasternet=0, pg3=0;
	ihandler_t se_ihandle;

	/*
	 * Initialize the multicast table and clear out the in-use flag 
	 * for each entry.
	 */
	for (i=0; i<SE_NMULTI; i++) {
		bzero(&sc->is_multi[i], 6);
		sc->is_muse[i] = 0;
	}
	sc->is_nmulti = 0;

	/*
	 * Throw away the pending data if we're resetting the interface.
	 */

	if( !ifs->regs ) { 
	    /* 
	     * Setup I/O space pointer.
	     */
	    ifs->regs = DEV_ETHER_REGISTERS;
	    /*
	     * Read configuration from controller
	     */
	    ifs->numSndChunks = ifs->regs->numSndChunks;
	    ifs->numSndEntries = ifs->regs->numSndEntries;
	    ifs->numRcvEntries = ifs->regs->numRcvEntries;
        
	    /*
	     * Initialize internal state 
	     */
	    ifs->firstFreeChunk = 0;
	    ifs->numFreeChunks =  ifs->numSndChunks;
	    ifs->rcvPtr = 0;
	    ifs->sndPtr = 0;
	    ifs->deallocPtr = 0;

 	    /* 
	     * initialize receive ring buffer 
	     */
	    for(i=0;i<ifs->numRcvEntries;i++) {
		ifsim_SetRcvBuffer(i, (struct mbuf*)0, ifs);
	    }
	}


#ifdef REAL_INTR_CODE
	    /*
	     * Register our interrupt routine. Also declare ourselves as being 
	     * capable of sharing interrupts. Note: We'll enable interrupts at
	     * the end of attach.
	     */
	    se_intr_info.configuration_st = (caddr_t)ctlr;
	    se_intr_info.intr = seintr;
	    se_intr_info.param = (caddr_t)unit;
	    se_intr_info.config_type = CONTROLLER_CONFIG_TYPE | SHARED_INTR_CAPABLE;

	    se_ihandle.ih_bus = ctlr->bus_hd;
	    se_ihandle.ih_bus_info = (char *)&se_intr_info;

	    sc->hid = handler_add(&se_ihandle);
	    if (sc->hid == (ihandler_id_t *)(NULL)) {
		printf("simether%d: interrrupt handler add failed\n", unit);
		return EIO;
	    }
#endif

	{
	    vm_offset_t vecaddr;
	    if(allocvec(1, &vecaddr) != KERN_SUCCESS)
		panic("simether: coudl not alloc vector\n");
	    printf("simos: vecaddr=0x%lx offset=0x%x \n",
		   vecaddr,vecoffset(vecaddr));
	    intrsetvec(vecoffset(vecaddr), seintr, se_softc);
	}


	/*
	 * Display the Tulip chip's revision as: step.step-revision.
	 */
	printf("simether%d:Probe complete\n", unit);
	
	return ESUCCESS;
}



/*
 * se_read_enet_id()
 *	Read the Ethernet ROM ID for the device and save it in softc.
 *
 * Arguments:
 *	ctlr		pointer to controller struct for the interface.
 *
 * Return Value:
 *	- EIO
 *	- ESUCCESS
 */
se_read_enet_id(struct controller *ctlr)
{
    register int unit = ctlr->ctlr_num;
    register struct se_softc *sc = &se_softc[unit];
    register struct IfsimState *ifs = &sc->ifsimState;
    volatile u_int value;
    int i;

    /* Need to set up the correct io space entry here */
    for (i=0; i<6; i++) {
	value = ifs->regs->etheraddr[i];
	sc->is_addr[i] = (u_char) value;
	sc->se_rom_addr[i] = (u_char) value;
    } 

    return ESUCCESS;
}



static int ifsim_GetRcvBuffer(int *len, struct IfsimState *ifs)
{
    int retval = ifs->rcvPtr;
    DevRegister x = ifs->regs->rcvEntries[ifs->rcvPtr].flag;
    if( x != DEV_ETHER_OS_OWNED ) return -1;
    *len = ifs->regs->rcvEntries[ifs->rcvPtr].len;
    
    ifs->rcvPtr++;
    if( ifs->rcvPtr >= ifs->numRcvEntries ) {
        ifs->rcvPtr -= ifs->numRcvEntries;
    }
    return retval;
}

static void ifsim_SetRcvBuffer(int index, struct mbuf *m, struct IfsimState *ifs)
{
    char *start;
    vm_offset_t va, pAddr;

    if( !m ) { 
	/*
	 * Attempt to get a 2k cluster mbuf: wait if necessary
	 */
	TUMCLGET(m, M_WAIT);
/*        n = m_vget(M_DONTWAIT, sizeof(struct rcvbuf), MT_DATA); */
	if (m == NULL) {
	    printf("ERROR: simether no memory for receive mbufs\n");
	}
    } 
    /*
     * Map the mbuf data address for Tulip's consumption.
     */
    va = mtod(m, vm_offset_t);


    /* Skip the ethernnet header. We don't have one */
    ifs->regs->rcvEntries[index].maxLen = (DevRegister) (m->m_len);
    svatophys(va, &pAddr);
    ifs->regs->rcvEntries[index].pAddr = (DevRegister)pAddr;
    ifs->regs->rcvEntries[index].flag = DEV_ETHER_CONTROLLER_OWNED;
    ifs->rcvBuffers[index] = m;
}

static void ifsim_DeallocSent(struct IfsimState *ifs)
{
    struct mbuf *m;

    while( 1 ) { 
        int x;
        if( !ifs->sndBuffers[ifs->deallocPtr] )
            break;

        x = ifs->regs->sndEntries[ifs->deallocPtr].flag;
        if( x != DEV_ETHER_OS_OWNED ) 
            break;
        /* 
         *  deallocate snd mbuf 
         */
        m_freem(ifs->sndBuffers[ifs->deallocPtr]);
        ifs->sndBuffers[ifs->deallocPtr] = 0;
        /*
         * clain free chunks 
         */
        ifs->numFreeChunks += ifs->numChunks[ifs->deallocPtr];
        ifs->deallocPtr++;
        if( ifs->deallocPtr >= ifs->numSndEntries ) 
            ifs->deallocPtr -= ifs->numSndEntries;
        
    }
}




/*
 * tuattach()
 * 	Interface exists: make available by filling in network interface
 * 	record.  System will initialize the interface when it is ready
 * 	to accept packets. Print ethernet address for unit.
 *
 * Arguments:
 *	ctlr		pointer to controller struct for the interface.
 *
 * Return Value:
 *	None.
 */
seattach(struct controller *ctlr)
{
	register int unit = ctlr->ctlr_num;
	register struct se_softc *sc = &se_softc[unit];
	register struct ifnet *ifp = &sc->is_if;
        register struct sockaddr_in *sin;

	ifp->if_addrlen = 6;	/* media address len */
	ifp->if_hdrlen = sizeof(struct ether_header) + 8  ;

	sc->is_ac.ac_bcastaddr = (u_char *)etherbroadcastaddr;
	sc->is_ac.ac_arphrd = ARPHRD_ETHER; 
	ifp->if_unit = unit;
	ifp->if_name = "simether";
	ifp->if_mtu = ETHERMTU;
	ifp->if_mediamtu = ETHERMTU;
	ifp->if_type = IFT_ETHER;
	ifp->if_flags = IFF_BROADCAST|IFF_MULTICAST|IFF_NOTRAILERS|IFF_SIMPLEX;
	((struct arpcom *)ifp)->ac_flag = 0;
        sin = (struct sockaddr_in *)&ifp->if_addr;
	sin->sin_family = AF_INET;
	ifp->if_init = se_init;
	ifp->if_output = ether_output;
	ifp->if_start = se_start;
	ifp->if_ioctl = seioctl;
	ifp->if_timer = 0;
	ifp->if_watchdog = sewatch;
	ifp->if_reset = se_reset;
	ifp->if_sysid_type = sc->se_sysid_type;

	/*
	 * Setup baudrate as we know it: updated again later in tuinit()
	 */
	ifp->if_baudrate = ETHER_BANDWIDTH_10MB;

#if NETSYNC_LOCK
        ifp->if_affinity = NETALLCPU;
	ifp->lk_softc = &sc->lk_se_softc;
#endif
	simple_lock_setup(&sc->lk_se_softc, se_softc_lockinfo);

	printf("simether%d: %s, hardware address: %s\n", unit,
	       ifp->if_version, ether_sprintf(sc->is_addr));

	attachpfilter(&(sc->is_ed));
	if_attach(ifp);

	/* All set to take interrupts now.
	 * Don't need this any more
	handler_enable(sc->hid);
	*/
}


/* 
 * tuinit()
 * 	Initialize interface. Make sure things are cleaned-up from past
 *	instances of the driver and ready descriptors in both rings for 
 *	a hand-off to Tulip.
 *
 *	Turn on the appropriate control bits in the Tulip's command register
 *	based on what is requested of the interface. If promiscuous mode is
 *	not selected, then execute the setup frame for address filtering.
 *
 *	The following registers are setup in this routine:
 *
 *		CSR3	- Receive ring address
 *		CSR4	- Transmit ring address
 *		CSR5	- Status register (cleared)
 *		CSR6	- Command register
 *		CSR7	- Interrupt mask register
 *
 *	If the initialization succeeds, the interface will be marked as being
 *	in the running state, and Tulip will have started working on the rings.
 *
 * Arguments:
 *	unit		The unit number of the interface
 *
 * Return Value:
 *	- EIO
 *	- ENOMEM
 *	- ENOBUFS
 *	- ESUCCESS
 */
seinit(int unit)
{
	register struct controller *ctlr = seinfo[unit];
	register struct se_softc *sc = &se_softc[unit];
	register struct ifnet *ifp = &sc->is_if;
	register struct IfsimState *ifs = &sc->ifsimState;
	register int i, j, k, cmdcsr, status;


	/* 
	 * Setup I/O space pointer.
	 */
	ifs->regs = DEV_ETHER_REGISTERS;
	/*
	     * Read configuration from controller
	     */
	ifs->numSndChunks = ifs->regs->numSndChunks;
	ifs->numSndEntries = ifs->regs->numSndEntries;
	ifs->numRcvEntries = ifs->regs->numRcvEntries;
        
	/*
	     * Initialize internal state 
	     */
	ifs->firstFreeChunk = 0;
	ifs->numFreeChunks =  ifs->numSndChunks;
	ifs->rcvPtr = 0;
	ifs->sndPtr = 0;
	ifs->deallocPtr = 0;

	/* 
	 * initialize receive ring buffer 
	 */
	for(i=0;i<ifs->numRcvEntries;i++) {
	    if (!ifs->rcvBuffers[i]) {
		ifsim_SetRcvBuffer(i, 0, ifs);
	    }
	}

	/*	
	 * We are active and running now.
	 */
	ifp->if_flags |= IFF_RUNNING;
	ifp->if_flags &= ~IFF_OACTIVE;

	/*
	 * Before we return, start the output process to ensure that
	 * requeued packets don't sit in the ifnet queue forever.
	 */
	if (ifp->if_snd.ifq_head)
 		tustart(ifp);

	return ESUCCESS;
}


/*
 * tustart(ifp)  
 * 	Interface start routine - start output on the interface 
 *
 * Arguments:
 *	ifp	The ifnet pointer, pointing to the intf. to o/p on.
 *
 * Return Value:
 * 	None.
 */
sestart(register struct ifnet *ifp)
{
	register int unit = ifp->if_unit;
	register struct se_softc *sc = &se_softc[unit];
	register struct IfsimState *ifs = &sc->ifsimState;
	register int ntpendg = sc->ntpendg;

	/*
	 * Tell the world we're busy processing the interface send-queue.
	 */
	ifp->if_flags |= IFF_OACTIVE;

	/*
	 * Remove packets from the interface queue and put them into the
	 * transmit ring while we can.
	 */
	while (1) {
		struct mbuf *m;

		IF_DEQUEUE(&ifp->if_snd, m);

		if (m)
			/*
			 * Have an mbuf chain to give to the device
			 */
			if (se_transmit(sc, m))
				/*
				 * Was put on the ring successfully, or was
				 * dropped; in either case, we want to continue.
				 */
				continue;
			else {
				/*
				 * Packet was put back on the interface queue.
				 * Keep OACTIVE set since there's no point in
				 * coming here again until after we have room
				 * in the transmit ring.
				 */
				break;
			}
		else {
			/*
			 * Interface queue has been emptied. Check the queue
			 * head again, however, before we quit. This check is 
			 * necessary to account for the race where the caller 
			 * of if_start finds active set (and therefore does not
			 * call start) *after* we've decided that the queue is 
			 * empty but before we get here to clear active.
			 */
			ifp->if_flags &= ~IFF_OACTIVE;
			mb();
			if (ifp->if_snd.ifq_head) {
				ifp->if_flags |= IFF_OACTIVE;
				continue;
			} else /* done */
				break;
		}
	}
}

	/* 
	 * check for sent buffers
	 * this check done during interrupt processing
	 * ifsim_DeallocSent(ifs);
	 * /



/*
 * se_put_transmit
 * 	Process an mbuf chain representing an ethernet packet (from our point
 *	of view) into the transmit ring.
 *
 * Arguments:
 *	sc	The controller structure for the interface.
 *	m	The mbuf chain to be copied to the RAM buffer.
 *
 * Return Value:
 *	1	If the packet was successfully placed in the ring or if it was
 *		dropped (transmission should continue)
 *      0	If the packet was put back on the interface queue (transmission
 *		should not continue in this case)
 */
se_transmit(register struct se_softc *sc, register struct mbuf *m)
{
    register struct ifnet *ifp = &sc->is_if;
    register struct IfsimState *ifs = &sc->ifsimState;
    register int unit = ifp->if_unit;
    register struct controller *ctlr = seinfo[unit];
    int i = 0, j,  numChunks=0, lastChunk=0 , firstChunk;
    struct mbuf *n;
    DevRegister dmaAddr[DEV_ETHER_MAX_SND_CHUNKS];
    DevRegister dmaLen [DEV_ETHER_MAX_SND_CHUNKS];

    if( ifs->sndBuffers[ifs->sndPtr] ) 
        return 0;

    /* 
     * determine the chunks 
     * !!! there can be more chunks than mbufs because of 
     * possible page boundaries
     */

    for(n=m; n; n=n->m_next) {
      vm_offset_t vAddr = mtod(n,vm_offset_t);
      vm_offset_t pAddr1,pAddr2;


      svatophys(vAddr, &pAddr1);
      svatophys((vAddr+n->m_len-1), &pAddr2);
      if (trunc_page(pAddr1) == trunc_page(pAddr2)) {
	  dmaAddr[numChunks] = pAddr1;
	  dmaLen[numChunks]  = n->m_len;
	  numChunks++;
      } else {
            int onFirstPage = page_size - ((uint)(pAddr1 & PAGE_MASK));
            dmaAddr[numChunks]   = pAddr1;
            dmaLen[numChunks]    = onFirstPage;
            dmaAddr[numChunks+1] = pAddr2;
            dmaLen[numChunks+1]  = n->m_len - onFirstPage;
            if( dmaLen[numChunks+1] > page_size ) 
              printf("ERROR: ifsim: chunk maps more than 2 pages \n");
            numChunks +=2;
      }
    }
    
    if( numChunks >= ifs->numSndChunks ) 
      printf("ERROR: if_Send: mbuf:0x%x numChunks=%d exceeds max (%d)\n",
              m,numChunks,ifs->numSndChunks);
    if( ifs->numFreeChunks < numChunks ) 
      return 0; /* not enough free chunks */
    
    /* keept a pointer to the mbuf */
    ifs->sndBuffers[ifs->sndPtr] = m;
    ifs->numChunks[ifs->sndPtr] = numChunks;
    ifs->numFreeChunks -= numChunks;
   /* write out the chunks */
    firstChunk =  ifs->firstFreeChunk;
    for( i=0; i<numChunks;i++ )  {
      ifs->regs->sndChunks[ifs->firstFreeChunk].pAddr = dmaAddr[i];
      ifs->regs->sndChunks[ifs->firstFreeChunk].len = dmaLen[i];
      lastChunk = ifs->firstFreeChunk;
      ifs->firstFreeChunk++;
      if( ifs->firstFreeChunk >= ifs->numSndChunks ) 
        ifs->firstFreeChunk -= ifs->numSndChunks;
    }
    /* write chunk boundaries */
    ifs->regs->sndEntries[ifs->sndPtr].firstChunk = firstChunk;
    ifs->regs->sndEntries[ifs->sndPtr].lastChunk = lastChunk;
    /* write flag */
    ifs->regs->sndEntries[ifs->sndPtr].flag = DEV_ETHER_CONTROLLER_OWNED;
    /* move sndPtr */
    ifs->sndPtr++;
    if ( ifs->sndPtr >= ifs->numSndEntries ) {
        ifs->sndPtr  -= ifs->numSndEntries;
    }   

    return 1;
}


/*
 * tuwatch() 
 *	Watchdog timer routine - goes off whenever a transmit times-out.
 *
 * Arguments:
 *	unit		The unit number of the interface
 *
 * Return Value:
 * 	None.
 */
sewatch(int unit)
{
    printf("simether%d: watchdog timer called\n",unit);
}


/*
 * tureset()
 * 	Reset the interface.
 *
 * Argument
 *	unit		The unit number of the interface
 *
 * Return Value 
 *	none
 */
sereset(int unit)
{
	register struct controller *ctlr = seinfo[unit];
	register struct se_softc *sc = &se_softc[unit];
	register struct ifnet *ifp = &sc->is_if;

	/*
	 * Stop the network activity and start things over again.
	 */
	ifp->if_flags &= ~IFF_RUNNING; 	
	seinit(unit);
}	


/* tuioctl()
 * 	Process an ioctl request. In doing so, the interface is
 * 	restarted and reinitialized (in most cases).
 *
 * Arguments:
 *	ifp		Pointer to the ifnet structure
 *	cmd		The ioctl to be performed
 *	data		The corresponding data for the ioctl.
 *
 * Return Value:
 *	- EIO
 *	- EINVAL
 *	- ENOMEM
 *	- ENOBUFS
 *	- ESUCCESS
 */
seioctl(register struct ifnet *ifp, register u_int cmd, register caddr_t data)
{
	register struct se_softc *sc = &se_softc[ifp->if_unit];
	register struct controller *ctlr = seinfo[ifp->if_unit];
	register unit = ifp->if_unit;

	struct ifreq *ifr = (struct ifreq *)data;
	struct ifdevea *ifd = (struct ifdevea *)data;
	struct ctrreq *ctr = (struct ctrreq *)data;
	struct ifaddr *ifa = (struct ifaddr *)data;
	int s, i, j, lock_on = 1, status = ESUCCESS;
	u_short ifmtu, speed;

        printf("simether%d: ioctl called (cmd = %d)\n",unit, cmd&0x0ff);

	s = splimp();
	simple_lock(&sc->lk_se_softc);

	switch (cmd) {
		case SIOCENABLBACK:
			ifp->if_flags |= IFF_LOOPBACK;
			if (ifp->if_flags & IFF_RUNNING)
				sereset(unit);
			break;

		case SIOCDISABLBACK:
			ifp->if_flags &= ~IFF_LOOPBACK;
			if (ifp->if_flags & IFF_RUNNING)
				sereset(unit);
			break;
	 
		case SIOCRPHYSADDR:
			/*
			 * Return the current and default physical addresses.
			 */
			bcopy(sc->is_addr, ifd->current_pa, 6);
			bcopy(sc->se_rom_addr, ifd->default_pa, 6);
			break;
	 
		case SIOCSPHYSADDR:
			/*
			 * Set (over-ride) the hardware physical address
			 */
			bcopy(ifr->ifr_addr.sa_data, sc->is_addr, 6);
			pfilt_newaddress(sc->is_ed.ess_enetunit, sc->is_addr);

			/*
			 * Since the physical address has changed, issue a new
			 * setup filter to the controller.
			 * Don't need this
			if (ifp->if_flags & IFF_RUNNING)
				status = se_setup_frame(ctlr);
				*/

			/* 
			 * If an IP address has been configured then an ARP 
			 * packet must be broadcast to tell all hosts which 
			 * currently have our address in their ARP tables to 
			 * update their information.
			 */
			simple_unlock(&sc->lk_se_softc);
			splx(s);

			if (((struct arpcom *)ifp)->ac_flag & AC_IPUP) {
				rearpwhohas((struct arpcom *)ifp);
			}

			/* Also call in to update the af_link address. */
			if_sphyaddr(ifp, ifr);
			lock_on = 0;
			break;

		case SIOCDELMULTI:
			/*
			 * Scan the multicast list/table to find the specified 
			 * mcast address. If the caller wasn't kidding, and this
			 * was the last reference to it, delete the entry by 
			 * zeroing out the field and down-counting the number 
			 * of active entries.
			 */
			for (i=0; i<SE_NMULTI; i++) {
				if (bcmp(&sc->is_multi[i],
					 ifr->ifr_addr.sa_data, 6) == 0) {
					if (--sc->is_muse[i] == 0) {
					   	bzero(&sc->is_multi[i], 6);
					   	sc->is_nmulti--;

						/*
						 * Register this with TULIP
						 * Not yet
						if (ifp->if_flags & IFF_RUNNING)
						  	status = 
							   se_setup_frame(ctlr);
							   */
						break;
					}
				}
			}
			break;

		case SIOCADDMULTI:
			/*
			 * Scan the multicast list/table and see if the mcast 
			 * address is already registered. If so, we can just 
			 * bump the in-use counter and exit. At the same time, 
			 * look for an empty slot in the table and remember the
			 * first one that's available.
			 */
			for (i=0,j=-1; i<SE_NMULTI; i++) {
				if (bcmp(&sc->is_multi[i],
					 ifr->ifr_addr.sa_data, 6) == 0) {
					sc->is_muse[i]++;
					break;
				} else if (j<0 && sc->is_muse[i]==0)
					j = i;
			}

			/*
			 * If we broke out of the above loop, then it indicates
			 * that the addr was previously added; take no action.
			 * If we didn't, then see if we encountered any unused 
			 * entries.
			 */
			if (i == SE_NMULTI) {
				if (j >= 0) {
					bcopy(ifr->ifr_addr.sa_data,
					      &sc->is_multi[j], 6);
					sc->is_muse[j]++;
					sc->is_nmulti++;

					/*
					 * Register this addition with TULIP
					 * Not yet
					if (ifp->if_flags & IFF_RUNNING)
						status = se_setup_frame(ctlr);
						*/
				} else {
					printf("se%d: multicast limit\n", unit);
					status = EIO;
				}
			}
			break;

		case SIOCRDCTRS:
		case SIOCRDZCTRS:
			ctr->ctr_ether = sc->ctrblk;
			ctr->ctr_type = CTR_ETHER;
			ctr->ctr_ether.est_seconds = 
				(time.tv_sec - sc->ztime) > 0xfffe ? 
					0xffff : (time.tv_sec - sc->ztime);
			if (cmd == SIOCRDZCTRS) {
				sc->ztime = time.tv_sec;
				bzero(&sc->ctrblk, sizeof(struct estat));
			}
			break;

		case SIOCSIFADDR:
			ifp->if_flags |= IFF_UP;
			sereset(unit);
			break;

		case SIOCSIFFLAGS:
			if (ifp->if_flags & IFF_RUNNING)
				sereset(unit);
			break;

		case SIOCSIPMTU:
			bcopy(ifr->ifr_data, (u_char *)&ifmtu, sizeof(u_short));

			if (ifmtu > ETHERMTU || ifmtu < 1)
				status = EINVAL;
			else
				ifp->if_mtu = ifmtu;
			break;

		case SIOCSMACSPEED:
			/*
			 * Until we have ether_config [hack]:
			 * 	Speed   10 = Half-Duplex 10Mbps
			 * 	Speed   20 = Full-Duplex 10Mbps
			 * 	Speed  100 = Half-Duplex 100Mbps
			 * 	Speed  200 = Full-Duplex 100Mbps
			 * 	Speed 1000 = Capture Effect Resolution
			 */
			bcopy(ifr->ifr_data, (u_char *)&speed, sizeof(u_short));

			sc->se_flags.ce_resolve = 0;
			sc->se_flags.full_duplex = 0;

			switch (speed) {
				case 20:
					sc->se_flags.full_duplex = 1;
				case 10:
					sc->se_flags.fast_speed = 0;
					break;

				case 200:
					sc->se_flags.full_duplex = 1;
				case 100:
					sc->se_flags.fast_speed = 1;
					break;

				case 1000:
					sc->se_flags.ce_resolve = 1;
					break;

				default:
					status = EINVAL;
			}

			sereset(unit);
			break;

		default:
			status = EINVAL;
	} 

	/*
	 * If we've already unlocked and restored the spl, don't do so again.
	 */
	if (lock_on) {
		simple_unlock(&sc->lk_se_softc);
		splx(s);
	}

	return (status);
}


/*
 * tuintr()
 *	Interface interrupt routine.
 *
 * Arguments:
 *	unit		The unit number of the interface
 *
 * Return Value:
 * 	None.
 */
seintr(struct se_softc *sctable)
{
	register u_int s, status;
	register struct se_softc *sc = &se_softc[0];
	register struct IfsimState *ifs = &sc->ifsimState;
	register struct ifnet *ifp = &sc->is_if;


	s = splimp();
	simple_lock(&sc->lk_se_softc);

	/*
	 * Tell the world we're busy processing transmit & receive completions.
	 */
	ifp->if_flags |= IFF_OACTIVE;


	/* ignore early interrupts */
	if( !ifs->regs ) {
	    simple_unlock(&sc->lk_se_softc);
	    splx(s);
	    return 1;
	}


	se_receive_int(sc);

        
        /*
         * Deallocate the buffers that have been fully sent 
         */
        ifsim_DeallocSent(ifs);
        
	/*
	 * If there's room in the transmit ring, and we have something 
	 * pending in the interface send-queue, then process it. NOTE: The
	 * ordering below is important: refer to similar code & comments
	 * in the start routine.
	 */
	ifp->if_flags &= ~IFF_OACTIVE;
	mb();
	if (ifp->if_snd.ifq_head)
	    sestart(ifp);

	simple_unlock(&sc->lk_se_softc);
	splx(s);

	return INTR_SERVICED;
}



/*
 * se_receive_int() 
 *	Process a receive interrupt completion.
 *
 * Arguments:
 *	sc	Pointer to the softc structure for this unit
 *
 * Return Value:
 * 	None.
 */
se_receive_int(register struct se_softc *sc)
{
	register struct ifnet *ifp = &sc->is_if;
	register struct IfsimState *ifs = &sc->ifsimState;
	register unit = ifp->if_unit;
	register int index;
        int still_to_do = 1;
	int found = 0;

	/* process as many packets as you can get */
	while( still_to_do ) { 
	    int len, index;
	    struct mbuf *m;
	    struct mbuf *n;
	    struct ether_header eh;
	    still_to_do = 0;

	    index = ifsim_GetRcvBuffer(&len, ifs);
	    if( index >= 0  ) { 
		found++;
		still_to_do = 1;
	
		m =ifs->rcvBuffers[index];
		/*
		 * Load the ethernet header for the ether_input call 
		 * below. Also re-order the two bytes of ether_type 
		 * within the header before we forget (ether_input 
		 * depends on this).
		 */
		eh = *(mtod(m, struct ether_header *));
		eh.ether_type = ntohs((u_short)eh.ether_type);	
		
		/*
		 * If the packet can be accommodated in a small mbuf 
		 * (with pkthdr), then get one of those. Otherwise, get
		 * a large 2k buffer.
		 */
		if (len <= MHLEN) {
		    MGETHDR(n, M_DONTWAIT, MT_DATA);
		} else {
		    TUMCLGET(n, M_DONTWAIT);
		}
		
		/*
		 * Do the ugly copying and setup the pkthdr fields if 
		 * there was no problem in getting the mbuf. Drop the 
		 * packet, otherwise.
		 */
		if (n) {
		    /*
		     * Drop the header and CRC bytes
		     */
		    len = len - sizeof(struct ether_header);
		    bcopy((mtod(m, vm_offset_t) + sizeof(struct ether_header)), 
			  mtod(n, vm_offset_t), len);
		    
                    /*
		     * Receive buffer is free: hand it over for 
		     * a new receive. No need to delay this until
		     * after ether_input is called.
		     */
		    ifsim_SetRcvBuffer(index, m, ifs);

		    n->m_pkthdr.len = n->m_len = len;
		    n->m_pkthdr.rcvif = &sc->is_if;

                    /*
		     * Bump up intrface counters. Input packets for 
		     * "netstat" include ALL directed, multicast, 
		     * and error inputs. For DECnet, only error-free
		     * input packets are counted.
		     */
		    sc->is_if.if_ipackets++;
		    if (sc->ctrblk.est_bytercvd + len > 
			sc->ctrblk.est_bytercvd)
			sc->ctrblk.est_bytercvd += len;
		    INCR(sc->ctrblk.est_blokrcvd, 0xffffffff);
		    
		    if (eh.ether_dhost[0] & 1) {
			if (sc->ctrblk.est_mbytercvd + len >
			    sc->ctrblk.est_mbytercvd)
			    sc->ctrblk.est_mbytercvd += len;
			INCR(sc->ctrblk.est_mblokrcvd, 
			     0xffffffff);
		    }
		    
		    /* Dispatch this packet */
		    ether_input(&sc->is_if, &eh, n);
		} else {
		    ifsim_SetRcvBuffer(index, m, ifs);
		}
	    }
	}	    
}



