/*
 * @DEC_COPYRIGHT@
 */
/*
 * This file has defines etc. from the simos tree
 * It needs to be kept consistent with original files:
 *
 * cpus-alpha/simos/machine_defs.h
 *
 */
#ifndef _IF_SEREG_H_
#define _IF_SEREG_H_


typedef unsigned long DevRegister;

#define DEV_ETHER_MAX_RCV_ENTRIES    64
#define DEV_ETHER_MAX_SND_ENTRIES    64
#define DEV_ETHER_MAX_SND_CHUNKS     128

#define DEV_ETHER_MAX_TRANSFER_SIZE  1800
#define DEV_ETHER_OS_OWNED           (DevRegister)1
#define DEV_ETHER_CONTROLLER_OWNED   (DevRegister)2

#define DEV_ETHER_REGISTERS  ((volatile DevEtherRegisters*)(0xfffffcc510000000))


typedef struct DevEtherRegisters {
  DevRegister etheraddr[6];       /* controller tells OS its ethernet addr */
  DevRegister numRcvEntries;      /* read by OS, indicates how many receive
				   * ring buffer entries will be used. OS must
				   * allocate a receive buffer for each of
				   * these entries */
  DevRegister numSndEntries;      /* read by OS, indicates how many send ring
				   * buffer entries will be used so it knows
				   * when to wrap its index pointer */
  DevRegister numSndChunks;       /* same as numSndEntries */

  struct {
    DevRegister pAddr;
    DevRegister maxLen;
    DevRegister len;
    DevRegister flag;         
  } rcvEntries[ DEV_ETHER_MAX_RCV_ENTRIES ];

  struct {
    DevRegister firstChunk;
    DevRegister lastChunk;
    DevRegister flag; /* triggers send */
  } sndEntries[ DEV_ETHER_MAX_SND_ENTRIES ];

  struct { 
    DevRegister pAddr;
    DevRegister len;
  } sndChunks[ DEV_ETHER_MAX_SND_CHUNKS ]; 

  /* note: sndChunks is last because we might extend the number of
   * send chunks in the future and don't want to break OS compatibility when
   * we do it */
} DevEtherRegisters;


typedef struct IfsimState {
    volatile DevEtherRegisters *regs;
    int numRcvEntries, numSndEntries, numSndChunks;
    int rcvPtr;
    int sndPtr;
    int deallocPtr;
    int firstFreeChunk;
    int numFreeChunks;
    struct mbuf *rcvBuffers[DEV_ETHER_MAX_RCV_ENTRIES];
    struct mbuf *sndBuffers[DEV_ETHER_MAX_SND_ENTRIES];
    int numChunks[DEV_ETHER_MAX_SND_ENTRIES];
} IfsimState;

#endif	/* _IF_SEREG_H_ */
