/*
 * @DEC_COPYRIGHT@
 */
/*
 * HISTORY
 * $Log: lockinfo.c,v $
 * Revision 1.1.318.41  1997/11/11  16:46:41  Jim_Beltz
 * 	Move the vdT.del_list_lk above the BMT.bfAccess.mcellList_lk as part of the
 * 	extent map locking.
 * 	[1997/11/05  18:07:57  Jim_Beltz]
 *
 * 	Move page0_mcell_lk above BMT.bfAccess.xtntMap_lk as part of the extent
 * 	map locking re-work.
 * 	[1997/11/05  13:37:32  Jim_Beltz]
 *
 * Revision 1.1.318.40  1997/10/27  22:29:44  David_Winchell
 * 	Lower the rm_event_buff lock.
 * 	[1997/10/27  19:54:27  David_Winchell]
 * 
 * Revision 1.1.318.39  1997/10/09  23:22:08  Tony_Bono
 * 	Added tssmap_lock for sosend wire table.
 * 	[1997/10/09  22:37:53  Tony_Bono]
 * 
 * Revision 1.1.318.38  1997/10/08  19:12:23  Arti_Kumar
 * 	 	** Merge Information **
 * 		** Command used:	bsubmit **
 * 		** Ancestor revision:	1.1.318.36 **
 * 		** Merge revision:	1.1.318.37 **
 * 	 	** End **
 * 	Add putpage_lk to ADVfs hierarchy and remove probe for bs_page_inval
 * 	routine (qar 54481).
 * 	[1997/10/06  23:43:11  Arti_Kumar]
 * 
 * Revision 1.1.318.37  1997/10/07  13:39:22  Larry_Scott
 * 	Disable hierarchy checks for logDescT.descLock.
 * 	- descLock must be below rnode_table_lock and above sock_lock
 * 	- rnode_table_lock is below sock_lock
 * 
 * 	Change logDesc mutex to logDescTbl complex lock; logDescTbl must be held
 * 	when taking descriptor (complex) lock.
 * 	[1997/10/01  18:01:48  Larry_Scott]
 * 
 * Revision 1.1.318.36  1997/09/11  20:43:35  Shashi_Mangalat
 * 	Remove ump_lockinfo.
 * 	[1997/09/11  14:54:46  Shashi_Mangalat]
 * 
 * Revision 1.1.318.35  1997/09/05  18:46:24  Jonathan_Wistar
 * 	Fix QAR #55074 by relocating the "slrecv_smp_lock" lock.
 * 	(Also eliminate obsolete lock "sl_timers_lock".)
 * 	[1997/09/05  18:44:11  Jonathan_Wistar]
 * 
 * Revision 1.1.318.34  1997/08/27  15:20:28  Tim_Mark
 * 	Moved migTruncLk above BfSetTblLock.  QAR 49607.
 * 	[1997/08/22  15:27:14  Tim_Mark]
 * 
 * Revision 1.1.318.33  1997/08/11  20:58:38  Ted_Qian
 * 	added request_q_complex_lock_info for boot time support.
 * 	[1997/08/06  18:55:22  Ted_Qian]
 * 
 * Revision 1.1.318.32  1997/07/10  21:08:16  Robert_Picco
 * 	Incorporate RM related locking.
 * 	[1997/07/10  20:29:06  Robert_Picco]
 * 
 * 	Add lock support for disaster tolerant masking driver.
 * 	[1997/07/10  14:29:55  Robert_Picco]
 * 
 * Revision 1.1.318.31  1997/07/03  11:56:36  Jim_Beltz
 * 	 	** Merge Information **
 * 		** Command used:	bmerge -rPTCOS;PTHW1OS_BL3;<> **
 * 		** Ancestor revision:	1.1.318.28 **
 * 		** Merge revision:	1.1.318.30 **
 * 	 	** End **
 * 	[1997/07/03  11:52:39  Jim_Beltz]
 * 
 * 	Move trunc_xfer_lk below vm_umap.vm_lock.  QAR 53670.
 * 	[1997/07/03  11:48:24  Jim_Beltz]
 * 
 * Revision 1.1.318.30  1997/07/02  15:59:38  James_Woodward
 * 	 	** Merge Information **
 * 		** Command used:	bsubmit **
 * 		** Ancestor revision:	1.1.318.28 **
 * 		** Merge revision:	1.1.318.29 **
 * 	 	** End **
 * 	reverse hierarchy position of waitqueue and thread lock
 * 	[1997/07/01  15:08:18  James_Woodward]
 * 
 * Revision 1.1.318.29  1997/07/02  14:05:41  Julie_Dewandel
 * 	Added otto_softc and otto_rdy_q locks for ATM driver.
 * 	[1997/06/30  20:20:33  Julie_Dewandel]
 * 
 * Revision 1.1.318.28  1997/06/25  15:32:41  Jonathan_Wistar
 * 	Add DLMSL (DLM session layer) locks
 * 	[1997/06/19  13:23:47  Jonathan_Wistar]
 * 
 * Revision 1.1.318.27  1997/06/04  17:05:28  Craig_Wright
 * 	Make logDescT.descLock ORDERED. Two can be held simultaneously
 * 	without hierarchy violation; deadlock is avoided by code in
 * 	lgr_flush_start().  QAR 46471.
 * 	Added probe stuff for new routines for log half full fix.
 * 	Added new trunc_xfer_lk. Qar 38423 & 32666.
 * 	[1997/06/03  23:09:50  Craig_Wright]
 * 
 * Revision 1.1.318.26  1997/05/27  16:50:14  Jim_Beltz
 * 	Added new AdvFS probe for fs_flush_saved_stats().
 * 	[1997/05/20  19:08:12  Jim_Beltz]
 * 
 * Revision 1.1.318.25  1997/05/16  12:48:25  James_Woodward
 * 	remove duplicate lockinfo structures.
 * 	[1997/05/13  14:47:37  James_Woodward]
 * 
 * Revision 1.1.318.24  1997/05/06  14:09:02  Dave_Gerson
 * 	Conditionally compile 'i2c_lockinfo' reference for Alphaserver
 * 	4100 only.
 * 	[1997/05/06  13:56:54  Dave_Gerson]
 * 
 * Revision 1.1.318.23  1997/05/02  20:52:44  Dave_Gerson
 * 	Added lock for I2C bus access.
 * 	[1997/05/02  18:19:09  Dave_Gerson]
 * 
 * Revision 1.1.318.22  1997/05/02  02:00:37  David_Winchell
 * 	Added locks rm_init_state_lock and rm_error_shutting_down_state_lock.
 * 	[1997/05/01  18:11:37  David_Winchell]
 * 
 * Revision 1.1.318.21  1997/04/24  21:02:01  Eric_Werme
 * 	 	** Merge Information **
 * 		** Command used:	bsubmit **
 * 		** Ancestor revision:	1.1.318.19 **
 * 		** Merge revision:	1.1.318.20 **
 * 	 	** End **
 * 	Elevate nfs_xprtfree_lock since now used by SPLIMP callers.
 * 	Add LI() for existing lock lock_for_exported.
 * 	[1997/04/24  18:07:55  Eric_Werme]
 * 
 * Revision 1.1.318.20  1997/04/17  18:54:40  James_Woodward
 * 	fix link undefined of cam_htm_nexuslk_li on pza only platform
 * 	[1997/04/17  18:41:16  James_Woodward]
 * 
 * Revision 1.1.318.19  1997/04/03  20:16:57  Dick_Bagley
 * 	 	** Merge Information **
 * 		** Command used:	bsubmit -visual **
 * 		** Ancestor revision:	1.1.318.16 **
 * 		** Merge revision:	1.1.318.18 **
 * 	 	** End **
 * 	Added "new" lock from sim_htm.c and addtional lock from
 * 	the qlogic driver.
 * 	[1997/04/03  20:06:46  Dick_Bagley]
 * 
 * Revision 1.1.318.18  1997/04/03  18:40:28  Shashi_Mangalat
 * 	 	** Merge Information **
 * 		** Command used:	bsubmit **
 * 		** Ancestor revision:	1.1.318.16 **
 * 		** Merge revision:	1.1.318.17 **
 * 	 	** End **
 * 	Remove ubc_pfree_lock.
 * 	[1997/03/31  22:32:09  Shashi_Mangalat]
 * 
 * Revision 1.1.318.17  1997/04/03  13:39:04  Jim_Hamel
 * 	remove references to rmspur driver lock info.
 * 	[1997/04/02  18:33:45  Jim_Hamel]
 * 
 * Revision 1.1.318.16  1997/03/10  18:15:51  Barry_Carlson
 * 	Added probe for tagdir_tag_to_freelist for QAR 49841.
 * 	[1997/03/10  17:17:55  Barry_Carlson]
 * 
 * Revision 1.1.318.15  1997/03/06  18:12:10  Gary_Grebus
 * 	 	** Merge Information **
 * 		** Command used:	bsubmit -visual **
 * 		** Ancestor revision:	1.1.318.11 **
 * 		** Merge revision:	1.1.318.14 **
 * 	 	** End **
 * 	Move atm_cmm_mmi_list lock higher in the hierarchy.
 * 	[1997/03/06  00:12:39  Gary_Grebus]
 * 
 * 	Added atm_cmm_mmi_list lock needed by ATM management interface.
 * 	Fix naming inconsistency for atm_cmm_driver_list lock info.
 * 	[1997/03/04  19:11:39  Gary_Grebus]
 * 
 * Revision 1.1.318.14  1997/03/05  22:08:00  Ronald_Menner
 * 	added dlbq_list_lock
 * 	[1997/03/04  19:38:10  Ronald_Menner]
 * 
 * Revision 1.1.318.13  1997/02/26  18:09:20  Jeff_Denham
 * 	 	** Merge Information **
 * 		** Command used:	bmerge -jPTBOS_BL4:V40BSUPPORTOS_PRE_BL4 **
 * 		** Ancestor revision:	1.1.291.3 **
 * 		** Merge revision:	1.1.346.2 **
 * 	 	** End **
 * 	merge of v40bsupportos.pre.bl4 changes
 * 	[1997/02/26  16:12:49  Jeff_Denham]
 * 
 * Revision 1.1.318.12  1997/02/25  15:23:31  Tim_Mark
 * 	Removed lockinfo stuff for ftx_log_data(),
 * 	bs_bf_data_logging, and bs_unpinpg_logif().
 * 	QAR 49881.
 * 	[1997/02/12  20:43:51  Tim_Mark]
 * 
 * Revision 1.1.318.11  1997/01/29  21:42:33  Jim_Hamel
 * 	Add RM related lock structures needed by updated cluster
 * 	code
 * 	[1997/01/29  21:29:19  Jim_Hamel]
 * 
 * Revision 1.1.318.10  1997/01/28  14:15:48  Caroline_Fleming
 * 	     Fix for QAR 51191: Change spl for cam_pool_ctrl_li and
 * 	                        cam_pool_queue_li to SPLNONE.
 * 	[1997/01/28  14:12:52  Caroline_Fleming]
 * 
 * Revision 1.1.318.9  1997/01/23  20:28:08  David_Cherkus
 * 	Fix QAR 51158: Add lockinfo for tcr mcnet parallel send lock
 * 	[1997/01/23  18:07:51  David_Cherkus]
 * 
 * Revision 1.1.318.8  1997/01/10  19:37:54  Caroline_Fleming
 * 	    Fix for QAR 50306: Move the cam_pool_ctrl_li
 * 	    and cam_pool_queue_li locks below the spo_xx locks,
 * 	    to fix SIMPORT driver lock hierachy violation.
 * 	[1997/01/10  19:35:50  Caroline_Fleming]
 * 
 * Revision 1.1.318.7  1996/12/20  16:23:54  William_Dallas
 * 	Added Disk response monitoring queue locks.
 * 	[1996/12/20  03:32:58  William_Dallas]
 * 
 * Revision 1.1.318.6  1996/12/03  21:48:55  Jeff_Denham
 * 	Remove procfs trace and aio init locks.
 * 	[1996/12/03  21:23:27  Jeff_Denham]
 * 
 * Revision 1.1.346.2  1996/11/27  20:54:27  Charles_Katz
 * 	Add simple lock for AdvFS to manage its MsfsMemUse variable
 * 	CLD ZPOB82799 (merge from v40a)
 * 	[1996/11/18  21:38:31  Charles_Katz]
 * 
 * Revision 1.1.318.5  1996/11/22  18:24:05  Raul_Kacirek
 * 	QAR 49195 - Inserted cam_pool_ctrl_li & cam_pool_queue_li
 * 	Fixed missing clossing quote in LI("ev_lock", ev_lock_info, SPLNONE, DEFAULT);
 * 	[1996/11/22  18:17:51  Raul_Kacirek]
 * 
 * Revision 1.1.318.4  1996/11/01  21:14:17  Anton_Verhulst
 * 	Add class.lock
 * 	[1996/11/01  21:06:17  Anton_Verhulst]
 * 
 * Revision 1.1.318.3  1996/10/18  20:32:01  John_Meneghini
 * 	 	** Merge Information **
 * 		** Command used:	bsubmit **
 * 		** Ancestor revision:	1.1.308.2 **
 * 		** Merge revision:	1.1.318.2 **
 * 	 	** End **
 * 	Added CAM RCI spin locks
 * 	[1996/10/07  13:22:21  John_Meneghini]
 * 
 * Revision 1.1.318.2  1996/10/18  17:45:16  James_Woodward
 * 	combine two lockinfos used for buf lock
 * 	[1996/10/18  11:52:11  James_Woodward]
 * 
 * Revision 1.1.308.2  1996/09/18  20:25:40  Jeff_Denham
 * 	 	** Merge Information **
 * 		** Command used:	bmerge -jPTAOS_BL3:PTAOS_BL4 **
 * 		** Ancestor revision:	1.1.242.5 **
 * 		** Merge revision:	1.1.242.7 **
 * 	 	** End **
 * 	merge of ptbos and pta bl4
 * 	[1996/09/18  14:26:16  Jeff_Denham]
 * 
 * Revision 1.1.291.3  1996/09/18  20:12:44  Jeff_Denham
 * 	 	** Merge Information **
 * 		** Command used:	bmerge -jPTAOS_BL3:PTAOS_BL4 **
 * 		** Ancestor revision:	1.1.242.5 **
 * 		** Merge revision:	1.1.242.7 **
 * 	 	** End **
 * 	merge of ptbos and pta bl4
 * 	[1996/09/18  14:26:16  Jeff_Denham]
 * 
 * Revision 1.1.291.2  1996/08/05  21:56:25  William_Evers
 * 	add console environment variable lock ev_lock.
 * 	[1996/08/05  19:22:28  William_Evers]
 * 
 * Revision 1.1.242.7  1996/07/30  18:50:15  David_Cherkus
 * 	Discovered that rm_mgmt_locks needs to be declared as ORDERED
 * 	[1996/07/30  18:32:09  David_Cherkus]
 * 
 * Revision 1.1.242.6  1996/07/24  19:36:01  David_Cherkus
 * 	Remove extra rm_mgmt_lock declaration, use DEFAULT for rm_mgmt_locks
 * 	[1996/07/24  15:04:34  David_Cherkus]
 * 
 * Revision 1.1.242.5  1996/07/08  12:01:20  David_Cherkus
 * 	Fix lockmode 4 panic with TCR and log_lock
 * 	[1996/07/03  20:10:26  David_Cherkus]
 * 
 * 	Fix lockmode 4 panic with TCR and log_lock
 * 	[1996/07/03  18:19:28  David_Cherkus]
 * 
 * Revision 1.1.242.4  1996/05/22  19:53:29  James_Woodward
 * 	merge of TCR2 changes
 * 	[1996/05/22  19:19:08  James_Woodward]
 * 
 * Revision 1.1.242.3  1996/04/12  15:40:19  Stuart_Hollander
 * 	Continuing HW6 to PTHW1 merge.
 * 	[1996/04/12  15:20:16  Stuart_Hollander]
 * 
 * Revision 1.1.248.3  1996/03/28  20:21:46  James_Woodward
 * 	fixes to rev 1.1.248.2
 * 	[1996/03/28  17:04:58  James_Woodward]
 * 
 * Revision 1.1.248.2  1996/03/28  13:35:57  John_H_Williams
 * 	Seeding tcr2os with base modules
 * 	[1996/03/28  13:32:17  John_H_Williams]
 * 
 * Revision 1.1.76.83  1996/03/14  19:01:18  Arti_Kumar
 * 	   Remove putpage_lk from ADVfs hierarchy (qar 44706)
 * 	   [1996/03/13  19:42:17  Arti_Kumar]
 * 
 * Revision 1.1.242.2  1996/03/13  21:01:25  Stuart_Hollander
 * 	HW6 to PTHW1 merge
 * 	[1996/03/13  19:02:25  Stuart_Hollander]
 * 
 * Revision 1.1.76.82  1996/03/06  22:27:46  Arti_Kumar
 * 	Add putpage_lk to ADVfs hierarchy (qar 36473)
 * 	[1996/03/06  22:23:15  Arti_Kumar]
 * 
 * Revision 1.1.76.81  1996/03/05  16:56:49  Steve_Neuner
 * 	Moved logDescT.descLock below rnode_table_lock (QAR 44294).
 * 	[1996/03/01  23:15:07  Steve_Neuner]
 * 
 * Revision 1.1.76.80  1996/03/05  16:45:01  Shashi_Mangalat
 * 	Remove duplicate and incorrect VM lockinfo entries.
 * 	[1996/03/04  19:36:10  Shashi_Mangalat]
 * 
 * Revision 1.1.76.79  1996/02/27  21:44:56  Jan_Daddamio
 * 	Move files for QAR 23146 to ZK for submission
 * 	[1996/02/23  00:58:14  Jan_Daddamio]
 * 
 * 	Bmerge with nightly pool
 * 	[1996/02/09  20:36:21  Jan_Daddamio]
 * 
 * 	Insert probes added to bs_params.c and bs_inmem_map.c for new
 * 	routines for vdump sparse file handling. QAR 23146
 * 	[1996/02/08  00:40:57  Jan_Daddamio]
 * 
 * Revision 1.1.76.78  1996/02/08  18:28:50  Jim_Beltz
 * 	Bmerge with nightly backing tree.
 * 	[1996/01/17  15:33:12  Jim_Beltz]
 * 
 * 	Added probe for cleanup_closed_list() in msfs/bs/bs_access.c. QAR 41281
 * 	[1996/01/17  15:29:42  Jim_Beltz]
 * 
 * Revision 1.1.176.13  1996/02/08  15:34:13  David_Cherkus
 * 	Lock hierarchy fixes
 * 	[1996/02/08  14:47:52  David_Cherkus]
 * 
 * Revision 1.1.176.12  1996/02/06  00:02:17  David_Cherkus
 * 	Add lockinfo for mcnet
 * 	[1996/02/05  23:17:32  David_Cherkus]
 * 
 * Revision 1.1.176.11  1996/02/05  20:37:01  David_Cherkus
 * 	Add lockinfo for CComSub
 * 	[1996/02/05  20:33:03  David_Cherkus]
 * 
 * Revision 1.1.176.10  1996/01/31  18:40:42  Joseph_Amato
 * 	fix for qar 43144 and add lockinfo structs
 * 	[1996/01/31  18:24:21  Joseph_Amato]
 * 
 * Revision 1.1.176.9  1996/01/17  17:30:35  William_Grava
 * 	Specify SPLDEVLOW as minimum IPL for Memory Channel simple locks
 * 	[1996/01/17  17:22:30  William_Grava]
 * 
 * Revision 1.1.76.77  1996/01/12  14:27:15  John_Meneghini
 * 	Merged branch with backing tree
 * 	[1996/01/10  16:47:04  John_Meneghini]
 * 
 * 	Merge branch with backing tree.
 * 	[1995/12/14  20:09:11  John_Meneghini]
 * 
 * 	Add cxalloc_lock for use with Shared Magnetic Tape allocation facility.
 * 	See io/common/alloc_smt.c for more information.
 * 	[1995/12/14  20:06:44  John_Meneghini]
 * 
 * Revision 1.1.76.76  1996/01/11  18:09:19  Stuart_Hollander
 * 	(QAR 41090) Change procfs_mount_lock to complex lock.
 * 	[1996/01/10  20:52:33  Stuart_Hollander]
 * 
 * Revision 1.1.76.75  1996/01/04  17:23:06  Tim_Mark
 * 	Bmerge.
 * 	[1996/01/04  13:22:29  Tim_Mark]
 * 
 * 	Added probe for check_vd_sizes in msfs/osf/msfs_vfsops.c
 * 	QAR 39225.
 * 	[1995/12/26  19:31:24  Tim_Mark]
 * 
 * Revision 1.1.76.74  1996/01/03  14:26:58  Jim_Beltz
 * 	Removed several ULI_PROBE() declarations for now-defunct AdvFS routines;
 * 	added two new ones for newly-added probes.  QAR 41857
 * 	[1995/12/29  15:25:25  Jim_Beltz]
 * 
 * Revision 1.1.176.8  1995/12/20  21:35:37  Timothy_Burke
 * 	Changed DRD map locks from complex to simple.
 * 	[1995/12/20  20:39:38  Timothy_Burke]
 * 
 * Revision 1.1.76.73  1995/12/20  14:59:39  Jin_Chen
 * 	Move the callback_list_lock one position up in the hierarchy.
 * 	[1995/12/20  14:34:57  Jin_Chen]
 * 
 * Revision 1.1.76.72  1995/12/18  22:09:18  Jeff_Denham
 * 	Switch proc_lock and aio_lock in the hierarchy. Fix some
 * 	other cruft.
 * 	[1995/12/16  00:09:23  Jeff_Denham]
 * 
 * Revision 1.1.76.71  1995/12/18  17:43:52  Shashi_Mangalat
 * 	Add segmented shared memory (SSM) locks.
 * 	[1995/12/18  00:26:18  Shashi_Mangalat]
 * 
 * Revision 1.1.76.70  1995/12/13  17:31:16  Peter_Woods
 * 	Add in lock info structures for cbus2_pci_int_lock. (QAR # 31545)
 * 	[1995/12/12  18:42:32  Peter_Woods]
 * 
 * Revision 1.1.176.7  1995/12/01  17:30:52  William_Grava
 * 	Add locks for Memory Channel scatter-gather and PCT access.
 * 	[1995/12/01  05:43:44  William_Grava]
 * 
 * Revision 1.1.176.6  1995/12/01  17:02:06  Brian_Stevens
 * 	Restoring to latest MP1 version.
 * 	[1995/12/01  16:56:32  Brian_Stevens]
 * 
 * Revision 1.1.76.69  1995/11/16  21:48:33  William_Dallas
 * 	Added separate lock for cam_disk recovery que thread
 * 	[1995/11/16  21:41:34  William_Dallas]
 * 
 * Revision 1.1.76.68  1995/11/16  14:52:30  Jim_Hamel
 * 	make pwrmgr locks dependent on PWRMGR_ENABLED
 * 	[1995/11/14  20:19:42  Jim_Hamel]
 * 
 * Revision 1.1.76.67  1995/11/15  16:46:12  Michael_Schloss
 * 	SMP Hierarchy work: restore BfSetTblLock to hierarchy
 * 	[1995/11/14  21:36:49  Michael_Schloss]
 * 
 * Revision 1.1.76.66  1995/11/14  19:17:52  Jeff_Denham
 * 	Make the proc locks ordered.
 * 	[1995/11/13  20:36:04  Jeff_Denham]
 * 
 * Revision 1.1.76.65  1995/11/13  19:06:44  Stuart_Hollander
 * 	merged pcmcia to nightly
 * 
 * 	Revision 1.1.194.2  1995/10/03  21:55:50  Theresa_Chin
 * 	Added I/O resource management locks.
 * 	[1995/11/03  21:30:23  Stuart_Hollander]
 * 
 * Revision 1.1.199.4  1995/11/06  16:27:16  David_Winchell
 * 	Granularity hints update.
 * 	[1995/11/01  14:55:42  David_Winchell]
 * 
 * Revision 1.1.76.64  1995/10/30  22:05:34  David_Winchell
 * 	Granularity hints
 * 	[1995/10/30  21:44:59  David_Winchell]
 * 
 * Revision 1.1.76.63  1995/10/30  20:54:36  Paul_Shaughnessy
 * 	Final? merge to nightly
 * 	[1995/10/30  20:46:11  Paul_Shaughnessy]
 * 
 * 	Rework file set lock hierarchy, merge to nightly
 * 	[1995/10/27  21:04:52  Paul_Shaughnessy]
 * 
 * 	Merge to nightly
 * 	[1995/10/18  22:17:35  Paul_Shaughnessy]
 * 
 * 	AdvFS SMP - Add probes and SMP asserts
 * 	[1995/10/06  18:23:26  Paul_Shaughnessy]
 * 
 * Revision 1.1.76.62  1995/10/26  21:20:33  atmproj_Peter_Schulter
 * 	Merged  changes  from  shared  SB
 * 	[1995/10/25  20:36:35  atmproj_Peter_Schulter]
 * 
 * 	moved atmsig_uni3x_lock above vm_umap.vm_lock
 * 	[1995/10/13  02:54:11  Ronald_Menner]
 * 
 * 	make ATM lockinfos dependent on appropriate ATM
 * 	subsystems (e.g.ATMIP)
 * 	[1995/10/06  18:45:48  Ronald_Menner]
 * 
 * 	Removed atm_arp_conevt_lock.
 * 	[1995/10/05  22:30:17  Julia_Fey]
 * 
 * 	{** Merge Information **}
 * 		{** Command used:	bsubmit -visual **}
 * 		{** Ancestor revision:	1.1.161.8 **}
 * 		{** Merge revision:	1.1.161.9 **}
 * 	{** End **}
 * 	added esi, map, and ppa locks for ATMIP
 * 	[1995/08/28  01:04:06  Ronald_Menner]
 * 
 * 	added more CMM locks
 * 	[1995/08/22  16:11:34  Peter_Schulter]
 * 
 * 	BL7 Merge
 * 	[1995/08/09  01:07:45  Peter_Schulter]
 * 
 * Revision 1.1.76.61  1995/10/23  14:27:44  Jim_Hamel
 * 	Add the lockinfo for the pwrmgr subsystem lock
 * 	[1995/10/19  17:56:35  Jim_Hamel]
 * 
 * Revision 1.1.199.3  1995/10/21  13:23:53  David_Winchell
 * 	Add gh_stats lock.
 * 	[1995/10/20  20:33:55  David_Winchell]
 * 
 * Revision 1.1.76.60  1995/10/20  20:52:42  Marcus_Barrow
 * 	Correct the name of the mounttab_lockinfo struct, was mounttab_lock.
 * 	[1995/10/20  20:40:36  Marcus_Barrow]
 * 
 * Revision 1.1.199.2  1995/10/18  21:38:29  David_Winchell
 * 	Resubmit of V32CSUPPORT changes
 * 	[1995/10/18  20:57:31  David_Winchell]
 * 
 * Revision 1.1.197.2  1995/10/18  20:31:25  David_Winchell
 * 	Add shmid_lock, granularity hint locks.
 * 	[1995/10/18  16:51:15  David_Winchell]
 * 
 * Revision 1.1.76.59  1995/09/27  20:28:57  Ann_Milicia
 * 	Remove probes for rm_clone() - no longer used as of QAR 35936 fix.
 * 	[1995/09/20  14:07:18  Ann_Milicia]
 * 
 * Revision 1.1.76.58  1995/09/26  21:41:44  Steve_Duerksen
 * 	Add 6 locks for PPP.
 * 	[1995/09/25  22:21:08  Steve_Duerksen]
 * 
 * Revision 1.1.76.57  1995/09/07  17:03:32  Paul_Shaughnessy
 * 	Removed ADVlogDescT_rdPgSem_info. Fix for QAR 36967/37002.
 * 	Submit for Brian Tsao.
 * 	[1995/09/07  12:54:05  Paul_Shaughnessy]
 * 
 * Revision 1.1.76.56  1995/09/01  17:28:29  James_Smart
 * 	Fix QAR 37005/36951/36882: cam ccfg lock heirarchy needs to be above
 * 	   cam common routines.
 * 	[1995/08/31  16:46:46  James_Smart]
 * 
 * Revision 1.1.76.55  1995/08/14  20:26:51  Rudy_Cano
 * 	{** Merge Information **}
 * 		{** Command used:	bsubmit **}
 * 		{** Ancestor revision:	1.1.76.53 **}
 * 		{** Merge revision:	1.1.76.54 **}
 * 	{** End **}
 * 	Added the lockinfo for the following locks which were converted
 * 	to OSF locks: ddlActiveLk, waitLock, DmnTblLock, InitLock, rdPgSem,
 * 	qiLock, scLock, pqd_main.lock, pqd_lowpri.lock and eliminated ADVInitMutex_lockinfo
 * 	[1995/08/11  22:13:59  Rudy_Cano]
 * 
 * Revision 1.1.76.54  1995/08/02  17:37:27  James_Woodward
 * 	make lwc routines per-cpu
 * 	[1995/08/02  17:31:26  James_Woodward]
 * 
 * Revision 1.1.76.53  1995/07/28  14:43:45  Shashi_Mangalat
 * 	Move SystemV ipc {shm,msg,sem} lockinfo setup to simple locks.
 * 	Add ubc_iocount_lockinfo.
 * 	[1995/07/28  14:43:13  Shashi_Mangalat]
 * 
 * Revision 1.1.76.52  1995/07/20  12:31:46  James_Woodward
 * 	merge of Platinum Lite BL6 changes
 * 	[1995/07/20  12:02:16  James_Woodward]
 * 
 * Revision 1.1.176.3  1995/07/17  13:12:31  Wayne_Cardoza
 * 	Add rm locks.
 * 	[1995/07/14  17:40:33  Wayne_Cardoza]
 * 
 * Revision 1.1.176.2  1995/07/13  12:47:04  David_Winchell
 * 	Add vm_rm_object.links_lock.
 * 	[1995/07/12  21:37:19  David_Winchell]
 * 
 * Revision 1.1.170.3  1995/07/06  19:20:37  Peter_Woods
 * 	Merge in changes from ptlite BL5 to ptlite BL6 into HW5
 * 	[1995/07/06  19:09:33  Peter_Woods]
 * 
 * Revision 1.1.76.51  1995/07/05  15:19:55  James_Smart
 * 	Add Device Naming mods to lock heirarchies.
 * 	[1995/07/04  11:37:34  James_Smart]
 * 
 * Revision 1.1.76.50  1995/06/30  19:18:59  Rudy_Cano
 * 	Added the lockinfo for following locks which were converted to OSF locks
 * 	cow_lk, flushLK, mss_lk, migTruncLk, RawBufFreeWait, shlvLock
 * 	[1995/06/23  03:26:48  Rudy_Cano]
 * 
 * Revision 1.1.76.49  1995/06/30  11:49:41  Patrick_ORourke
 * 	Merge to latest
 * 	[1995/06/28  20:31:59  Patrick_ORourke]
 * 
 * 	Merge to ptos.nightly
 * 	[1995/06/23  18:20:31  Patrick_ORourke]
 * 
 * 	Swap structure reorganization
 * 	[1995/06/22  17:15:30  Patrick_ORourke]
 * 
 * Revision 1.1.76.48  1995/06/22  19:14:59  Peter_Woods
 * 	Update lock into structure for T2 on Sable.  Add in two structures (one
 * 	for each PCI) and also allow those structures to be used on Gamma
 * 	(DEC2100_C500)
 * 	[1995/06/19  17:26:15  Peter_Woods]
 * 
 * Revision 1.1.76.47  1995/06/15  20:28:41  Ajay_Kachrani
 * 	bmerged again.
 * 	[1995/06/14  19:32:37  Ajay_Kachrani]
 * 
 * 	bmerged
 * 	[1995/06/13  20:54:50  Ajay_Kachrani]
 * 
 * 	Changed MRT locks naming to MFC for IP multicast and deleted
 * 	mrtvcache lock.
 * 	[1995/05/25  14:20:19  Ajay_Kachrani]
 * 
 * Revision 1.1.76.46  1995/06/15  16:27:16  James_Woodward
 * 	remove nxm_lock
 * 	[1995/06/14  12:56:20  James_Woodward]
 * 
 * Revision 1.1.165.3  1995/06/13  19:11:56  David_Winchell
 * 	Include vm_rm_object lock under RMVM ifdef.
 * 	[1995/06/13  19:07:08  David_Winchell]
 * 
 * Revision 1.1.99.18  1995/06/13  19:00:08  David_Winchell
 * 	Lower the vm_map_hint lock in the hierarchy and add a new entry for rm
 * 	objects, vm_rm_object.ob.
 * 	[1995/06/13  18:35:40  David_Winchell]
 * 
 * Revision 1.1.76.45  1995/06/12  19:45:17  Jeff_Denham
 * 	Remove task->thread_list_lock.
 * 	[1995/06/09  19:24:21  Jeff_Denham]
 * 
 * Revision 1.1.76.44  1995/06/08  20:48:08  Diane_Lebel
 * 	Moved rnode_table_lock below sock_lock.
 * 	[1995/06/08  16:45:46  Diane_Lebel]
 * 
 * Revision 1.1.165.2  1995/06/08  20:23:50  David_Winchell
 * 	Lower the vm_map_hint lock in the hierarchy and add a new entry for rm
 * 	objects, vm_rm_ibject.ob.
 * 	[1995/06/08  19:14:31  David_Winchell]
 * 
 * Revision 1.1.76.43  1995/06/05  11:26:31  Tim_Mark
 * 	Changed the name of one of the probes in msfs/bs/bs_stripe.c
 * 	Fix for QAR 31793.
 * 	[1995/06/02  18:27:27  Tim_Mark]
 * 
 * Revision 1.1.76.42  1995/06/01  00:27:38  Diane_Lebel
 * 	Added rnode complex lock r_rwlock.
 * 	[1995/05/30  23:27:34  Diane_Lebel]
 * 
 * Revision 1.1.76.41  1995/05/30  17:36:38  Paul_Shaughnessy
 * 	Added BfsDeleteLock lockinfo struct
 * 	[1995/05/17  18:46:06  Paul_Shaughnessy]
 * 
 * Revision 1.1.76.40  1995/05/24  13:09:11  James_Woodward
 * 	merge of Platinum Lite BL4 changes
 * 	[1995/05/23  19:46:05  James_Woodward]
 * 
 * Revision 1.1.76.39  1995/04/28  20:13:51  Diane_Lebel
 * 	Added locks for NFS client.
 * 	[1995/04/27  23:28:44  Diane_Lebel]
 * 
 * Revision 1.1.76.38  1995/04/28  19:54:13  James_Smart
 * 	Add ddr locks.
 * 	[1995/04/27  11:45:25  James_Smart]
 * 
 * Revision 1.1.76.37  1995/04/28  19:23:28  Blue_Tabor
 * 	Turn off RT_SEM code.  Binary semaphores are no longer supported.
 * 	[1995/04/27  13:36:04  Blue_Tabor]
 * 
 * Revision 1.1.76.36  1995/04/28  16:43:08  Jack_McCann
 * 	merge for submit
 * 	[1995/04/28  16:11:44  Jack_McCann]
 * 
 * 	Path MTU Discovery - add pmtu_lock
 * 	[1995/04/26  22:33:19  Jack_McCann]
 * 
 * Revision 1.1.76.35  1995/04/27  22:20:39  Brian_Tsao
 * 	Ran bmerge.
 * 	[1995/04/27  22:15:39  Brian_Tsao]
 * 
 * 	Added Pmsfs_unmount, Padvfs_mountfs probes and
 * 	deleted obsolete Ppage_error_invalidate probe.
 * 	[1995/04/27  20:58:31  Brian_Tsao]
 * 
 * Revision 1.1.76.34  1995/04/27  16:36:17  James_Woodward
 * 	Add nxm_lock to support 2-level scheduling.
 * 	[1995/04/26  15:57:12  James_Woodward]
 * 
 * Revision 1.1.76.33  1995/04/24  21:04:24  Eric_Werme
 * 	RPC: Change chlist lock to simple lock.
 * 	NFS: Add tslist lock.
 * 	[1995/04/24  18:24:09  Eric_Werme]
 * 
 * Revision 1.1.76.32  1995/04/24  18:56:03  Rudy_Cano
 * 	Added lockinfo stuctures for all converted AdvFS FTX locks and remaining AdvFS mutexes.
 * 	[1995/04/24  07:39:10  Rudy_Cano]
 * 
 * Revision 1.1.99.17  1995/04/21  03:08:04  Brian_Stevens
 * 	Merge in of new drd lock.
 * 	[1995/04/20  04:09:42  Brian_Stevens]
 * 
 * Revision 1.1.76.31  1995/04/19  23:49:43  Dennis_Doherty
 * 		Commented out probe references to functions that have been
 * 		removed.   The functions are return_bsacc_refcnt and
 * 		access_test.
 * 	[1995/04/07  20:15:56  Dennis_Doherty]
 * 
 * Revision 1.1.99.16  1995/04/19  16:49:57  Robin_Lewis
 * 	Fixed type-o, changed NRPTY to the coorect value RPTY
 * 	[1995/04/19  14:38:49  Robin_Lewis]
 * 
 * Revision 1.1.76.30  1995/04/17  19:54:45  James_Smart
 * 	Add kio locks
 * 	[1995/04/17  19:49:44  James_Smart]
 * 
 * Revision 1.1.76.29  1995/04/17  19:01:37  Robin_Lewis
 * 	Changed #if NRPTY > 0   to   #if RPTY
 * 	[1995/04/17  18:48:01  Robin_Lewis]
 * 
 * Revision 1.1.99.15  1995/04/06  17:59:06  James_Woodward
 * 	add missing includes for scsi, lsm, and xtiso subsystems.
 * 	[1995/04/03  19:07:46  James_Woodward]
 * 
 * Revision 1.1.76.28  1995/04/06  15:07:23  Ann_Milicia
 * 	bmerge to latest
 * 	[1995/04/05  14:17:57  Ann_Milicia]
 * 
 * 	Changing procfs.pr_trace_lock to simple lock for QAR 29804.
 * 	[1995/04/04  14:00:26  Ann_Milicia]
 * 
 * Revision 1.1.76.27  1995/03/31  16:39:39  James_Woodward
 * 	merge of Platinum Lite BL2 changes
 * 	[1995/03/31  16:32:38  James_Woodward]
 * 
 * Revision 1.1.99.14  1995/03/27  21:15:04  Geraldine_Harter
 * 	Positions of the unp_mutex and chtabhead locks need to be reversed
 * 	[1995/03/27  18:55:49  Geraldine_Harter]
 * 
 * Revision 1.1.99.13  1995/03/27  19:46:08  William_Burns
 * 	Added shared interrupt locks.
 * 	[1995/03/23  13:06:37  William_Burns]
 * 
 * Revision 1.1.76.26  1995/03/21  14:52:31  Jin_Chen
 * 	Add the lock for callback_list
 * 	[1995/03/17  16:19:57  Jin_Chen]
 * 
 * Revision 1.1.76.25  1995/03/21  14:29:46  Jie_Yao
 * 	Fix lockinfo's for procfs
 * 	[1995/03/20  19:25:08  Jie_Yao]
 * 
 * Revision 1.1.76.24  1995/03/18  23:44:38  Arti_Kumar
 * 	bmerge to latest
 * 	[1995/03/18  23:39:04  Arti_Kumar]
 * 
 * 	bci again
 * 	[1995/03/17  22:19:50  Arti_Kumar]
 * 
 * 	Ric: Add ULI call for chlist_lock
 * 	[1995/03/08  23:15:19  Arti_Kumar]
 * 
 * Revision 1.1.99.12  1995/03/13  16:47:29  Ashoke_Rampuria
 * 	Added contig_malloc_lock.
 * 	[1995/03/11  16:01:41  Ashoke_Rampuria]
 * 
 * Revision 1.1.99.11  1995/03/09  19:00:16  Brian_Stevens
 * 	DRD, RMVM, DLM updates.
 * 	[1995/03/08  03:15:46  Brian_Stevens]
 * 
 * Revision 1.1.76.23  1995/03/07  23:26:03  Jeff_Denham
 * 	Remove sig lockinfo.
 * 	[1995/03/06  20:26:03  Jeff_Denham]
 * 
 * Revision 1.1.88.18  1995/03/02  22:16:28  Maria_Vella
 * 	Changed priority level for drd_retry_que_lock.
 * 	[1995/03/02  21:55:08  Maria_Vella]
 * 
 * Revision 1.1.88.17  1995/02/28  22:08:37  David_Cherkus
 * 	Improved SMP lockinfo entries for DRD
 * 	[1995/02/28  22:06:32  David_Cherkus]
 * 
 * Revision 1.1.76.22  1995/02/28  22:05:28  Paula_Long
 * 	Merging PTLITEOS.BL1 to PTOS
 * 	[1995/02/28  21:54:29  Paula_Long]
 * 
 * Revision 1.1.88.16  1995/02/27  22:57:22  David_Cherkus
 * 	Improved RM allocation code, plus SMP lockinfo fixes.
 * 	[1995/02/27  22:51:39  David_Cherkus]
 * 
 * Revision 1.1.88.15  1995/02/27  21:53:50  Maria_Vella
 * 	Added DRD bsc_rm_lock and bss_rm_pend_lock spin locks.
 * 	[1995/02/27  21:36:16  Maria_Vella]
 * 
 * Revision 1.1.99.10  1995/02/24  18:16:58  Ronald_Menner
 * 	associate splimp with dlb_lock
 * 	[1995/02/24  18:10:52  Ronald_Menner]
 * 
 * Revision 1.1.99.9  1995/02/17  21:34:55  Jeff_Denham
 * 	Add lockinfo for aio super bufs.
 * 	[1995/02/17  18:00:53  Jeff_Denham]
 * 
 * Revision 1.1.76.21  1995/02/17  01:32:25  Brian_Tsao
 * 	Bmerge.
 * 	[1995/02/17  00:45:34  Brian_Tsao]
 * 
 * 	Added new probes.
 * 	[1995/02/16  23:38:35  Brian_Tsao]
 * 
 * Revision 1.1.88.14  1995/02/11  18:15:00  Brian_Stevens
 * 	Merge to PTLITEOS BL1.
 * 	[1995/02/11  17:58:41  Brian_Stevens]
 * 
 * Revision 1.1.99.8  1995/02/08  20:37:04  James_Woodward
 * 	convert system V semaphores to use simple locks
 * 	[1995/02/07  18:00:50  James_Woodward]
 * 
 * Revision 1.1.76.20  1995/02/08  16:14:20  Brian_Tsao
 * 	Merge recent changes.
 * 	[1995/02/03  22:48:13  Brian_Tsao]
 * 
 * 	Add initial probes.
 * 	[1995/02/03  22:42:19  Brian_Tsao]
 * 
 * Revision 1.1.99.7  1995/02/07  20:44:29  Ashoke_Rampuria
 * 	Removed heap allocator locks.
 * 	[1995/01/31  14:43:21  Ashoke_Rampuria]
 * 
 * Revision 1.1.76.19  1995/02/03  21:51:18  Dennis_Doherty
 * 	Added first order lock probes for AdvFS.
 * 	[1995/02/03  21:26:01  Dennis_Doherty]
 * 
 * Revision 1.1.76.18  1995/02/03  18:31:19  Mike_Brady
 * 	added more PROBE locks
 * 	[1995/02/03  00:09:54  Mike_Brady]
 * 
 * Revision 1.1.99.6  1995/01/30  22:48:13  Ronald_Menner
 * 	removed tmtab_tb_lock (not used)
 * 	[1995/01/30  20:34:01  Ronald_Menner]
 * 
 * Revision 1.1.99.5  1995/01/30  19:48:24  Ashoke_Rampuria
 * 	vm_page_free_lock is now acquired at splhigh.
 * 	[1995/01/30  19:22:07  Ashoke_Rampuria]
 * 
 * 	Removed zone locks. Changed malloc locks spl level
 * 	to splhigh. Changed binlog_selq_lock spl level to splnone.
 * 	[1995/01/27  21:24:08  Ashoke_Rampuria]
 * 
 * Revision 1.1.76.16  1995/01/30  19:39:01  Rudy_Cano
 * 	Changed four advfs complex locks into OSF locks and added four lock classes
 * 	for four spin locks. The ULI lockinfo lines were added to this file for
 * 	those mentioned above.
 * 	[1995/01/28  01:38:40  Rudy_Cano]
 * 
 * Revision 1.1.76.17  1995/02/01  22:39:01  Dennis_Doherty
 * 	Added first order probes for SMP testing.
 * 	[1995/02/01  20:46:13  Dennis_Doherty]
 * 
 * Revision 1.1.99.4  1995/01/25  20:59:04  Paula_Long
 * 	Merging in goldminos.bl5 changes into to ptliteos
 * 	[1995/01/25  14:10:51  Paula_Long]
 * 
 * Revision 1.1.99.3  1995/01/24  01:49:26  Paula_Long
 * 	Merging in ClusterWave3 Base System Enablers
 * 	[1995/01/24  01:20:04  Paula_Long]
 * 
 * Revision 1.1.88.8  1995/01/20  21:19:31  David_Winchell
 * 	Added  a lock for rm dual window support.
 * 	[1995/01/20  19:58:49  David_Winchell]
 * 
 * Revision 1.1.99.2  1995/01/19  23:47:15  Paula_Long
 * 	Backed out probe points and lk_cred_busy lock for ptos.lite
 * 	[1995/01/17  02:57:38  Paula_Long]
 * 
 * Revision 1.1.88.7  1995/01/19  20:42:48  Maria_Vella
 * 	Added locks for drd strategy retry handling.
 * 	[1995/01/19  20:23:06  Maria_Vella]
 * 
 * Revision 1.1.76.15  1995/01/19  14:57:28  Paula_Long
 * 	merge of Gold Minor BL5 changes
 * 	[1995/01/19  14:45:45  Paula_Long]
 * 
 * Revision 1.1.88.6  1995/01/13  03:34:09  Brian_Stevens
 * 	Conditional compile of vm support for rm.
 * 	[1995/01/12  21:39:49  Brian_Stevens]
 * 
 * Revision 1.1.53.32  1995/01/11  22:10:58  John_Meneghini
 * 	Rename cam_pd_unit_table_li entry to cam_pd_unit_li.
 * 	Re-add cam_pd_unit_tabel_li as a simple lock to maintain
 * 	binary compatability with third party drivers.
 * 	[1995/01/10  21:15:46  John_Meneghini]
 * 
 * Revision 1.1.88.5  1995/01/09  13:54:36  Timothy_Burke
 * 	Added include of drd.h.
 * 	[1995/01/09  13:10:13  Timothy_Burke]
 * 
 * Revision 1.1.76.14  1995/01/04  17:44:05  Mike_Brady
 * 	Merge
 * 	[1995/01/04  17:42:40  Mike_Brady]
 * 
 * 	Added SMP probe stuff for msfs/fs/fs_dir_init.c
 * 	[1995/01/03  21:48:19  Mike_Brady]
 * 
 * Revision 1.1.76.13  1995/01/03  23:11:06  Robin_Lewis
 * 	bmerge of code
 * 	[1994/12/19  19:53:04  Robin_Lewis]
 * 
 * 	code change to make streams a real kernel option that comes
 * 	     &  goes in the kernel
 * 	[1994/12/19  12:46:10  Robin_Lewis]
 * 
 * Revision 1.1.88.4  1994/12/16  20:38:43  David_Winchell
 * 	Added VM support for RM user mappings.
 * 	[1994/12/16  19:14:34  David_Winchell]
 * 
 * Revision 1.1.76.12  1994/12/16  15:36:47  Ashoke_Rampuria
 * 	vm_page_wire_lock is now acquired at spldevhigh.
 * 	[1994/12/15  18:40:59  Ashoke_Rampuria]
 * 
 * 	vm_page_free_lock is now acquired at spldevhigh.
 * 	[1994/12/15  18:39:52  Ashoke_Rampuria]
 * 
 * Revision 1.1.88.3  1994/12/13  22:02:11  Timothy_Burke
 * 	Added locks for DRD (Distributed Raw Disk).
 * 	[1994/12/13  20:32:16  Timothy_Burke]
 * 
 * Revision 1.1.53.31  1994/12/13  12:06:30  James_Woodward
 * 	remove unneeded rpc cred lock
 * 	[1994/12/02  17:07:04  James_Woodward]
 * 
 * Revision 1.1.76.11  1994/12/09  07:39:26  Tim_Hoskins
 * 	merge of Gold Minor BL4 changes
 * 	[1994/12/09  06:30:17  Tim_Hoskins]
 * 
 * Revision 1.1.76.10  1994/11/29  16:00:15  Ashoke_Rampuria
 * 	Lowered the priority for heap_lock below vm_kmap lock.
 * 	[1994/11/28  15:53:20  Ashoke_Rampuria]
 * 
 * Revision 1.1.76.9  1994/11/29  00:28:31  Rudy_Cano
 * 	For AdvFS, added "#if MSFS > 0" around probe locks code (i.e.
 * 	      ULI_PROBE lines)
 * 	to prevent errors when building kernel w/o AdvFS.
 * 	[1994/11/28  23:16:29  Rudy_Cano]
 * 
 * Revision 1.1.76.8  1994/11/23  21:50:21  Shashi_Mangalat
 * 	Remove unused ubc_dead_object_lockinfo.
 * 	[1994/11/23  19:19:55  Shashi_Mangalat]
 * 
 * Revision 1.1.53.30  1994/11/22  18:32:25  Naresh_Dharnidharka
 * 	merge after Robin's submit
 * 	[1994/11/22  18:22:54  Naresh_Dharnidharka]
 * 
 * 	merger mania
 * 	[1994/11/21  23:45:40  Naresh_Dharnidharka]
 * 
 * 	Change DECTLASER to DEC21000
 * 	[1994/11/21  22:44:31  Naresh_Dharnidharka]
 * 
 * Revision 1.1.76.7  1994/11/22  00:46:07  Rudy_Cano
 * 	Added Initial Probe Locks for file msfs/osf/msfs_vnops.c
 * 	[1994/11/22  00:43:57  Rudy_Cano]
 * 
 * 	Added probe locks for AdvFS routines in msfs_vnops.c
 * 	[1994/11/19  00:09:37  Rudy_Cano]
 * 
 * Revision 1.1.53.29  1994/11/22  00:40:17  Ronald_Menner
 * 	Bmerged CAM, pcia_ctl_lock, and qlogic changes
 * 	[1994/11/22  00:37:53  Ronald_Menner]
 * 
 * 	Added dltimer_lock for the MS dlpi subsystem
 * 	[1994/11/22  00:32:34  Ronald_Menner]
 * 
 * Revision 1.1.53.28  1994/11/21  21:54:09  John_Meneghini
 * 	Bug fix QAR 25590 - Change cam_pd_unit_table lock from simple to
 * 	                    complex lock.
 * 	[1994/11/18  17:25:36  John_Meneghini]
 * 
 * Revision 1.1.53.27  1994/11/21  21:07:26  Brian_Nadeau
 * 	Add qlogic isp1020 (isp) to the lock hierarchy.
 * 	[1994/11/21  20:52:16  Brian_Nadeau]
 * 
 * Revision 1.1.53.26  1994/11/21  16:51:23  Naresh_Dharnidharka
 * 	merge before submit
 * 	[1994/11/21  16:49:46  Naresh_Dharnidharka]
 * 
 * 	Add pcia_ctl_lock to hierarchy.
 * 	[1994/11/17  20:16:46  Naresh_Dharnidharka]
 * 
 * Revision 1.1.53.25  1994/11/17  20:59:56  Ronald_Menner
 * 	Added dlbq lock
 * 	[1994/11/17  20:28:25  Ronald_Menner]
 * 
 * Revision 1.1.76.6  1994/11/10  22:52:51  Hsin_Lee
 * 	Add probe locks for AdvFS routines in fs_fset_create.c and msfs_misc.c.
 * 	[1994/11/10  22:04:34  Hsin_Lee]
 * 
 * Revision 1.1.76.5  1994/11/08  17:39:08  Andrew_Duane
 * 	Removed #include "msi.h"
 * 	[1994/11/02  16:36:04  Andrew_Duane]
 * 
 * Revision 1.1.53.24  1994/11/07  15:10:17  Janet_Schank
 * 	Fixed a typo in cam_spo_debug lock.
 * 	[1994/11/03  13:45:43  Janet_Schank]
 * 
 * Revision 1.1.53.23  1994/11/03  21:24:00  Jeff_Denham
 * 	Add sys_exitp_lockinfo to base of complex lock hierarchy.
 * 	[1994/11/03  18:46:12  Jeff_Denham]
 * 
 * Revision 1.1.53.22  1994/11/03  20:28:13  John_Meneghini
 * 	Bug fix QAR 26405 - added simple lock 'error_event_lock' to hierachy.
 * 	[1994/11/01  14:00:17  John_Meneghini]
 * 
 * Revision 1.1.76.4  1994/11/02  16:26:53  Stan_Luke
 * 	Added global 'probe_smp' variable for ULI_PROBE.
 * 	[1994/10/26  15:18:21  Stan_Luke]
 * 
 * Revision 1.1.76.3  1994/10/29  02:43:09  Tim_Hoskins
 * 	merge of Gold Minor BL3 changes
 * 	[1994/10/29  02:39:02  Tim_Hoskins]
 * 
 * Revision 1.1.76.2  1994/10/21  21:53:59  Paula_Long
 * 	added code to support lock_probe_points
 * 	[1994/10/21  17:11:19  Paula_Long]
 * 
 * Revision 1.1.53.21  1994/10/17  18:32:54  Shashi_Mangalat
 * 	Enable lock hierarchy checking for akl_slock.
 * 	[1994/10/17  15:01:57  Shashi_Mangalat]
 * 
 * Revision 1.1.53.20  1994/10/04  15:47:49  James_Smart
 * 	Removed reference to 810 buffer pool lock (cam_pfree_buf_li)
 * 	[1994/09/16  15:06:02  James_Smart]
 * 
 * Revision 1.1.53.19  1994/10/03  19:56:08  Patrick_ORourke
 * 	Merge before bsubmit
 * 	[1994/10/03  19:43:51  Patrick_ORourke]
 * 
 * 	Bmerge to latest.
 * 	[1994/09/28  19:12:00  Patrick_ORourke]
 * 
 * 	Added hierarchy for pmap_memdsc_lock.
 * 	[1994/09/26  23:19:42  Patrick_ORourke]
 * 
 * Revision 1.1.53.18  1994/10/03  14:37:00  Janet_Schank
 * 	Changed names of SimPort locks to use the same naming
 * 	convention used by other CAM drivers.  Changed the heirarchy
 * 	of two of the SimPort locks.
 * 	[1994/09/29  13:53:03  Janet_Schank]
 * 
 * Revision 1.1.53.17  1994/09/22  12:01:53  James_Woodward
 * 	merge of Hardware BL4 changes
 * 	[1994/09/22  11:04:40  James_Woodward]
 * 
 * 		ipl6 requeue mechinism using lwc logic
 * 		[1994/08/26  15:36:32  James_Woodward]
 * 
 * 		Added hierarchy for SIMport locks.
 * 		[1994/08/23  13:07:45  Janet_Schank]
 * 
 * Revision 1.1.53.16  1994/09/13  19:53:16  Ashoke_Rampuria
 * 	Changed spl level for vm_page_free_lock.
 * 	[1994/09/13  19:47:52  Ashoke_Rampuria]
 * 
 * Revision 1.1.53.15  1994/09/12  19:02:51  Rajul_Shah
 * 	Eliminated fta's (complex) command buffer q lock and added
 * 	a simple softc lock as is done with rest of the netdrivers.
 * 	[1994/09/08  11:56:45  Rajul_Shah]
 * 
 * Revision 1.1.53.14  1994/09/12  18:49:45  William_Burns
 * 	Added dma scatter/gather resource map lock.
 * 	[1994/09/01  13:18:46  William_Burns]
 * 
 * Revision 1.1.53.13  1994/09/12  12:55:01  Janet_Schank
 * 	Added hierarchy for SIMport locks.
 * 	[1994/09/07  20:40:13  Janet_Schank]
 * 
 * Revision 1.1.53.12  1994/09/10  19:34:22  Anton_Verhulst
 * 	Make processor callout and processor pset locks ordered.
 * 	[1994/09/07  17:06:00  Anton_Verhulst]
 * 
 * Revision 1.1.53.11  1994/09/09  21:09:10  Paul_Shaughnessy
 * 	AdvFS ASMP support
 * 	[1994/09/09  13:58:07  Paul_Shaughnessy]
 * 
 * Revision 1.1.53.10  1994/09/07  18:23:28  Geraldine_Harter
 * 	Fix socklocks/vm_umap & chtabhead lock hierarchy violations
 * 	[1994/09/06  19:48:43  Geraldine_Harter]
 * 
 * Revision 1.1.53.9  1994/09/01  21:49:21  Ronald_Menner
 * 	Add minimal spls levels to streams_log_lock, mblk_lock, osrq_lock,
 * 	q.lock, and sqh_lock.  And adjust posotion in the heirarchy
 * 	accordingly.
 * 	[1994/08/31  19:20:54  Ronald_Menner]
 * 
 * Revision 1.1.53.8  1994/08/25  18:20:25  James_Woodward
 * 	make callout queues parallel
 * 	[1994/08/25  10:32:28  James_Woodward]
 * 
 * 	changes for parallel callout queues
 * 	[1994/08/22  12:25:22  James_Woodward]
 * 
 * Revision 1.1.53.7  1994/08/25  15:25:03  James_Woodward
 * 	performance clean-up of nfs server code
 * 	[1994/08/25  11:05:46  James_Woodward]
 * 
 * Revision 1.1.53.6  1994/08/24  19:06:56  Geraldine_Harter
 * 	Fix socklocks/pgrphash lock hierarchy violation
 * 	(socklock.sock_lock must be taken before pgrphash_lock)
 * 	[1994/08/24  14:12:33  Geraldine_Harter]
 * 
 * Revision 1.1.53.5  1994/08/23  22:27:06  Julia_Fey
 * 	add dlb_slock for dlb.
 * 	[1994/08/22  22:44:47  Julia_Fey]
 * 
 * Revision 1.1.53.4  1994/08/18  16:46:07  Ashoke_Rampuria
 * 	merge of Hardware BL3 changes
 * 	[1994/08/18  15:25:33  Ashoke_Rampuria]
 * 
 * Revision 1.1.53.3  1994/08/03  21:27:02  Ashoke_Rampuria
 * 	Backed out of last change.
 * 	[1994/08/03  21:20:28  Ashoke_Rampuria]
 * 
 * Revision 1.1.53.2  1994/08/03  20:52:40  Ashoke_Rampuria
 * 	Added last_type_lockinfo for malloc. Removed ktp_kt_lockinfo.
 * 	[1994/08/03  18:37:20  Ashoke_Rampuria]
 * 
 * Revision 1.1.2.126  1994/07/25  20:11:02  Larry_Woodman
 * 	Added the ubc_dead_object_lock.
 * 	[1994/07/19  22:08:31  Larry_Woodman]
 * 
 * Revision 1.1.2.125  1994/07/01  18:15:54  Geraldine_Harter
 * 	Add unp_mutex_lock for unix domain sockets
 * 	[1994/06/30  16:03:23  Geraldine_Harter]
 * 
 * Revision 1.1.2.124  1994/06/23  18:22:15  Shashi_Mangalat
 * 	Add hierarchy and ipl for malloc locks.
 * 	[1994/06/23  16:09:26  Shashi_Mangalat]
 * 
 * Revision 1.1.2.123  1994/06/21  20:22:52  Brian_Nadeau
 * 	Remove DSA lockinfo.
 * 	[1994/06/21  20:14:48  Brian_Nadeau]
 * 
 * Revision 1.1.2.122  1994/06/20  19:15:45  Tim_Hoskins
 * 	Move definition of lockmode to here so that SAS
 * 	can have a different initialized value.
 * 	[1994/06/20  17:15:29  Tim_Hoskins]
 * 
 * Revision 1.1.2.121  1994/06/17  18:17:29  Ronald_Menner
 * 	spl for str_to_lock changed to SPLDEVHIGH inorder
 * 	to allow calls to streams_timeout in interrupt context.
 * 	[1994/06/17  11:07:28  Ronald_Menner]
 * 
 * Revision 1.1.2.120  1994/06/14  15:20:07  Eric_Werme
 * 	Add a lock for RPC client handle list control.
 * 	[1994/06/13  19:51:50  Eric_Werme]
 * 
 * Revision 1.1.2.119  1994/06/09  11:54:17  Mary_Walker
 * 	made str_to_lock splsoftc
 * 	[1994/06/08  14:36:55  Mary_Walker]
 * 
 * Revision 1.1.2.118  1994/06/07  17:52:55  Mary_Walker
 * 	comment out str_to_lock for now - need to not use SIMPLE_LOCK when
 * 	taking this lock
 * 	[1994/06/07  17:45:43  Mary_Walker]
 * 
 * Revision 1.1.2.117  1994/06/03  14:56:42  Mary_Walker
 * 	moved str_to_lock and made it SPLHIGH
 * 	[1994/06/03  11:44:43  Mary_Walker]
 * 
 * Revision 1.1.2.116  1994/05/31  20:21:36  Mike_Larson
 * 	Add ASE/AM lock information.
 * 	[1994/05/31  20:19:54  Mike_Larson]
 * 
 * Revision 1.1.2.115  1994/05/30  00:01:21  Heather_Gray
 * 	Arptab lock changed from complex to simple.
 * 	[1994/05/26  20:09:16  Heather_Gray]
 * 
 * Revision 1.1.2.114  1994/05/27  20:30:17  Eric_Werme
 * 	Delete mh_ref_lock_info data, no longer used.
 * 	[1994/05/27  14:41:11  Eric_Werme]
 * 
 * Revision 1.1.2.113  1994/05/26  21:33:50  Geraldine_Harter
 * 	Rename the ifaddr lock
 * 	[1994/05/26  18:04:08  Geraldine_Harter]
 * 
 * Revision 1.1.2.112  1994/05/26  13:36:15  Farrell_Woods
 * 	Move packetfilter lock to the other side of the various
 * 	netdriver softc locks in order to prevent heirarchy
 * 	violations when the packetfilter is used.  Also change
 * 	the packetfilter's IPL from SPLNET to SPLIMP since
 * 	the packetfilter does its critical processing at splimp
 * 	already.
 * 	[1994/05/23  23:34:27  Farrell_Woods]
 * 
 * Revision 1.1.2.111  1994/05/23  22:05:21  Shashi_Mangalat
 * 	Ooops, u_addr_lockinfo must be above umap lock!
 * 	[1994/05/23  22:03:25  Shashi_Mangalat]
 * 
 * Revision 1.1.2.110  1994/05/20  18:07:11  Ronald_Menner
 * 	Moved bufc_lock (bufcall lock) above qactive_lock
 * 	[1994/05/19  01:02:50  Ronald_Menner]
 * 
 * Revision 1.1.2.109  1994/05/18  18:52:19  Shashi_Mangalat
 * 	Move u_map_lockinfo before pgrp_lockinfo.
 * 	Disable hierarchy checking of akl_slock_lockinfo.
 * 	[1994/05/17  21:09:09  Shashi_Mangalat]
 * 
 * Revision 1.1.2.108  1994/05/16  21:01:26  Geraldine_Harter
 * 	Add lockinfo for ifaddr quick access lock
 * 	[1994/05/12  22:44:02  Geraldine_Harter]
 * 
 * Revision 1.1.2.107  1994/05/16  18:52:26  Carol_Sheridan
 * 	Removed cblock_freelist_lock from lockinfo
 * 	specs for simple locks since lock has been
 * 	removed from tty driver.
 * 	[1994/05/12  21:37:13  Carol_Sheridan]
 * 
 * Revision 1.1.2.106  1994/05/11  18:00:19  Donald_Dutile
 * 	Change/fix T2_hae0_3_li conditional on DEC2100_A500
 * 	not NPCI.
 * 	[1994/05/11  17:59:27  Donald_Dutile]
 * 
 * Revision 1.1.2.105  1994/05/10  23:36:24  Jeff_Denham
 * 	Move process group signal lock back down below proc lock
 * 	now that other adjustments have been made.
 * 	[1994/05/10  17:44:17  Jeff_Denham]
 * 
 * Revision 1.1.2.104  1994/05/10  21:26:52  Donald_Dutile
 * 	Add lockinfo for SABLE HAE/PCI Config TYPE1 accesses.
 * 	[1994/05/09  20:31:31  Donald_Dutile]
 * 
 * Revision 1.1.2.103  1994/05/09  21:48:10  Jeff_Denham
 * 	Move process group signal lock below proc lock.
 * 	[1994/05/09  21:47:10  Jeff_Denham]
 * 
 * Revision 1.1.2.102  1994/05/06  14:17:52  Andrew_Duane
 * 	Fix #ifdef LDTTY to #if to fix build.
 * 	Moved xtiso_lockinfo and xtisoc_lockinfo to #if XTISO
 * 	[1994/05/06  14:17:31  Andrew_Duane]
 * 
 * Revision 1.1.2.101  1994/05/05  19:28:22  Mary_Walker
 * 	added STREAMS locking hierarchy
 * 	[1994/05/05  19:17:14  Mary_Walker]
 * 
 * Revision 1.1.2.100  1994/05/05  18:38:14  Shashi_Mangalat
 * 	Add spl and hierarchy to anon_klock.
 * 	[1994/05/05  18:31:39  Shashi_Mangalat]
 * 
 * Revision 1.1.2.99  1994/05/04  22:09:15  Jeff_Denham
 * 	Add pgrp_sig_lockinfo.
 * 	[1994/04/29  20:45:51  Jeff_Denham]
 * 
 * Revision 1.1.2.98  1994/04/25  22:38:47  Shashi_Mangalat
 * 	Add spl and hierarchy to pmap locks.
 * 	[1994/04/25  22:27:30  Shashi_Mangalat]
 * 
 * Revision 1.1.2.97  1994/04/25  22:12:50  Heather_Gray
 * 	Match some stat lock names with the global names.
 * 	[1994/04/25  22:09:57  Heather_Gray]
 * 
 * Revision 1.1.2.96  1994/04/25  21:54:17  Eric_Werme
 * 	Fix hierarchy for udpms_lock.
 * 	[1994/04/25  20:18:48  Eric_Werme]
 * 
 * Revision 1.1.2.95  1994/04/25  19:41:22  Heather_Gray
 * 	Add infos for multicast routing locks.
 * 	[1994/04/25  17:50:39  Heather_Gray]
 * 
 * Revision 1.1.2.94  1994/04/23  22:09:23  Ernie_Petrides
 * 	Add LDTTY conditional around "ldtty.t_intimeout_lock" lockinfo.
 * 	[1994/04/23  22:07:15  Ernie_Petrides]
 * 
 * Revision 1.1.2.93  1994/04/22  19:42:42  Anton_Verhulst
 * 	Move lk_lmf after the socket locks.
 * 	[1994/04/22  19:40:22  Anton_Verhulst]
 * 
 * Revision 1.1.2.92  1994/04/22  13:08:15  Eric_Werme
 * 	Add entry for udpms_lock
 * 	[1994/04/21  17:55:20  Eric_Werme]
 * 
 * Revision 1.1.2.91  1994/04/16  02:41:25  Paula_Long
 * 	cleaned/synced up lockinfo changes
 * 	[1994/04/15  06:55:37  Paula_Long]
 * 
 * 	merged with latest revision
 * 	[1994/04/12  08:47:52  Paula_Long]
 * 
 * 	Added additional locks (under NOCHECK)
 * 	[1994/04/12  01:48:10  Paula_Long]
 * 
 * 	Added LI_NOCHECK macro.
 * 	Created, placeholder lockinfo structures for those locks which did not
 * 	have lockinfo structures (no heirarchy or spl checking is done for
 * 	these locks).  This was done so that we could get stats for the locks.
 * 	[1994/04/10  23:33:23  Paula_Long]
 * 
 * Revision 1.1.2.90  1994/04/15  21:33:06  Heather_Gray
 * 	Correct inifaddr through arp ordering.
 * 	[1994/04/15  20:13:32  Heather_Gray]
 * 
 * Revision 1.1.2.89  1994/04/15  15:11:11  Ashoke_Rampuria
 * 	Registered lockinfo for semidq, shmidq and msgidq locks.
 * 	[1994/04/14  20:43:43  Ashoke_Rampuria]
 * 
 * Revision 1.1.2.88  1994/04/11  21:57:50  Julia_Fey
 * 	register lockinfo for srt_thread_lock.
 * 	[1994/04/11  21:15:06  Julia_Fey]
 * 
 * Revision 1.1.2.87  1994/04/08  19:57:48  Eric_Werme
 * 	Add NFS V3 write gathering locks.
 * 	[1994/04/07  19:47:43  Eric_Werme]
 * 
 * Revision 1.1.2.86  1994/04/07  13:15:56  Ernie_Petrides
 * 	Remove lockinfo specs for ip_misc_lock and ipreass.ipr_lock.
 * 	[1994/04/07  13:12:09  Ernie_Petrides]
 * 
 * Revision 1.1.2.85  1994/04/07  13:03:34  Ernie_Petrides
 * 	Change/move netisr_slock hierarchy specification for use at splimp().
 * 	[1994/04/07  12:58:26  Ernie_Petrides]
 * 
 * Revision 1.1.2.84  1994/04/07  11:08:47  Rajul_Shah
 * 	Registered lockinfo for the TGEC, TULIP, DEMNA, and DEMFA
 * 	network drivers.
 * 	[1994/04/01  22:13:44  Rajul_Shah]
 * 
 * Revision 1.1.2.83  1994/04/04  21:16:34  Ernie_Petrides
 * 	Add lockinfo for incore file structure lock.
 * 	[1994/04/04  17:08:51  Ernie_Petrides]
 * 
 * Revision 1.1.2.82  1994/03/24  21:33:01  Geraldine_Harter
 * 	Remove ORDNEXT flag from domain lockinfo. The domain class
 * 	now strictly precedes the socket lock class in the hierarchy.
 * 	Split domain lock in two to support DECnet/OSI & X.25.
 * 	Move dli_input_lock after domain lock.
 * 	[1994/03/24  19:52:10  Geraldine_Harter]
 * 
 * Revision 1.1.2.81  1994/03/24  16:49:06  Shashi_Mangalat
 * 	Moved shmid_lock to below vm_umap_lock.
 * 	[1994/03/24  16:14:57  Shashi_Mangalat]
 * 
 * Revision 1.1.2.80  1994/03/23  18:30:44  Brian_Nadeau
 * 	Add astro to lock hierarchy.
 * 	[1994/03/23  18:28:05  Brian_Nadeau]
 * 
 * Revision 1.1.2.79  1994/03/18  21:50:22  Ronald_Menner
 * 	Move dli_dlif_lock above FAA lock (needs to be above all
 * 	locks associated with a network interface)
 * 	[1994/03/16  22:46:35  Ronald_Menner]
 * 
 * Revision 1.1.2.78  1994/03/03  16:12:37  Shashi_Mangalat
 * 	Split vm_map_lockinfo to k_map_lockinfo and u_map_lockinfo.  Changed
 * 	hostname_lock to simple lock.
 * 	[1994/03/02  20:18:00  Shashi_Mangalat]
 * 
 * Revision 1.1.2.77  1994/03/02  20:51:00  Brian_Nadeau
 * 	Change all CAM sleep locks to spin locks.  Also add SCSI 810
 * 	and AHA 1740 to the lock hierarchy.
 * 	[1994/02/28  21:17:25  Brian_Nadeau]
 * 
 * Revision 1.1.2.76  1994/03/02  02:52:29  Ernie_Petrides
 * 	Move global definition of lockstats_table[] from here to lock.c
 * 	to force allocation of space for the full possible range of NCPUS
 * 	(since lockinfo.c is built according to the tailored configuration).
 * 	[1994/03/01  00:03:52  Ernie_Petrides]
 * 
 * Revision 1.1.2.75  1994/02/28  01:05:29  Ernie_Petrides
 * 	Add lockinfo for bk_lock fields of vm_page_bucket structures.
 * 	[1994/02/27  18:54:13  Ernie_Petrides]
 * 
 * Revision 1.1.2.74  1994/02/22  17:08:41  Shashi_Mangalat
 * 	Removed vm_swap_list_lockinfo and added vm_swap_circular_lockinfo.
 * 	[1994/02/21  22:16:43  Shashi_Mangalat]
 * 
 * Revision 1.1.2.73  1994/02/22  16:24:22  David_Scoda
 * 	redo procfs locking; fixes for PIOCTxxx ioctls
 * 	[1994/02/22  16:15:08  David_Scoda]
 * 
 * Revision 1.1.2.72  1994/02/19  19:25:45  Paula_Long
 * 	Merged with changes for u_timer and cfg
 * 	[1994/02/19  19:01:19  Paula_Long]
 * 
 * 	Added lockinfo for sysdev_write_lock
 * 	[1994/02/18  07:12:57  Paula_Long]
 * 
 * Revision 1.1.2.71  1994/02/18  21:03:59  James_Woodward
 * 	Move u_timer from utask structure to proc.  Combine utask timer lock and
 * 	proc timer lock.
 * 	[1994/02/18  19:23:44  James_Woodward]
 * 
 * Revision 1.1.2.70  1994/02/18  20:06:50  Brian_Stevens
 * 	Change cfg_subsys_lock to be ORDERED.
 * 	[1994/02/18  19:29:00  Brian_Stevens]
 * 
 * Revision 1.1.2.69  1994/02/15  16:05:04  James_Woodward
 * 	cancel itimer signals off indirect psignal queue.
 * 	[1994/02/15  14:27:32  James_Woodward]
 * 
 * Revision 1.1.2.68  1994/02/14  18:12:05  Timothy_Burke
 * 	Removed nfs_lbolt_lock.
 * 	[1994/02/14  13:58:22  Timothy_Burke]
 * 
 * Revision 1.1.2.67  1994/02/11  17:25:13  Geraldine_Harter
 * 	Remove global_arp_lock (unused).
 * 	[1994/02/10  21:34:26  Geraldine_Harter]
 * 
 * Revision 1.1.2.66  1994/01/21  23:35:42  Aju_John
 * 	dynamic vnode alloc-dealloc changes
 * 	[1994/01/21  21:48:06  Aju_John]
 * 
 * Revision 1.1.2.65  1994/01/20  18:23:41  Julia_Fey
 * 	changed DLPI to DLB since sterling's dlpi is called dlb now.
 * 	[1994/01/19  20:11:02  Julia_Fey]
 * 
 * Revision 1.1.2.64  1994/01/11  21:20:54  John_Dustin
 * 	change NPACKETFILTER to PACKETFILTER, reflects option only
 * 	[1994/01/10  22:52:47  John_Dustin]
 * 
 * 	add inp_head locks for udp and tcp
 * 	[1994/01/07  20:28:27  John_Dustin]
 * 
 * Revision 1.1.2.63  1994/01/09  18:39:33  David_Scoda
 * 	unfunnel proc code, change trace locking, misc bug fixes
 * 	[1994/01/09  18:39:07  David_Scoda]
 * 
 * Revision 1.1.2.62  1994/01/05  21:20:01  James_Woodward
 * 	remove p_sdata_lock
 * 	[1994/01/05  21:05:51  James_Woodward]
 * 
 * Revision 1.1.2.61  1994/01/05  12:36:28  Brian_Stevens
 * 	Initial Gold support for loadable modules.
 * 	[1994/01/05  12:35:47  Brian_Stevens]
 * 
 * Revision 1.1.2.60  1994/01/04  18:25:11  Jeff_Denham
 * 	Merge to latest.
 * 	[1994/01/04  18:21:52  Jeff_Denham]
 * 
 * 	Remove ORDNEXT flag from wait queue lockinfo. The wait queue class
 * 	now strictly precedes the thread lock class in the hierarchy.
 * 	[1994/01/03  14:58:30  Jeff_Denham]
 * 
 * Revision 1.1.2.59  1994/01/03  19:11:52  John_Dustin
 * 	move hostname_lock after socket locks to avoid hierarchy violation
 * 	[1993/12/23  15:49:45  John_Dustin]
 * 
 * Revision 1.1.2.58  1993/12/30  17:07:12  James_Woodward
 * 	remove quatum adj lock.  change ordering timer lock
 * 	[1993/12/21  13:15:45  James_Woodward]
 * 
 * Revision 1.1.2.57  1993/12/14  22:35:37  John_Dustin
 * 	resolve ifqueue and fxx_intr ordering problems
 * 	[1993/12/14  21:46:20  John_Dustin]
 * 
 * Revision 1.1.2.56  1993/12/11  22:03:54  Shashi_Mangalat
 * 	Added u_seg_cache_lockinfo, vop_seglockinfo and h_kmem_wanted_lockinfo.
 * 	[1993/12/11  22:02:23  Shashi_Mangalat]
 * 
 * Revision 1.1.2.55  1993/12/06  23:04:22  Stan_Luke
 * 	Oops, dropped global_raw_lock in my merging, for shame, for shame.
 * 	[1993/12/06  22:52:57  Stan_Luke]
 * 
 * Revision 1.1.2.54  1993/12/06  22:30:44  Stan_Luke
 * 	Moved socklocks back to original place in locking hierarchy;
 * 	disabled hierarchy checking for BL8 spin for i_io_lock,
 * 	because of confict:
 * 		i_io_lock > nfs_daemon_lock	(i.e. ufs_lock)
 * 		sock_lock > i_io_lock		(i.e. vrele)
 * 	but:	nfs_daemon_lock > sock_lock	(i.e. rfs_writeg)
 * 	[1993/12/06  22:14:02  Stan_Luke]
 * 
 * Revision 1.1.2.53  1993/12/06  22:13:05  John_Dustin
 * 	take out the inp_head locks for tcb and udb until we get order straight
 * 	[1993/12/06  21:59:59  John_Dustin]
 * 
 * 	add remaining 22 lockinfo network locks
 * 	[1993/12/06  21:00:57  John_Dustin]
 * 
 * Revision 1.1.2.52  1993/12/03  15:59:07  Stan_Luke
 * 	Moved up socklocks higher than i_io_lock since soo_close() can call
 * 	through to ufs_inactive() (via vrele).
 * 	[1993/12/02  20:07:49  Stan_Luke]
 * 
 * Revision 1.1.2.51  1993/12/02  21:59:48  Brian_Nadeau
 * 	Change all cam locks to be splbio.
 * 	[1993/12/02  21:58:17  Brian_Nadeau]
 * 
 * Revision 1.1.2.50  1993/12/01  17:48:15  Stan_Luke
 * 	Added inode.i_io_lock, ihash_lock, inode.i_incore_lock,
 * 	namei.nch_lockinfo, nc_freelist_lockinfo, nc_purge_lockinfo.
 * 	[1993/12/01  16:54:28  Stan_Luke]
 * 
 * Revision 1.1.2.49  1993/11/24  20:02:45  Julie_McGray
 * 	Remove rtlock and ipclock
 * 	[1993/11/24  18:57:06  Julie_McGray]
 * 
 * Revision 1.1.2.48  1993/11/17  23:23:49  Ernie_Petrides
 * 	Remove spaces in names of unknown simple and complex lock classes.
 * 	[1993/11/17  23:10:10  Ernie_Petrides]
 * 
 * Revision 1.1.2.47  1993/11/16  23:35:06  James_Woodward
 * 	remove merge markers
 * 	[1993/11/16  23:34:18  James_Woodward]
 * 
 * Revision 1.1.2.46  1993/11/16  22:57:16  James_Woodward
 * 	update with nightly
 * 	[1993/11/16  14:15:58  James_Woodward]
 * 
 * 	sync up with nightly
 * 	[1993/11/15  13:13:42  James_Woodward]
 * 
 * Revision 1.1.2.45  1993/11/16  20:44:01  Ernie_Petrides
 * 	Reorganize lockinfo/lockstats data for new 8-byte simple lock format.
 * 	[1993/11/16  19:27:13  Ernie_Petrides]
 * 
 * Revision 1.1.2.44  1993/11/16  00:24:42  Shashi_Mangalat
 * 	Renamed vm_swap_buffree_lock to vm_swap_io_lock.
 * 	[1993/11/15  23:46:06  Shashi_Mangalat]
 * 
 * Revision 1.1.2.43  1993/11/15  21:48:14  Anna_Salzberg
 * 	Removed attach array - shm_attch_lockinfo no longer needed.
 * 	[1993/11/12  14:28:15  Anna_Salzberg]
 * 
 * Revision 1.1.2.42  1993/11/11  23:07:19  Brian_Nadeau
 * 	Add CAM locks
 * 	[1993/11/11  22:29:14  Brian_Nadeau]
 * 
 * Revision 1.1.2.41  1993/11/11  17:44:18  Anton_Verhulst
 * 	Raise the hierarchy of unp_misc_lock.
 * 	[1993/11/11  17:43:40  Anton_Verhulst]
 * 
 * Revision 1.1.2.40  1993/11/01  23:29:07  Ernie_Petrides
 * 	Remove lockinfo specs for habitat syscall counter locks.
 * 	[1993/11/01  22:42:27  Ernie_Petrides]
 * 
 * Revision 1.1.2.39  1993/11/01  20:36:57  Anton_Verhulst
 * 	Removed ledger info.
 * 	[1993/11/01  20:17:24  Anton_Verhulst]
 * 
 * Revision 1.1.2.38  1993/11/01  18:14:55  Larry_Scott
 * 	adjust lock ordering around audit locks
 * 	[1993/10/29  20:37:20  Larry_Scott]
 * 
 * Revision 1.1.2.37  1993/10/29  16:15:40  Timothy_Burke
 * 	Cleanup prior to submit.
 * 	[1993/10/27  17:47:07  Timothy_Burke]
 * 
 * 	Added nfs_sv_active_lock.
 * 	Changed presto lock from complex to simple.
 * 	[1993/10/27  16:54:13  Timothy_Burke]
 * 
 * Revision 1.1.2.36  1993/10/22  21:52:29  Shashi_Mangalat
 * 	Added UBC lockinfo initializations.
 * 	[1993/10/22  21:04:43  Shashi_Mangalat]
 * 
 * Revision 1.1.2.35  1993/10/22  20:48:16  Sashi_Sashidhar
 * 	Added fdfs lock
 * 	[1993/10/22  17:11:30  Sashi_Sashidhar]
 * 
 * Revision 1.1.2.34  1993/09/30  21:23:37  Alaa_Zeineldine
 * 	Conditionally compile out ult_bin_ctr_lockinfo
 * 	[1993/09/30  21:22:40  Alaa_Zeineldine]
 * 
 * Revision 1.1.2.33  1993/09/29  13:55:25  Alaa_Zeineldine
 * 	SMP: Add *_ctr_lockinfo for habitat counters.
 * 	[1993/09/28  05:22:28  Alaa_Zeineldine]
 * 
 * Revision 1.1.2.32  1993/09/28  23:36:27  Larry_Scott
 * 	security merge
 * 	[1993/09/28  15:40:52  Larry_Scott]
 * 
 * Revision 1.1.2.31  1993/09/28  15:54:50  David_Scoda
 * 	/proc Sterling merge & locking
 * 	[1993/09/28  15:51:25  David_Scoda]
 * 
 * Revision 1.1.2.30  1993/09/27  19:48:17  John_Dustin
 * 	make dli locks conditional on #if DLI || #if DLPI
 * 	[1993/09/22  19:34:44  John_Dustin]
 * 
 * 	make dli locks conditional on #ifdef DLI
 * 	[1993/09/22  13:48:05  John_Dustin]
 * 
 * 	Add dli_input_lock and dli_dlif_lock entries
 * 	[1993/09/21  20:44:21  John_Dustin]
 * 
 * Revision 1.1.2.29  1993/09/24  15:12:28  Julie_McGray
 * 	Add thread safe AIO lock
 * 	[1993/09/24  15:10:29  Julie_McGray]
 * 
 * Revision 1.1.2.28  1993/09/23  20:16:19  Shashi_Mangalat
 * 	Added lockinfo for swap locks.
 * 	[1993/09/23  19:49:19  Shashi_Mangalat]
 * 
 * Revision 1.1.2.27  1993/09/23  18:15:01  Vikas_Sontakke
 * 	Add real_time locks for semaphores and IPC etc
 * 	[1993/09/23  18:13:32  Vikas_Sontakke]
 * 
 * Revision 1.1.2.26  1993/09/16  00:09:36  Anna_Salzberg
 * 	Added vm_map_lockinfo, shm_stats_lockinfo, and shm_attch_lockinfo.
 * 	[1993/09/16  00:06:55  Anna_Salzberg]
 * 
 * Revision 1.1.2.25  1993/09/15  18:02:55  Ronald_Menner
 * 	Bmerge most recent changes
 * 	[1993/09/15  17:19:56  Ronald_Menner]
 * 
 * 	add lockinfo entries for Streams ptys (rptys) and conditionize
 * 	entry for clist ptys (ptys)
 * 	[1993/09/14  13:48:01  Ronald_Menner]
 * 
 * Revision 1.1.2.24  1993/09/14  20:08:07  John_Dustin
 * 	added #include "packetfilter.h" so NPACKETFILTER is defined
 * 	[1993/09/10  18:50:36  John_Dustin]
 * 
 * 	Add packetfilter lockinfo
 * 	[1993/09/10  18:48:44  John_Dustin]
 * 
 * Revision 1.1.2.23  1993/09/14  19:31:51  Brian_Stevens
 * 	Added structure definitions for maintaining lists of
 * 	statically configurable modules that are manageable
 * 	using the configuration manager framework.
 * 	[1993/09/13  19:22:25  Brian_Stevens]
 * 
 * Revision 1.1.2.22  1993/09/09  17:34:56  Timothy_Burke
 * 	Added lockinfo for prestoserve NFS driver.
 * 	[1993/09/07  17:56:47  Timothy_Burke]
 * 
 * 	Moved several nfs and rpc locks down in the heirarchy as
 * 	appropriate due to changing the SPLNONE level to SPLIMP.
 * 	[1993/09/03  18:17:51  Timothy_Burke]
 * 
 * Revision 1.1.2.21  1993/08/31  20:58:18  Ernie_Petrides
 * 	Add 15 more lockinfo specs referenced in kern directory sources,
 * 	enhance lockinfo macros for conditional compilation and disabling
 * 	of hierarchy checking, enable socket lock hierarchy checking, and
 * 	touch up numerous locking class names.
 * 	[1993/08/31  20:08:38  Ernie_Petrides]
 * 
 * Revision 1.1.2.20  1993/08/30  19:10:39  Shashi_Mangalat
 * 	Temporarily disabled vm_map_lockinfo.
 * 	[1993/08/30  19:06:56  Shashi_Mangalat]
 * 
 * Revision 1.1.2.19  1993/08/26  15:05:25  Shashi_Mangalat
 * 	More vm lockinfo.
 * 	[1993/08/25  20:33:36  Shashi_Mangalat]
 * 
 * Revision 1.1.2.18  1993/08/24  21:25:53  Stuart_Hollander
 * 	For faa, changed class names, moved entries lower,
 * 	and added minspl checking for interrupt lock.
 * 	[1993/08/23  23:17:56  Stuart_Hollander]
 * 
 * Revision 1.1.2.17  1993/08/18  16:25:28  Anna_Salzberg
 * 	Added lockinfo.
 * 	[1993/08/18  15:58:50  Anna_Salzberg]
 * 
 * Revision 1.1.2.16  1993/08/17  02:28:49  Paula_Long
 * 	faa_interrupt_level lock needs to be declared using the SLI macro to
 * 	ensure proper compilation occurs for uni-processor variants.
 * 	[1993/08/17  02:27:45  Paula_Long]
 * 
 * Revision 1.1.2.15  1993/08/14  23:00:20  Stuart_Hollander
 * 	Added 2 faa lockinfo classes
 * 	[1993/08/14  22:03:16  Stuart_Hollander]
 * 
 * Revision 1.1.2.14  1993/08/10  22:04:58  Timothy_Burke
 * 	Added nfs lockinfo for nfs_rrok_lockinfo.
 * 	[1993/08/10  21:38:16  Timothy_Burke]
 * 
 * Revision 1.1.2.13  1993/08/09  16:01:56  Shashi_Mangalat
 * 	mbuf_lock was incorrectly placed during VM locks merge.
 * 	[1993/08/09  15:55:26  Shashi_Mangalat]
 * 
 * Revision 1.1.2.12  1993/08/06  23:06:43  Shashi_Mangalat
 * 	Added VM lockinfo initializations for some locks.
 * 	[1993/08/06  22:27:38  Shashi_Mangalat]
 * 
 * Revision 1.1.2.11  1993/08/06  14:38:36  Timothy_Burke
 * 	Added in nfs & rpc locks.
 * 	[1993/08/05  18:25:33  Timothy_Burke]
 * 
 * Revision 1.1.2.10  1993/08/06  13:51:36  Ajay_Kachrani
 * 	Enable mbuf_sli hierarchy checking.
 * 	[1993/08/06  13:30:35  Ajay_Kachrani]
 * 
 * Revision 1.1.2.9  1993/08/06  13:22:20  Anna_Salzberg
 * 	Remove msgrcv_lock as it is no longer used.
 * 	[1993/08/06  13:19:26  Anna_Salzberg]
 * 
 * Revision 1.1.2.8  1993/08/06  12:33:20  James_Woodward
 * 	fix locking around processor state and idle dispatching
 * 	[1993/08/06  11:35:11  James_Woodward]
 * 
 * Revision 1.1.2.7  1993/08/04  18:42:25  Ajay_Kachrani
 * 	Surround sock lock info w/ NETSYNC_LOCK to cover .ext;
 * 	Remove commented NLIs.
 * 	[1993/08/04  18:28:41  Ajay_Kachrani]
 * 
 * Revision 1.1.2.6  1993/08/03  16:34:48  Ajay_Kachrani
 * 	Convert n/w locks to new lock stats using LI
 * 	[1993/07/22  19:37:01  Ajay_Kachrani]
 * 
 * Revision 1.1.2.5  1993/07/30  21:38:20  Ernie_Petrides
 * 	Add 23 more lockinfo specs referenced in sys and bsd directory sources
 * 	and move a couple of task and zone related specs from last revision.
 * 	[1993/07/30  20:57:22  Ernie_Petrides]
 * 
 * Revision 1.1.2.4  1993/07/29  19:21:53  James_Woodward
 * 	Add lockinfo changes.
 * 	[1993/07/29  17:48:01  James_Woodward]
 * 
 * Revision 1.1.2.3  1993/07/26  19:52:36  Ernie_Petrides
 * 	Add 16 lockinfo specs for proc related locks and create conditionally
 * 	compiled SLI, ULI, and NLI macros.
 * 	[1993/07/26  19:35:08  Ernie_Petrides]
 * 
 * Revision 1.1.2.2  1993/06/21  23:17:58  Ernie_Petrides
 * 	Creation of lockinfo source file.
 * 	[1993/06/21  22:14:08  Ernie_Petrides]
 * 
 * $EndLog$
 */
#pragma ident "@(#)$RCSfile: lockinfo.c,v $ $Revision: 1.1.318.41 $ (DEC) $Date: 1997/11/11 16:46:41 $"

#include <kern/lock.h>
#include <machine/cpu.h>
#include <unix_locks.h>
#include <net/net_globals.h>
#include <sys/lock_probe.h>

#if 0
#include "rt_sem.h"
#endif


#include "rt_ipc.h"
#include "rt_csem.h"
#include "te.h"
#include "tu.h"
#include "simether.h"
#include "xna.h" 
#include "mfa.h"
#include "faa.h"
#include "packetfilter.h"
#include "gwscreen.h"
#include "pty.h"
#include "rpty.h"
#include "dli.h"
#include "dlb.h"
#include  "dlpi.h"
#include "dec_audit.h"
#include "siop.h"
#include "xza.h"
#include "aha.h"
#include "pza.h"
#include "tza.h"
#include "isp.h"
#include "psiop.h"
#include "xcr.h"
#include "tra.h"
#include "fta.h"
#include "lta.h"
#include "trsrcf.h"
#include "mlog.h"
#include "cdfs.h"
#include "ffm_fs.h"
#include "tcds.h"
#include "streams.h"
#include "strifnet.h"
#include "ci.h"
#include "uq.h"
#include "ldtty.h"
#include "pci.h"
#include "msfs.h"
#include "drd.h"
#include "dlm.h"
#include "dlmsl.h"
#include "ccomsub.h"
#include "mcnet.h"
#include "rm.h"
#include "rmvm.h"
#include "procfs.h"
#include "lsm.h"
#include "scsi.h"
#include "xtiso.h"
#include "ppp.h"
#include "atm.h"
#include "atmip.h"
#include "uni3x.h"
#include "pwrmgr_enabled.h"

/*
 * Define lockmode here so that we can have a different initial value
 * for the SAS kernel.  When we make osf_boot smart enough to read MFS,
 * then this definition could go back to lock.c.
 */
#include "sas.h"
#if SAS > 0
long lockmode =  0; /* 0: uni, 1: rt, 2: smp, 3: rt/smp, 4: rt/smp/dbg/stats */
#else
long lockmode = -1; /* to be determined at bootstrap time */
#endif /* SAS > 0 */

#if MACH_SLOCKS
#define SCC(xxx) xxx
#else
#define SCC(xxx)
#endif

#if UNIX_LOCKS
#define UCC(xxx) xxx
#else
#define UCC(xxx)
#endif

#if NETSYNC_LOCK
#define NCC(xxx) xxx
#else
#define NCC(xxx)
#endif

#define LI(name, extlip, minspl, flags) { \
	extern struct lockinfo *extlip; \
	lip->li_name = name; \
	lip->li_count = 0; \
	lip->li_flgspl = (minspl) | (flags); \
	extlip = lip++; \
}
#define SLI(a,b,c,d) SCC(LI(a,b,c,d))
#define ULI(a,b,c,d) UCC(LI(a,b,c,d))
#define NLI(a,b,c,d) NCC(LI(a,b,c,d))

#define LI_DISABLE_HIERARCHY_CHECKING(p) (p)->li_flgspl &= ~LI_DOHIER
#define SLI_DISABLE_HIERARCHY_CHECKING(p) SCC(LI_DISABLE_HIERARCHY_CHECKING(p))
#define ULI_DISABLE_HIERARCHY_CHECKING(p) UCC(LI_DISABLE_HIERARCHY_CHECKING(p))
#define NLI_DISABLE_HIERARCHY_CHECKING(p) NCC(LI_DISABLE_HIERARCHY_CHECKING(p))

#define DEFAULT (LI_VALID | LI_DOHIER) /* default lockinfo flags */
#define PROBE   (LI_VALID | LI_PROBE)  /* Flags set probing */
#define ORDERED (DEFAULT | LI_ORDERED)
#define ORDNEXT (DEFAULT | LI_ORDNEXT)

#define ULI_NOCHECK(name, extlip) { \
	ULI(name, extlip, SPLNONE, DEFAULT); \
	ULI_DISABLE_HIERARCHY_CHECKING(extlip); \
}

#ifdef LOCK_PROBE
int probe_smp = 1;	/* 0 == disabled at PROBE_START/END macro */
#define ULI_PROBE(name, extlip) { \
	ULI(name, extlip, SPLNONE, PROBE); \
}
#else
#define ULI_PROBE(name, extlip)
#endif /* LOCK_PROBE */

extern int lockinfo_count;
extern struct lockinfo *lockinfo_table;
extern struct lockstats *lockstats_table[];


/*
 * The following function contains lockinfo specs for initializing the
 * complex and simple lock data structures associated with lock debugging
 * and statistics.  The order of the LI() macro invocations determines the
 * relative locking hierarchy positions between classes of locks, with specs
 * nearer the beginning of the function referring to higher (or outermost)
 * locks and specs nearer the end of the function referring to lower (or
 * innermost) locks.  To add a new lockinfo spec, follow these steps:
 *
 *	1) Determine where the class of locks fits in below, and add a new
 *		LI() line in the function below specifying the class name,
 *		the name of an external lockinfo structure pointer, the
 *		minimum required spl level for lock acquisition, and the
 *		default trace flag constant.  The suggested class naming
 *		convention is to use the name of the global lock instance
 *		for single-lock classes or <structure_name>.<field_name>
 *		for classes of locks which are fields in other structures.
 *		If the associated lock instances are conditionally compiled,
 *		use the SLI(), ULI(), or NLI() macro in place of LI() as
 *		appropriate.
 *
 *	2) Add the global (struct lockinfo *) variable to the source file
 *		related to the locking class.  Use conditionally compiled
 *		declarations (decl_simple_lock_info/udecl_simple_lock_info)
 *		if the associated lock instances are conditionally compiled.
 *
 *	3) Replace the associated lock_init() or simple_lock_init() calls
 *		with lock_setup() or simple_lock_setup() calls naming the
 *		global (struct lockinfo *) variable as the 2nd argument.
 *
 *	4) If the lock class is a field of a data structure which is
 *		dynamically allocated and freed, add a complementary call
 *		to lock_terminate() or simple_lock_terminate() just before
 *		the data structure is freed.
 */

lockinfo_bootstrap(lip_start)
struct lockinfo *lip_start;
{
	register struct lockinfo *lip;
	register struct lockstats *lsp;
	register int i;

	lip = lip_start;
	lip->li_name = "unknown_simple_lock";
	lip->li_count = 0;
	lip->li_flgspl = 0;
	lip++;
	lip->li_name = "unknown_complex_lock";
	lip->li_count = 0;
	lip->li_flgspl = 0;
	lip++;

	/*********************************************/
	/* start of lockinfo specs for complex locks */
	/*********************************************/

	LI("lockdev_lock", lockdev_lockinfo, SPLNONE, DEFAULT);
	LI("sys_exitp_lock", sys_exitp_lockinfo, SPLNONE, DEFAULT);
	LI("fdfs_lock", fdfs_lockinfo, SPLNONE, DEFAULT);
	LI("cfg_subsys_lock", cfg_subsys_lockinfo, SPLNONE, ORDERED);
	LI("callbk_list_lock", callbk_list_lockinfo, SPLNONE, DEFAULT);
	LI("subsys_tbl_lock", subsys_tbl_lockinfo, SPLNONE, DEFAULT);
#if DLM > 0
	LI("dlm_smp_lock", dlm_smp_lockinfo, SPLNONE, DEFAULT);
#endif /* DLM */
#if DLMSL > 0
	LI("sl_smp_lock", sl_smp_lockinfo, SPLNONE, DEFAULT);
#endif /* DLMSL */
#if DRD > 0
	LI("bsc_rm_node.bn_node_lock", bsc_node_lockinfo, SPLNONE, DEFAULT);
#endif
	LI("lock_for_exported", lockinfo_for_exported, SPLNONE, DEFAULT);
	LI("inode.i_io_lock", i_io_lockinfo, SPLNONE, ORDERED);
	LI_DISABLE_HIERARCHY_CHECKING(i_io_lockinfo);
	LI("nfs_daemon_lock", nfs_daemon_lockinfo, SPLNONE, DEFAULT);

#if PROCFS > 0
	LI("procnode.prc_ioctl_lock", procnode_ioctl_lockinfo,SPLNONE,ORDERED);
	LI("procfs_mount_lock", procfs_mount_lockinfo, SPLNONE, DEFAULT);
#endif
	LI("semidq_lock", semidq_lockinfo, SPLNONE, DEFAULT);
	LI("msgidq_lock", msgidq_lockinfo, SPLNONE, DEFAULT);
	LI("utask.uu_addr_lock", u_addr_lockinfo, SPLNONE, DEFAULT);
#if ATM
#if UNI3X
	NLI("atmsig_uni3x_lock", atmsig_uni3x_lock_info, SPLNONE, DEFAULT);
#endif
#if ATMIP
        NLI("request_q_complex_lock", request_q_complex_lock_info, SPLNONE, DEFAULT);
#endif
#endif
#if MSFS > 0
	/***********************************************
	 * AdvFS Complex (sleep) locks in hierarchy
	 ***********************************************/
	LI("RawBufFreeWait", ADVRawBufFreeWait_lockinfo, SPLNONE, DEFAULT );
        LI("waitLock", ADVwaitLock_info, SPLNONE, DEFAULT );

	LI("BfsDeleteLock",Bfs_delete_lockinfo, SPLNONE, DEFAULT);
	LI("DmnTblLock", ADVDmnTblLock_info, SPLNONE, DEFAULT );
	LI("InitLock", ADVInitLock_info, SPLNONE, DEFAULT );
        LI("fsContext.file_lock", ADVfsContext_file_lock_info, SPLNONE,
	   ORDERED | ORDNEXT );
    /* DO NOT PUT ANY LOCK BETWEEN fsContext.file_lock AND vm_umap.vm_lock
     * BECAUSE OF THE ORDNEXT FLAG ON THE FILE_LOCK!!  THESE MUST BE AT THE
     * SAME HIERARCHY LEVEL.  OR ELSE.
     */
#endif
	LI("vm_umap.vm_lock", u_map_lockinfo, SPLNONE, DEFAULT);
#if MSFS > 0
	LI("bfAccessT.trunc_xfer_lk", ADVbfAccessT_trunc_xfer_lk_info, SPLNONE, DEFAULT );
#endif
	LI("rnode.r_rwlock", rnode_rw_lockinfo, SPLNONE, ORDERED);
	NLI("unp_mutex_lock", unp_mutex_li, SPLNONE, DEFAULT);
#if MSFS > 0
	/***********************************************
	 * AdvFS Complex (sleep) locks in hierarchy - cont
	 ***********************************************/
	LI("ddlActiveLk", ADVvdT_ddlActiveLk_info, SPLNONE, DEFAULT );
	LI("bsInMemXtntT.migTruncLk", ADVbsInMemXtntT_migTruncLk_info, SPLNONE, DEFAULT );
        LI("BfSetTblLock", ADVBfSetTblLock_info, SPLNONE, DEFAULT );
	LI("quotaInfoT_qiLock", ADVquotaInfoT_qiLock, SPLNONE, DEFAULT );
	LI("bfSetT.fragLock", ADVbfSetT_fragLock_info, SPLNONE, DEFAULT );
	LI("bfSetT.dirLock", ADVbfSetT_dirLock_info, SPLNONE, DEFAULT );
/*	LI("bfAccessT.cow_lk", ADVbfAccessT_cow_lk_info, SPLNONE, DEFAULT );*/
	LI("bfAccessT.mcellList_lk", ADVbfAccessT_mcellList_lk_info, SPLNONE, ORDERED );
	LI("bfAccessT.putpage_lk",ADVbfAccessT_putpage_lk_info, SPLNONE, DEFAULT );
	LI("bfAccessT.xtntMap_lk", ADVbfAccessT_xtntMap_lk_info, SPLNONE, DEFAULT );
	LI("FilesetLock", ADVFilesetLock_lockinfo, SPLNONE, DEFAULT );
	LI("domainT.scLock", ADVdomainT_scLock_info, SPLNONE, DEFAULT );
/*	LI("dQuot.dqLock", ADVdQuot_dqLock_info, SPLNONE, DEFAULT ); */
	LI("vdT.mcell_lk", ADVvdT_mcell_lk_info, SPLNONE, DEFAULT );
	LI("vdT.del_list_lk", ADVvdT_del_list_lk_info, SPLNONE, DEFAULT );
	LI("BMT.bfAccessT.mcellList_lk", ADV_BMT_bfAccessT_mcellList_lk_info, SPLNONE, DEFAULT );
	LI("vdT.page0_mcell_lk", ADVvdT_page0_mcell_lk_info, SPLNONE, DEFAULT );
	LI("BMT.bfAccessT.xtntMap_lk", ADV_BMT_bfAccessT_xtntMap_lk_info, SPLNONE, DEFAULT );
	LI("vdT.stgMap_lk", ADVvdT_stgMap_lk_info, SPLNONE, DEFAULT );
#endif
	ULI("chtabhead_lock", chtabhead_lockinfo, SPLNONE, DEFAULT);
	NLI("global_raw_lock", global_raw_li, SPLNONE, DEFAULT);
	NLI("domain_lock", domain_li, SPLNONE, DEFAULT);
	NLI("pmtu_lock", pmtu_li, SPLNONE, DEFAULT);
#if DLI > 0 || DLB > 0
	NLI("dli_input_lock", dli_input_li, SPLNONE, DEFAULT);
#endif
	NLI("socklocks.sock_lock", sock_li, SPLNONE, ORDERED);
	NLI("socklocks.snd_lock", snd_li, SPLNONE, DEFAULT);
	NLI("socklocks.rcv_lock", rcv_li, SPLNONE, DEFAULT);
	ULI("rnode_table_lock", rnode_table_lockinfo, SPLNONE, DEFAULT);
	LI("pgrphash_lock", pgrphash_lockinfo, SPLNONE, DEFAULT);
	ULI("proc_relation_lock", proc_relation_lockinfo, SPLNONE, DEFAULT);
	ULI("pgrp.pg_lock", pgrp_lockinfo, SPLNONE, ORDERED);
	ULI("sysdev_write_lock", sysdev_write_lockinfo, SPLNONE, DEFAULT);
	LI("cm_lock", cm_lockinfo, SPLNONE, DEFAULT);
	LI("shmidq_lock", shmidq_lockinfo, SPLNONE, DEFAULT);
	LI("lk_lmf", lk_lmf_lockinfo, SPLNONE, DEFAULT);
	NLI("pr_dom_lock", pr_dom_li, SPLNONE, DEFAULT);
	/* inp_lock must follow socket lock */
	NLI("tcb.inp_lock", inp_tcphead_li, SPLNONE, DEFAULT);
	NLI("udb.inp_lock", inp_udphead_li, SPLNONE, DEFAULT);
	NLI("inifaddr_lock", inifaddr_li, SPLNONE, DEFAULT);
	NLI("igmp_lock", igmp_li, SPLNONE, DEFAULT);
	NLI("mfc_lock", mfc_li, SPLNONE, DEFAULT);
	NLI("route_lock", route_li, SPLNONE, DEFAULT);
	NLI("ip_frag_lock", ip_frag_li, SPLNONE, DEFAULT);
#if DLI > 0 || DLB > 0
	NLI("dli_dlif_lock", dli_dlif_li, SPLNONE, DEFAULT);
#endif
#if NTRA > 0 || TRSRCF > 0
	/* srt_thread_lock needs to be after dli_dlif_lock */
	NLI("srt_thread_lock", srtab_li, SPLNONE, DEFAULT);
#endif

#if NFAA > 0
	LI("fxx_softc.lk_thread_level",
		faa_thread_level_lockinfo, SPLNONE, DEFAULT);
#endif
#if DEC_AUDIT > 0
	ULI("audinit.lock", audinit_lockinfo, SPLNONE, DEFAULT);
#endif

#if NSCSI > 0
	ULI("cam_pd_unit",cam_pd_unit_li,   SPLBIO, DEFAULT);
	ULI("scsi_did",   scsi_did_li,      SPLBIO, DEFAULT);
#if NSIOP > 0
	/* CAM siop */
	ULI("cam_siop_jalloc",  cam_siop_jalloc_li,     SPLBIO, DEFAULT);
	ULI("cam_siop_balloc",  cam_siop_balloc_li,     SPLBIO, DEFAULT);
#endif
#endif
	LI("vm_kmap.vm_lock", k_map_lockinfo, SPLNONE, ORDERED);
#if RPTY > 0
	LI("pty_allocation_lock", pty_opcl_lockinfo, SPLNONE, DEFAULT);
#endif

#ifdef ALPHA
	LI("ev_lock", ev_lock_info, SPLNONE, DEFAULT);
#endif ALPHA

	/*******************************************/
	/* end of lockinfo specs for complex locks */
	/*******************************************/
	/******************************************/
        /* start of lockinfo specs for NOCHECK    */
	/* complex locks.			  */
	/******************************************/
#if	(CDFS > 0)
	ULI_NOCHECK("cdnode.cd_io_lock", cdfs_inio_lockinfo);
#endif	/* CDFS > 0 */
	ULI_NOCHECK("bdevlock.dsw_lock", bdev_lockinfo);
	ULI_NOCHECK("cdevlock.dsw_lock", cdev_lockinfo);
	ULI_NOCHECK("handler_id_lock", handler_id_lockinfo);
	ULI_NOCHECK("handler_key.lock", handler_key_lockinfo);
	ULI_NOCHECK("ldbl_module_list_lock", ldbl_module_list_lockinfo);
	ULI_NOCHECK("topology_tree_lock", topology_tree_lockinfo);

	/* I/O resource mgmt locks */
	ULI_NOCHECK("res_handle_list_lock", res_handle_list_lockinfo);
	ULI_NOCHECK("resource_key.lock", resource_key_lockinfo);
	ULI_NOCHECK("intr_db_lock", intr_db_lockinfo);
	ULI_NOCHECK("mem_db_lock", mem_db_lockinfo);
	ULI_NOCHECK("io_db_lock", io_db_lockinfo);
	ULI_NOCHECK("io_rsrc_list.lock", resource_entry_lockinfo);
	ULI_NOCHECK("io_res_map_lock", io_res_map_lockinfo);
	ULI_NOCHECK("mem_res_map_lock", mem_res_map_lockinfo);


#if	(NVBA > 0) || (CVBA > 0)
	ULI_NOCHECK("vhp_bus_adapt.shoebox_lk", vhp_shobox_lockinfo);
	ULI_NOCHECK("vhp_bus_adapt.adapt_c_lk", vhp_adapt_c_lockinfo);
#endif	/* (NVBA > 0) || (CVBA > 0) */
#if	(NLV > 0)
	ULI_NOCHECK("volgrp.vg_lock", vg_lockinfo);
	ULI_NOCHECK("lvol.lv_lock", lvtmp_lockinfo);
#endif	/* NLV > 0 */
#if	(NLSM_TED > 0)
	ULI_NOCHECK("ted_cmd_slepp_lock", ted_cmd_slepp_lockinfo);
#endif	/* (NLSM_TED > 0) */
	ULI_NOCHECK("ufsmount.um_lock", umpq_lockinfo);
	ULI_NOCHECK("dqhead.dq_lock", dq_lockinfo);
	ULI_NOCHECK("fifonode.fn_rdio_lock", fifo_rdio_lockinfo);
	ULI_NOCHECK("fifonode.fn_wrio_lock", fifo_wrio_lockinfo);
	ULI_NOCHECK("bp.b_lock", buf_lockinfo);
	ULI_NOCHECK("sleeplck.fichain_lock", sleeplcks_fichain_lockinfo);
	ULI_NOCHECK("grantlcks.fichain_lock", grantlcks_fichain_lockinfo);
	ULI_NOCHECK("klmlcks.fichain_lock", klmlcks_fichain_lock);
	ULI_NOCHECK("mount.m_unmount_lock", unmount_lockinfo);
	ULI_NOCHECK("vfssw_lock", vfssw_lockinfo);
#if	(NLSM > 0)
	ULI_NOCHECK("volcpybuf_sleep.sl_sleep", vol_alloc_sleep_lockinfo);
#endif	/* NLSM	> 0) */

#if MSFS > 0
	/***************************************************/
	/* start of lockinfo specs for NOCHECK AdvFS locks */
	/***************************************************/

	/* HSM locks */
        ULI_NOCHECK("bfAccessT.mss_lk", ADVbfAccessT_mss_lk_info );
        ULI_NOCHECK("bfSetT.shlvLock", ADVbfSetT_shlvLock_info );
	ULI_NOCHECK("pqd_main_lock", ADVpqd_main_lock_info );
        ULI_NOCHECK("pqd_lowpri_lock", ADVpqd_lowpri_lock_info );

	/* Locks with hierarchy violations */
	ULI_NOCHECK("dQuot.dqLock", ADVdQuot_dqLock_info );
	ULI_NOCHECK("bfAccessT.cow_lk", ADVbfAccessT_cow_lk_info );
	LI("logDescT.descLock", ADVlogDescT_descLock_info, SPLNONE, ORDERED ); 
	LI_DISABLE_HIERARCHY_CHECKING(ADVlogDescT_descLock_info);
	ULI_NOCHECK("LogDescTbl", ADVLogDescTbl_lockinfo );
#endif /* MSFS */

#if	(DEC4100 == 1)
	/*********************************************/
        /* start of lockinfo spec for I2C Bus lock   */
        /*********************************************/

        ULI_NOCHECK("i2c_lock", i2c_lockinfo);
#endif /* DEC4100 */
	/******************************************/
        /* end of lockinfo specs for NOCHECK      */
	/* complex locks.			  */
	/******************************************/

	LI("lock.l_lock", interlock_lockinfo, SPLNONE, DEFAULT);
	LI_DISABLE_HIERARCHY_CHECKING(interlock_lockinfo);

#if MSFS > 0
	/******************************************/
	/* start of lockinfo specs for PROBE      */
	/* complex locks.			  */
	/******************************************/

	/* fs/fs_create.c PROBES */
	ULI_PROBE("Pfs_create_file", Pfs_create_file_lockinfo);
	ULI_PROBE("Pfs_update_stats", Pfs_update_stats_lockinfo);
    ULI_PROBE("Pfs_flush_saved_stats", Pfs_flush_saved_stats_lockinfo);
	ULI_PROBE("Pfs_trunc_test", Pfs_trunc_test_lockinfo);

	/* fs/fs_dir_init.c PROBES */
	ULI_PROBE("Pfs_init_directory", Pfs_init_directory_lockinfo);
	ULI_PROBE("Pfs_assemble_dir", Pfs_assemble_dir_lockinfo);
	ULI_PROBE("Pdir_empty", Pdir_empty_lockinfo);
	ULI_PROBE("Pfs_create_file_set", Pfs_create_file_set_lockinfo);
	ULI_PROBE("Pcreate_root_file", Pcreate_root_file_lockinfo);
	ULI_PROBE("Pfs_init_ftx", Pfs_init_ftx_lockinfo);
	ULI_PROBE("Pfs_insert_undo", Pfs_insert_undo_lockinfo);
	
	/* kernel/msfs/fs/fs_file_sets.c */
	ULI_PROBE("Pfs_fset_create", Pfs_fset_create_lockinfo);
	ULI_PROBE("Pfs_fset_get_id", Pfs_fset_get_id_lockinfo);
	ULI_PROBE("Pfs_fset_get_stats", Pfs_fset_get_stats_lockinfo);
	ULI_PROBE("Pfs_fset_delete", Pfs_fset_delete_lockinfo);
	ULI_PROBE("Pfs_fset_clone", Pfs_fset_clone_lockinfo);
	ULI_PROBE("Pfs_fset_get_info", Pfs_fset_get_info_lockinfo);
	ULI_PROBE("Pfs_fset_name_change", Pfs_fset_name_change_lockinfo);
	ULI_PROBE("Pfs_create_frag", Pfs_create_frag_lockinfo);
	ULI_PROBE("Pfs_quick_frag_test", Pfs_quick_frag_test_lockinfo);
	ULI_PROBE("Pfs_delete_frag", Pfs_delete_frag_lockinfo);

        /* kernel/msfs/fs/fs_read_write.c */
        ULI_PROBE("Pfs_zero_fill_pages", Pfs_zero_fill_pages_lockinfo);
        ULI_PROBE("Pfs_read", Pfs_read_lockinfo);
        ULI_PROBE("Pfs_read_cntxt_access", Pfs_read_cntxt_access_lockinfo);
        ULI_PROBE("Pfs_read_cntxt_vp", Pfs_read_cntxt_vp_lockinfo);
        ULI_PROBE("Pfs_read_cntxt", Pfs_read_cntxt_lockinfo);
        ULI_PROBE("Pfs_read_vp", Pfs_read_vp_lockinfo);
        ULI_PROBE("Pfs_write", Pfs_write_lockinfo);
        ULI_PROBE("Pfs_write_cntxt_vp", Pfs_write_cntxt_vp_lockinfo);
        ULI_PROBE("Pfs_write_add_stg", Pfs_write_add_stg_lockinfo);
        ULI_PROBE("Pcopy_and_del_frag", Pcopy_and_del_frag_lockinfo);
        ULI_PROBE("Pbcopy_frag", Pbcopy_frag_lockinfo);
        ULI_PROBE("Puiomove_frag", Puiomove_frag_lockinfo);

        /* kernel/msfs/fs/fs_quota.c */
        ULI_PROBE("PquotaUndoObsolete",PquotaUndoObsolete_lockinfo);
        ULI_PROBE("PquotaUndo", PquotaUndo_lockinfo);
        ULI_PROBE("Pchk_quota_write", Pchk_quota_write_lockinfo);
        ULI_PROBE("Pquota_init", Pquota_init_lockinfo);
        ULI_PROBE("Pquota_files_init", Pquota_files_init_lockinfo);
        ULI_PROBE("Pattach_quota", Pattach_quota_lockinfo);
        ULI_PROBE("Pdetach_quota", Pdetach_quota_lockinfo);
        ULI_PROBE("Pchk_blk_quota", Pchk_blk_quota_lockinfo);
        ULI_PROBE("Pchk_bf_quota", Pchk_bf_quota_lockinfo);
        ULI_PROBE("Pchk_bf_quota_chg", Pchk_bf_quota_chg_lockinfo);
        ULI_PROBE("Pvalidate_quota_status", Pvalidate_quota_status_lockinfo);
        ULI_PROBE("Pflush_time_limits", Pflush_time_limits_lockinfo);
        ULI_PROBE("Pget_quota_config", Pget_quota_config_lockinfo);
        ULI_PROBE("Pfs_quota_blks", Pfs_quota_blks_lockinfo);
        ULI_PROBE("Pchkdquot", Pchkdquot_lockinfo);
        ULI_PROBE("Pget_quota_blks_used", Pget_quota_blks_used_lockinfo);
        ULI_PROBE("Popmntque", Popmntque_lockinfo);
        ULI_PROBE("Pqi_init", Pqi_init_lockinfo);
        ULI_PROBE("Pqf_get", Pqf_get_lockinfo);
        ULI_PROBE("Pquota_activate", Pquota_activate_lockinfo);
        ULI_PROBE("Pquota_deactivate", Pquota_deactivate_lockinfo);
        ULI_PROBE("Padvfs_enforce_on", Padvfs_enforce_on_lockinfo);
        ULI_PROBE("Padvfs_enforce_off", Padvfs_enforce_off_lockinfo);
        ULI_PROBE("Padvfs_maint_on", Padvfs_maint_on_lockinfo);
        ULI_PROBE("Padvfs_maint_off", Padvfs_maint_off_lockinfo);
        ULI_PROBE("Padvfs_quota_chown", Padvfs_quota_chown_lockinfo);
        ULI_PROBE("Padvfs_get_quota", Padvfs_get_quota_lockinfo);
        ULI_PROBE("Padvfs_set_quota", Padvfs_set_quota_lockinfo);
        ULI_PROBE("Padvfs_set_use", Padvfs_set_use_lockinfo);
        ULI_PROBE("Padvfs_quota_sync", Padvfs_quota_sync_lockinfo);
        ULI_PROBE("Pmutex_lock_try", Pmutex_lock_try_lockinfo);
        ULI_PROBE("Pdqinit", Pdqinit_lockinfo);
        ULI_PROBE("Pdqget", Pdqget_lockinfo);
        ULI_PROBE("Pdqput", Pdqput_lockinfo);
        ULI_PROBE("Pdqsync",Pdqsync_lockinfo);
        ULI_PROBE("Pdqflush", Pdqflush_lockinfo);

	/* kernel/msfs/osf/msfs_misc.c */
	ULI_PROBE("Posf_fd_to_bfh", Posf_fd_to_bfh_lockinfo);
	ULI_PROBE("Pfscontext_lock_init", Pfscontext_lock_init_lockinfo);
	ULI_PROBE("Pvnode_fscontext_allocate", Pvnode_fscontext_allocate_lockinfo);
	ULI_PROBE("Pvnode_fscontext_deallocate", Pvnode_fscontext_deallocate_lockinfo);
	ULI_PROBE("Pbf_get_l", Pbf_get_l_lockinfo);
	ULI_PROBE("Pmsfs_inactive", Pmsfs_inactive_lockinfo);
	ULI_PROBE("Pmsfs_reclaim", Pmsfs_reclaim_lockinfo);
	ULI_PROBE("Pzero_vm_page", Pzero_vm_page_lockinfo);
	ULI_PROBE("Pcopy_frag_into_vm_page", Pcopy_frag_into_vm_page_lockinfo);
	ULI_PROBE("Pmsfs_putpage", Pmsfs_putpage_lockinfo);
	ULI_PROBE("Pmsfs_getpage", Pmsfs_getpage_lockinfo);
	ULI_PROBE("Pcopy_frag_into_page", Pcopy_frag_into_page_lockinfo);
	ULI_PROBE("Pmsfs_mmap", Pmsfs_mmap_lockinfo);
	ULI_PROBE("Pmsfs_bread", Pmsfs_bread_lockinfo);
	ULI_PROBE("Pmsfs_brelse", Pmsfs_brelse_lockinfo);

	/* kernel/msfs/osf/msfs_vnops.c */
        ULI_PROBE("Pmsfs_create", Pmsfs_create_lockinfo);
        ULI_PROBE("Pmsfs_mknod", Pmsfs_mknod_lockinfo);
        ULI_PROBE("Pmsfs_close", Pmsfs_close_lockinfo);
        ULI_PROBE("Pmsfs_access", Pmsfs_access_lockinfo);
        ULI_PROBE("Pmsfs_getattr", Pmsfs_getattr_lockinfo);
        ULI_PROBE("Pmsfs_setattr", Pmsfs_setattr_lockinfo);
        ULI_PROBE("Pmsfs_read", Pmsfs_read_lockinfo);
        ULI_PROBE("Pmsfs_write", Pmsfs_write_lockinfo);
        ULI_PROBE("Pmsfs_fsync", Pmsfs_fsync_lockinfo);
        ULI_PROBE("Pmsfs_seek", Pmsfs_seek_lockinfo);
        ULI_PROBE("Pmsfs_remove", Pmsfs_remove_lockinfo);
        ULI_PROBE("Pmsfs_link", Pmsfs_link_lockinfo);
        ULI_PROBE("Pmsfs_rename", Pmsfs_rename_lockinfo);
        ULI_PROBE("Pdecr_link_count", Pdecr_link_count_lockinfo);
        ULI_PROBE("Pmsfs_mkdir", Pmsfs_mkdir_lockinfo);
        ULI_PROBE("Pmsfs_rmdir", Pmsfs_rmdir_lockinfo);
        ULI_PROBE("Pmsfs_symlink", Pmsfs_symlink_lockinfo);
        ULI_PROBE("Pmsfs_readdir", Pmsfs_readdir_lockinfo);
        ULI_PROBE("Pmsfs_readlink", Pmsfs_readlink_lockinfo);
        ULI_PROBE("Pmsfs_page_read", Pmsfs_page_read_lockinfo);
        ULI_PROBE("Pmsfs_page_write", Pmsfs_page_write_lockinfo);
        ULI_PROBE("Pmsfsspec_read", Pmsfsspec_read_lockinfo);
        ULI_PROBE("Pmsfsspec_write", Pmsfsspec_write_lockinfo);
        ULI_PROBE("Pmsfsspec_close", Pmsfsspec_close_lockinfo);
        ULI_PROBE("Pmsfsspec_reclaim", Pmsfsspec_reclaim_lockinfo);
        ULI_PROBE("Pmsfsfifo_read", Pmsfsfifo_read_lockinfo);
        ULI_PROBE("Pmsfsfifo_write", Pmsfsfifo_write_lockinfo);
        ULI_PROBE("Pmsfsfifo_close", Pmsfsfifo_close_lockinfo);
        ULI_PROBE("Pmsfsfifo_getattr", Pmsfsfifo_getattr_lockinfo);
        ULI_PROBE("Pmsfs_chown", Pmsfs_chown_lockinfo);
        ULI_PROBE("Pmsfs_chmod", Pmsfs_chmod_lockinfo);
        ULI_PROBE("Pattach_undel_dir", Pattach_undel_dir_lockinfo);
        ULI_PROBE("Pdetach_undel_dir", Pdetach_undel_dir_lockinfo);
        ULI_PROBE("Pget_undel_dir", Pget_undel_dir_lockinfo);
        ULI_PROBE("Pget_name", Pget_name_lockinfo);
        ULI_PROBE("Pget_name2", Pget_name2_lockinfo);
        ULI_PROBE("Pmsfs_lockctl", Pmsfs_lockctl_lockinfo);
        ULI_PROBE("Pmsfs_setvlocks", Pmsfs_setvlocks_lockinfo);
        ULI_PROBE("Pmsfs_syncdata", Pmsfs_syncdata_lockinfo);

	/* kernel/msfs/bs/bs_access.c */

	ULI_PROBE("Pbs_bf_htop", Pbs_bf_htop_lockinfo);
	ULI_PROBE("Pinit_access", Pinit_access_lockinfo);
	ULI_PROBE("Palloc_sublist", Palloc_sublist_lockinfo);
	ULI_PROBE("Pget_free_acc", Pget_free_acc_lockinfo);
	ULI_PROBE("Pfind_bfap", Pfind_bfap_lockinfo);
	ULI_PROBE("Paccess_invalidate", Paccess_invalidate_lockinfo);
	ULI_PROBE("Pbs_inval_rsvd_acc_st", Pbs_inval_rsvd_acc_st_lockinfo);
	ULI_PROBE("Pbs_init_area", Pbs_init_area_lockinfo);
	ULI_PROBE("Pbs_map_bf", Pbs_map_bf_lockinfo);
	ULI_PROBE("Pbs_insmntque", Pbs_insmntque_lockinfo);
	ULI_PROBE("Pbs_access", Pbs_access_lockinfo);
	ULI_PROBE("Prbf_access", Prbf_access_lockinfo);
	ULI_PROBE("Prbf_vfs_access", Prbf_vfs_access_lockinfo);
	ULI_PROBE("Pbfm_open_ms", Pbfm_open_ms_lockinfo);
	ULI_PROBE("Pbs_del_access", Pbs_del_access_lockinfo);
	ULI_PROBE("Prbf_access_int", Prbf_access_int_lockinfo);
	ULI_PROBE("Prbf_access_one_int", Prbf_access_one_int_lockinfo);
	ULI_PROBE("Pget_n_setup_new_vnode", Pget_n_setup_new_vnode_lockinfo);
	ULI_PROBE("Pgrab_bsacc", Pgrab_bsacc_lockinfo);
	ULI_PROBE("Pbs_close", Pbs_close_lockinfo);
	ULI_PROBE("Pbs_vfs_close", Pbs_vfs_close_lockinfo);
	ULI_PROBE("Pbs_close_one", Pbs_close_one_lockinfo);
	ULI_PROBE("Pbs_close_options", Pbs_close_options_lockinfo);
	ULI_PROBE("Pclose_int", Pclose_int_lockinfo);
	ULI_PROBE("Pclose_one_int", Pclose_one_int_lockinfo);
	ULI_PROBE("Pbs_unhook_vnode", Pbs_unhook_vnode_lockinfo);
	ULI_PROBE("Pbf_setup_truncation", Pbf_setup_truncation_lockinfo);
	/* ULI_PROBE("Preturn_bsacc_refcnt", Preturn_bsacc_refcnt_lockinfo);*/
	ULI_PROBE("Pbs_get_current_tag", Pbs_get_current_tag_lockinfo);
	/* ULI_PROBE("Paccess_test", Paccess_test_lockinfo); */
	ULI_PROBE("Pcheck_busy", Pcheck_busy_lockinfo);
	ULI_PROBE("Pcheck_mv_bfap_to_free", Pcheck_mv_bfap_to_free_lockinfo);
	ULI_PROBE("Pcleanup_closed_list", Pcleanup_closed_list_lockinfo);

	/* kernel/msfs/bs/bs_bmt_util.c		*/

	ULI_PROBE("Palloc_link_mcell_undo", Palloc_link_mcell_undo_lockinfo);
	ULI_PROBE("Palloc_mcell", Palloc_mcell_lockinfo);
	ULI_PROBE("Palloc_mcell_undo", Palloc_mcell_undo_lockinfo);
	ULI_PROBE("Palloc_page0_mcell", Palloc_page0_mcell_lockinfo);
	ULI_PROBE("Pbmt_alloc_link_mcell", Pbmt_alloc_link_mcell_lockinfo);
	ULI_PROBE("Pbmt_alloc_mcell", Pbmt_alloc_mcell_lockinfo);
	ULI_PROBE("Pbmt_alloc_prim_mcell", Pbmt_alloc_prim_mcell_lockinfo);
	ULI_PROBE("Pbmt_alloc_prim_mcell", Pbmt_alloc_prim_mcell_lockinfo);
	ULI_PROBE("Pbmt_close", Pbmt_close_lockinfo);
	ULI_PROBE("Pbmt_extend", Pbmt_extend_lockinfo);
	ULI_PROBE("Pbmt_find_mcell", Pbmt_find_mcell_lockinfo);
	ULI_PROBE("Pbmt_free_bf_mcells", Pbmt_free_bf_mcells_lockinfo);
	ULI_PROBE("Pbmt_free_bf_mcells_2", Pbmt_free_bf_mcells_2_lockinfo);
	ULI_PROBE("Pbmt_free_bf_mcells_i", Pbmt_free_bf_mcells_i_lockinfo);
	ULI_PROBE("Pbmt_free_mcell", Pbmt_free_mcell_lockinfo);
	ULI_PROBE("Pbmt_get_vd_bf_descs", Pbmt_get_vd_bf_descs_lockinfo);
	ULI_PROBE("Pbmt_init_page", Pbmt_init_page_lockinfo);
	ULI_PROBE("Pbmt_link_mcells", Pbmt_link_mcells_lockinfo);
	ULI_PROBE("Pbmt_open", Pbmt_open_lockinfo);
	ULI_PROBE("Pbmt_read", Pbmt_read_lockinfo);
	ULI_PROBE("Pbmt_set_mcell_free_list", Pbmt_set_mcell_free_list_lockinfo);
	ULI_PROBE("Pbmt_unlink_mcells", Pbmt_unlink_mcells_lockinfo);
	ULI_PROBE("Pbmt_upd_mcell_free_list", Pbmt_upd_mcell_free_list_lockinfo);
	ULI_PROBE("Pbmtr_assign", Pbmtr_assign_lockinfo);
	ULI_PROBE("Pbmtr_assign_rec", Pbmtr_assign_rec_lockinfo);
	ULI_PROBE("Pbmtr_clone_mcell", Pbmtr_clone_mcell_lockinfo);
	ULI_PROBE("Pbmtr_clone_recs", Pbmtr_clone_recs_lockinfo);
	ULI_PROBE("Pbmtr_create_rec", Pbmtr_create_rec_lockinfo);
	ULI_PROBE("Pbmtr_find", Pbmtr_find_lockinfo);
	ULI_PROBE("Pbmtr_get_rec_n_lk", Pbmtr_get_rec_n_lk_lockinfo);
	ULI_PROBE("Pbmtr_get_rec_ptr", Pbmtr_get_rec_ptr_lockinfo);
	ULI_PROBE("Pbmtr_put_rec_n_unlk", Pbmtr_put_rec_n_unlk_lockinfo);
	ULI_PROBE("Pbmtr_put_rec_n_unlk_int", Pbmtr_put_rec_n_unlk_int_lockinfo);
	ULI_PROBE("Pbmtr_put_rec_undo_opx", Pbmtr_put_rec_undo_opx_lockinfo);
	ULI_PROBE("Pbmtr_scan", Pbmtr_scan_lockinfo);
	ULI_PROBE("Pbmtr_scan_mcells", Pbmtr_scan_mcells_lockinfo);
	ULI_PROBE("Pbmtr_update_rec", Pbmtr_update_rec_lockinfo);
	ULI_PROBE("Pbmtr_update_rec_int", Pbmtr_update_rec_int_lockinfo);
	ULI_PROBE("Pbmtr_update_rec_n_unlk", Pbmtr_update_rec_n_unlk_lockinfo);
	ULI_PROBE("Pextend_bmt_redo_opx", Pextend_bmt_redo_opx_lockinfo);
	ULI_PROBE("Pfree_mcell_chains_opx", Pfree_mcell_chains_opx_lockinfo);
	ULI_PROBE("Pinit_bs_bmt_util_opx", Pinit_bs_bmt_util_opx_lockinfo);
	ULI_PROBE("Pinit_mcell", Pinit_mcell_lockinfo);
	ULI_PROBE("Plink_mcells", Plink_mcells_lockinfo);
	ULI_PROBE("Plink_unlink_mcells_undo", Plink_unlink_mcells_undo_lockinfo);
	ULI_PROBE("Pallocate_link_new_mcell", Pallocate_link_new_mcell_lockinfo);
	ULI_PROBE("Pbmtr_find_check", Pbmtr_find_check_lockinfo);

	/* kernel/msfs/bs/bs_domain.c		*/

	ULI_PROBE("Pbs_bfdmn_activate", Pbs_bfdmn_activate_lockinfo);
	ULI_PROBE("Pbs_bfdmn_deactivate", Pbs_bfdmn_deactivate_lockinfo);
	ULI_PROBE("Pbs_bfdmn_flush", Pbs_bfdmn_flush_lockinfo);
	ULI_PROBE("Pbs_bfdmn_flush_all", Pbs_bfdmn_flush_all_lockinfo);
	ULI_PROBE("Pbs_bfdmn_flush_bfrs", Pbs_bfdmn_flush_bfrs_lockinfo);
	ULI_PROBE("Pbs_bfdmn_flush_sync", Pbs_bfdmn_flush_sync_lockinfo);
	ULI_PROBE("Pbs_bfdmn_id_activate", Pbs_bfdmn_id_activate_lockinfo);
	ULI_PROBE("Pbs_bfdmn_tbl_activate", Pbs_bfdmn_tbl_activate_lockinfo);
	ULI_PROBE("Pbs_bfset_get_dev", Pbs_bfset_get_dev_lockinfo);
	ULI_PROBE("Pbs_dmn_change", Pbs_dmn_change_lockinfo);
	ULI_PROBE("Pbs_domain_access", Pbs_domain_access_lockinfo);
	ULI_PROBE("Pbs_domain_close", Pbs_domain_close_lockinfo);
	ULI_PROBE("Pbs_get_dmntbl_params", Pbs_get_dmntbl_params_lockinfo);
	ULI_PROBE("Pbs_sync", Pbs_sync_lockinfo);
	ULI_PROBE("Pbs_vd_add_active", Pbs_vd_add_active_lockinfo);
	ULI_PROBE("Pbs_vd_remove_active", Pbs_vd_remove_active_lockinfo);
	ULI_PROBE("Pclear_bmt", Pclear_bmt_lockinfo);
	ULI_PROBE("Pdmn_alloc", Pdmn_alloc_lockinfo);
	ULI_PROBE("Pdmn_dealloc", Pdmn_dealloc_lockinfo);
	ULI_PROBE("Pdomain_lookup", Pdomain_lookup_lockinfo);
	ULI_PROBE("Pdomain_name_lookup", Pdomain_name_lookup_lockinfo);
	ULI_PROBE("Pfind_del_entry", Pfind_del_entry_lockinfo);
	ULI_PROBE("Pfree_vdds", Pfree_vdds_lockinfo);
	ULI_PROBE("Pget_domain_disks", Pget_domain_disks_lockinfo);
	ULI_PROBE("Pget_domain_major", Pget_domain_major_lockinfo);
	ULI_PROBE("Pget_raw_vd_attrs", Pget_raw_vd_attrs_lockinfo);
	ULI_PROBE("Pget_vdd", Pget_vdd_lockinfo);
	ULI_PROBE("Pmega_presto_start", Pmega_presto_start_lockinfo);
	ULI_PROBE("Pmega_touch_presto", Pmega_touch_presto_lockinfo);
	ULI_PROBE("Pscan_rsvd_file_xtnt_map", Pscan_rsvd_file_xtnt_map_lockinfo);
	ULI_PROBE("Pset_disk_count", Pset_disk_count_lockinfo);
	ULI_PROBE("Pset_vd_mounted", Pset_vd_mounted_lockinfo);
	ULI_PROBE("Psetup_vd", Psetup_vd_lockinfo);
	ULI_PROBE("Pstrcat", Pstrcat_lockinfo);
	ULI_PROBE("Pvd_alloc", Pvd_alloc_lockinfo);
	ULI_PROBE("Pvd_alloc_index", Pvd_alloc_index_lockinfo);
	ULI_PROBE("Pvd_dup", Pvd_dup_lockinfo);
	ULI_PROBE("Pvd_free", Pvd_free_lockinfo);
	ULI_PROBE("Pvd_remove", Pvd_remove_lockinfo);
	ULI_PROBE("Pwait_for_ddl_active_entry", Pwait_for_ddl_active_entry_lockinfo);
	ULI_PROBE("Pset_recovery_failed", Pset_recovery_failed_lockinfo);

	/* kernel/msfs/bs/bs_extents.c		*/

	ULI_PROBE("Pcreate_xtnt_map_hdr", Pcreate_xtnt_map_hdr_lockinfo);
	ULI_PROBE("Pinit_bs_xtnts_opx", Pinit_bs_xtnts_opx_lockinfo);
	ULI_PROBE("Pinsert_xtnt_chain", Pinsert_xtnt_chain_lockinfo);
	ULI_PROBE("Pload_from_bmt_xtra_xtnt_rec", Pload_from_bmt_xtra_xtnt_rec_lockinfo);
	ULI_PROBE("Pload_from_shadow_rec", Pload_from_shadow_rec_lockinfo);
	ULI_PROBE("Pload_from_xtnt_array", Pload_from_xtnt_array_lockinfo);
	ULI_PROBE("Pload_from_xtnt_rec", Pload_from_xtnt_rec_lockinfo);
	ULI_PROBE("Pload_from_xtra_xtnt_rec", Pload_from_xtra_xtnt_rec_lockinfo);
	ULI_PROBE("Pload_inmem_xtnt_map", Pload_inmem_xtnt_map_lockinfo);
	ULI_PROBE("Podm_check_xtnt_map", Podm_check_xtnt_map_lockinfo);
	ULI_PROBE("Podm_create_xtnt_map", Podm_create_xtnt_map_lockinfo);
	ULI_PROBE("Podm_create_xtnt_rec", Podm_create_xtnt_rec_lockinfo);
	ULI_PROBE("Podm_fix_rsvd_file_xtnt_map", Podm_fix_rsvd_file_xtnt_map_lockinfo);
	ULI_PROBE("Podm_print_xtnt_map", Podm_print_xtnt_map_lockinfo);
	ULI_PROBE("Podm_remove_mcells_from_xtnt_map", Podm_remove_mcells_from_xtnt_map_lockinfo);
	ULI_PROBE("Podm_rewrite_xtnt_map", Podm_rewrite_xtnt_map_lockinfo);
	ULI_PROBE("Premove_xtnt_chain", Premove_xtnt_chain_lockinfo);
	ULI_PROBE("Pundo_insert_remove_xtnt_chain", Pundo_insert_remove_xtnt_chain_lockinfo);
	ULI_PROBE("Pundo_mcell_cnt", Pundo_mcell_cnt_lockinfo);
	ULI_PROBE("Pundo_upd_xtnt_rec", Pundo_upd_xtnt_rec_lockinfo);
	ULI_PROBE("Pundo_zero", Pundo_zero_lockinfo);
	ULI_PROBE("Pupdate_mcell_cnt", Pupdate_mcell_cnt_lockinfo);
	ULI_PROBE("Pupdate_xtnt_array", Pupdate_xtnt_array_lockinfo);
	ULI_PROBE("Pupdate_xtnt_rec", Pupdate_xtnt_rec_lockinfo);
	ULI_PROBE("Px_create_inmem_xtnt_map", Px_create_inmem_xtnt_map_lockinfo);
	ULI_PROBE("Px_create_shadow_rec", Px_create_shadow_rec_lockinfo);
	ULI_PROBE("Px_delete_shadow_rec", Px_delete_shadow_rec_lockinfo);
	ULI_PROBE("Px_load_inmem_xtnt_map", Px_load_inmem_xtnt_map_lockinfo);
	ULI_PROBE("Px_update_ondisk_xtnt_map", Px_update_ondisk_xtnt_map_lockinfo);

	/* kernel/msfs/bs/bs_init.c		*/

	ULI_PROBE("Pbs_dmn_init", Pbs_dmn_init_lockinfo);
	ULI_PROBE("Pbs_disk_init", Pbs_disk_init_lockinfo);
	ULI_PROBE("Pbmt_init_mcell_free_list", Pbmt_init_mcell_free_list_lockinfo);
	ULI_PROBE("Pdel_init_mcell_list", Pdel_init_mcell_list_lockinfo);
	ULI_PROBE("Ptagdir_init_pg0", Ptagdir_init_pg0_lockinfo);
	ULI_PROBE("Pinit_sbm", Pinit_sbm_lockinfo);

	/* kernel/msfs/bs/bs_inmem_map.c	*/

	ULI_PROBE("Pimm_create_xtnt_map", Pimm_create_xtnt_map_lockinfo);
	ULI_PROBE("Pimm_init_xtnt_map", Pimm_init_xtnt_map_lockinfo);
	ULI_PROBE("Pimm_extend_xtnt_map", Pimm_extend_xtnt_map_lockinfo);
	ULI_PROBE("Pimm_get_first_xtnt_desc", Pimm_get_first_xtnt_desc_lockinfo);
	ULI_PROBE("Pimm_get_xtnt_desc", Pimm_get_xtnt_desc_lockinfo);
	ULI_PROBE("Pimm_get_next_xtnt_desc", Pimm_get_next_xtnt_desc_lockinfo);
	ULI_PROBE("Pimm_delete_xtnt_map", Pimm_delete_xtnt_map_lockinfo);
	ULI_PROBE("Pimm_delete_sub_xtnt_maps", Pimm_delete_sub_xtnt_maps_lockinfo);
	ULI_PROBE("Pimm_init_sub_xtnt_map", Pimm_init_sub_xtnt_map_lockinfo);
	ULI_PROBE("Pimm_copy_sub_xtnt_map", Pimm_copy_sub_xtnt_map_lockinfo);
	ULI_PROBE("Pimm_extend_sub_xtnt_map", Pimm_extend_sub_xtnt_map_lockinfo);
	ULI_PROBE("Pimm_load_sub_xtnt_map", Pimm_load_sub_xtnt_map_lockinfo);
	ULI_PROBE("Pimm_unload_sub_xtnt_map", Pimm_unload_sub_xtnt_map_lockinfo);
	ULI_PROBE("Pimm_copy_xtnt_descs", Pimm_copy_xtnt_descs_lockinfo);
	ULI_PROBE("Pimm_split_desc", Pimm_split_desc_lockinfo);
	ULI_PROBE("Pmm_get_alloc_page_cnt", Pmm_get_alloc_page_cnt_lockinfo);
	ULI_PROBE("Pget_sub_xtnt_map_alloc_page_cnt", Pget_sub_xtnt_map_alloc_page_cnt_lockinfo);
	ULI_PROBE("Pimm_get_next_page", Pimm_get_next_page_lockinfo);
	ULI_PROBE("Pimm_get_first_hole", Pimm_get_first_hole_lockinfo);
	ULI_PROBE("Pimm_get_hole_size", Pimm_get_hole_size_lockinfo);
	ULI_PROBE("Pimm_get_page_type", Pimm_get_page_type_lockinfo);
	ULI_PROBE("Pimm_set_next_valid_copy_page", Pimm_set_next_valid_copy_page_lockinfo);
	ULI_PROBE("Pimm_page_to_sub_xtnt_map", Pimm_page_to_sub_xtnt_map_lockinfo);
	ULI_PROBE("Pimm_page_to_xtnt", Pimm_page_to_xtnt_lockinfo);
	ULI_PROBE("Pimm_overlay_xtnt_map", Pimm_overlay_xtnt_map_lockinfo);
	ULI_PROBE("Pimm_merge_xtnt_map", Pimm_merge_xtnt_map_lockinfo);
	ULI_PROBE("Pimm_remove_page_map", Pimm_remove_page_map_lockinfo);
	ULI_PROBE("Pimm_check_xtnt_map", Pimm_check_xtnt_map_lockinfo);
	ULI_PROBE("Pimm_print_xtnt_map", Pimm_print_xtnt_map_lockinfo);

	/* kernel/msfs/bs/bs_migrate.c		*/

	ULI_PROBE("Pmig_register_migrate_agent", Pmig_register_migrate_agent_lockinfo);
	ULI_PROBE("Pbs_migrate", Pbs_migrate_lockinfo);
	ULI_PROBE("Pmig_migrate", Pmig_migrate_lockinfo);
	ULI_PROBE("Pmigrate_normal_one_disk", Pmigrate_normal_one_disk_lockinfo);
	ULI_PROBE("Pmigrate_normal", Pmigrate_normal_lockinfo);
	ULI_PROBE("Pmigrate_stripe", Pmigrate_stripe_lockinfo);
	ULI_PROBE("Pget_xm_page_range_info", Pget_xm_page_range_info_lockinfo);
	ULI_PROBE("Pextend_page_range_list", Pextend_page_range_list_lockinfo);
	ULI_PROBE("Pswitch_stg", Pswitch_stg_lockinfo);
	ULI_PROBE("Pbs_move_metadata", Pbs_move_metadata_lockinfo);
	ULI_PROBE("Pmove_metadata", Pmove_metadata_lockinfo);
	ULI_PROBE("Pset_block_map", Pset_block_map_lockinfo);
	ULI_PROBE("Palloc_copy_stg", Palloc_copy_stg_lockinfo);

	/* kernel/msfs/bs/bs_misc.c		*/

	ULI_PROBE("Powner_or_suser", Powner_or_suser_lockinfo);
	ULI_PROBE("Pcopyin_dmntbl_setname", Pcopyin_dmntbl_setname_lockinfo);
	ULI_PROBE("Popen_domain", Popen_domain_lockinfo);
	ULI_PROBE("Popen_set", Popen_set_lockinfo);
	ULI_PROBE("Pmsfs_real_syscall", Pmsfs_real_syscall_lockinfo);
	ULI_PROBE("Pbs_inherit", Pbs_inherit_lockinfo);
	ULI_PROBE("Pbs_inherit_init", Pbs_inherit_init_lockinfo);

	/* kernel/msfs/bs/bs_msg_queue.c */
	ULI_PROBE("Pmsgq_create", Pmsgq_create_lockinfo);
	ULI_PROBE("Pmsgq_alloc_msg", Pmsgq_alloc_msg_lockinfo);
	ULI_PROBE("Pmsgq_send_msg", Pmsgq_send_msg_lockinfo);
	ULI_PROBE("Pmsgq_recv_msg", Pmsgq_recv_msg_lockinfo);
	ULI_PROBE("Pmsgq_free_msg", Pmsgq_free_msg_lockinfo);
	ULI_PROBE("Pulmq_alloc_msg", Pulmq_alloc_msg_lockinfo);
	ULI_PROBE("Pulmq_send_msg", Pulmq_send_msg_lockinfo);
	ULI_PROBE("Pulmq_recv_msg", Pulmq_recv_msg_lockinfo);
	ULI_PROBE("Pulmq_free_msg", Pulmq_free_msg_lockinfo);

    /* bs/bs_buffer2.c PROBE stuff */
    ULI_PROBE("Pbs_cache_init", Pbs_cache_init_lockinfo);
    ULI_PROBE("Ppage_lookup", Ppage_lookup_lockinfo);
    ULI_PROBE("Pmap_pages", Pmap_pages_lockinfo);
    ULI_PROBE("Pblkmap", Pblkmap_lockinfo);
    ULI_PROBE("Pbs_refpg_unshelve", Pbs_refpg_unshelve_lockinfo);
    ULI_PROBE("Pbs_refpg_get", Pbs_refpg_get_lockinfo);
    ULI_PROBE("Pbs_refpg", Pbs_refpg_lockinfo);
    ULI_PROBE("Pbs_refpg_fetch", Pbs_refpg_fetch_lockinfo);
    ULI_PROBE("Pbs_refpg_int", Pbs_refpg_int_lockinfo);
    ULI_PROBE("Pbs_derefpg", Pbs_derefpg_lockinfo);
    ULI_PROBE("Pbs_pinpg", Pbs_pinpg_lockinfo);
    ULI_PROBE("Pbs_pinpg_ftx", Pbs_pinpg_ftx_lockinfo);
    ULI_PROBE("Pbs_pinpg_put", Pbs_pinpg_put_lockinfo);
    ULI_PROBE("Pbs_pinpg_unshelve", Pbs_pinpg_unshelve_lockinfo);
    ULI_PROBE("Pbs_pinpg_clone", Pbs_pinpg_clone_lockinfo);
    ULI_PROBE("Pbs_pinpg_int", Pbs_pinpg_int_lockinfo);
    ULI_PROBE("Pbs_pinpg_one_int", Pbs_pinpg_one_int_lockinfo);
    ULI_PROBE("Pbs_unpinpg", Pbs_unpinpg_lockinfo);
    ULI_PROBE("Pbs_find_page", Pbs_find_page_lockinfo);
    ULI_PROBE("Pfind_page", Pfind_page_lockinfo);
    ULI_PROBE("P_state_block", P_state_block_lockinfo);
    ULI_PROBE("Pbs_set_bufstate", Pbs_set_bufstate_lockinfo);
    ULI_PROBE("Pbs_clear_bufstate", Pbs_clear_bufstate_lockinfo);
    ULI_PROBE("Pbs_get_bufstate", Pbs_get_bufstate_lockinfo);
    ULI_PROBE("Pset_state", Pset_state_lockinfo);
    ULI_PROBE("Pwait_state", Pwait_state_lockinfo);
    ULI_PROBE("Pclear_state", Pclear_state_lockinfo);
    ULI_PROBE("Pbuf_remap", Pbuf_remap_lockinfo);
    ULI_PROBE("Pget_freebuf", Pget_freebuf_lockinfo);
    ULI_PROBE("Pclear_buf", Pclear_buf_lockinfo);
    ULI_PROBE("Pseq_ahead_cont", Pseq_ahead_cont_lockinfo);
    ULI_PROBE("Pprefetch", Pprefetch_lockinfo);
    ULI_PROBE("Pseq_ahead_start", Pseq_ahead_start_lockinfo);
    ULI_PROBE("Pseq_ahead", Pseq_ahead_lockinfo);
    ULI_PROBE("Pbs_real_invalidate_pg", Pbs_real_invalidate_pg_lockinfo);
    ULI_PROBE("Pbs_purge_dirty", Pbs_purge_dirty_lockinfo);
    ULI_PROBE("Pbs_cache_rundown", Pbs_cache_rundown_lockinfo);
    ULI_PROBE("Pbfr_trace", Pbfr_trace_lockinfo);
    ULI_PROBE("Pbs_get_rawbuf", Pbs_get_rawbuf_lockinfo);
    ULI_PROBE("Pbs_put_rawbuf", Pbs_put_rawbuf_lockinfo);
    ULI_PROBE("Pbs_clear_rawbuf", Pbs_clear_rawbuf_lockinfo);
    ULI_PROBE("Punpin_sync", Punpin_sync_lockinfo);
    ULI_PROBE("Pbs_put_freebuf", Pbs_put_freebuf_lockinfo);

    /* bs/bs_copy.c probe stuff */
    ULI_PROBE("Pcp_copy_page_range", Pcp_copy_page_range_lockinfo);
    ULI_PROBE("Pset_block_map", Pset_block_map_lockinfo);
    ULI_PROBE("Pverify_page_write", Pverify_page_write_lockinfo);
    ULI_PROBE("Pforce_write", Pforce_write_lockinfo);
    ULI_PROBE("Pcompare_copies", Pcompare_copies_lockinfo);
    ULI_PROBE("Pcp_insert_onto_xtnt_ma", Pcp_insert_onto_xtnt_ma_lockinfo);
    ULI_PROBE("Pcp_remove_from_xtnt_ma", Pcp_remove_from_xtnt_ma_lockinfo);
    ULI_PROBE("Pcp_alloc_stg", Pcp_alloc_stg_lockinfo);

    /* bs/bs_create.c probe stuff */
    ULI_PROBE("Prbf_create", Prbf_create_lockinfo);
    ULI_PROBE("Prbf_int_create", Prbf_int_create_lockinfo);
    ULI_PROBE("Pcreate_rtdn_opx", Pcreate_rtdn_opx_lockinfo);
    ULI_PROBE("Pmake_mcell_valid", Pmake_mcell_valid_lockinfo);
    ULI_PROBE("Pcreate_undo_opx", Pcreate_undo_opx_lockinfo);
    ULI_PROBE("Pbs_create", Pbs_create_lockinfo);
    ULI_PROBE("Pinit_bscreate_opx", Pinit_bscreate_opx_lockinfo);
    ULI_PROBE("Pnew_mcell", Pnew_mcell_lockinfo);
    ULI_PROBE("Pkill_mcell", Pkill_mcell_lockinfo);
    ULI_PROBE("Pdealloc_mcells", Pdealloc_mcells_lockinfo);

    /* bs/bs_delete.c probe stuff */
    ULI_PROBE("Pbs_deferred_delete_und", Pbs_deferred_delete_und_lockinfo);
    ULI_PROBE("Prbf_delete", Prbf_delete_lockinfo);
    ULI_PROBE("Prbf_delete_int", Prbf_delete_int_lockinfo);
    ULI_PROBE("Pbs_delete_undo_opx", Pbs_delete_undo_opx_lockinfo);
    ULI_PROBE("Preset_ondisk_bf_state", Preset_ondisk_bf_state_lockinfo);
    ULI_PROBE("Pbs_delete_rtdn_opx", Pbs_delete_rtdn_opx_lockinfo);
    ULI_PROBE("Pbs_delete", Pbs_delete_lockinfo);
    ULI_PROBE("Pinit_bs_delete_opx", Pinit_bs_delete_opx_lockinfo);
    ULI_PROBE("Pdel_add_to_del_list", Pdel_add_to_del_list_lockinfo);
    ULI_PROBE("Pdel_remove_from_del_li", Pdel_remove_from_del_li_lockinfo);
    ULI_PROBE("Pdel_bitfile_list_undo", Pdel_bitfile_list_undo_lockinfo);
    ULI_PROBE("Ppin_link", Ppin_link_lockinfo);
    ULI_PROBE("Ppin_hdr", Ppin_hdr_lockinfo);
    ULI_PROBE("Pdel_clean_mcell_list", Pdel_clean_mcell_list_lockinfo);
    ULI_PROBE("Pdel_dealloc_stg", Pdel_dealloc_stg_lockinfo);
    ULI_PROBE("Pdel_xtnt_array", Pdel_xtnt_array_lockinfo);
    ULI_PROBE("Pxfer_xtnts_to_clone", Pxfer_xtnts_to_clone_lockinfo);
    ULI_PROBE("Pdel_part_xtnt", Pdel_part_xtnt_lockinfo);
    ULI_PROBE("Pdel_range", Pdel_range_lockinfo);
    ULI_PROBE("Pdel_ftx_start", Pdel_ftx_start_lockinfo);
    ULI_PROBE("Pdel_ftx_done", Pdel_ftx_done_lockinfo);
    ULI_PROBE("Psbm_howmany_blks", Psbm_howmany_blks_lockinfo);
    ULI_PROBE("Pdel_find_del_entry", Pdel_find_del_entry_lockinfo);

	/* bs/bs_qio.c probe stuff */
	ULI_PROBE("Pcheck_queue", Pcheck_queue_lockinfo);
	ULI_PROBE("Pbs_q_blocking", Pbs_q_blocking_lockinfo);
	ULI_PROBE("Pbs_q_list", Pbs_q_list_lockinfo);
	ULI_PROBE("Pbs_io_complete", Pbs_io_complete_lockinfo);
	ULI_PROBE("Pcheck_cont_bits", Pcheck_cont_bits_lockinfo);
	ULI_PROBE("Pbs_pinblock", Pbs_pinblock_lockinfo);
	ULI_PROBE("Pbs_pinblock_sync", Pbs_pinblock_sync_lockinfo);
	ULI_PROBE("Plsn_io_list", Plsn_io_list_lockinfo);
	ULI_PROBE("Pbs_q_lazy", Pbs_q_lazy_lockinfo);
	ULI_PROBE("Pwait_to_readyq", Pwait_to_readyq_lockinfo);
	ULI_PROBE("Pready_to_consolq", Pready_to_consolq_lockinfo);
	ULI_PROBE("Psort_to_dev", Psort_to_dev_lockinfo);
	ULI_PROBE("Pcache_logflush", Pcache_logflush_lockinfo);
	ULI_PROBE("Pcall_logflush", Pcall_logflush_lockinfo);
	ULI_PROBE("Pbs_logpage_dirty", Pbs_logpage_dirty_lockinfo);
	ULI_PROBE("Pbs_logflush_start", Pbs_logflush_start_lockinfo);
	ULI_PROBE("Plogflush_cont", Plogflush_cont_lockinfo);
	ULI_PROBE("Pbs_logflush_sync", Pbs_logflush_sync_lockinfo);
	ULI_PROBE("Pbfflush_start", Pbfflush_start_lockinfo);
	ULI_PROBE("Pbfflush_sync", Pbfflush_sync_lockinfo);
	ULI_PROBE("Pbs_sync_bf_flush", Pbs_sync_bf_flush_lockinfo);
	ULI_PROBE("Pbfflush", Pbfflush_lockinfo);
	ULI_PROBE("Pbfflush_sync_nowait", Pbfflush_sync_nowait_lockinfo);
	ULI_PROBE("Pbs_bf_flush_nowait", Pbs_bf_flush_nowait_lockinfo);
	ULI_PROBE("Pbs_bflush", Pbs_bflush_lockinfo);
	ULI_PROBE("Pbs_bflush_sync", Pbs_bflush_sync_lockinfo);
	ULI_PROBE("Pflush_vols", Pflush_vols_lockinfo);
	ULI_PROBE("Prm_or_moveq", Prm_or_moveq_lockinfo);
	ULI_PROBE("Plink_write_req", Plink_write_req_lockinfo);
	ULI_PROBE("Prm_ioq", Prm_ioq_lockinfo);
	ULI_PROBE("Pbs_raw_page", Pbs_raw_page_lockinfo);
	ULI_PROBE("Psendtoiothread", Psendtoiothread_lockinfo);

    /* bs/bs_tagdir.c probe stuff */
    ULI_PROBE("Ptagdir_get_info", Ptagdir_get_info_lockinfo);
    ULI_PROBE("Pinit_tagdir_opx", Pinit_tagdir_opx_lockinfo);
    ULI_PROBE("Ptagdir_remove_tag", Ptagdir_remove_tag_lockinfo);
    ULI_PROBE("Ptagdir_tag_to_freelist", Ptagdir_tag_to_freelist_lockinfo);
    ULI_PROBE("Pinit_next_tag_page", Pinit_next_tag_page_lockinfo);
    ULI_PROBE("Ptagdir_alloc_tag", Ptagdir_alloc_tag_lockinfo);
    ULI_PROBE("Pextend_tagdir_redo_opx", Pextend_tagdir_redo_opx_lockinfo);
    ULI_PROBE("Ptagdir_insert_tag", Ptagdir_insert_tag_lockinfo);
    ULI_PROBE("Ptagdir_set_next_tag", Ptagdir_set_next_tag_lockinfo);
    ULI_PROBE("Ptagdir_reset_tagmap", Ptagdir_reset_tagmap_lockinfo);
    ULI_PROBE("Ptagdir_stuff_tagmap", Ptagdir_stuff_tagmap_lockinfo);
    ULI_PROBE("Ptagdir_set_tagmap_comm", Ptagdir_set_tagmap_comm_lockinfo);
    ULI_PROBE("Ptagdir_write_undo_opx", Ptagdir_write_undo_opx_lockinfo);
    ULI_PROBE("Ptagdir_lookup", Ptagdir_lookup_lockinfo);
    ULI_PROBE("Ptagdir_lookup2", Ptagdir_lookup2_lockinfo);
    ULI_PROBE("Ptagdir_lookup_next", Ptagdir_lookup_next_lockinfo);
    ULI_PROBE("Pbs_switch_root_tagdir", Pbs_switch_root_tagdir_lockinfo);
    ULI_PROBE("Pswitch_root_tagdir_red", Pswitch_root_tagdir_red_lockinfo);

    /* bs/bs_params probe stuff */
    ULI_PROBE("Pbs_get_bf_xtnt_map", Pbs_get_bf_xtnt_map_lockinfo);
    ULI_PROBE("Pbs_get_bkup_xtnt_map", Pbs_get_bkup_xtnt_map_lockinfo);
    ULI_PROBE("Pbs_delete_with_clone", Pbs_delete_with_clone_lockinfo);
    ULI_PROBE("Pbs_get_bf_page_cnt", Pbs_get_bf_page_cnt_lockinfo);
    ULI_PROBE("Pbs_get_bf_next_page", Pbs_get_bf_next_page_lockinfo);
    ULI_PROBE("Pbs_get_bf_page_sz", Pbs_get_bf_page_sz_lockinfo);
    ULI_PROBE("Pbs_get_bf_params", Pbs_get_bf_params_lockinfo);
    ULI_PROBE("Pbs_set_bf_params", Pbs_set_bf_params_lockinfo);
    ULI_PROBE("Pbs_get_bf_iparams", Pbs_get_bf_iparams_lockinfo);
    ULI_PROBE("Pbs_set_bf_iparams", Pbs_set_bf_iparams_lockinfo);
    ULI_PROBE("Pbs_get_dmn_vd_list", Pbs_get_dmn_vd_list_lockinfo);
    ULI_PROBE("Pbs_get_dmn_params", Pbs_get_dmn_params_lockinfo);
    ULI_PROBE("Pbs_set_dmn_params", Pbs_set_dmn_params_lockinfo);
    ULI_PROBE("Pbs_get_vd_params", Pbs_get_vd_params_lockinfo);
    ULI_PROBE("Pbs_set_vd_params", Pbs_set_vd_params_lockinfo);
    ULI_PROBE("Pbs_get_avail_mcells", Pbs_get_avail_mcells_lockinfo);

    /* bs/bs_service_class.c probe stuff */
    ULI_PROBE("Psc_init_sc_tbl", Psc_init_sc_tbl_lockinfo);
    ULI_PROBE("Psc_find_super_class", Psc_find_super_class_lockinfo);
    ULI_PROBE("Psc_find_alt_class", Psc_find_alt_class_lockinfo);
    ULI_PROBE("Psc_binSearchForClass", Psc_binSearchForClass_lockinfo);
    ULI_PROBE("Psc_add_class", Psc_add_class_lockinfo);
    ULI_PROBE("Psc_add_vd_to_class", Psc_add_vd_to_class_lockinfo);
    ULI_PROBE("Psc_add_vd", Psc_add_vd_lockinfo);
    ULI_PROBE("Psc_remove_class", Psc_remove_class_lockinfo);
    ULI_PROBE("Psc_remove_vd", Psc_remove_vd_lockinfo);
    ULI_PROBE("Psc_select_class", Psc_select_class_lockinfo);
    ULI_PROBE("Psc_valid_vd", Psc_valid_vd_lockinfo);
    ULI_PROBE("Psc_select_vd_for_mcell", Psc_select_vd_for_mcell_lockinfo);
    ULI_PROBE("Psc_select_vd_for_stg", Psc_select_vd_for_stg_lockinfo);

    /* bs/bs_shadow.c probe stuff */
    ULI_PROBE("Psha_register_shadow_ag", Psha_register_shadow_ag_lockinfo);
    ULI_PROBE("Pundo_bitfile_type", Pundo_bitfile_type_lockinfo);
    ULI_PROBE("Pget_old_members", Pget_old_members_lockinfo);
    ULI_PROBE("Pextend_shadow_desc", Pextend_shadow_desc_lockinfo);
    ULI_PROBE("Pcreate_new_members", Pcreate_new_members_lockinfo);

    /* bs/bs_stripe.c probe_stuff */
    ULI_PROBE("Pstr_register_stripe_ag", Pstr_register_stripe_ag_lockinfo);
    ULI_PROBE("Pbs_stripe", Pbs_stripe_lockinfo);
    ULI_PROBE("Pstr_stripe", Pstr_stripe_lockinfo);
    ULI_PROBE("Pstripe_zero_size", Pstripe_zero_size_lockinfo);
    ULI_PROBE("Pstripe_nonzero_size", Pstripe_nonzero_size_lockinfo);
    ULI_PROBE("Pundo_xtnt_rec", Pundo_xtnt_rec_lockinfo);
    ULI_PROBE("Pupdate_xtnt_rec_fields", Pupdate_xtnt_rec_fields_lockinfo);
    ULI_PROBE("Pstr_create_stripe_hdr", Pstr_create_stripe_hdr_lockinfo);
    ULI_PROBE("Pstr_delete_stripe_hdr", Pstr_delete_stripe_hdr_lockinfo);
    ULI_PROBE("Pstr_calc_page_alloc", Pstr_calc_page_alloc_lockinfo);
    ULI_PROBE("Pstr_create_bf_rel_xtnt", Pstr_create_bf_rel_xtnt_lockinfo);
    ULI_PROBE("Pcreate_bf_rel_xtnt_des", Pcreate_bf_rel_xtnt_des_lockinfo);

    /* bs/ftx_recovery.c probe stuff */
    ULI_PROBE("Pftx_bfmeta_rec_redo", Pftx_bfmeta_rec_redo_lockinfo);
    ULI_PROBE("Pftx_bfdata_rec_redo", Pftx_bfdata_rec_redo_lockinfo);
    ULI_PROBE("Pftx_bfdmn_recovery", Pftx_bfdmn_recovery_lockinfo);
    ULI_PROBE("Pftx_recovery_pass", Pftx_recovery_pass_lockinfo);
    ULI_PROBE("Pnum_of_log_recs", Pnum_of_log_recs_lockinfo);

    /* msfs/bs/ftx_routines.c probe stuff */
    ULI_PROBE("Pftx_register_agent", Pftx_register_agent_lockinfo);
    ULI_PROBE("Pftx_register_agent_n", Pftx_register_agent_n_lockinfo);
    ULI_PROBE("Pftx_register_agent_n2", Pftx_register_agent_n2_lockinfo);
    ULI_PROBE("P_ftx_start_meta", P_ftx_start_meta_lockinfo);
    ULI_PROBE("P_ftx_start", P_ftx_start_lockinfo);
    ULI_PROBE("P_ftx_start_n", P_ftx_start_n_lockinfo);
    ULI_PROBE("P_ftx_start_exc", P_ftx_start_exc_lockinfo);
    ULI_PROBE("P_ftx_start_i", P_ftx_start_i_lockinfo);
    ULI_PROBE("Pnewftx", Pnewftx_lockinfo);
    ULI_PROBE("Pftx_done", Pftx_done_lockinfo);
    ULI_PROBE("Pftx_done_fs", Pftx_done_fs_lockinfo);
    ULI_PROBE("Pftx_done_n", Pftx_done_n_lockinfo);
    ULI_PROBE("Pftx_done_u", Pftx_done_u_lockinfo);
    ULI_PROBE("Pftx_done_urd", Pftx_done_urd_lockinfo);
    ULI_PROBE("Pftx_done_urdr", Pftx_done_urdr_lockinfo);
    ULI_PROBE("Pftx_quit", Pftx_quit_lockinfo);
    ULI_PROBE("Pftx_fail", Pftx_fail_lockinfo);
    ULI_PROBE("Pftx_fail_2", Pftx_fail_2_lockinfo);
    ULI_PROBE("Pftx_set_continuation", Pftx_set_continuation_lockinfo);
    ULI_PROBE("Pftx_special_done_mode", Pftx_special_done_mode_lockinfo);
    ULI_PROBE("Prbf_pinpg", Prbf_pinpg_lockinfo);
    ULI_PROBE("Prbf_deref_page", Prbf_deref_page_lockinfo);
    ULI_PROBE("Prbf_pin_record", Prbf_pin_record_lockinfo);
    ULI_PROBE("Paddone_undo", Paddone_undo_lockinfo);
    ULI_PROBE("Paddone_redo", Paddone_redo_lockinfo);
    ULI_PROBE("Paddone_rtdn", Paddone_rtdn_lockinfo);
    ULI_PROBE("Paddone_cont", Paddone_cont_lockinfo);
    ULI_PROBE("Paddone_recredo", Paddone_recredo_lockinfo);
    ULI_PROBE("Plog_donerec_nunpin", Plog_donerec_nunpin_lockinfo);
    ULI_PROBE("Pdo_ftx_continuations", Pdo_ftx_continuations_lockinfo);
    ULI_PROBE("Pget_perlvl_p", Pget_perlvl_p_lockinfo);
    ULI_PROBE("P_ftx_lock", P_ftx_lock_lockinfo);
    ULI_PROBE("P_ftx_add_lock", P_ftx_add_lock_lockinfo);
    ULI_PROBE("P_ftx_shared_lock", P_ftx_shared_lock_lockinfo);
    ULI_PROBE("P_ftx_exclusive_lock", P_ftx_exclusive_lock_lockinfo);
    ULI_PROBE("P_ftx_set_state", P_ftx_set_state_lockinfo);
    ULI_PROBE("Pftx_unlock", Pftx_unlock_lockinfo);
    ULI_PROBE("Prelease_ftx_locks", Prelease_ftx_locks_lockinfo);
    ULI_PROBE("Pget_ftx_id", Pget_ftx_id_lockinfo);
    ULI_PROBE("Pget_ftx_dmnh", Pget_ftx_dmnh_lockinfo);
    ULI_PROBE("Pftx_init", Pftx_init_lockinfo);
    ULI_PROBE("Pftx_free", Pftx_free_lockinfo);
    ULI_PROBE("Pftx_init_recovery_loga", Pftx_init_recovery_loga_lockinfo);
    ULI_PROBE("Pftx_get_dirtybufla", Pftx_get_dirtybufla_lockinfo);
    ULI_PROBE("Pftx_set_oldestftxla", Pftx_set_oldestftxla_lockinfo);
    ULI_PROBE("Pftx_get_oldestftxla", Pftx_get_oldestftxla_lockinfo);
    ULI_PROBE("Pftx_set_firstla", Pftx_set_firstla_lockinfo);
    ULI_PROBE("Preset_oldest_lsn", Preset_oldest_lsn_lockinfo);

    /* msfs/osf/msfs_proplist.c probe stuff */
    ULI_PROBE("Pmsfs_pl_register_agnts", Pmsfs_pl_register_agnts_lockinfo);
    ULI_PROBE("Pmsfs_pl_init_cur", Pmsfs_pl_init_cur_lockinfo);
    ULI_PROBE("Pmsfs_pl_cur_to_pnt", Pmsfs_pl_cur_to_pnt_lockinfo);
    ULI_PROBE("Pmsfs_pl_deref_cur", Pmsfs_pl_deref_cur_lockinfo);
    ULI_PROBE("Pmsfs_pl_pin_cur", Pmsfs_pl_pin_cur_lockinfo);
    ULI_PROBE("Pmsfs_pl_seek_cur", Pmsfs_pl_seek_cur_lockinfo);
    ULI_PROBE("Pmsfs_pl_next_mc_cur", Pmsfs_pl_next_mc_cur_lockinfo);
    ULI_PROBE("Pmsfs_setproplist", Pmsfs_setproplist_lockinfo);
    ULI_PROBE("Pmsfs_pl_set_entry", Pmsfs_pl_set_entry_lockinfo);
    ULI_PROBE("Pmsfs_pl_fill_hdr_image", Pmsfs_pl_fill_hdr_image_lockinfo);
    ULI_PROBE("Pmsfs_getproplist", Pmsfs_getproplist_lockinfo);
    ULI_PROBE("Pmsfs_pl_create_rec", Pmsfs_pl_create_rec_lockinfo);
    ULI_PROBE("Pmsfs_pl_get_entry", Pmsfs_pl_get_entry_lockinfo);
    ULI_PROBE("Pmsfs_pl_get_name", Pmsfs_pl_get_name_lockinfo);
    ULI_PROBE("Pmsfs_pl_get_data", Pmsfs_pl_get_data_lockinfo);
    ULI_PROBE("Pmsfs_delproplist", Pmsfs_delproplist_lockinfo);
    ULI_PROBE("Pmsfs_pl_del_root_done", Pmsfs_pl_del_root_done_lockinfo);
    ULI_PROBE("Pmsfs_pl_del_data_chain", Pmsfs_pl_del_data_chain_lockinfo);
    ULI_PROBE("Pmsfs_pl_unlink_mcells", Pmsfs_pl_unlink_mcells_lockinfo);

	/* kernel/msfs/fs/fs_dir_lookup.c	*/

	ULI_PROBE("Pinsert_seq", Pinsert_seq_lockinfo);
	ULI_PROBE("Premove_dir_ent", Premove_dir_ent_lockinfo);
	ULI_PROBE("Premove_dots", Premove_dots_lockinfo);
	ULI_PROBE("Pseq_search", Pseq_search_lockinfo);
	ULI_PROBE("Premove_bf", Premove_bf_lockinfo);
	ULI_PROBE("Ptag_search", Ptag_search_lockinfo);
	ULI_PROBE("Premove_entry", Premove_entry_lockinfo);
	ULI_PROBE("Pglom_dir_entries", Pglom_dir_entries_lockinfo);
	ULI_PROBE("Pdir_trunc_start", Pdir_trunc_start_lockinfo);
	ULI_PROBE("Pdir_trunc_finish", Pdir_trunc_finish_lockinfo);

        /* kernel/msfs/osf/msfs_vfsops.c */
        ULI_PROBE("Pmsfs_vptofh", Pmsfs_vptofh_lockinfo);
        ULI_PROBE("Padvfs_quotactl", Padvfs_quotactl_lockinfo);
        ULI_PROBE("Padvfs_quotactl_mp", Padvfs_quotactl_mp_lockinfo);
        ULI_PROBE("Padvfs_mountfs", Padvfs_mountfs_lockinfo);
        ULI_PROBE("Pmsfs_unmount", Pmsfs_unmount_lockinfo);
        ULI_PROBE("Pcheck_vd_sizes", Pcheck_vd_sizes_lockinfo);

        /* kernel/msfs/bs/bs_bitfile_sets.c */
        ULI_PROBE("Pbs_fragbf_thread", Pbs_fragbf_thread_lockinfo);
        ULI_PROBE("Pfrag_group_dealloc",Pfrag_group_dealloc_lockinfo);
        ULI_PROBE("Pfrag_group_init",Pfrag_group_init_lockinfo);
        ULI_PROBE("Pfrag_list_extend",Pfrag_list_extend_lockinfo);
        ULI_PROBE("Pbs_frag_alloc",Pbs_frag_alloc_lockinfo);
        ULI_PROBE("Pbs_frag_dealloc",Pbs_frag_dealloc_lockinfo);
        ULI_PROBE("Pbs_frag_has_stg",Pbs_frag_has_stg_lockinfo);
        ULI_PROBE("Pbs_fragbf_open",Pbs_fragbf_open_lockinfo);
        ULI_PROBE("Pbs_fragbf_close",Pbs_fragbf_close_lockinfo);
        ULI_PROBE("Pbs_bfs_init",Pbs_bfs_init_lockinfo);
        ULI_PROBE("Pbfs_lookup",Pbfs_lookup_lockinfo);
        ULI_PROBE("Pbfs_invalidate",Pbfs_invalidate_lockinfo);
        ULI_PROBE("Pbfs_alloc",Pbfs_alloc_lockinfo);
        ULI_PROBE("Pbfs_dealloc",Pbfs_dealloc_lockinfo);
        ULI_PROBE("Pbs_bfs_remove_all",Pbs_bfs_remove_all_lockinfo);
        ULI_PROBE("Pbs_bfs_get_handle",Pbs_bfs_get_handle_lockinfo);
        ULI_PROBE("Pbfs_create",Pbfs_create_lockinfo);
        ULI_PROBE("Prbf_bfs_create",Prbf_bfs_create_lockinfo);
        ULI_PROBE("Pbfs_close",Pbfs_close_lockinfo);
        ULI_PROBE("Pbs_bfs_close",Pbs_bfs_close_lockinfo);
        ULI_PROBE("Pbfs_access",Pbfs_access_lockinfo);
        ULI_PROBE("Pbfs_open",Pbfs_open_lockinfo);
        ULI_PROBE("Pdelete_orig_set_tags",Pdelete_orig_set_tags_lockinfo);
        ULI_PROBE("Pdelete_clone_set_tags",Pdelete_clone_set_tags_lockinfo);
        ULI_PROBE("Pbfs_delete_pending_list_add",Pbfs_delete_pending_list_add_lockinfo);
        ULI_PROBE("Pdel_list_remove_undo",Pdel_list_remove_undo_lockinfo);
        ULI_PROBE("Pbfs_delete_pending_list_remove",Pbfs_delete_pending_list_remove_lockinfo);
        ULI_PROBE("Pbfs_delete_pending_list_finish_all",Pbfs_delete_pending_list_finish_all_lockinfo);
        ULI_PROBE("Punlink_clone_undo",Punlink_clone_undo_lockinfo);
        ULI_PROBE("Punlink_clone",Punlink_clone_lockinfo);
        ULI_PROBE("Pbs_bfs_delete",Pbs_bfs_delete_lockinfo);
        ULI_PROBE("Pclone_tagdir",Pclone_tagdir_lockinfo);
        ULI_PROBE("Pbs_bfs_clone",Pbs_bfs_clone_lockinfo);
        ULI_PROBE("Pbs_bfs_out_of_sync",Pbs_bfs_out_of_sync_lockinfo);
        ULI_PROBE("Pbs_bfs_get_set_id",Pbs_bfs_get_set_id_lockinfo);
        ULI_PROBE("Pbs_bfs_get_clone_info",Pbs_bfs_get_clone_info_lockinfo);
        ULI_PROBE("Pbs_bfs_get_desc",Pbs_bfs_get_desc_lockinfo);
        ULI_PROBE("Pbs_bfs_add_root",Pbs_bfs_add_root_lockinfo);
        ULI_PROBE("Pbs_bfs_remove_root",Pbs_bfs_remove_root_lockinfo);
        ULI_PROBE("Pbs_bfs_switch_root_tagdir",Pbs_bfs_switch_root_tagdir_lockinfo);
        ULI_PROBE("Pbs_bfset_activate",Pbs_bfset_activate_lockinfo);
        ULI_PROBE("Pbs_bfset_deactivate",Pbs_bfset_deactivate_lockinfo);
        ULI_PROBE("Pbs_bfs_get_info",Pbs_bfs_get_info_lockinfo);
        ULI_PROBE("Pbs_bfs_find_set",Pbs_bfs_find_set_lockinfo);
        ULI_PROBE("Pnew_clone_mcell",Pnew_clone_mcell_lockinfo);
        ULI_PROBE("Pnew_clone_mcell_undo_opx",Pnew_clone_mcell_undo_opx_lockinfo);
        ULI_PROBE("Pinit_crmcell_opx",Pinit_crmcell_opx_lockinfo);
        ULI_PROBE("Pclone",Pclone_lockinfo);
        ULI_PROBE("Pbs_have_clone",Pbs_have_clone_lockinfo);
        ULI_PROBE("Pprint_out_of_sync_msg",Pprint_out_of_sync_msg_lockinfo);
        ULI_PROBE("Pbs_cow_pg",Pbs_cow_pg_lockinfo);
        ULI_PROBE("Pbs_cow",Pbs_cow_lockinfo);
        ULI_PROBE("Pbfs_upd_shelving_thresholds",Pbfs_upd_shelving_thresholds_lockinfo);
        ULI_PROBE("Pbs_get_bfset_params",Pbs_get_bfset_params_lockinfo);
        ULI_PROBE("Pbs_set_bfset_params",Pbs_set_bfset_params_lockinfo);
        ULI_PROBE("Prbf_set_bfset_params",Prbf_set_bfset_params_lockinfo);

        /* kernel/msfs/bs/ms_logger.c */
	ULI_PROBE("Plgr_lsn_gt", Plgr_lsn_gt_lockinfo);
	ULI_PROBE("Plgr_lsn_lt", Plgr_lsn_lt_lockinfo);
	ULI_PROBE("Plsn_add", Plsn_add_lockinfo);
	ULI_PROBE("Plsn_gt", Plsn_gt_lockinfo);
	ULI_PROBE("Plsn_lt", Plsn_lt_lockinfo);
	ULI_PROBE("Plgr_make_pg_safe", Plgr_make_pg_safe_lockinfo);
	ULI_PROBE("Plgr_valid_blk", Plgr_valid_blk_lockinfo);
	ULI_PROBE("Plgr_repair_pg", Plgr_repair_pg_lockinfo);
	ULI_PROBE("Plgr_complete_pg", Plgr_complete_pg_lockinfo);
	ULI_PROBE("Plgr_restore_pg", Plgr_restore_pg_lockinfo);
	ULI_PROBE("Plgr_valid_pg", Plgr_valid_pg_lockinfo);
	ULI_PROBE("Plgr_refpg", Plgr_refpg_lockinfo);
	ULI_PROBE("Plgr_unpinpg", Plgr_unpinpg_lockinfo);
	ULI_PROBE("Plgr_init", Plgr_init_lockinfo);
	ULI_PROBE("Padd_lsn_list", Padd_lsn_list_lockinfo);
	ULI_PROBE("Plgr_writev_ftx", Plgr_writev_ftx_lockinfo);
	ULI_PROBE("Pget_clean_pg", Pget_clean_pg_lockinfo);
	ULI_PROBE("Plgr_retest_firstrec", Plgr_retest_firstrec_lockinfo);
	ULI_PROBE("Prelease_dirty_pg", Prelease_dirty_pg_lockinfo);
	ULI_PROBE("Plgr_write", Plgr_write_lockinfo);
	ULI_PROBE("Plgr_read_open", Plgr_read_open_lockinfo);
	ULI_PROBE("Plgr_dmn_read_open", Plgr_dmn_read_open_lockinfo);
	ULI_PROBE("Plgr_read_close", Plgr_read_close_lockinfo);
	ULI_PROBE("Plgr_read", Plgr_read_lockinfo);
	ULI_PROBE("Pget_pg_lsn", Pget_pg_lsn_lockinfo);
	ULI_PROBE("Pfind_log_end_pg", Pfind_log_end_pg_lockinfo);
	ULI_PROBE("Psetup_log_desc", Psetup_log_desc_lockinfo);
	ULI_PROBE("Plog_init", Plog_init_lockinfo);
	ULI_PROBE("Plgr_open", Plgr_open_lockinfo);
	ULI_PROBE("Plgr_close", Plgr_close_lockinfo);
	ULI_PROBE("Plgr_flush", Plgr_flush_lockinfo);
	ULI_PROBE("Plgr_flush_start", Plgr_flush_start_lockinfo);
	ULI_PROBE("Plgr_flush_sync", Plgr_flush_sync_lockinfo);
	ULI_PROBE("Plog_flush", Plog_flush_lockinfo);
	ULI_PROBE("Plog_flush_sync", Plog_flush_sync_lockinfo);
	ULI_PROBE("Plgr_get_last_rec", Plgr_get_last_rec_lockinfo);
	ULI_PROBE("Plgr_dmn_get_last_rec", Plgr_dmn_get_last_rec_lockinfo);
	ULI_PROBE("Plgr_get_first_rec", Plgr_get_first_rec_lockinfo);
	ULI_PROBE("Plgr_dmn_get_first_rec", Plgr_dmn_get_first_rec_lockinfo);
	ULI_PROBE("Plgr_dmn_get_pseudo_first_rec", Plgr_dmn_get_pseudo_first_rec_lockinfo);
	ULI_PROBE("Plgr_checkpoint_log", Plgr_checkpoint_log_lockinfo);
	ULI_PROBE("Plgr_switch_vol", Plgr_switch_vol_lockinfo);
	ULI_PROBE("Plgr_calc_num_pgs", Plgr_calc_num_pgs_lockinfo);
    
	/* kernel/msfs/bs/bs_sbm.c */
	ULI_PROBE("Pbitmap_undo_opx", Pbitmap_undo_opx_lockinfo);
        ULI_PROBE("Pinit_bs_bitmap_opx", Pinit_bs_bitmap_opx_lockinfo);
	ULI_PROBE("Psbm_set_pg_bits", Psbm_set_pg_bits_lockinfo);
        ULI_PROBE("Palloc_bits", Palloc_bits_lockinfo);
	ULI_PROBE("Palloc_bits_page", Palloc_bits_page_lockinfo);
        ULI_PROBE("Pdealloc_bits_no_sub_ftx", Pdealloc_bits_no_sub_ftx_lockinfo);
	ULI_PROBE("Pdealloc_bits_page", Pdealloc_bits_page_lockinfo);
        ULI_PROBE("Psbm_find_space", Psbm_find_space_lockinfo);
	ULI_PROBE("Psbm_remove_space", Psbm_remove_space_lockinfo);
	ULI_PROBE("Psbm_return_space_no_sub_ftx", Psbm_return_space_no_sub_ftx_lockinfo);
	ULI_PROBE("Premove_desc", Premove_desc_lockinfo);
	ULI_PROBE("Padd_cache", Padd_cache_lockinfo);
	ULI_PROBE("Premove_cache", Premove_cache_lockinfo);
	ULI_PROBE("Pload_x_cache", Pload_x_cache_lockinfo);
	ULI_PROBE("Psbm_init", Psbm_init_lockinfo);
	ULI_PROBE("Psbm_clear_cache", Psbm_clear_cache_lockinfo);
	ULI_PROBE("Psbm_total_free_space", Psbm_total_free_space_lockinfo);
	ULI_PROBE("Psbm_dump", Psbm_dump_lockinfo);

	/* kernel/msfs/bs/bs_stg.c */
	
	ULI_PROBE("Padd_stg_undo", Padd_stg_undo_lockinfo);
	ULI_PROBE("Pinit_bs_stg_opx", Pinit_bs_stg_opx_lockinfo);
	ULI_PROBE("Prbf_add_overlapping_stg", Prbf_add_overlapping_stg_lockinfo);
	ULI_PROBE("Prbf_add_overlapping_clone_stg", Prbf_add_overlapping_clone_stg_lockinfo);
	ULI_PROBE("Prbf_add_stg", Prbf_add_stg_lockinfo);
	ULI_PROBE("Pstg_add_stg", Pstg_add_stg_lockinfo);
	ULI_PROBE("Pstg_add_stg_no_cow", Pstg_add_stg_no_cow_lockinfo);
	ULI_PROBE("Pstg_set_alloc_disk", Pstg_set_alloc_disk_lockinfo);
	ULI_PROBE("Padd_stg", Padd_stg_lockinfo);
	ULI_PROBE("Padd_rsvd_stg", Padd_rsvd_stg_lockinfo);
	ULI_PROBE("Palloc_stg", Palloc_stg_lockinfo);
	ULI_PROBE("Palloc_append_stg", Palloc_append_stg_lockinfo);
	ULI_PROBE("Pget_first_mcell", Pget_first_mcell_lockinfo);
	ULI_PROBE("Palloc_hole_stg", Palloc_hole_stg_lockinfo);
	ULI_PROBE("Pstg_alloc_from_svc_class", Pstg_alloc_from_svc_class_lockinfo);
	ULI_PROBE("Pextend_skip", Pextend_skip_lockinfo);
	ULI_PROBE("Pstg_alloc_from_one_disk", Pstg_alloc_from_one_disk_lockinfo);
	ULI_PROBE("Palloc_from_one_disk", Palloc_from_one_disk_lockinfo);
	ULI_PROBE("Palloc_from_bitmap", Palloc_from_bitmap_lockinfo);
	ULI_PROBE("Pxfer_stg", Pxfer_stg_lockinfo);
	ULI_PROBE("Pxfer_stg_to_bf", Pxfer_stg_to_bf_lockinfo);
	ULI_PROBE("Pxfer_stg_to_xtnts", Pxfer_stg_to_xtnts_lockinfo);
	ULI_PROBE("Pxfer_append_stg", Pxfer_append_stg_lockinfo);
	ULI_PROBE("Pxfer_hole_stg", Pxfer_hole_stg_lockinfo);
	ULI_PROBE("Pxfer_stg_on_one_disk", Pxfer_stg_on_one_disk_lockinfo);
	ULI_PROBE("Pxfer_stg_in_one_subxtnt", Pxfer_stg_in_one_subxtnt_lockinfo);
	ULI_PROBE("Pstg_alloc_new_mcell", Pstg_alloc_new_mcell_lockinfo);
	ULI_PROBE("Px_page_to_iolist", Px_page_to_iolist_lockinfo);
	ULI_PROBE("Px_page_to_blkmap", Px_page_to_blkmap_lockinfo);
	ULI_PROBE("Px_copypage_to_blkmap", Px_copypage_to_blkmap_lockinfo);
	ULI_PROBE("Pscan_xtnt_map_list", Pscan_xtnt_map_list_lockinfo);
	ULI_PROBE("Px_page_to_blk", Px_page_to_blk_lockinfo);
	ULI_PROBE("Px_page_mapped", Px_page_mapped_lockinfo);
	ULI_PROBE("Ppage_is_mapped", Ppage_is_mapped_lockinfo);
	ULI_PROBE("Pstg_remove_stg_start", Pstg_remove_stg_start_lockinfo);
	ULI_PROBE("Pstg_remove_stg_finish", Pstg_remove_stg_finish_lockinfo);
	ULI_PROBE("Premove_stg", Premove_stg_lockinfo);
	ULI_PROBE("Pdealloc_stg", Pdealloc_stg_lockinfo);
	ULI_PROBE("Pclip_del_xtnt_map", Pclip_del_xtnt_map_lockinfo);
	ULI_PROBE("Pmerge_xtnt_maps", Pmerge_xtnt_maps_lockinfo);
	ULI_PROBE("Pmerge_sub_xtnt_maps", Pmerge_sub_xtnt_maps_lockinfo);
	ULI_PROBE("Punload_sub_xtnt_maps", Punload_sub_xtnt_maps_lockinfo);

	/* kernel/msfs/osf/msfs_io.c */
	ULI_PROBE("Pdomain_panic", Pdomain_panic_lockinfo);
	ULI_PROBE("Pbs_osf_complete", Pbs_osf_complete_lockinfo);
	ULI_PROBE("Pmsfs_iodone", Pmsfs_iodone_lockinfo);
	ULI_PROBE("Pmsfs_async_iodone_lwc", Pmsfs_async_iodone_lwc_lockinfo);
	ULI_PROBE("Pcheck_consol_list", Pcheck_consol_list_lockinfo);
	ULI_PROBE("Plist_map", Plist_map_lockinfo);
	ULI_PROBE("Pconsecutive_list_io", Pconsecutive_list_io_lockinfo);
	ULI_PROBE("Pcall_disk", Pcall_disk_lockinfo);
	ULI_PROBE("Pbs_startio", Pbs_startio_lockinfo);
	ULI_PROBE("Pread_raw_page", Pread_raw_page_lockinfo);
	ULI_PROBE("Pwrite_raw_page", Pwrite_raw_page_lockinfo);
	ULI_PROBE("Psetbits", Psetbits_lockinfo);
	ULI_PROBE("Pclearbits", Pclearbits_lockinfo);
	ULI_PROBE("Pfindbits", Pfindbits_lockinfo);

	/* kernel/msfs/osf/msfs_lookup.c */
	ULI_PROBE("Pstrtoul", Pstrtoul_lockinfo);
	ULI_PROBE("Pmsfs_lookup", Pmsfs_lookup_lockinfo);
	ULI_PROBE("Pcheck_path_back", Pcheck_path_back_lockinfo);
	ULI_PROBE("Pnew_parent", Pnew_parent_lockinfo);

	/* kernel/msfs/osf/msfs_syscalls.c */
	ULI_PROBE("Pmsfs_syscall", Pmsfs_syscall_lockinfo);

#endif /* MSFS */

	/********************************************/
	/* start of lockinfo specs for simple locks */
	/********************************************/

#if DLM > 0
        ULI("dlm_pdb_lock",         dlm_pdb_lockinfo,         SPLNONE,DEFAULT);
        ULI("dlm_lkid_lock",        dlm_lkid_lockinfo,        SPLNONE,DEFAULT);
        ULI("dlm_lkb_lock",         dlm_lkb_lockinfo,         SPLNONE,DEFAULT);
        ULI("dlm_rsb_lock",         dlm_rsb_lockinfo,         SPLNONE,DEFAULT);
        ULI("dlm_rsb_incarn_lock",  dlm_rsb_incarn_lockinfo,  SPLNONE,DEFAULT);
        ULI("dlm_rhash_chain_lock", dlm_rhash_chain_lockinfo, SPLNONE,DEFAULT);
        ULI("dlm_rsbinfo_respq_lock",dlm_rsbinfo_respq_lockinfo,SPLNONE,DEFAULT);
        ULI("dlm_msg_respq_lock",   dlm_msg_respq_lockinfo,   SPLNONE,DEFAULT);
        ULI("dlm_deadlockq_lock",   dlm_deadlockq_lockinfo,   SPLNONE,DEFAULT);
        ULI("dlm_timeoutq_lock",    dlm_timeoutq_lockinfo,    SPLNONE,DEFAULT);
#endif /* DLM */
#if DLMSL > 0
	ULI("malloc_smp_lock", malloc_smp_lockinfo,    SPLNONE, DEFAULT);
#endif /* DLMSL */
#if DRD > 0
	ULI("drd_maphd_lock",  drd_maphd_lockinfo,     SPLNONE, DEFAULT);
        ULI("drd_map_lock",  drd_map_lockinfo,     SPLNONE, DEFAULT);
	ULI("drd_bsc_chtabhead_lock", bsc_chtabhead_lockinfo, SPLNONE, DEFAULT);
	ULI("drd_bssdupreq_lock", bssdupreq_lock_info, SPLNONE, DEFAULT);
	ULI("drd_bss_sv_active_lock", bss_sv_active_lockinfo, SPLNONE, DEFAULT);
	ULI("drd_bp_lock", drd_bp_lockinfo, SPLNONE, DEFAULT);
	ULI("bsc_rm_serial_lock", bsc_rm_serial_lockinfo, SPLNONE, DEFAULT);
	ULI("bsc_biod_lock", bsc_biod_lock_info, SPLNONE, DEFAULT);
#endif

	SLI("hostname_lock", hostname_lockinfo, SPLNONE, DEFAULT);
	SLI("arptab_lock", arptab_li, SPLNONE, DEFAULT);
	NLI("inifaddr_spin_lock", inifaddr_spin_li, SPLNONE, DEFAULT);
	NLI("ifqueue.ifq_slock_splnone", ifqueue_splnone_li, SPLNONE, DEFAULT);
	NLI("domain.dom_rc_lock", dom_rc_li, SPLNONE, DEFAULT);
	NLI("global_unpconn_lock", global_unpconn_li, SPLNONE, DEFAULT);

	ULI("rpc_cl_xid_lock", rpc_cl_xid_lock_info, SPLNONE, DEFAULT);
	ULI("lk_rpcrqcred", lk_rpcrqcred_info, SPLNONE, DEFAULT);
	ULI("lk_rpccallout", lk_rpccallout_info, SPLNONE, DEFAULT);
	ULI("lk_rpcdupreq", lk_rpcdupreq_info, SPLNONE, DEFAULT);

	ULI("nfs_activewrites_lock",nfs_activewrites_lockinfo,SPLNONE,DEFAULT);
	ULI("nfs3_activewrites_lock",nfs3_activewrites_lockinfo,SPLNONE,DEFAULT);
	ULI("nfs_writefree_lock", nfs_writefree_lockinfo, SPLNONE, DEFAULT);
	ULI("nfs3_writefree_lock", nfs3_writefree_lockinfo, SPLNONE, DEFAULT);
	ULI("nfs_sys_lock", nfs_sys_lockinfo, SPLNONE, DEFAULT);
	ULI("tslist_lock", nfs_tslist_lockinfo, SPLNONE, DEFAULT);
	ULI("chlist_lock", chlist_lockinfo, SPLNONE, DEFAULT);
	ULI("asyncdaemon_lock", asyncdaemon_lockinfo, SPLNONE, DEFAULT);
	ULI("unixauthtab_lock", unixauthtab_lockinfo, SPLNONE, DEFAULT);
	ULI("nfs_minor_lock", nfs_minor_lockinfo, SPLNONE, DEFAULT);
	ULI("nfs_newnum_lock", nfs_newnum_lockinfo, SPLNONE, DEFAULT);
	ULI("rnode.r_statelock", rnode_state_lockinfo, SPLNONE, DEFAULT);
	ULI("mntinfo.mi_lock", mntinfo_lockinfo, SPLNONE, DEFAULT);
	ULI("presto_lock", presto_lockinfo, SPLNONE, DEFAULT);

	SLI("shm_stats_lock", shm_stats_lockinfo, SPLNONE, DEFAULT);
	SLI("shmid_lock", shmid_lockinfo, SPLNONE, DEFAULT);
	SLI("msgid_lock", msgid_lockinfo, SPLNONE, DEFAULT);
	SLI("semid_lock", semid_lockinfo, SPLNONE, DEFAULT);
	SLI("undo_lock", undo_lockinfo, SPLNONE, DEFAULT);
	SLI("cur_msghdrs", cur_msghdrs_lockinfo, SPLNONE, DEFAULT);
	SLI("all_zones_lock", all_zones_lockinfo, SPLNONE, DEFAULT);
	SLI("stack_queue_lock", stack_queue_lockinfo, SPLNONE, DEFAULT);
	SLI("all_psets_lock", all_psets_lockinfo, SPLNONE, DEFAULT);

	ULI("pgrp.pg_sig_lock", pgrp_sig_lockinfo, SPLNONE, DEFAULT);
	ULI("accounting_lock", accounting_lockinfo, SPLNONE, DEFAULT);
	ULI("aiosb_lock", aiosb_lockinfo, SPLNONE, DEFAULT);
	ULI("gidhash_lock", gidhash_lockinfo, SPLNONE, DEFAULT);
	ULI("uidhash_lock", uidhash_lockinfo, SPLNONE, DEFAULT);
	ULI("ruidhash_lock", ruidhash_lockinfo, SPLNONE, DEFAULT);
	ULI("pid_lock", pid_lockinfo, SPLNONE, DEFAULT);
	ULI("proc.p_lock", proc_lockinfo, SPLNONE, ORDERED);
	ULI("aio_lock", aio_lockinfo, SPLNONE, DEFAULT);
	ULI("session.s_lock", sess_lockinfo, SPLNONE, DEFAULT);

	SLI("task_swap_queues_lock",task_swap_queues_lockinfo,SPLNONE,DEFAULT);
	SLI("task.lock", task_lockinfo, SPLNONE, ORDERED);
	SLI("processor_set.lock", pset_lockinfo, SPLNONE, ORDERED);
	SLI("task.ipc_translation_lock", ipc_task_lockinfo, SPLNONE, DEFAULT);
	SLI("thread.ipc_state_lock", ipc_thread_lockinfo, SPLNONE, DEFAULT);
	SLI("kern_port.port_data_lock", kern_port_lockinfo, SPLNONE, DEFAULT);
	SLI("kern_set.set_data_lock", kern_set_lockinfo, SPLNONE, ORDERED);
	SLI("msg_queue.lock", msg_queue_lockinfo, SPLNONE, ORDERED);
	SLI("port_hash_bucket.lock", port_hash_lockinfo, SPLNONE, DEFAULT);
	SLI("reply_port_cache_lock", reply_pcache_lockinfo, SPLNONE, DEFAULT);
	SLI("task_ref_count_lock", task_ref_count_lockinfo, SPLNONE, DEFAULT);

	ULI("kiorequest.req_lk", kio_reqs_lk_info, SPLNONE, DEFAULT);
	ULI("kio_sys_lk", kio_sys_lockinfo, SPLNONE, DEFAULT);
	ULI("kiohandle.kio_handle_lk", kio_handles_lk_info, SPLNONE, DEFAULT);
#if MSFS > 0
	/********************************************************
	 * AdvFS simple locks in the hierarchy
	 ********************************************************/
	ULI("DmnTblMutex", ADVDmnTblMutex_lockinfo, SPLNONE, DEFAULT );
	ULI("BfSetTblMutex", ADVBfSetTblMutex_lockinfo, SPLNONE, DEFAULT );
	ULI("BfAccessTblMutex", ADVBfAccessTblMutex_lockinfo, SPLNONE, DEFAULT );
	ULI("DomainMutex", ADVdomainT_mutex_lockinfo, SPLNONE, DEFAULT );
	ULI("LookupMutex", ADVLookupMutex_lockinfo, SPLNONE, DEFAULT );
	ULI("IoQueueMutex", ADVIoQueueMutex_lockinfo, SPLNONE, DEFAULT );
	ULI("LockMgrMutex.mutex",  msfs_lockmgrmutex_lockinfo, SPLNONE, DEFAULT );
	ULI("mutexT.mutex",  msfs_mp_mutex_lockinfo, SPLNONE, DEFAULT );
	ULI("filesetMutex", ADVfilesetMutex_lockinfo, SPLNONE, DEFAULT );
	ULI("dquotMutex", ADVdquotMutex_lockinfo, SPLNONE, DEFAULT );
	ULI("fsContext_mutex", ADVfsContext_mutex_lockinfo, SPLNONE, DEFAULT );
	ULI("dirMutex", ADVbfSetT_dirMutex_lockinfo, SPLNONE, DEFAULT );
	ULI("bfSetT.dirMutex", ADVbfSetT_dirMutex_lockinfo, SPLNONE, DEFAULT );
	ULI("FtxMutex", ADVFtxMutex_lockinfo, SPLNONE, DEFAULT );
	ULI("BfAccessTraceMutex", ADVBfAccessTraceMutex_lockinfo, SPLNONE, DEFAULT );
	ULI("msgQT.mutex", ADVmsgQT_mutex_lockinfo, SPLNONE, DEFAULT );
	ULI("dqhead_dqh_mutex", ADVdqhead_dqh_mutex_lockinfo, SPLNONE, DEFAULT );
	ULI("Ms_malloc_mutex", ADVMs_malloc_mutex_lockinfo, SPLNONE, DEFAULT );
#endif
	ULI("ufile_state.uf_ofile_lock", ufd_lockinfo, SPLNONE, DEFAULT);
	ULI("utask_nd.utnd_lock", utnd_lockinfo, SPLNONE, DEFAULT);
	ULI("ihash_lock", ihash_lockinfo, SPLNONE, DEFAULT);
	ULI("inode.i_incore_lock", i_incore_lockinfo, SPLNONE, ORDERED);
	ULI("utask.uu_handy_lock", u_handy_lockinfo, SPLNONE, DEFAULT);
	ULI("real_ucred.lock", ucred_lockinfo, SPLNONE, DEFAULT);
	ULI("file.f_incore_lock", file_lockinfo, SPLNONE, DEFAULT);
	ULI("vnode.v_lock", vnode_lockinfo, SPLNONE, DEFAULT);
	ULI("namei.nch_lockinfo", nch_lockinfo, SPLNONE, DEFAULT);
	ULI("vn_free_lock", vn_free_lockinfo, SPLNONE, DEFAULT);
	ULI("nc_freelist_lockinfo", nc_freelist_lockinfo, SPLNONE, DEFAULT);
	ULI("nc_purge_lockinfo", nc_purge_lockinfo, SPLNONE, DEFAULT);

	NLI("unp_misc_lock", unp_misc_li, SPLNONE, DEFAULT);
	NLI("tssmap_lock", tssmap_li, SPLNONE, DEFAULT);
	NLI("tcb.inp_rc_lock", inp_tcp_rc_li, SPLNONE, DEFAULT);
	NLI("udb.inp_rc_lock", inp_udp_rc_li, SPLNONE, DEFAULT);
	NLI("mfclencap_lock", mfclencap_li, SPLNONE, DEFAULT);
#if LOCK_NETSTATS
	NLI("rtstat.rts_lock", rtstat_li, SPLNONE, DEFAULT);
	NLI("icmpstat.icps_lock", icmpstat_li, SPLNONE, DEFAULT);
	NLI("igmpstat.igps_lock", igmpstat_li, SPLNONE, DEFAULT);
	NLI("mrtstat.mrts_lock", mrtstat_li, SPLNONE, DEFAULT);
	NLI("ipstat.ips_lock", ipstat_li, SPLNONE, DEFAULT);
#endif
	NLI("misc_tcp_lock", misc_tcp_li, SPLNONE, DEFAULT);
#if LOCK_NETSTATS
	NLI("tcpstat.tcps_lock", tcpstat_li, SPLNONE, DEFAULT);
	NLI("udpstat.udps_lock", udpstat_li, SPLNONE, DEFAULT);
#endif
#if NPTY > 0
	ULI("pty_lock", pty_lockinfo, SPLNONE, DEFAULT);
#endif
#if RPTY > 0
	SLI("pty_lock", pty_lockinfo, SPLNONE, DEFAULT);
#endif
#if DEC_AUDIT > 0
	ULI("audlog.lock", audlog_lockinfo, SPLNONE, DEFAULT);
	ULI("audselq.lock", audselq_lockinfo, SPLNONE, DEFAULT);
	ULI("audbuf.lock", audbuf_lockinfo, SPLNONE, DEFAULT);
	ULI("audmask.lock", audmask_lockinfo, SPLNONE, DEFAULT);
#endif

	SLI("consvp_lock", consvp_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_map.ref_lock", vm_map_ref_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_map_entry.vme_faultlock", vme_fault_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_vp_object.vu_seglock", vop_seglockinfo, SPLNONE, DEFAULT);
	SLI("vm_object.ob_lock", vm_object_lockinfo, SPLNONE, ORDERED);
	SLI("vm_map.hint_lock", vm_map_hint_lockinfo, SPLNONE, DEFAULT);
	SLI("gr_mapping_lock", mapping_lock_lockinfo, SPLNONE, DEFAULT);
	SLI("gh_stats_lock", mapping_lock_lockinfo, SPLNONE, DEFAULT);
	SLI("gr_pagezeroer_lock", gr_pagezeroer_lockinfo, SPLNONE, DEFAULT);
	SLI("gr_collect_lock", gr_collect_lockinfo, SPLNONE, DEFAULT);
	SLI("granhint_lock", granhint_lockinfo, SPLNONE, DEFAULT);
#if RMVM > 0
	SLI("vm_rm_object.links_lock", vm_rm_object_links_lockinfo, SPLNONE, ORDERED);
	SLI("rm_object_map_recv_lock",rm_object_map_recv_lockinfo, SPLNONE, DEFAULT); 
	SLI("rm_object_map_trans_base_lock",rm_object_map_trans_base_lockinfo, SPLNONE, DEFAULT);
 	SLI("rm_object_map_dual_base_lock",rm_object_map_dual_base_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_rm_object.ob_lock", vm_rm_object_lockinfo, SPLNONE, DEFAULT);
#endif
	SLI("shared_segment.ss_lock", shared_segment_lockinfo, SPLNONE,DEFAULT);
	SLI("a_lock", a_lock_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_swap_circular_lock", vm_swap_circular_lockinfo,SPLNONE,DEFAULT);
	SLI("vm_swap.vs_lock", vm_swap_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_swap_io_lock", vm_swap_io_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_swap_space_lock", vm_swap_space_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_page_queue_lock", vm_page_queue_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_page_lock_data", vm_page_lockinfo, SPLNONE, DEFAULT);
	SLI("ubc_lru_lock", ubc_lru_lockinfo, SPLNONE, DEFAULT);
	SLI("ubc_pages_lock", ubc_pages_lockinfo, SPLNONE, DEFAULT);
	SLI("ubc_dirty_lock", ubc_dirty_lockinfo, SPLNONE, DEFAULT);
	SLI("ubc_iocount_lock", ubc_iocount_lockinfo, SPLNONE, DEFAULT);
	SLI("u_seg_cache_lock", u_seg_cache_lockinfo, SPLNONE, DEFAULT);
	SLI("ptepages_lock", ptepages_lockinfo, SPLNONE, DEFAULT);
	SLI("anon_klock.akl_slock", akl_slock_lockinfo, SPLNONE, DEFAULT);
	SLI("pmap_memdsc_lock", pmap_memdsc_lockinfo, SPLNONE, DEFAULT);
	SLI("binlog_selq_lock", binlog_selq_lockinfo, SPLNONE, DEFAULT);

#if STREAMS != 0 || TIMOD || SVVS || STRKINFO || STRIFNET || XTISO || DLB || DLPI || LDTTY || RPTTY || PCKT || LAT 
	SLI("modsw_lock", modsw_lockinfo, SPLNONE, DEFAULT);
	SLI("sth_pipestat.lock", stb_lockinfo, SPLNONE, DEFAULT);
	SLI("sad_lock", sad_lockinfo, SPLNONE, DEFAULT);
	SLI("muxid_lock", streams_muxid_lockinfo, SPLNONE, DEFAULT);
	SLI("iocblk_id_lock", iocblk_id_lockinfo, SPLNONE, DEFAULT);
	SLI("weldq_lock", weldq_lockinfo, SPLNONE, DEFAULT);
	SLI("bufc_lock",streams_bufc_lockinfo, SPLNONE, DEFAULT);
	SLI("active_qlock_lock", active_qlock_lockinfo, SPLNONE, DEFAULT);
	SLI("streams_open_lock", streams_open_lockinfo, SPLNONE, DEFAULT);
	SLI("sth_s.sth_ext_flags_lock", sth_ext_flags_lockinfo, SPLNONE, DEFAULT);
#endif	/* STREAMS || TIMOD || SVVS || STRKINFO || STRIFNET || XTISO || DLB || DLPI || LDTTY || RPTTY || PCKT || LAT */

#if PPP
	SLI("ppp_if_table_lock", ppp_if_table_lock_info, SPLNONE, DEFAULT);
	SLI("ppp_if_info.ppp_if_element_lock", ppp_if_element_lock_info, SPLNONE, DEFAULT);
	SLI("ppp_comp_table_lock", ppp_comp_table_lock_info, SPLNONE, DEFAULT);
	SLI("ppp_comp_info.ppp_comp_element_lock", ppp_comp_element_lock_info, SPLNONE, DEFAULT);
	SLI("ppp_async_table_lock", ppp_async_table_lock_info, SPLNONE, DEFAULT);
	SLI("ppp_async_info.ppp_async_element_lock", ppp_async_element_lock_info, SPLNONE, DEFAULT);
#endif	/* PPP */

#if XTISO
	SLI("xtiso_lock", xtiso_lockinfo, SPLNONE, DEFAULT);
	SLI("xtisoc_lock", xtisoc_lockinfo, SPLNONE, DEFAULT);
#endif	/* XTISO */

#if LDTTY
	SLI("ldtty.t_intimeout_lock", tp_intimeout_lockinfo, SPLNONE, DEFAULT);
#endif

#if 0
#if RT_SEM
	SLI("real_time.p4bsem_lock", p4bsem_lockinfo, SPLNONE, DEFAULT);
#endif
#endif
#if	(DLB > 0)
	NLI("dlbq_list_head_slock", dlbq_list_sli, SPLNONE, DEFAULT);
#endif
#if RT_CSEM || RT_IPC
	SLI("real_time.p4csem_lock", p4csem_lockinfo, SPLNONE, DEFAULT);
#endif
#if MCNET > 0
        LI("mcnet_psend_lock", mcnet_psend_lockinfo, SPLNONE, DEFAULT);
#endif
#if RM > 0
	LI("rm_mgmt_locks", rm_mgmt_lockinfo, SPLNONE, DEFAULT);
	LI("rm_mgmt_tag_lock", rm_mgmt_tag_lockinfo, SPLNONE, DEFAULT);
	LI("rm_mgmt_bitmap_lock", rm_mgmt_bitmap_lockinfo, SPLNONE, DEFAULT);
	LI("rm_map_data_lock", rm_map_data_lockinfo, SPLNONE, DEFAULT);
	LI("rm_callback_lock", rm_callback_lockinfo, SPLNONE, DEFAULT);
#endif
	ULI("cxalloc_lock ", cxalloc_lockinfo, SPLSOFTC, DEFAULT);
	ULI("rci_registry_lock", rci_registry_lockinfo, SPLSOFTC, DEFAULT);
	ULI("rci_event_lock", rci_event_lockinfo, SPLSOFTC, DEFAULT);
	ULI("error_event_lock", event_cntrl_li, SPLSOFTC, DEFAULT);

#if DLPI
	NLI("dltimer_lock", dltimer_lockinfo, SPLSOFTC, DEFAULT);
#endif

#ifdef	ASE
	ULI("amlog",            amlog_lock_li,          SPLSOFTC, DEFAULT);
	ULI("amcntrl",          am_cntrl_lock_li,       SPLSOFTC, DEFAULT);
	ULI("amcntrl.hflink",   am_host_lk_li,          SPLSOFTC, DEFAULT);
	ULI("amcntrl.dflink",   am_device_lk_li,        SPLSOFTC, DEFAULT);
	ULI("ase_table",        ase_table_lock_li,      SPLSOFTC, DEFAULT);
#endif

#ifdef	DTM
	ULI("dtm_stack_lock",	dtm_stack_lockinfo,  SPLSOFTC, DEFAULT);
	ULI("dtm_list_lock",	dtm_list_lockinfo,   SPLSOFTC, DEFAULT);
	ULI("dtm_control",	dtm_control_lockinfo, SPLNONE, DEFAULT);
#endif

#if ATM
        ULI("atm_cmm_mmi_list",atm_cmm_mmi_list_info, SPLIMP,DEFAULT);
#if ATMIP
        ULI("atm_arp_delevt_lock", atm_arp_delevt_lock_li, SPLSOFTC, DEFAULT);
#endif
#endif

	SLI("log_lock", log_lockinfo, SPL1, DEFAULT);
	NLI("ifqueue.ifq_slock_splnet", ifqueue_splnet_li, SPLNET, DEFAULT);

#if RM > 0
        /*
         * MC interrupts (like all PCI interrupts on Alpha) can come in at 
         * SPLDEVLOW or SPLDEVHIGH.  We use SPLDEVLOW in this declaration 
         * to cover the case when they come in at SPLDEVLOW.
         */
	SLI("rm_error_lock", rm_error_lockinfo, SPLDEVLOW, DEFAULT);
	SLI("rm_init_state_lock", rm_init_state_lockinfo, SPLDEVLOW, DEFAULT);
	SLI("rm_thread_lock", rm_thread_lockinfo, SPLDEVLOW, DEFAULT);
	SLI("rm_error_shutting_down_state_lock", rm_error_shutting_down_state_lockinfo, SPLDEVLOW, DEFAULT);
	SLI("rm_object_map_trans_intr_lock",rm_object_map_trans_intr_lockinfo, SPLDEVLOW, DEFAULT);
 	SLI("rm_object_map_dual_intr_lock",rm_object_map_dual_intr_lockinfo, SPLDEVLOW, DEFAULT);
	SLI("vm_rm_object_mappers_lock", vm_rm_object_mappers_lockinfo, SPLDEVLOW, DEFAULT);
	SLI("vm_rm_base_address_lock", vm_rm_base_address_lockinfo, SPLDEVLOW, DEFAULT);
#endif
#if MCNET > 0
        LI("mcnet_reflock", mcnet_reflockinfo, SPLDEVLOW, DEFAULT);
#endif
#if CCOMSUB > 0
        SLI("CComGlbSlock", CComGlbSlockInfo, SPLDEVLOW, DEFAULT);
        SLI("CComQueSlock", CComQueSlockInfo, SPLDEVLOW, DEFAULT);
#endif
#if DRD > 0
	LI("bsc_rm_segment.bs_seg_lock", bsc_seg_lockinfo, SPLDEVLOW, DEFAULT);
	ULI("bsc_rm_lock", bsc_rm_lock_info, SPLDEVLOW, DEFAULT);
	ULI("bss_rm_pend_lock", bss_rm_pend_lock_info, SPLDEVLOW, DEFAULT);
	ULI("drd_retry_que_lock",  drd_retry_que_lockinfo,     SPLSOFTC, DEFAULT);
#endif
#if RM > 0
	SLI("rm_event_buff_lock", rm_event_buff_lockinfo, SPLDEVLOW, DEFAULT);
#endif
#if DLMSL > 0
	ULI("slrecv_smp_lock", slrecv_smp_lockinfo,  SPLMEMCHN, DEFAULT);
#endif /* DLMSL */

#if NSCSI > 0
	ULI("cam_pd_unit_table",cam_pd_unit_table_li,   SPLBIO, DEFAULT);
	ULI("cam_pd_device",    cam_pd_device_li,       SPLBIO, DEFAULT);
	ULI("cam_resp_que",     cam_resp_que_li,        SPLBIO, DEFAULT);
	ULI("cam_pd_device3",   cam_pd_device3_li,      SPLBIO, DEFAULT);
	ULI("cam_disk",    	cam_dsk_lock_li,        SPLBIO, DEFAULT);
	ULI("cam_disk_recq",   	cam_recq_lock_li,       SPLBIO, DEFAULT);
	ULI("cam_c_ctrl",       cam_c_ctrl_li,          SPLBIO, DEFAULT);
	ULI("cam_c_qhead",      cam_c_qhead_li,         SPLBIO, DEFAULT);
	ULI("cam_c_edt",        cam_c_edt_li,           SPLBIO, DEFAULT);
	ULI("cam_ccmn_bp_pool", cam_ccmn_bp_pool_li,    SPLBIO, DEFAULT);
	ULI("cam_ccmn_data_pool",cam_ccmn_data_pool_li, SPLBIO, DEFAULT);
	ULI("cam_d3_ctrl",      cam_d3_ctrl_li,         SPLBIO, DEFAULT);
	ULI("cam_x_qhead",      cam_x_qhead_li,         SPLBIO, DEFAULT);
	ULI("cam_x_ctrl",       cam_x_ctrl_li,          SPLBIO, DEFAULT);
	ULI("cam_softc",        cam_softc_li,           SPLBIO, DEFAULT);
	ULI("cam_reg",          cam_reg_li,             SPLBIO, DEFAULT);
	ULI("cam_cplt",         cam_cplt_li,            SPLBIO, DEFAULT);
	ULI("cam_sm",           cam_sm_li,              SPLBIO, DEFAULT);
#endif
#if NASC > 0 || NTCDS > 0 || NPSIOP > 0 || NSIOP > 0 || NAHA > 0 ||  NXZA > 0 || NISP > 0
	ULI("cam_htm",          cam_htm_nexuslk_li,     SPLBIO, DEFAULT);
#endif

	ULI("ddr_dbase", ddr_dbase_open_lkinfo, SPLNONE, DEFAULT);
	ULI("ddr_scsidevs", ddr_scsidevs_lockinfo, SPLNONE, DEFAULT);
	ULI("dev_desc.lk", cam_ddr_dev_desc_lkinfo, SPLNONE, DEFAULT);

#if NXZA > 0
	ULI("cam_xza_target",   cam_xza_target_li,      SPLBIO, DEFAULT);
#endif
#if NSIOP > 0
	ULI("cam_siop_doneq",   cam_siop_doneq_li,      SPLBIO, DEFAULT);
	ULI("cam_free_job",     cam_free_job_li,        SPLBIO, DEFAULT);
	ULI("cam_free_buf",     cam_free_buf_li,        SPLBIO, DEFAULT);
	ULI("cam_sh",           cam_sh_li,              SPLBIO, DEFAULT);
	ULI("cam_siopq",        cam_siopq_li,           SPLBIO, DEFAULT);
#endif
#if NPSIOP > 0
        ULI("cam_psiop_doneq",   cam_psiop_doneq_li,    SPLBIO, DEFAULT);
        ULI("cam_pfree_job",     cam_pfree_job_li,      SPLBIO, DEFAULT);
        ULI("cam_psh",           cam_psh_li,            SPLBIO, DEFAULT);
        ULI("cam_psiopq",        cam_psiopq_li,         SPLBIO, DEFAULT);
#endif
#if NAHA > 0
	ULI("cam_aha_softc",	 cam_aha_softc_li,	SPLBIO, DEFAULT);
	ULI("cam_aha_freeq",	 cam_aha_freeq_li,	SPLBIO, DEFAULT);
	ULI("cam_aha_activeq",	 cam_aha_activeq_li,	SPLBIO, DEFAULT);
#endif
#if NTZA > 0 || NPZA > 0
	ULI("cam_spo_generic",	 cam_spo_genericlk_li,	SPLBIO, DEFAULT);
	ULI("cam_spo_resource",	 cam_spo_resourcelk_li,	SPLBIO, DEFAULT);
	ULI("cam_spo_nexus",	 cam_spo_nexuslk_li,	SPLBIO, DEFAULT);
	ULI("cam_spo_iowaitq",	 cam_spo_iowaitq_li,	SPLBIO, DEFAULT);
	ULI("cam_spo_dacq",	 cam_spo_dacqlk_li,	SPLBIO, DEFAULT);
	ULI("cam_spo_dafq",	 cam_spo_dafqlk_li,	SPLBIO, DEFAULT);
	ULI("cam_spo_adrq",	 cam_spo_adrqlk_li,	SPLBIO, DEFAULT);
	ULI("cam_spo_adfq",	 cam_spo_adfqlk_li,	SPLBIO, DEFAULT);
	ULI("cam_spo_debug",	 cam_spo_debugqlk_li,	SPLBIO, DEFAULT);
#endif /* NTZA > 0 || NPZA > 0 */
#if NISP > 0 
	ULI("isp_abort",         cam_isp_abort_li,      SPLBIO, DEFAULT);
	ULI("cam_isp",		 cam_isp_li,		SPLBIO, DEFAULT);
#endif /* NISP > 0 */
#if NXCR > 0
	ULI("astro_re_table",	re_table_li,		SPLBIO, DEFAULT);
	ULI("astro_re_device",	re_device_li,		SPLBIO, DEFAULT);
	ULI("astro_xcr_softc",	xcr_softc_li,		SPLBIO, DEFAULT);
	ULI("astro_xcr_reg",	xcr_reg_li,		SPLBIO, DEFAULT);
	ULI("astro_xcr_cb",	xcr_cb_li,		SPLBIO, DEFAULT);
#endif
#if NSCSI > 0
	ULI("cam_pool_ctrl",    cam_pool_ctrl_li,       SPLNONE, DEFAULT);
	ULI("cam_pool_queue",   cam_pool_queue_li,      SPLNONE, DEFAULT);
#endif

#if STREAMS != 0 || TIMOD || SVVS || STRKINFO || STRIFNET || XTISO || DLB || DLPI || LDTTY || RPTTY || PCKT || LAT 
	SLI("osrq_lock", osrq_lockinfo, SPLDEVHIGH,DEFAULT);
	SLI("streams_log_lock", streamsm_log_lockinfo, SPLDEVHIGH,DEFAULT);
	SLI("mblk_lock",streams_mblk_lockinfo, SPLDEVHIGH, DEFAULT);
	SLI("queue(streams).q_lock", streams_q_lockinfo, SPLDEVHIGH, DEFAULT);
	SLI("sqh_s.sqh_lock", sqh_lockinfo, SPLDEVHIGH, DEFAULT);
	SLI("str_to_lock", str_to_lockinfo, SPLDEVHIGH, DEFAULT);
#endif /* STREAMS != 0 || TIMOD || SVVS || STRKINFO || STRIFNET || XTISO || DLB || DLPI || LDTTY || RPTTY || PCKT || LAT */
#if	(DLB > 0)
	NLI("dlb_slock", dlb_sli, SPLIMP, DEFAULT);
	NLI("dlbq_lock", dlbq_sli, SPLIMP, DEFAULT);
#endif
#if MCNET > 0
        NLI("mcnet_sclock", mcnet_sclockinfo, SPLIMP, DEFAULT);
#endif
#if NGWSCREEN > 0
	NLI("gwscreen_lock", gwscreen_li, SPLIMP, DEFAULT);
#endif
#if NTE > 0	/* TEGEC */
	SLI("te_softc.lk_te_softc", te_softc_lockinfo, SPLIMP, DEFAULT);
#endif
#if NTU > 0	/* TULIP */
	SLI("tu_softc.lk_tu_softc", tu_softc_lockinfo, SPLIMP, DEFAULT);
#endif
#if NSIMETHER > 0	/* SIMETHER */
	SLI("se_softc.lk_se_softc", se_softc_lockinfo, SPLIMP, DEFAULT);
#endif
#if NXNA > 0	/* DEMNA */
	SLI("xna_softc.lk_xna_softc", xna_softc_lockinfo, SPLIMP, DEFAULT);
#endif
#if NMFA > 0	/* DEMFA */
	SLI("mfa_softc.lk_mfa_softc", mfa_softc_lockinfo, SPLIMP, DEFAULT);
#endif
#if NFTA > 0	/* DEFTA, DEFEA, DEFPA */
	SLI("fta_softc.lk_fta_softc", fta_softc_lockinfo, SPLIMP, DEFAULT);
#endif
#if NFAA > 0	/* DEFAA */
	SLI("fxx_softc.lk_interrupt_level",
		faa_interrupt_level_lockinfo, SPLIMP, DEFAULT);
#endif
#if NLTA > 0    /* DGLTA, DGLPB */
        SLI("otto_softc.lk_otto_softc", otto_softc_lockinfo, SPLIMP, DEFAULT);
        SLI("otto_softc.lk_ready_q", otto_rdy_q_lockinfo, SPLIMP, DEFAULT);
#endif
#if PACKETFILTER > 0
	NLI("pfilt_lock", pfilt_lockinfo, SPLIMP, DEFAULT);
#endif
#if	defined(DEC2100_A500) || defined(DEC2100_C500)
	ULI("PCI0_hae0_3_lock",        PCI0_hae0_3_li, SPLDEVHIGH, DEFAULT);
	ULI("PCI1_hae0_3_lock",        PCI1_hae0_3_li, SPLDEVHIGH, DEFAULT);
	ULI("cbus2_pci_int_lock",      cbus2_pci_int_li, SPLDEVHIGH, DEFAULT);
#endif	/* defined(DEC2100_A500) || defined(DEC2100_C500) */
#ifdef	DEC21000
	ULI("pcia_ctl_lock",        pcia_ctl_li, SPLDEVHIGH, DEFAULT);
#endif	/* DEC21000	*/
#if NPCI > 0
	ULI("dma_hw_sg_rmap_lock", sg_rmap_lockinfo, SPLDEVHIGH, DEFAULT);
	ULI("shared_intr_lock", intr_add, SPLDEVHIGH, DEFAULT);
	ULI("shared_intr_act_lock", intr_act, SPLDEVHIGH, DEFAULT);
#endif	/* NPCI	*/
#if PWRMGR_ENABLED
	ULI("pwrmgr_data_lock", pwrmgr_lock_info, SPLDEVHIGH, DEFAULT);
#endif
	NLI("ifqueue.ifq_slock", ifqueue_li, SPLIMP, DEFAULT);
	NLI("ifnet.if_slock", ifnet_li, SPLIMP, DEFAULT);
	NLI("mbuf_slock", mbuf_sli, SPLIMP, DEFAULT);
	ULI("cku_private.lk_outbuf", cku_lk_outbuf_info, SPLIMP, DEFAULT);
	ULI("udp_data.lk_udpdata", lk_udpdata_info, SPLIMP, DEFAULT);
	ULI("nfs_rrok_lock", nfs_rrok_lockinfo, SPLIMP, DEFAULT);
	ULI("nfs_xprtfree_lock", nfs_xprtfree_lockinfo, SPLIMP, DEFAULT);
	NLI("netisr_slock", netisr_li, SPLIMP, DEFAULT);

        /* ATM LOCKS */
#if ATM
#if UNI3X
        ULI("atmsig_callref_lock",atmsig_callref_lock_info,SPLIMP,DEFAULT);
#endif
#if ATMIP
        ULI("atmip_driver_q_lock",atmip_driver_q_li,SPLIMP, DEFAULT);
        ULI("atmip_zombie_vc_lock",atmip_vc_zombie_li,SPLIMP, DEFAULT);
        ULI("atmip_zombie_ppa_lock",atmip_ppa_zombie_li,SPLIMP, DEFAULT);
        ULI("atmip_rtupdt_lock",atmip_rtupdt_li,SPLIMP, DEFAULT);
        ULI("atmip_drv_ppaq_lock",atmip_drv_ppaq_li,SPLIMP, DEFAULT);
        ULI("atmip_drv_esiq_lock",atmip_drv_esiq_li,SPLIMP, DEFAULT);
        ULI("atmip_drv_ifq_lock",atmip_drv_ifq_li,SPLIMP, DEFAULT);
        ULI("atmip_esi_ifq_lock",atmip_esi_ifq_li,SPLIMP, DEFAULT);
        ULI("atmip_map_lock",atmip_map_li,SPLIMP, DEFAULT);
        ULI("atmip_map_sec_lock",atmip_map_sec_li,SPLIMP, DEFAULT);
        ULI("atmip_ppa_lock", atmip_ppa_li, SPLIMP, DEFAULT);
        ULI("atm_arp_rcv_q_lock", atm_arp_rcv_q_lock_li, SPLIMP, DEFAULT);
        ULI("atm_arp_table_lock", atm_arp_table_lock_li, SPLIMP, DEFAULT);
        ULI("atm_arp_bucket_lock", atm_arp_bucket_lock_li, SPLIMP, DEFAULT);
        ULI("atm_arp_data_lock", atm_arp_data_lock_li, SPLIMP, DEFAULT);
        ULI("atmip_vc_lock", atmip_vc_li, SPLIMP, DEFAULT);
        ULI("atmip_ifnet_lock", atmip_ifnet_li, SPLIMP, DEFAULT);
        ULI("inarp_pending_q_lock", inarp_pending_q_lock_li, SPLIMP, DEFAULT);
#endif
#if UNI3x
        ULI("ifatmsig_q_lock",ifatmsig_q_lock_info,SPLIMP,DEFAULT);
#endif
        ULI("atm_cmm_driver_list",atm_cmm_driver_list_info,SPLIMP,DEFAULT);
        ULI("atm_cmm_ppa_list",atm_cmm_ppa_list_info,SPLIMP,DEFAULT);
        ULI("atm_cmm_converge_list",atm_cmm_converge_list_info,SPLIMP,DEFAULT);
        ULI("atm_cmm_signal_list",atm_cmm_signal_list_info,SPLIMP,DEFAULT);
        ULI("atm_cmm_vc_zombies_list",atm_cmm_vc_zombies_list_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_vcs_zombies_list",atm_cmm_vcs_zombies_list_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_bind_zombies_list",atm_cmm_bind_zombies_list_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_ppa_zombies_list",atm_cmm_ppa_zombies_list_info,
                                        SPLIMP,DEFAULT);

        ULI("atm_cmm_device_lock",atm_cmm_device_lock_info,SPLIMP,DEFAULT);
        ULI("atm_cmm_device.esi_list",atm_cmm_device_esi_list_lock_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_device.ppas_list",atm_cmm_device_ppas_list_lock_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_device.vchash_list",atm_cmm_device_vchash_list_lock_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_device.vcs_list",atm_cmm_device_vcs_list_lock_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_device.unalloc_list",atm_cmm_device_unalloc_list_lock_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_device.unused_list",atm_cmm_device_unused_list_lock_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_res_exp_list",atm_cmm_res_exp_list_info, SPLIMP,DEFAULT);

        ULI("atm_cmm_esi_lock",atm_cmm_esi_lock_info,SPLIMP,DEFAULT);

        ULI("atm_cmm_ppa_lock",atm_cmm_ppa_lock_info,SPLIMP,DEFAULT);
        ULI("atm_cmm_ppa_lock.binds_list",atm_cmm_ppa_binds_list_lock_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_ppa_lock.dbinds_list",atm_cmm_ppa_dbinds_list_lock_info,
                                        SPLIMP,DEFAULT);


        ULI("atm_cmm_cvg_lock",atm_cmm_cvg_lock_info,SPLIMP,DEFAULT);
        ULI("atm_cmm_cvg.esis_list",atm_cmm_cvg_esis_list_lock_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_cvg.binds_list",atm_cmm_cvg_binds_list_lock_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_cvg.vcs_list",atm_cmm_cvg_vcs_list_lock_info,
                                        SPLIMP,DEFAULT);

        ULI("atm_cmm_sig_lock",atm_cmm_sig_lock_info,SPLIMP,DEFAULT);
        ULI("atm_cmm_sig.ppas_list",atm_cmm_sig_ppas_list_lock_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_sig_lock.vcs_list",atm_cmm_sig_vcs_list_lock_info,
                                        SPLIMP,DEFAULT);

        ULI("atm_cmm_bind_lock",atm_cmm_bind_lock_info,SPLIMP,DEFAULT);
        ULI("atm_cmm_bind.vcs_list",atm_cmm_bind_vcs_list_lock_info,
                                        SPLIMP,DEFAULT);
	ULI("atm_cmm_bind.rvcs_list",atm_cmm_bind_rvcs_list_lock_info,
					SPLIMP,DEFAULT);
	ULI("atm_cmm_start_queue",atm_cmm_start_q_list_lock_info,
					SPLIMP,DEFAULT);
        ULI("atm_cmm_vc_lock",atm_cmm_vc_lock_info,SPLIMP,DEFAULT);
        ULI("atm_cmm_vc.eps_list",atm_cmm_vc_eps_list_lock_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_vc.ieps_list",atm_cmm_vc_ieps_list_lock_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_vc.ceps_list",atm_cmm_vc_ceps_list_lock_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_vc.deps_list",atm_cmm_vc_deps_list_lock_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_vc.neps_list",atm_cmm_vc_neps_list_lock_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_vc.causes_list",atm_cmm_vc_causes_list_lock_info,
                                        SPLIMP,DEFAULT);
        ULI("atm_cmm_vc.log_list",atm_cmm_vc_log_list_lock_info,
                                        SPLIMP,DEFAULT);

        ULI("atm_cmm_vcs_lock",atm_cmm_vcs_lock_info,SPLIMP,DEFAULT);
#if UNI3X
        NLI("atmsig_eventq_lock", atmsig_event_q_lock_info, SPLIMP, DEFAULT);
#endif
#endif
        /* End of ATM locks */

	SLI("pmap_seg.ps_seg_lock", pmap_seg_ps_seglockinfo,
							SPLIMP, DEFAULT);
	SLI("pmap_ssm.pss_lock", pmap_ssm_lockinfo, SPLIMP, DEFAULT);
	SLI("pmap.lock", user_pmap_lockinfo, SPLIMP, DEFAULT);
	SLI("pmap.other_lock", user_pmap_other_lockinfo, SPLIMP, DEFAULT);
	SLI("kernel_pmap.lock", kernel_pmap_lockinfo, SPLIMP, DEFAULT);
	SLI("kernel_pmap.other_lock", kernel_pmap_other_lockinfo,
							SPLIMP, DEFAULT);
	SLI("reaper_lock", reaper_lockinfo, SPLSCHED, DEFAULT);
	SLI("event.ev_slock", event_lockinfo, SPLSCHED, DEFAULT);
	SLI("thread_callq.tcq_lock", tcq_lockinfo, SPLHIGH, DEFAULT);
	ULI("proc.p_timer_lock", proc_timer_lockinfo, SPLHIGH, DEFAULT);
	SLI("mpqueue_head.lock", mpqueue_lockinfo, SPLHIGH, DEFAULT);
	SLI("kmemdata.waitlock", kmemdata_waitlock_info, SPLHIGH, DEFAULT);
	SLI("kmembuckets.kb_lock", kdp_kb_lockinfo, SPLHIGH, DEFAULT);
	SLI("vm_page_wire_lock", vm_page_wire_lockinfo, SPLHIGH, DEFAULT);
	SLI("vm_page_free_lock", vm_page_free_lockinfo, SPLHIGH, DEFAULT);
	SLI("vm_page_bucket.bk_lock", vm_page_bucket_lockinfo, SPLHIGH,DEFAULT);
	SLI("contig_malloc_lock", contig_malloc_lockinfo, SPLHIGH, DEFAULT);

	/* lock classes below here cannot use assert_wait, wakeup, etc. */

	SLI("class.lock", class_lockinfo, SPLSCHED, ORDERED);
	SLI("thread.lock", thread_lockinfo, SPLSCHED, ORDERED);
	SLI("wait_queue.lock", wait_queue_lockinfo, SPLSCHED, DEFAULT);
	SLI("swapper_lock_data", swapper_lockinfo, SPLSCHED, DEFAULT);
	SLI("callout_lock", callout_lockinfo, SPLHIGH, DEFAULT);
	SLI("processor_set.idle_lock", pset_idle_lockinfo, SPLSCHED, DEFAULT);
	SLI("processor.lock", processor_lockinfo, SPLSCHED, DEFAULT);
	SLI("processor.runq.lock", proc_runq_lockinfo, SPLSCHED, DEFAULT);
	SLI("processor_set.runq.lock", pset_runq_lockinfo, SPLSCHED, DEFAULT);
	SLI("action_lock", action_lockinfo, SPLSCHED, DEFAULT);

	ULI("session.s_fpgrp_lock", sess_fpgrp_lockinfo, SPLHIGH, DEFAULT);
	ULI("pgrp.pg_ref_lock", pgrp_refcnt_lockinfo, SPLHIGH, DEFAULT);
	SLI("xprlock", xprlockinfo, SPLHIGH, DEFAULT);
	SLI("processor.callout_lock", processor_callout_lockinfo, SPLDEVRT, ORDERED);
	SLI("callout_free_lock", callout_free_lockinfo, SPLDEVRT, DEFAULT);

	SLI("binlog_lock", binlog_lockinfo, SPLEXTREME, DEFAULT);
	SLI("msgbuf_lock", msgbuf_lockinfo, SPLEXTREME, DEFAULT);

	/******************************************/
	/* end of lockinfo specs for simple locks */
	/******************************************/

	/******************************************/
        /* start of lockinfo specs for NOCHECK    */
	/* simple locks.			  */
	/******************************************/

#if	(DEC4000 == 1)
	ULI_NOCHECK("kn430_fprint_lock", kn430_fprint_lockinfo);
	ULI_NOCHECK("kn430_watch_lock", kn430_watch_lockinfo);
#endif	/* DEC4000 */
#if	(NTRA > 0)
	ULI_NOCHECK("tra_softc.scb_clear_lock",  tra_lockinfo);
#endif 	/* (NTRA > 0 */
	ULI_NOCHECK("bp_map.map_lock", bp_map_lockinfo);
	ULI_NOCHECK("cluster_bl_lock", cluster_bl_lockinfo);
	ULI_NOCHECK("exec_switch_lock", exec_switch_lockinfo);
#if	(NMLOG > 0)
	ULI_NOCHECK("mem_lock", mem_lockinfo);
#endif	/* NMLOG > 0 */
#if	(CDFS > 0)
	ULI_NOCHECK("cdfs.lock", cdfs_lockinfo);
	ULI_NOCHECK("cdhead.hash_lock", cdfs_hash_lockinfo);
	ULI_NOCHECK("cdfs.mount_lock", cdfsmount_lockinfo);
	ULI_NOCHECK("cnode.cd_incore_lock", cdfs_incore_lockinfo);
#endif	/* CDFS > 0 */
#if	(FFM_FS > 0)
	ULI_NOCHECK("ffm_mounttab_lock", ffm_mounttab_lockinfo);
#endif	/* FFM_FS > 0 */
#if	(NTCDS > 0)
	ULI_NOCHECK("DMA_CTRL.d3_lk_ctrl", d3cntrl_lockinfo);
#endif	/* NTCDS > 0 */
#if	(NPSIOP > 0)
	ULI_NOCHECK("siop_lock", siop_lockinfo);
#endif	/* NPSIOP > 0 */
	ULI_NOCHECK("nfs_export_lock", nfs_export_lockinfo);
#if	(VAGUE_STATS == 0)
	ULI_NOCHECK("rpc_cl_stat_lock", rpc_cl_stat_lockinfo);
	ULI_NOCHECK("rpc_sv_stat_lock", rpc_sv_stat_lockinfo);
#endif
#if	(NVBA > 0) || (CVBA > 0)
	ULI_NOCHECK("vba_bus_adapt.adapt_s_lock", vhp_adapt_s_lockinfo);
#endif	/* NVBA > 0 || CVBA > 0 */
#if	(STRIFNET > 0)
	ULI_NOCHECK("contextPtr.context_lock", streamsm_context_lockinfo);
	ULI_NOCHECK("list(streams)_lock", streamsm_list_lockinfo);
#endif	/* STRIFNET > 0 */
#if	(NLSM_TED > 0)
	ULI_NOCHECK("ted_lockp", ted_lockp_lockinfo);
	ULI_NOCHECK("ted_event.e_condp", ted_act_lockinfo);
#endif	/* NLSM_TED > 0 */
	ULI_NOCHECK("mfs_minor_lock", mfs_minor_lockinfo);
	ULI_NOCHECK("gen_lock", gen_lockinfo);
	ULI_NOCHECK("dhp.dqh_lock", dqh_lockinfo);
	ULI_NOCHECK("dqfree_lock", dqfree_lockinfo);
	ULI_NOCHECK("ump_qsync_lock", ump_qsync_lockinfo);
        ULI_NOCHECK("mounttab_lock", mounttab_lockinfo);
	ULI_NOCHECK("fs.fs_lock", fs_lockinfo);
	ULI_NOCHECK("fifo_fileid_lock", fifo_fileid_lockinfo);
	ULI_NOCHECK("fifonode.fn_fifo_lock", fifo_lockinfo);
	ULI_NOCHECK("speclisth.sh_lock", spechash_lockinfo);
	ULI_NOCHECK("devlist.lock",devlist_lockinfo);
	ULI_NOCHECK("bfreelist_lock", bfreelist_lockinfo);
	ULI_NOCHECK("bufhd.bhd_lock", bhash_lockinfo);
	ULI_NOCHECK("flckinfo.flckinfo_lock", flckinfo_lockinfo);
	ULI_NOCHECK("flino.flino_lock", flino_lockinfo);
	ULI_NOCHECK("fids_lock", fids_lockinfo);
	ULI_NOCHECK("vn_dealloc_cache_lockup_ref",
		    vn_dealloc_cache_lookup_ref_lockinfo);
	ULI_NOCHECK("mount.m_lock", mount_lockinfo);
	ULI_NOCHECK("mount.m_vlist_lock", mount_vlist_lockinfo);
	ULI_NOCHECK("vnode.v_output_lock", vnode_output_lockinfo);
	ULI_NOCHECK("vnode.v_aux_lock", vnode_aux_lockinfo);
	ULI_NOCHECK("vnode.v_buflists_lock", buflists_lockinfo);
	ULI_NOCHECK("mountlist_lock", mountlist_lockinfo);
#if	MACH_LDEBUG
	ULI_NOCHECK("vnode_trace_lock", vnode_trace_lockinfo);
#endif	/* MACH_LDEBUG */
#if	(MACH_XP_FPD > 0)
	ULI_NOCHECK("memory_manager_default_lock", 
	  	    memory_manager_default_lockinfo);
#endif	/* (MACH_XP_FPD > 0) */
	ULI_NOCHECK("vm_perf_lock", vm_perf_lockinfo);
	ULI_NOCHECK("vm_swap_list.vsl_lock", vm_swap_list_vsl_lockinfo);
#if	(NLSM > 0)
	ULI_NOCHECK("volspin.vol_alloc_spin_lock", vol_alloc_spin_lockinfo);
	ULI_NOCHECK("volsync.sy_lock", vol_allocsync_lockinfo);
#endif	/* NLSM	> 0) */

	/******************************************/
        /* end of lockinfo specs for NOCHECK      */
	/* simple locks.			  */
	/******************************************/

	i = (1 << LOCKSTATS_SHIFT) - 1;
	lsp = (struct lockstats *)(((unsigned long)lip + i) & ~i);
	lockinfo_count = lip - lip_start;
	lockinfo_table = lip_start;

	if (lockmode==4) {
		for (i = 0; i < ncpus; i++, lsp += lockinfo_count)
			lockstats_table[i] = lsp;
		bzero((char *)lip, (char *)lsp - (char *)lip);
	}
	return((char *)lsp - (char *)lip_start);
}
